      SUBROUTINE DDASUM(N,DXH,DXL,INCX,DDH,DDL)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      INTEGER INCX,N
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION DXH(*),DXL(*)
*     ..
*
*  Purpose
*  =======
*
*     DDASUM takes the sum of the absolute values.
*
*  Advancesoft corp.
*  =================
*
*     jack dongarra, linpack, 3/11/78.
*     modified 3/93 to return if incx .le. 0.
*     modified 12/3/93, array(1) declarations changed to array(*)
*     modified by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. Local Scalars ..
      DOUBLE PRECISION DDH,DDL
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
!     DOUBLE PRECISION DTEMP
      INTEGER I,M,MP1,NINCX
*     ..
*     .. Intrinsic Functions ..
!     INTRINSIC DABS,MOD
      INTRINSIC MOD
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDASUM"
#endif
      DDH = 0.0d0
      DDL = 0.0d0
!     DASUM = 0.0d0
!     DTEMP = 0.0d0
      IF (N.LE.0 .OR. INCX.LE.0) RETURN
      IF (INCX.EQ.1) GO TO 20
*
*        code for increment not equal to 1
*
      NINCX = N*INCX
!$omp parallel
!$omp do
!$omp& private(I,t1,t2,p1,p2,q1,q2)
      DO 10 I = 1,NINCX,INCX
!         DTEMP = DTEMP + DABS(DX(I))
          if (DXH(I) .lt. 0.0D+0) then
            t1 = -DXH(I)
            t2 = -DXL(I)
          else
            t1 = DXH(I)
            t2 = DXL(I)
          endif
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   10 CONTINUE
!$omp end parallel
!     DASUM = DTEMP
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDASUM"
#include "fout.h"
#endif
      RETURN
*
*        code for increment equal to 1
*
*
*        clean-up loop
*
   20 M = MOD(N,6)
      IF (M.EQ.0) GO TO 40
      DO 30 I = 1,M
 !        DTEMP = DTEMP + DABS(DX(I))
          if (DXH(I) .lt. 0.0D+0) then
            t1 = -DXH(I)
            t2 = -DXL(I)
          else
            t1 = DXH(I)
            t2 = DXL(I)
          endif
!         call dd_selfadd(DDH,DDL,t1,t2)
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   30 CONTINUE
      IF (N.LT.6) GO TO 60
   40 MP1 = M + 1
!$omp parallel
!$omp do
!$omp& private(I,t1,t2,p1,p2,q1,q2)
      DO 50 I = MP1,N,6
!         DTEMP = DTEMP + DABS(DX(I)) + DABS(DX(I+1)) + DABS(DX(I+2)) +
!    +            DABS(DX(I+3)) + DABS(DX(I+4)) + DABS(DX(I+5))
          if (DXH(I) .lt. 0.0D+0) then
            t1 = -DXH(I)
            t2 = -DXL(I)
          else
            t1 = DXH(I)
            t2 = DXL(I)
          endif
!$omp critical
!      PRINT *, ' I= ',I,' thread = ', TID
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!
          if (DXH(I+1) .lt. 0.0D+0) then
            t1 = -DXH(I+1)
            t2 = -DXL(I+1)
          else
            t1 = DXH(I+1)
            t2 = DXL(I+1)
          endif
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!
          if (DXH(I+2) .lt. 0.0D+0) then
            t1 = -DXH(I+2)
            t2 = -DXL(I+2)
          else
            t1 = DXH(I+2)
            t2 = DXL(I+2)
          endif
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!
          if (DXH(I+3) .lt. 0.0D+0) then
            t1 = -DXH(I+3)
            t2 = -DXL(I+3)
          else
            t1 = DXH(I+3)
            t2 = DXL(I+3)
          endif
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!
          if (DXH(I+4) .lt. 0.0D+0) then
            t1 = -DXH(I+4)
            t2 = -DXL(I+4)
          else
            t1 = DXH(I+4)
            t2 = DXL(I+4)
          endif
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!
          if (DXH(I+5) .lt. 0.0D+0) then
            t1 = -DXH(I+5)
            t2 = -DXL(I+5)
          else
            t1 = DXH(I+5)
            t2 = DXL(I+5)
          endif
!$omp critical
          p1 = DDH + t1
          p2 = p1 - DDH
          q2 = (DDH - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DDL
          q2 = q2 + t2
          DDH = q1 + q2
          DDL = q2 - (DDH - q1)
!$omp end critical
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50 CONTINUE
!$omp end parallel
   60 CONTINUE
!  60 DASUM = DTEMP
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDASUM"
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
      END
