      SUBROUTINE DDAXPY(N,DAH,DAL,DXH,DXL,INCX,DYH,DYL,INCY)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION DAH,DAL
      INTEGER INCX,INCY,N
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION DXH(*),DXL(*),DYH(*),DYL(*)
*     ..
*
*  Purpose
*  =======
*
*     DDAXPY constant times a vector plus a vector.
*     uses unrolled loops for increments equal to one.
*
*  Advancesoft corp.
*  =================
*
*     First written by Osamu Hino 01/04/2010
*  =====================================================================
*
*     .. Local Scalars ..
      INTEGER I,IX,IY,M,MP1
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
      integer IXG,IYG
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MOD
*     ..
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "DDAXPY"
#endif
      IF (N.LE.0) RETURN
!     IF (DA.EQ.0.0d0) RETURN
      IF (DAH.EQ.0.0d0.AND.DAL.EQ.0.0d0) RETURN
      IF (INCX.EQ.1 .AND. INCY.EQ.1) GO TO 20
*
*        code for unequal increments or equal increments
*          not equal to 1
*
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      IXG = IX
      IYG = IY
!$omp parallel
!$omp do
!!!!!!$omp& private(I,IX,IY,s1,t1,t2,u1,p1,p2,q1,q2,r1,r2)
!$omp& private(I,IX,IY)
!$omp& private(s1,t1,t2,u1,p1,p2,q1,q2,r1,r2)
      DO 10 I = 1,N
          IX = IXG + INCX*(I-1)
          IY = IYG + INCY*(I-1)
!         DY(IY) = DY(IY) + DA*DX(IX)
          t1 = DAH * DXH(IX)
          u1 = 134217729.0*DAH
          p1 = u1 - (u1-DAH)
          p2 = DAH - p1
          u1 = 134217729.0*DXH(IX)
          r1 = u1 - (u1-DXH(IX))
          r2 = DXH(IX) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DAH * DXL(IX) + DAL * DXH(IX))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
          p1 = DYH(IY) + t1
          p2 = p1 - DYH(IY)
          q2 = (DYH(IY) - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DYL(IY)
          q2 = q2 + t2
          s1 = q1 + q2
          DYL(IY) = q2 - (s1 - q1)
          DYH(IY) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!
!          IX = IX + INCX
!          IY = IY + INCY
   10 CONTINUE
!$omp end parallel
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDAXPY"
#include "fout.h"
#endif
      RETURN
*
*        code for both increments equal to 1
*
*
*        clean-up loop
*
   20 M = MOD(N,4)
      IF (M.EQ.0) GO TO 40
      DO 30 I = 1,M
!         DY(I) = DY(I) + DA*DX(I)
          t1 = DAH * DXH(I)
          u1 = 134217729.0*DAH
          p1 = u1 - (u1-DAH)
          p2 = DAH - p1
          u1 = 134217729.0*DXH(I)
          r1 = u1 - (u1-DXH(I))
          r2 = DXH(I) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DAH * DXL(I) + DAL * DXH(I))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
          p1 = DYH(I) + t1
          p2 = p1 - DYH(I)
          q2 = (DYH(I) - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DYL(I)
          q2 = q2 + t2
          s1 = q1 + q2
          DYL(I) = q2 - (s1 - q1)
          DYH(I) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   30 CONTINUE
      IF (N.LT.4) RETURN
   40 MP1 = M + 1
!$omp parallel
!$omp do
!$omp& private(I,s1,t1,t2,u1,p1,p2,q1,q2,r1,r2)
      DO 50 I = MP1,N,4
!         DY(I) = DY(I) + DA*DX(I)
          t1 = DAH * DXH(I)
          u1 = 134217729.0*DAH
          p1 = u1 - (u1-DAH)
          p2 = DAH - p1
          u1 = 134217729.0*DXH(I)
          r1 = u1 - (u1-DXH(I))
          r2 = DXH(I) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DAH * DXL(I) + DAL * DXH(I))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
          p1 = DYH(I) + t1
          p2 = p1 - DYH(I)
          q2 = (DYH(I) - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DYL(I)
          q2 = q2 + t2
          s1 = q1 + q2
          DYL(I) = q2 - (s1 - q1)
          DYH(I) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         DY(I+1) = DY(I+1) + DA*DX(I+1)
          t1 = DAH * DXH(I+1)
          u1 = 134217729.0*DAH
          p1 = u1 - (u1-DAH)
          p2 = DAH - p1
          u1 = 134217729.0*DXH(I+1)
          r1 = u1 - (u1-DXH(I+1))
          r2 = DXH(I+1) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DAH * DXL(I+1) + DAL * DXH(I+1))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
          p1 = DYH(I+1) + t1
          p2 = p1 - DYH(I+1)
          q2 = (DYH(I+1) - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DYL(I+1)
          q2 = q2 + t2
          s1 = q1 + q2
          DYL(I+1) = q2 - (s1 - q1)
          DYH(I+1) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         DY(I+2) = DY(I+2) + DA*DX(I+2)
          t1 = DAH * DXH(I+2)
          u1 = 134217729.0*DAH
          p1 = u1 - (u1-DAH)
          p2 = DAH - p1
          u1 = 134217729.0*DXH(I+2)
          r1 = u1 - (u1-DXH(I+2))
          r2 = DXH(I+2) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DAH * DXL(I+2) + DAL * DXH(I+2))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
          p1 = DYH(I+2) + t1
          p2 = p1 - DYH(I+2)
          q2 = (DYH(I+2) - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DYL(I+2)
          q2 = q2 + t2
          s1 = q1 + q2
          DYL(I+2) = q2 - (s1 - q1)
          DYH(I+2) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!         DY(I+3) = DY(I+3) + DA*DX(I+3)
          t1 = DAH * DXH(I+3)
          u1 = 134217729.0*DAH
          p1 = u1 - (u1-DAH)
          p2 = DAH - p1
          u1 = 134217729.0*DXH(I+3)
          r1 = u1 - (u1-DXH(I+3))
          r2 = DXH(I+3) - r1
          t2 = ((p1*r1-t1) + p1*r2 + p2*r1) + p2*r2
          t2 = t2 + (DAH * DXL(I+3) + DAL * DXH(I+3))
          s1 = t1 + t2
          t2 = t2 - (s1 - t1)
          t1 = s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
          p1 = DYH(I+3) + t1
          p2 = p1 - DYH(I+3)
          q2 = (DYH(I+3) - (p1 - p2)) + (t1 - p2)
          q1 = p1
          q2 = q2 + DYL(I+3)
          q2 = q2 + t2
          s1 = q1 + q2
          DYL(I+3) = q2 - (s1 - q1)
          DYH(I+3) = s1
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
   50 CONTINUE
!$omp end parallel
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDAXPY"
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
      END
