!     SUBROUTINE DROTMG(DD1,DD2,DX1,DY1,DPARAM)
      subroutine ddrotmg(
     +  dd1h,dd1l,dd2h,dd2l,dx1h,dx1l,dy1h,dy1l,dparamh,dparaml
     +)
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE PRECISION DD1,DD2,DX1,DY1
      double precision dd1h, dd1l
      double precision dd2h, dd2l
      double precision dx1h, dx1l
      double precision dy1h, dy1l
*     ..
*     .. Array Arguments ..
!     DOUBLE PRECISION DPARAM(5)
      double precision dparamh(5), dparaml(5)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     ..
*
*  Purpose
*  =======
*
*     CONSTRUCT THE MODIFIED GIVENS TRANSFORMATION MATRIX H WHICH ZEROS
*     THE SECOND COMPONENT OF THE 2-VECTOR  (DSQRT(DD1)*DX1,DSQRT(DD2)*
*     DY2)**T.
*     WITH DPARAM(1)=DFLAG, H HAS ONE OF THE FOLLOWING FORMS..
*
*     DFLAG=-1.D0     DFLAG=0.D0        DFLAG=1.D0     DFLAG=-2.D0
*
*       (DH11  DH12)    (1.D0  DH12)    (DH11  1.D0)    (1.D0  0.D0)
*     H=(          )    (          )    (          )    (          )
*       (DH21  DH22),   (DH21  1.D0),   (-1.D0 DH22),   (0.D0  1.D0).
*     LOCATIONS 2-4 OF DPARAM CONTAIN DH11, DH21, DH12, AND DH22
*     RESPECTIVELY. (VALUES OF 1.D0, -1.D0, OR 0.D0 IMPLIED BY THE
*     VALUE OF DPARAM(1) ARE NOT STORED IN DPARAM.)
*
*     THE VALUES OF GAMSQ AND RGAMSQ SET IN THE DATA STATEMENT MAY BE
*     INEXACT.  THIS IS OK AS THEY ARE ONLY USED FOR TESTING THE SIZE
*     OF DD1 AND DD2.  ALL ACTUAL SCALING OF DATA IS DONE USING GAM.
*
*
*  Arguments
*  =========
*
*  DD1    (input/output) DOUBLE PRECISION
*
*  DD2    (input/output) DOUBLE PRECISION
*
*  DX1    (input/output) DOUBLE PRECISION
*
*  DY1    (input) DOUBLE PRECISION
*
*  DPARAM (input/output)  DOUBLE PRECISION array, dimension 5
*     DPARAM(1)=DFLAG
*     DPARAM(2)=DH11
*     DPARAM(3)=DH21
*     DPARAM(4)=DH12
*     DPARAM(5)=DH22
*
*  =====================================================================
*
!  double double modefication 3/2011 (f.y.)
!
*     .. Local Scalars ..
!     DOUBLE PRECISION DFLAG,DH11,DH12,DH21,DH22,DP1,DP2,DQ1,DQ2,DTEMP,
!    +                 DU,GAM,GAMSQ,ONE,RGAMSQ,TWO,ZERO
      double precision dflagh, dflagl
      double precision dh11h, dh11l
      double precision dh12h, dh12l
      double precision dh21h, dh21l
      double precision dh22h, dh22l
      double precision dp1h, dp1l
      double precision dp2h, dp2l
      double precision dq1h, dq1l
      double precision dq2h, dq2l
      double precision dtemph, dtempl
      double precision duh, dul
      double precision gamh, gaml
      double precision gamsqh, gamsql
      double precision oneh, onel
      double precision rgamsqh, rgamsql
      double precision twoh, twol
      double precision zeroh, zerol
      double precision absdq1h, absdq1l
      double precision absdq2h, absdq2l
      double precision ddmh, ddml
      double precision ddah, ddal
      double precision absdd2h, absdd2l
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
      INTEGER IGO
!     dummy var for fpcounter
      integer n, incx
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC DABS
*     ..
*     .. Data statements ..
*
!     DATA ZERO,ONE,TWO/0.D0,1.D0,2.D0/
!     DATA GAM,GAMSQ,RGAMSQ/4096.D0,16777216.D0,5.9604645D-8/
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "ddrotmg"
#endif
      zeroh = 0.0d+0
      zerol = 0.0d+0
      oneh = 1.0d+0
      onel = 0.0d+0
      twoh = 2.d0
      twol = 0.0d+0
      gamh = 4096.d0
      gaml = 0.0d+0
      gamsqh = 16777216.d0
      gamsql = 0.0d+0
      rgamsqh = 5.9604645D-8
      rgamsql = 0.0d+0
*     ..

!     IF (.NOT.DD1.LT.ZERO) GO TO 10
      if(.not.
!       (dd1 .lt. zero)
     + ((dd1h .lt. zeroh) .or.
     +  ((dd1h .eq. zeroh) .and.
     +   (dd1l .lt. zerol)))
     +) goto 10
*       GO ZERO-H-D-AND-DX1..
      GO TO 60
   10 CONTINUE
*     CASE-DD1-NONNEGATIVE
!     DP2 = DD2*DY1
!<<<  dp2 = dd2 * dy1
      dp2h = dd2h * dy1h
      t_u1 = 134217729.0 * dd2h
      t_p1 = t_u1
     +         - (t_u1 - dd2h)
      t_p2 = dd2h - t_p1
      t_u1 = 134217729.0 * dy1h
      t_r1 = t_u1
     +         - (t_u1 - dy1h)
      t_r2 = dy1h - t_r1
      dp2l = ((t_p1*t_r1
     +        - dp2h)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      dp2l = dp2l
     +      + (dd2h*dy1l
     +      + dd2l*dy1h)
      t_s1 = dp2h + dp2l
      dp2l = dp2l - (t_s1 - dp2h)
      dp2h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  dp2 = dd2 * dy1
!     IF (.NOT.DP2.EQ.ZERO) GO TO 20
      if(.not.
!       (dp2 .eq. zero)
     + ((dp2h .eq. zeroh) .and.
     +  (dp2l .eq. zerol))
     +) goto 20
!     DFLAG = -TWO
!<<<  dflag = -two
      dflagh = -twoh
      dflagl = -twol
!>>>  dflag = -two
      GO TO 260
*     REGULAR-CASE..
   20 CONTINUE
!     DP1 = DD1*DX1
!<<<  dp1 = dd1 * dx1
      dp1h = dd1h * dx1h
      t_u1 = 134217729.0 * dd1h
      t_p1 = t_u1
     +         - (t_u1 - dd1h)
      t_p2 = dd1h - t_p1
      t_u1 = 134217729.0 * dx1h
      t_r1 = t_u1
     +         - (t_u1 - dx1h)
      t_r2 = dx1h - t_r1
      dp1l = ((t_p1*t_r1
     +        - dp1h)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      dp1l = dp1l
     +      + (dd1h*dx1l
     +      + dd1l*dx1h)
      t_s1 = dp1h + dp1l
      dp1l = dp1l - (t_s1 - dp1h)
      dp1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  dp1 = dd1 * dx1
!     DQ2 = DP2*DY1
!<<<  dq2 = dp2 * dy1
      dq2h = dp2h * dy1h
      t_u1 = 134217729.0 * dp2h
      t_p1 = t_u1
     +         - (t_u1 - dp2h)
      t_p2 = dp2h - t_p1
      t_u1 = 134217729.0 * dy1h
      t_r1 = t_u1
     +         - (t_u1 - dy1h)
      t_r2 = dy1h - t_r1
      dq2l = ((t_p1*t_r1
     +        - dq2h)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      dq2l = dq2l
     +      + (dp2h*dy1l
     +      + dp2l*dy1h)
      t_s1 = dq2h + dq2l
      dq2l = dq2l - (t_s1 - dq2h)
      dq2h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  dq2 = dp2 * dy1
!     DQ1 = DP1*DX1
!<<<  dq1 = dp1 * dx1
      dq1h = dp1h * dx1h
      t_u1 = 134217729.0 * dp1h
      t_p1 = t_u1
     +         - (t_u1 - dp1h)
      t_p2 = dp1h - t_p1
      t_u1 = 134217729.0 * dx1h
      t_r1 = t_u1
     +         - (t_u1 - dx1h)
      t_r2 = dx1h - t_r1
      dq1l = ((t_p1*t_r1
     +        - dq1h)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      dq1l = dq1l
     +      + (dp1h*dx1l
     +      + dp1l*dx1h)
      t_s1 = dq1h + dq1l
      dq1l = dq1l - (t_s1 - dq1h)
      dq1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  dq1 = dp1 * dx1
*
!     IF (.NOT.DABS(DQ1).GT.DABS(DQ2)) GO TO 40
!<<<  absdq1 = abs(dq1)
      if (dq1h .lt. 0.0d+0) then
        absdq1h = -dq1h
        absdq1l = -dq1l
      elseif (
     +  (dq1h .eq. 0.0d+0).and.
     +  (dq1l .lt. 0.0d+0)) then
        absdq1h = dq1h
        absdq1l = -dq1l
      else
        absdq1h = dq1h
        absdq1l = dq1l
      endif
!>>>  absdq1 = abs(dq1)
!<<<  absdq2 = abs(dq2)
      if (dq2h .lt. 0.0d+0) then
        absdq2h = -dq2h
        absdq2l = -dq2l
      elseif (
     +  (dq2h .eq. 0.0d+0).and.
     +  (dq2l .lt. 0.0d+0)) then
        absdq2h = dq2h
        absdq2l = -dq2l
      else
        absdq2h = dq2h
        absdq2l = dq2l
      endif
!>>>  absdq2 = abs(dq2)
      if(.not.
!       (absdq1 .gt. absdq2)
     + ((absdq1h .gt. absdq2h) .or.
     +  ((absdq1h .eq. absdq2h) .and.
     +   (absdq1l .gt. absdq2l)))
     +) goto 40
!     DH21 = -DY1/DX1
!<<<  dh21 = dy1 / dx1
      t_q1 = dy1h / dx1h
      dh21h = dx1h * t_q1
      t_u1 = 134217729.0 * dx1h
      t_p1 = t_u1
     +         - (t_u1 - dx1h)
      t_p2 = dx1h - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      dh21l = ((t_p1*t_r1
     +       - dh21h)
     +       + t_p1*t_r2
     +       + t_p2*t_r1)
     +       + t_p2*t_r2
      dh21l = dh21l + (dx1l*t_q1)
      t_s1 = dh21h + dh21l
      dh21l = dh21l - (t_s1 - dh21h)
      dh21h = t_s1
      t_s1 = dy1h - dh21h
      t_r1 = t_s1 - dy1h
      t_s2 = (dy1h 
     +         - (t_s1 - t_r1))
     +         - (dh21h + t_r1)
      t_s2 = t_s2 - dh21l
      t_s2 = t_s2 + dy1l
      t_q2 = (t_s1 + t_s2)
     +         / dx1h
      dh21h = t_q1 + t_q2
      dh21l = t_q2
     +      - (dh21h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dh21 = dy1 / dx1
!<<<  dh21 = -dh21
      dh21h = -dh21h
      dh21l = -dh21l
!>>>  dh21 = -dh21
!     DH12 = DP2/DP1
!<<<  dh12 = dp2 / dp1
      t_q1 = dp2h / dp1h
      dh12h = dp1h * t_q1
      t_u1 = 134217729.0 * dp1h
      t_p1 = t_u1
     +         - (t_u1 - dp1h)
      t_p2 = dp1h - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      dh12l = ((t_p1*t_r1
     +       - dh12h)
     +       + t_p1*t_r2
     +       + t_p2*t_r1)
     +       + t_p2*t_r2
      dh12l = dh12l + (dp1l*t_q1)
      t_s1 = dh12h + dh12l
      dh12l = dh12l - (t_s1 - dh12h)
      dh12h = t_s1
      t_s1 = dp2h - dh12h
      t_r1 = t_s1 - dp2h
      t_s2 = (dp2h 
     +         - (t_s1 - t_r1))
     +         - (dh12h + t_r1)
      t_s2 = t_s2 - dh12l
      t_s2 = t_s2 + dp2l
      t_q2 = (t_s1 + t_s2)
     +         / dp1h
      dh12h = t_q1 + t_q2
      dh12l = t_q2
     +      - (dh12h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dh12 = dp2 / dp1
*
!     DU = ONE - DH12*DH21
!<<<  ddm = dh12 * dh21
      ddmh = dh12h * dh21h
      t_u1 = 134217729.0 * dh12h
      t_p1 = t_u1
     +         - (t_u1 - dh12h)
      t_p2 = dh12h - t_p1
      t_u1 = 134217729.0 * dh21h
      t_r1 = t_u1
     +         - (t_u1 - dh21h)
      t_r2 = dh21h - t_r1
      ddml = ((t_p1*t_r1
     +        - ddmh)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      ddml = ddml
     +      + (dh12h*dh21l
     +      + dh12l*dh21h)
      t_s1 = ddmh + ddml
      ddml = ddml - (t_s1 - ddmh)
      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ddm = dh12 * dh21
!<<<  du = one - ddm
      t_p1 = oneh - ddmh
      t_p2 = t_p1 - oneh
      duh = t_p1
      dul = (oneh - (t_p1 - t_p2))
     +         - (ddmh + t_p2)
      dul = dul + onel - ddml
      t_s1 = duh + dul
      dul = dul - (t_s1 - duh)
      duh = t_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  du = one - ddm
*
!     IF (.NOT.DU.LE.ZERO) GO TO 30
      if(.not.
!       (du .le. zero)
     + ((duh .lt. zeroh) .or.
     +  ((duh .eq. zeroh) .and.
     +   (dul .le. zerol)))
     +) goto 30
*         GO ZERO-H-D-AND-DX1..
      GO TO 60
   30 CONTINUE
!     DFLAG = ZERO
      dflagh = zeroh
      dflagl = zerol
!     DD1 = DD1/DU
!<<<  dd1 = dd1 / du
      t_q1 = dd1h / duh
      t_ch = duh * t_q1
      t_u1 = 134217729.0 * duh
      t_p1 = t_u1
     +         - (t_u1 - duh)
      t_p2 = duh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (dul * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dd1h - t_ch
      t_r1 = t_s1 - dd1h
      t_s2 = (dd1h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dd1l
      t_q2 = (t_s1 + t_s2)
     +         / duh
      dd1h = t_q1 + t_q2
      dd1l = t_q2
     +      - (dd1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dd1 = dd1 / du
!     DD2 = DD2/DU
!<<<  dd2 = dd2 / du
      t_q1 = dd2h / duh
      t_ch = duh * t_q1
      t_u1 = 134217729.0 * duh
      t_p1 = t_u1
     +         - (t_u1 - duh)
      t_p2 = duh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (dul * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dd2h - t_ch
      t_r1 = t_s1 - dd2h
      t_s2 = (dd2h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dd2l
      t_q2 = (t_s1 + t_s2)
     +         / duh
      dd2h = t_q1 + t_q2
      dd2l = t_q2
     +      - (dd2h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dd2 = dd2 / du
!     DX1 = DX1*DU
!<<<  dx1 = dx1 * du
      t_ch = dx1h * duh
      t_u1 = 134217729.0 * dx1h
      t_p1 = t_u1
     +         - (t_u1 - dx1h)
      t_p2 = dx1h - t_p1
      t_u1 = 134217729.0 * duh
      t_r1 = t_u1
     +         - (t_u1 - duh)
      t_r2 = duh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (dx1h*dul
     +         +  dx1l*duh)
      t_s1 = t_ch + t_cl
      dx1l = t_cl - (t_s1 - t_ch)
      dx1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  dx1 = dx1 * du
*         GO SCALE-CHECK..
      GO TO 100
   40 CONTINUE
!     IF (.NOT.DQ2.LT.ZERO) GO TO 50
      if(.not.
!       (dq2 .lt. zero)
     + ((dq2h .lt. zeroh) .or.
     +  ((dq2h .eq. zeroh) .and.
     +   (dq2l .lt. zerol)))
     +) goto 50
*         GO ZERO-H-D-AND-DX1..
      GO TO 60
   50 CONTINUE
!     DFLAG = ONE
      dflagh = oneh
      dflagl = onel
!     DH11 = DP1/DP2
!<<<  dh11 = dp1 / dp2
      t_q1 = dp1h / dp2h
      dh11h = dp2h * t_q1
      t_u1 = 134217729.0 * dp2h
      t_p1 = t_u1
     +         - (t_u1 - dp2h)
      t_p2 = dp2h - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      dh11l = ((t_p1*t_r1
     +       - dh11h)
     +       + t_p1*t_r2
     +       + t_p2*t_r1)
     +       + t_p2*t_r2
      dh11l = dh11l + (dp2l*t_q1)
      t_s1 = dh11h + dh11l
      dh11l = dh11l - (t_s1 - dh11h)
      dh11h = t_s1
      t_s1 = dp1h - dh11h
      t_r1 = t_s1 - dp1h
      t_s2 = (dp1h 
     +         - (t_s1 - t_r1))
     +         - (dh11h + t_r1)
      t_s2 = t_s2 - dh11l
      t_s2 = t_s2 + dp1l
      t_q2 = (t_s1 + t_s2)
     +         / dp2h
      dh11h = t_q1 + t_q2
      dh11l = t_q2
     +      - (dh11h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dh11 = dp1 / dp2
!     DH22 = DX1/DY1
!<<<  dh22 = dx1 / dy1
      t_q1 = dx1h / dy1h
      dh22h = dy1h * t_q1
      t_u1 = 134217729.0 * dy1h
      t_p1 = t_u1
     +         - (t_u1 - dy1h)
      t_p2 = dy1h - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      dh22l = ((t_p1*t_r1
     +       - dh22h)
     +       + t_p1*t_r2
     +       + t_p2*t_r1)
     +       + t_p2*t_r2
      dh22l = dh22l + (dy1l*t_q1)
      t_s1 = dh22h + dh22l
      dh22l = dh22l - (t_s1 - dh22h)
      dh22h = t_s1
      t_s1 = dx1h - dh22h
      t_r1 = t_s1 - dx1h
      t_s2 = (dx1h 
     +         - (t_s1 - t_r1))
     +         - (dh22h + t_r1)
      t_s2 = t_s2 - dh22l
      t_s2 = t_s2 + dx1l
      t_q2 = (t_s1 + t_s2)
     +         / dy1h
      dh22h = t_q1 + t_q2
      dh22l = t_q2
     +      - (dh22h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dh22 = dx1 / dy1
!     DU = ONE + DH11*DH22
!<<<  ddm = dh11 * dh22
      ddmh = dh11h * dh22h
      t_u1 = 134217729.0 * dh11h
      t_p1 = t_u1
     +         - (t_u1 - dh11h)
      t_p2 = dh11h - t_p1
      t_u1 = 134217729.0 * dh22h
      t_r1 = t_u1
     +         - (t_u1 - dh22h)
      t_r2 = dh22h - t_r1
      ddml = ((t_p1*t_r1
     +        - ddmh)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      ddml = ddml
     +      + (dh11h*dh22l
     +      + dh11l*dh22h)
      t_s1 = ddmh + ddml
      ddml = ddml - (t_s1 - ddmh)
      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ddm = dh11 * dh22
!<<<  du = one + ddm
      t_p1 = oneh + ddmh
      t_p2 = t_p1 - oneh
      duh = t_p1
      dul = (oneh - (t_p1 - t_p2))
     +      + (ddmh - t_p2)
      dul = dul + onel + ddml
      t_s1 = duh + dul
      dul = dul - (t_s1 - duh)
      duh = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  du = one + ddm
!     DTEMP = DD2/DU
!<<<  dtemp = dd2 / du
      t_q1 = dd2h / duh
      dtemph = duh * t_q1
      t_u1 = 134217729.0 * duh
      t_p1 = t_u1
     +         - (t_u1 - duh)
      t_p2 = duh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      dtempl = ((t_p1*t_r1
     +       - dtemph)
     +       + t_p1*t_r2
     +       + t_p2*t_r1)
     +       + t_p2*t_r2
      dtempl = dtempl + (dul*t_q1)
      t_s1 = dtemph + dtempl
      dtempl = dtempl - (t_s1 - dtemph)
      dtemph = t_s1
      t_s1 = dd2h - dtemph
      t_r1 = t_s1 - dd2h
      t_s2 = (dd2h 
     +         - (t_s1 - t_r1))
     +         - (dtemph + t_r1)
      t_s2 = t_s2 - dtempl
      t_s2 = t_s2 + dd2l
      t_q2 = (t_s1 + t_s2)
     +         / duh
      dtemph = t_q1 + t_q2
      dtempl = t_q2
     +      - (dtemph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dtemp = dd2 / du
!     DD2 = DD1/DU
!<<<  dd2 = dd1 / du
      t_q1 = dd1h / duh
      dd2h = duh * t_q1
      t_u1 = 134217729.0 * duh
      t_p1 = t_u1
     +         - (t_u1 - duh)
      t_p2 = duh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      dd2l = ((t_p1*t_r1
     +       - dd2h)
     +       + t_p1*t_r2
     +       + t_p2*t_r1)
     +       + t_p2*t_r2
      dd2l = dd2l + (dul*t_q1)
      t_s1 = dd2h + dd2l
      dd2l = dd2l - (t_s1 - dd2h)
      dd2h = t_s1
      t_s1 = dd1h - dd2h
      t_r1 = t_s1 - dd1h
      t_s2 = (dd1h 
     +         - (t_s1 - t_r1))
     +         - (dd2h + t_r1)
      t_s2 = t_s2 - dd2l
      t_s2 = t_s2 + dd1l
      t_q2 = (t_s1 + t_s2)
     +         / duh
      dd2h = t_q1 + t_q2
      dd2l = t_q2
     +      - (dd2h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dd2 = dd1 / du
!     DD1 = DTEMP
      dd1h = dtemph
      dd1l = dtempl
!     DX1 = DY1*DU
!<<<  dx1 = dy1 * du
      dx1h = dy1h * duh
      t_u1 = 134217729.0 * dy1h
      t_p1 = t_u1
     +         - (t_u1 - dy1h)
      t_p2 = dy1h - t_p1
      t_u1 = 134217729.0 * duh
      t_r1 = t_u1
     +         - (t_u1 - duh)
      t_r2 = duh - t_r1
      dx1l = ((t_p1*t_r1
     +        - dx1h)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      dx1l = dx1l
     +      + (dy1h*dul
     +      + dy1l*duh)
      t_s1 = dx1h + dx1l
      dx1l = dx1l - (t_s1 - dx1h)
      dx1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  dx1 = dy1 * du
*         GO SCALE-CHECK
      GO TO 100
*     PROCEDURE..ZERO-H-D-AND-DX1..
   60 CONTINUE
!     DFLAG = -ONE
!<<<  dflag = -one
      dflagh = -oneh
      dflagl = -onel
!>>>  dflag = -one
!     DH11 = ZERO
      dh11h = zeroh
      dh11l = zerol
!     DH12 = ZERO
      dh12h = zeroh
      dh12l = zerol
!     DH21 = ZERO
      dh21h = zeroh
      dh21l = zerol
!     DH22 = ZERO
      dh22h = zeroh
      dh22l = zerol
*
!     DD1 = ZERO
      dd1h = zeroh
      dd1l = zerol
!     DD2 = ZERO
      dd2h = zeroh
      dd2l = zerol
!     DX1 = ZERO
      dx1h = zeroh
      dx1l = zerol
*         RETURN..
      GO TO 220
*     PROCEDURE..FIX-H..
   70 CONTINUE
!     IF (.NOT.DFLAG.GE.ZERO) GO TO 90
      if(.not.
!       (dflag .ge. zero)
     + ((dflagh .gt. zeroh) .or.
     +  ((dflagh .eq. zeroh) .and.
     +   (dflagl .ge. zerol)))
     +) goto 90
*
!     IF (.NOT.DFLAG.EQ.ZERO) GO TO 80
      if(.not.
!       (dflag .eq. zero)
     + ((dflagh .eq. zeroh) .and.
     +  (dflagl .eq. zerol))
     +) goto 80
!     DH11 = ONE
      dh11h = oneh
      dh11l = onel
!     DH22 = ONE
      dh22h = oneh
      dh22l = onel
!     DFLAG = -ONE
!<<<  dflag = -one
      dflagh = -oneh
      dflagl = -onel
!>>>  dflag = -one
      GO TO 90
   80 CONTINUE
!     DH21 = -ONE
!<<<  dh21 = -one
      dh21h = -oneh
      dh21l = -onel
!>>>  dh21 = -one
!     DH12 = ONE
      dh12h = oneh
      dh12l = onel
!     DFLAG = -ONE
!<<<  dflag = -one
      dflagh = -oneh
      dflagl = -onel
!>>>  dflag = -one
   90 CONTINUE
!     GO TO IGO(120,150,180,210)
      if(igo .eq. 120) goto 120
      if(igo .eq. 150) goto 150
      if(igo .eq. 180) goto 180
      if(igo .eq. 210) goto 210
*     PROCEDURE..SCALE-CHECK
  100 CONTINUE
  110 CONTINUE
!     IF (.NOT.DD1.LE.RGAMSQ) GO TO 130
      if(.not.
!       (dd1 .le. rgamsq)
     + ((dd1h .lt. rgamsqh) .or.
     +  ((dd1h .eq. rgamsqh) .and.
     +   (dd1l .le. rgamsql)))
     +) goto 130
!     IF (DD1.EQ.ZERO) GO TO 160
      if(
!       (dd1 .eq. zero)
     + ((dd1h .eq. zeroh) .and.
     +  (dd1l .eq. zerol))
     +) goto 160
!     ASSIGN 120 TO IGO
      igo = 120
*              FIX-H..
      GO TO 70
  120 CONTINUE
!     DD1 = DD1*GAM**2
!<<<  ddm = gam * gam
      ddmh = gamh * gamh
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      ddml = ((t_p1*t_r1
     +        - ddmh)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      ddml = ddml
     +      + (gamh*gaml
     +      + gaml*gamh)
      t_s1 = ddmh + ddml
      ddml = ddml - (t_s1 - ddmh)
      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ddm = gam * gam
!<<<  dd1 = dd1 * ddm
      t_ch = dd1h * ddmh
      t_u1 = 134217729.0 * dd1h
      t_p1 = t_u1
     +         - (t_u1 - dd1h)
      t_p2 = dd1h - t_p1
      t_u1 = 134217729.0 * ddmh
      t_r1 = t_u1
     +         - (t_u1 - ddmh)
      t_r2 = ddmh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (dd1h*ddml
     +         +  dd1l*ddmh)
      t_s1 = t_ch + t_cl
      dd1l = t_cl - (t_s1 - t_ch)
      dd1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  dd1 = dd1 * ddm
!     DX1 = DX1/GAM
!<<<  dx1 = dx1 / gam
      t_q1 = dx1h / gamh
      t_ch = gamh * t_q1
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (gaml * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dx1h - t_ch
      t_r1 = t_s1 - dx1h
      t_s2 = (dx1h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dx1l
      t_q2 = (t_s1 + t_s2)
     +         / gamh
      dx1h = t_q1 + t_q2
      dx1l = t_q2
     +      - (dx1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dx1 = dx1 / gam
!     DH11 = DH11/GAM
!<<<  dh11 = dh11 / gam
      t_q1 = dh11h / gamh
      t_ch = gamh * t_q1
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (gaml * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dh11h - t_ch
      t_r1 = t_s1 - dh11h
      t_s2 = (dh11h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dh11l
      t_q2 = (t_s1 + t_s2)
     +         / gamh
      dh11h = t_q1 + t_q2
      dh11l = t_q2
     +      - (dh11h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dh11 = dh11 / gam
!     DH12 = DH12/GAM
!<<<  dh12 = dh12 / gam
      t_q1 = dh12h / gamh
      t_ch = gamh * t_q1
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (gaml * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dh12h - t_ch
      t_r1 = t_s1 - dh12h
      t_s2 = (dh12h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dh12l
      t_q2 = (t_s1 + t_s2)
     +         / gamh
      dh12h = t_q1 + t_q2
      dh12l = t_q2
     +      - (dh12h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dh12 = dh12 / gam
      GO TO 110
  130 CONTINUE
  140 CONTINUE
!     IF (.NOT.DD1.GE.GAMSQ) GO TO 160
      if(.not.
!       (dd1 .ge. gamsq)
     + ((dd1h .gt. gamsqh) .or.
     +  ((dd1h .eq. gamsqh) .and.
     +   (dd1l .ge. gamsql)))
     +) goto 160
!     ASSIGN 150 TO IGO
      igo = 150
*              FIX-H..
      GO TO 70
  150 CONTINUE
!     DD1 = DD1/GAM**2
!<<<  ddm = gam * gam
      ddmh = gamh * gamh
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      ddml = ((t_p1*t_r1
     +        - ddmh)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      ddml = ddml
     +      + (gamh*gaml
     +      + gaml*gamh)
      t_s1 = ddmh + ddml
      ddml = ddml - (t_s1 - ddmh)
      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ddm = gam * gam
!<<<  dd1 = dd1 / ddm
      t_q1 = dd1h / ddmh
      t_ch = ddmh * t_q1
      t_u1 = 134217729.0 * ddmh
      t_p1 = t_u1
     +         - (t_u1 - ddmh)
      t_p2 = ddmh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (ddml * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dd1h - t_ch
      t_r1 = t_s1 - dd1h
      t_s2 = (dd1h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dd1l
      t_q2 = (t_s1 + t_s2)
     +         / ddmh
      dd1h = t_q1 + t_q2
      dd1l = t_q2
     +      - (dd1h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dd1 = dd1 / ddm
!     DX1 = DX1*GAM
!<<<  dx1 = dx1 * gam
      t_ch = dx1h * gamh
      t_u1 = 134217729.0 * dx1h
      t_p1 = t_u1
     +         - (t_u1 - dx1h)
      t_p2 = dx1h - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (dx1h*gaml
     +         +  dx1l*gamh)
      t_s1 = t_ch + t_cl
      dx1l = t_cl - (t_s1 - t_ch)
      dx1h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  dx1 = dx1 * gam
!     DH11 = DH11*GAM
!<<<  dh11 = dh11 * gam
      t_ch = dh11h * gamh
      t_u1 = 134217729.0 * dh11h
      t_p1 = t_u1
     +         - (t_u1 - dh11h)
      t_p2 = dh11h - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (dh11h*gaml
     +         +  dh11l*gamh)
      t_s1 = t_ch + t_cl
      dh11l = t_cl - (t_s1 - t_ch)
      dh11h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  dh11 = dh11 * gam
!     DH12 = DH12*GAM
!<<<  dh12 = dh12 * gam
      t_ch = dh12h * gamh
      t_u1 = 134217729.0 * dh12h
      t_p1 = t_u1
     +         - (t_u1 - dh12h)
      t_p2 = dh12h - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (dh12h*gaml
     +         +  dh12l*gamh)
      t_s1 = t_ch + t_cl
      dh12l = t_cl - (t_s1 - t_ch)
      dh12h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  dh12 = dh12 * gam
      GO TO 140
  160 CONTINUE
  170 CONTINUE
!     IF (.NOT.DABS(DD2).LE.RGAMSQ) GO TO 190
!<<<  absdd2 = abs(dd2)
      if (dd2h .lt. 0.0d+0) then
        absdd2h = -dd2h
        absdd2l = -dd2l
      elseif (
     +  (dd2h .eq. 0.0d+0).and.
     +  (dd2l .lt. 0.0d+0)) then
        absdd2h = dd2h
        absdd2l = -dd2l
      else
        absdd2h = dd2h
        absdd2l = dd2l
      endif
!>>>  absdd2 = abs(dd2)
      if(.not.
!       (absdd2 .le. rgamsq)
     + ((absdd2h .lt. rgamsqh) .or.
     +  ((absdd2h .eq. rgamsqh) .and.
     +   (absdd2l .le. rgamsql)))
     +) goto 190
!     IF (DD2.EQ.ZERO) GO TO 220
      if(
!       (dd2 .eq. zero)
     + ((dd2h .eq. zeroh) .and.
     +  (dd2l .eq. zerol))
     +) goto 220
!     ASSIGN 180 TO IGO
      igo = 180
*              FIX-H..
      GO TO 70
  180 CONTINUE
!     DD2 = DD2*GAM**2
!<<<  ddm = gam * gam
      ddmh = gamh * gamh
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      ddml = ((t_p1*t_r1
     +        - ddmh)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      ddml = ddml
     +      + (gamh*gaml
     +      + gaml*gamh)
      t_s1 = ddmh + ddml
      ddml = ddml - (t_s1 - ddmh)
      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ddm = gam * gam
!<<<  dd2 = dd2 * ddm
      t_ch = dd2h * ddmh
      t_u1 = 134217729.0 * dd2h
      t_p1 = t_u1
     +         - (t_u1 - dd2h)
      t_p2 = dd2h - t_p1
      t_u1 = 134217729.0 * ddmh
      t_r1 = t_u1
     +         - (t_u1 - ddmh)
      t_r2 = ddmh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (dd2h*ddml
     +         +  dd2l*ddmh)
      t_s1 = t_ch + t_cl
      dd2l = t_cl - (t_s1 - t_ch)
      dd2h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  dd2 = dd2 * ddm
!     DH21 = DH21/GAM
!<<<  dh21 = dh21 / gam
      t_q1 = dh21h / gamh
      t_ch = gamh * t_q1
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (gaml * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dh21h - t_ch
      t_r1 = t_s1 - dh21h
      t_s2 = (dh21h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dh21l
      t_q2 = (t_s1 + t_s2)
     +         / gamh
      dh21h = t_q1 + t_q2
      dh21l = t_q2
     +      - (dh21h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dh21 = dh21 / gam
!     DH22 = DH22/GAM
!<<<  dh22 = dh22 / gam
      t_q1 = dh22h / gamh
      t_ch = gamh * t_q1
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (gaml * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dh22h - t_ch
      t_r1 = t_s1 - dh22h
      t_s2 = (dh22h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dh22l
      t_q2 = (t_s1 + t_s2)
     +         / gamh
      dh22h = t_q1 + t_q2
      dh22l = t_q2
     +      - (dh22h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dh22 = dh22 / gam
      GO TO 170
  190 CONTINUE
  200 CONTINUE
!<<<  absdd2 = abs(dd2)
      if (dd2h .lt. 0.0d+0) then
        absdd2h = -dd2h
        absdd2l = -dd2l
      elseif (
     +  (dd2h .eq. 0.0d+0).and.
     +  (dd2l .lt. 0.0d+0)) then
        absdd2h = dd2h
        absdd2l = -dd2l
      else
        absdd2h = dd2h
        absdd2l = dd2l
      endif
!>>>  absdd2 = abs(dd2)
!     IF (.NOT.DABS(DD2).GE.GAMSQ) GO TO 220
      if(.not.
!       (absdd2 .ge. gamsq)
     + ((absdd2h .gt. gamsqh) .or.
     +  ((absdd2h .eq. gamsqh) .and.
     +   (absdd2l .ge. gamsql)))
     +) goto 220
!     ASSIGN 210 TO IGO
      igo = 210
*              FIX-H..
      GO TO 70
  210 CONTINUE
!     DD2 = DD2/GAM**2
!<<<  ddm = gam * gam
      ddmh = gamh * gamh
      t_u1 = 134217729.0 * gamh
      t_p1 = t_u1
     +         - (t_u1 - gamh)
      t_p2 = gamh - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      ddml = ((t_p1*t_r1
     +        - ddmh)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      ddml = ddml
     +      + (gamh*gaml
     +      + gaml*gamh)
      t_s1 = ddmh + ddml
      ddml = ddml - (t_s1 - ddmh)
      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ddm = gam * gam
!<<<  dd2 = dd2 / ddm
      t_q1 = dd2h / ddmh
      t_ch = ddmh * t_q1
      t_u1 = 134217729.0 * ddmh
      t_p1 = t_u1
     +         - (t_u1 - ddmh)
      t_p2 = ddmh - t_p1
      t_u1 = 134217729.0 * t_q1
      t_r1 = t_u1
     +         - (t_u1 - t_q1)
      t_r2 = t_q1 - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +         + t_p1*t_r2
     +         + t_p2*t_r1)
     +         + t_p2*t_r2
      t_cl = t_cl
     +         + (ddml * t_q1)
      t_s1 = t_ch + t_cl
      t_cl = t_cl
     +         - (t_s1 - t_ch)
      t_ch = t_s1
      t_s1 = dd2h - t_ch
      t_r1 = t_s1 - dd2h
      t_s2 = (dd2h
     +         - (t_s1 - t_r1))
     +         - (t_ch + t_r1)
      t_s2 = t_s2 - t_cl
      t_s2 = t_s2 + dd2l
      t_q2 = (t_s1 + t_s2)
     +         / ddmh
      dd2h = t_q1 + t_q2
      dd2l = t_q2
     +      - (dd2h - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  dd2 = dd2 / ddm
!     DH21 = DH21*GAM
!<<<  dh21 = dh21 * gam
      t_ch = dh21h * gamh
      t_u1 = 134217729.0 * dh21h
      t_p1 = t_u1
     +         - (t_u1 - dh21h)
      t_p2 = dh21h - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (dh21h*gaml
     +         +  dh21l*gamh)
      t_s1 = t_ch + t_cl
      dh21l = t_cl - (t_s1 - t_ch)
      dh21h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  dh21 = dh21 * gam
!     DH22 = DH22*GAM
!<<<  dh22 = dh22 * gam
      t_ch = dh22h * gamh
      t_u1 = 134217729.0 * dh22h
      t_p1 = t_u1
     +         - (t_u1 - dh22h)
      t_p2 = dh22h - t_p1
      t_u1 = 134217729.0 * gamh
      t_r1 = t_u1
     +         - (t_u1 - gamh)
      t_r2 = gamh - t_r1
      t_cl = ((t_p1*t_r1
     +         - t_ch)
     +        + t_p1 * t_r2
     +        + t_p2 * t_r1)
     +        + t_p2 * t_r2
      t_cl = t_cl
     +         + (dh22h*gaml
     +         +  dh22l*gamh)
      t_s1 = t_ch + t_cl
      dh22l = t_cl - (t_s1 - t_ch)
      dh22h = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  dh22 = dh22 * gam
      GO TO 200
  220 CONTINUE
!     IF (DFLAG) 250,230,240
      if(
!       (dflag .lt. 0.0)
     + ((dflagh .lt. 0.0d+0) .or.
     +  ((dflagh .eq. 0.0d+0) .and.
     +   (dflagl .lt. 0.0d+0)))
     +) goto 250
      if(
!       (dflag .eq. 0.0)
     + ((dflagh .eq. 0.0d+0) .and.
     +  (dflagl .eq. 0.0d+0))
     +) goto 230
      if(
!       (dflag .gt. 0.0)
     + ((dflagh .gt. 0.0d+0) .or.
     +  ((dflagh .eq. 0.0d+0) .and.
     +   (dflagl .gt. 0.0d+0)))
     +) goto 240
  230 CONTINUE
!     DPARAM(3) = DH21
      dparamh(3) = dh21h
      dparaml(3) = dh21l
!     DPARAM(4) = DH12
      dparamh(4) = dh12h
      dparaml(4) = dh12l
      GO TO 260
  240 CONTINUE
!     DPARAM(2) = DH11
      dparamh(2) = dh11h
      dparaml(2) = dh11l
!     DPARAM(5) = DH22
      dparamh(5) = dh22h
      dparaml(5) = dh22l
      GO TO 260
  250 CONTINUE
!     DPARAM(2) = DH11
      dparamh(2) = dh11h
      dparaml(2) = dh11l
!     DPARAM(3) = DH21
      dparamh(3) = dh21h
      dparaml(3) = dh21l
!     DPARAM(4) = DH12
      dparamh(4) = dh12h
      dparaml(4) = dh12l
!     DPARAM(5) = DH22
      dparamh(5) = dh22h
      dparaml(5) = dh22l
  260 CONTINUE
!     DPARAM(1) = DFLAG
      dparamh(1) = dflagh
      dparaml(1) = dflagl
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = ddrotmg"
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
      END
