!     SUBROUTINE DSBMV(UPLO,N,K,ALPHA,A,LDA,X,INCX,BETA,Y,INCY)
      subroutine ddsbmv(uplo, n, k, alphah, alphal,
     +                  ah, al, lda, xh, xl, incx,
     +                  betah, betal, yh, yl, incy)
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE PRECISION ALPHA,BETA
      double precision alphah, alphal
      double precision betah, betal
      INTEGER INCX,INCY,K,LDA,N
      CHARACTER UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE PRECISION A(LDA,*),X(*),Y(*)
      double precision ah(lda,*), al(lda,*)
      double precision xh(*), xl(*)
      double precision yh(*), yl(*)
!
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     ..
*
*  Purpose
*  =======
*
*  DSBMV  performs the matrix-vector  operation
*
*     y := alpha*A*x + beta*y,
*
*  where alpha and beta are scalars, x and y are n element vectors and
*  A is an n by n symmetric band matrix, with k super-diagonals.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the upper or lower
*           triangular part of the band matrix A is being supplied as
*           follows:
*
*              UPLO = 'U' or 'u'   The upper triangular part of A is
*                                  being supplied.
*
*              UPLO = 'L' or 'l'   The lower triangular part of A is
*                                  being supplied.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry, K specifies the number of super-diagonals of the
*           matrix A. K must satisfy  0 .le. K.
*           Unchanged on exit.
*
*  ALPHA  - DOUBLE PRECISION.
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with UPLO = 'U' or 'u', the leading ( k + 1 )
*           by n part of the array A must contain the upper triangular
*           band part of the symmetric matrix, supplied column by
*           column, with the leading diagonal of the matrix in row
*           ( k + 1 ) of the array, the first super-diagonal starting at
*           position 2 in row k, and so on. The top left k by k triangle
*           of the array A is not referenced.
*           The following program segment will transfer the upper
*           triangular part of a symmetric band matrix from conventional
*           full matrix storage to band storage:
*
*                 DO 20, J = 1, N
*                    M = K + 1 - J
*                    DO 10, I = MAX( 1, J - K ), J
*                       A( M + I, J ) = matrix( I, J )
*              10    CONTINUE
*              20 CONTINUE
*
*           Before entry with UPLO = 'L' or 'l', the leading ( k + 1 )
*           by n part of the array A must contain the lower triangular
*           band part of the symmetric matrix, supplied column by
*           column, with the leading diagonal of the matrix in row 1 of
*           the array, the first sub-diagonal starting at position 1 in
*           row 2, and so on. The bottom right k by k triangle of the
*           array A is not referenced.
*           The following program segment will transfer the lower
*           triangular part of a symmetric band matrix from conventional
*           full matrix storage to band storage:
*
*                 DO 20, J = 1, N
*                    M = 1 - J
*                    DO 10, I = J, MIN( N, J + K )
*                       A( M + I, J ) = matrix( I, J )
*              10    CONTINUE
*              20 CONTINUE
*
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           ( k + 1 ).
*           Unchanged on exit.
*
*  X      - DOUBLE PRECISION array of DIMENSION at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array X must contain the
*           vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  BETA   - DOUBLE PRECISION.
*           On entry, BETA specifies the scalar beta.
*           Unchanged on exit.
*
*  Y      - DOUBLE PRECISION array of DIMENSION at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array Y must contain the
*           vector y. On exit, Y is overwritten by the updated vector y.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*
!  double double modefication xxxx 3/2011
*     .. Parameters ..
!     DOUBLE PRECISION ONE,ZERO
!     PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
      double precision oneh, onel
      double precision zeroh, zerol
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP1,TEMP2
      double precision temp1h, temp1l
      double precision temp2h, temp2l
      INTEGER I,INFO,IX,IY,J,JX,JY,KPLUS1,KX,KY,L
      integer i0
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX,MIN
      double precision ddmh, ddml
      double precision ddah, ddal
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "ddsbmv"
#endif
*     ..
*
*     Test the input parameters.
*
      zeroh = 0.0d+0
      zerol = 0.0d+0
      oneh = 1.0d+0
      onel = 0.0d+0
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (K.LT.0) THEN
          INFO = 3
      ELSE IF (LDA.LT. (K+1)) THEN
          INFO = 6
      ELSE IF (INCX.EQ.0) THEN
          INFO = 8
      ELSE IF (INCY.EQ.0) THEN
          INFO = 11
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DSBMV ',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. ((ALPHA.EQ.ZERO).AND. (BETA.EQ.ONE))) RETURN
      if((n .eq. 0) .or. (
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +  .and.
!       (beta .eq. one)
     + ((betah .eq. oneh) .and.
     +  (betal .eq. onel))
     + )) goto 9999
*
*     Set up the start points in  X  and  Y.
*
      IF (INCX.GT.0) THEN
          KX = 1
      ELSE
          KX = 1 - (N-1)*INCX
      END IF
      IF (INCY.GT.0) THEN
          KY = 1
      ELSE
          KY = 1 - (N-1)*INCY
      END IF
*
*     Start the operations. In this version the elements of the array A
*     are accessed sequentially with one pass through A.
*
*     First form  y := beta*y.
*
!     IF (BETA.NE.ONE) THEN
      if(
!       (beta .ne. one)
     + ((betah .ne. oneh) .or.
     +  (betal .ne. onel))
     +) then
          IF (INCY.EQ.1) THEN
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                  DO 10 I = 1,N
!                     Y(I) = ZERO
                  yh(i) = zeroh
                  yl(i) = zerol
   10             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                  DO 20 I = 1,N
!                     Y(I) = BETA*Y(I)
!<<<              y(i) = beta * y(i)
                  t_ch = betah * yh(i)
                  t_u1 = 134217729.0 * betah
                  t_p1 = t_u1
     +                     - (t_u1 - betah)
                  t_p2 = betah - t_p1
                  t_u1 = 134217729.0 * yh(i)
                  t_r1 = t_u1
     +                     - (t_u1 - yh(i))
                  t_r2 = yh(i) - t_r1
                  t_cl = ((t_p1*t_r1
     +                     - t_ch)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  t_cl = t_cl
     +                     + (betah*yl(i)
     +                     +  betal*yh(i))
                  t_s1 = t_ch + t_cl
                  yl(i) = t_cl - (t_s1 - t_ch)
                  yh(i) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>              y(i) = beta * y(i)
   20             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          ELSE
              IY = KY
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I,iy)
                  DO 30 I = 1,N
                      iy = ky + (i - 1)*incy
!                     Y(IY) = ZERO
                      yh(IY) = zeroh
                      yl(IY) = zerol
!!                    IY = IY + INCY
   30             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I,iy)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                  DO 40 I = 1,N
                      iy = ky + (i - 1)*incy
!                     Y(IY) = BETA*Y(IY)
!<<<                  y(iy) = beta * y(iy)
                      t_ch = betah * yh(iy)
                      t_u1 = 134217729.0 * betah
                      t_p1 = t_u1
     +                         - (t_u1 - betah)
                      t_p2 = betah - t_p1
                      t_u1 = 134217729.0 * yh(iy)
                      t_r1 = t_u1
     +                         - (t_u1 - yh(iy))
                      t_r2 = yh(iy) - t_r1
                      t_cl = ((t_p1*t_r1
     +                         - t_ch)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      t_cl = t_cl
     +                         + (betah*yl(iy)
     +                         +  betal*yh(iy))
                      t_s1 = t_ch + t_cl
                      yl(iy) = t_cl - (t_s1 - t_ch)
                      yh(iy) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                  y(iy) = beta * y(iy)
!!                    IY = IY + INCY
   40             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          END IF
      END IF
!     IF (ALPHA.EQ.ZERO) RETURN
      if(
!                       (alpha .eq. zero)
     +                 ((alphah .eq. zeroh) .and.
     +                  (alphal .eq. zerol))
     +) goto 9999
      IF (LSAME(UPLO,'U')) THEN
*
*        Form  y  when upper triangle of A is stored.
*
          KPLUS1 = K + 1
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
              DO 60 J = 1,N
!                 TEMP1 = ALPHA*X(J)
!<<<          temp1 = alpha * x(j)
              temp1h = alphah * xh(j)
              t_u1 = 134217729.0 * alphah
              t_p1 = t_u1
     +                 - (t_u1 - alphah)
              t_p2 = alphah - t_p1
              t_u1 = 134217729.0 * xh(j)
              t_r1 = t_u1
     +                 - (t_u1 - xh(j))
              t_r2 = xh(j) - t_r1
              temp1l = ((t_p1*t_r1
     +                - temp1h)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              temp1l = temp1l
     +              + (alphah*xl(j)
     +              + alphal*xh(j))
              t_s1 = temp1h + temp1l
              temp1l = temp1l - (t_s1 - temp1h)
              temp1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          temp1 = alpha * x(j)
!                 TEMP2 = ZERO
              temp2h = zeroh
              temp2l = zerol
                  L = KPLUS1 - J
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                  DO 50 I = MAX(1,J-K),J - 1
!                     Y(I) = Y(I) + TEMP1*A(L+I,J)
!<<<              ddm = temp1 * a(l+i,j)
                  ddmh = temp1h * ah(l+i,j)
                  t_u1 = 134217729.0 * temp1h
                  t_p1 = t_u1
     +                     - (t_u1 - temp1h)
                  t_p2 = temp1h - t_p1
                  t_u1 = 134217729.0 * ah(l+i,j)
                  t_r1 = t_u1
     +                     - (t_u1 - ah(l+i,j))
                  t_r2 = ah(l+i,j) - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (temp1h*al(l+i,j)
     +                  + temp1l*ah(l+i,j))
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = temp1 * a(l+i,j)
!<<<              y(i) = y(i) + ddm
                  t_p1 = yh(i) + ddmh
                  t_p2 = t_p1 - yh(i)
                  t_q1 = t_p1
                  t_q2 = (yh(i) - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + yl(i) + ddml
                  yh(i) = t_q1 + t_q2
                  yl(i) = t_q2 - (yh(i) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(i) = y(i) + ddm
!                     TEMP2 = TEMP2 + A(L+I,J)*X(I)
!<<<              ddm = a(l+i,j) * x(i)
                  ddmh = ah(l+i,j) * xh(i)
                  t_u1 = 134217729.0 * ah(l+i,j)
                  t_p1 = t_u1
     +                     - (t_u1 - ah(l+i,j))
                  t_p2 = ah(l+i,j) - t_p1
                  t_u1 = 134217729.0 * xh(i)
                  t_r1 = t_u1
     +                     - (t_u1 - xh(i))
                  t_r2 = xh(i) - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (ah(l+i,j)*xl(i)
     +                  + al(l+i,j)*xh(i))
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = a(l+i,j) * x(i)
!$omp critical
!<<<              temp2 = temp2 + ddm
                  t_p1 = temp2h + ddmh
                  t_p2 = t_p1 - temp2h
                  t_q1 = t_p1
                  t_q2 = (temp2h - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + temp2l + ddml
                  temp2h = t_q1 + t_q2
                  temp2l = t_q2 - (temp2h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              temp2 = temp2 + ddm
!$omp end critical
   50             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(J) = Y(J) + TEMP1*A(KPLUS1,J) + ALPHA*TEMP2
!<<<          ddm = alpha * temp2
              ddmh = alphah * temp2h
              t_u1 = 134217729.0 * alphah
              t_p1 = t_u1
     +                 - (t_u1 - alphah)
              t_p2 = alphah - t_p1
              t_u1 = 134217729.0 * temp2h
              t_r1 = t_u1
     +                 - (t_u1 - temp2h)
              t_r2 = temp2h - t_r1
              ddml = ((t_p1*t_r1
     +                - ddmh)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              ddml = ddml
     +              + (alphah*temp2l
     +              + alphal*temp2h)
              t_s1 = ddmh + ddml
              ddml = ddml - (t_s1 - ddmh)
              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          ddm = alpha * temp2
!<<<          dda = temp1 * a(kplus1,j)
              ddah = temp1h * ah(kplus1,j)
              t_u1 = 134217729.0 * temp1h
              t_p1 = t_u1
     +                 - (t_u1 - temp1h)
              t_p2 = temp1h - t_p1
              t_u1 = 134217729.0 * ah(kplus1,j)
              t_r1 = t_u1
     +                 - (t_u1 - ah(kplus1,j))
              t_r2 = ah(kplus1,j) - t_r1
              ddal = ((t_p1*t_r1
     +                - ddah)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              ddal = ddal
     +              + (temp1h*al(kplus1,j)
     +              + temp1l*ah(kplus1,j))
              t_s1 = ddah + ddal
              ddal = ddal - (t_s1 - ddah)
              ddah = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          dda = temp1 * a(kplus1,j)
!<<<          dda = ddm + dda
              t_p1 = ddmh + ddah
              t_p2 = t_p1 - ddmh
              t_q1 = t_p1
              t_q2 = (ddmh - (t_p1 - t_p2))
     +                 + (ddah - t_p2)
              t_q2 = t_q2 + ddml + ddal
              ddah = t_q1 + t_q2
              ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>          dda = ddm + dda
!<<<          y(j) = y(j) + dda
              t_p1 = yh(j) + ddah
              t_p2 = t_p1 - yh(j)
              t_q1 = t_p1
              t_q2 = (yh(j) - (t_p1 - t_p2))
     +                 + (ddah - t_p2)
              t_q2 = t_q2 + yl(j) + ddal
              yh(j) = t_q1 + t_q2
              yl(j) = t_q2 - (yh(j) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>          y(j) = y(j) + dda
   60         CONTINUE
          ELSE
              JX = KX
              JY = KY
              DO 80 J = 1,N
!                 TEMP1 = ALPHA*X(JX)
!<<<          temp1 = alpha * x(jx)
              temp1h = alphah * xh(jx)
              t_u1 = 134217729.0 * alphah
              t_p1 = t_u1
     +                 - (t_u1 - alphah)
              t_p2 = alphah - t_p1
              t_u1 = 134217729.0 * xh(jx)
              t_r1 = t_u1
     +                 - (t_u1 - xh(jx))
              t_r2 = xh(jx) - t_r1
              temp1l = ((t_p1*t_r1
     +                - temp1h)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              temp1l = temp1l
     +              + (alphah*xl(jx)
     +              + alphal*xh(jx))
              t_s1 = temp1h + temp1l
              temp1l = temp1l - (t_s1 - temp1h)
              temp1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          temp1 = alpha * x(jx)
!                 TEMP2 = ZERO
              temp2h = zeroh
              temp2l = zerol
                  IX = KX
                  IY = KY
                  L = KPLUS1 - J
                  i0 = MAX(1,J-K)
!$omp parallel
!$omp do
!$omp+ private(I,ix,iy)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                  DO 70 I = MAX(1,J-K),J - 1
                      ix = kx + (i - i0)*incx
                      iy = ky + (i - i0)*incy
!                     Y(IY) = Y(IY) + TEMP1*A(L+I,J)
!<<<              ddm = temp1 * a(l+i,j)
                  ddmh = temp1h * ah(l+i,j)
                  t_u1 = 134217729.0 * temp1h
                  t_p1 = t_u1
     +                     - (t_u1 - temp1h)
                  t_p2 = temp1h - t_p1
                  t_u1 = 134217729.0 * ah(l+i,j)
                  t_r1 = t_u1
     +                     - (t_u1 - ah(l+i,j))
                  t_r2 = ah(l+i,j) - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (temp1h*al(l+i,j)
     +                  + temp1l*ah(l+i,j))
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = temp1 * a(l+i,j)
!<<<              y(iy) = y(iy) + ddm
                  t_p1 = yh(iy) + ddmh
                  t_p2 = t_p1 - yh(iy)
                  t_q1 = t_p1
                  t_q2 = (yh(iy) - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + yl(iy) + ddml
                  yh(iy) = t_q1 + t_q2
                  yl(iy) = t_q2 - (yh(iy) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(iy) = y(iy) + ddm
!                     TEMP2 = TEMP2 + A(L+I,J)*X(IX)
!<<<              ddm = a(l+i,j) * x(ix)
                  ddmh = ah(l+i,j) * xh(ix)
                  t_u1 = 134217729.0 * ah(l+i,j)
                  t_p1 = t_u1
     +                     - (t_u1 - ah(l+i,j))
                  t_p2 = ah(l+i,j) - t_p1
                  t_u1 = 134217729.0 * xh(ix)
                  t_r1 = t_u1
     +                     - (t_u1 - xh(ix))
                  t_r2 = xh(ix) - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (ah(l+i,j)*xl(ix)
     +                  + al(l+i,j)*xh(ix))
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = a(l+i,j) * x(ix)
!$omp critical
!<<<              temp2 = temp2 + ddm
                  t_p1 = temp2h + ddmh
                  t_p2 = t_p1 - temp2h
                  t_q1 = t_p1
                  t_q2 = (temp2h - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + temp2l + ddml
                  temp2h = t_q1 + t_q2
                  temp2l = t_q2 - (temp2h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              temp2 = temp2 + ddm
!$omp end critical
!!                    IX = IX + INCX
!!                    IY = IY + INCY
   70             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + TEMP1*A(KPLUS1,J) + ALPHA*TEMP2
!<<<          ddm = alpha * temp2
              ddmh = alphah * temp2h
              t_u1 = 134217729.0 * alphah
              t_p1 = t_u1
     +                 - (t_u1 - alphah)
              t_p2 = alphah - t_p1
              t_u1 = 134217729.0 * temp2h
              t_r1 = t_u1
     +                 - (t_u1 - temp2h)
              t_r2 = temp2h - t_r1
              ddml = ((t_p1*t_r1
     +                - ddmh)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              ddml = ddml
     +              + (alphah*temp2l
     +              + alphal*temp2h)
              t_s1 = ddmh + ddml
              ddml = ddml - (t_s1 - ddmh)
              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          ddm = alpha * temp2
!<<<          dda = temp1 * a(kplus1,j)
              ddah = temp1h * ah(kplus1,j)
              t_u1 = 134217729.0 * temp1h
              t_p1 = t_u1
     +                 - (t_u1 - temp1h)
              t_p2 = temp1h - t_p1
              t_u1 = 134217729.0 * ah(kplus1,j)
              t_r1 = t_u1
     +                 - (t_u1 - ah(kplus1,j))
              t_r2 = ah(kplus1,j) - t_r1
              ddal = ((t_p1*t_r1
     +                - ddah)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              ddal = ddal
     +              + (temp1h*al(kplus1,j)
     +              + temp1l*ah(kplus1,j))
              t_s1 = ddah + ddal
              ddal = ddal - (t_s1 - ddah)
              ddah = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          dda = temp1 * a(kplus1,j)
!<<<          dda = dda + ddm
              t_p1 = ddah + ddmh
              t_p2 = t_p1 - ddah
              t_q1 = t_p1
              t_q2 = (ddah - (t_p1 - t_p2))
     +                 + (ddmh - t_p2)
              t_q2 = t_q2 + ddal + ddml
              ddah = t_q1 + t_q2
              ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>          dda = dda + ddm
!<<<          y(jy) = y(jy) + dda
              t_p1 = yh(jy) + ddah
              t_p2 = t_p1 - yh(jy)
              t_q1 = t_p1
              t_q2 = (yh(jy) - (t_p1 - t_p2))
     +                 + (ddah - t_p2)
              t_q2 = t_q2 + yl(jy) + ddal
              yh(jy) = t_q1 + t_q2
              yl(jy) = t_q2 - (yh(jy) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>          y(jy) = y(jy) + dda
                  JX = JX + INCX
                  JY = JY + INCY
                  IF (J.GT.K) THEN
                      KX = KX + INCX
                      KY = KY + INCY
                  END IF
   80         CONTINUE
          END IF
      ELSE
*
*        Form  y  when lower triangle of A is stored.
*
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
              DO 100 J = 1,N
!                 TEMP1 = ALPHA*X(J)
!<<<          temp1 = alpha * x(j)
              temp1h = alphah * xh(j)
              t_u1 = 134217729.0 * alphah
              t_p1 = t_u1
     +                 - (t_u1 - alphah)
              t_p2 = alphah - t_p1
              t_u1 = 134217729.0 * xh(j)
              t_r1 = t_u1
     +                 - (t_u1 - xh(j))
              t_r2 = xh(j) - t_r1
              temp1l = ((t_p1*t_r1
     +                - temp1h)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              temp1l = temp1l
     +              + (alphah*xl(j)
     +              + alphal*xh(j))
              t_s1 = temp1h + temp1l
              temp1l = temp1l - (t_s1 - temp1h)
              temp1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          temp1 = alpha * x(j)
!                 TEMP2 = ZERO
              temp2h = zeroh
              temp2l = zerol
!                 Y(J) = Y(J) + TEMP1*A(1,J)
!<<<          ddm = temp1 * a(1,j)
              ddmh = temp1h * ah(1,j)
              t_u1 = 134217729.0 * temp1h
              t_p1 = t_u1
     +                 - (t_u1 - temp1h)
              t_p2 = temp1h - t_p1
              t_u1 = 134217729.0 * ah(1,j)
              t_r1 = t_u1
     +                 - (t_u1 - ah(1,j))
              t_r2 = ah(1,j) - t_r1
              ddml = ((t_p1*t_r1
     +                - ddmh)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              ddml = ddml
     +              + (temp1h*al(1,j)
     +              + temp1l*ah(1,j))
              t_s1 = ddmh + ddml
              ddml = ddml - (t_s1 - ddmh)
              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          ddm = temp1 * a(1,j)
!<<<          y(j) = y(j) + ddm
              t_p1 = yh(j) + ddmh
              t_p2 = t_p1 - yh(j)
              t_q1 = t_p1
              t_q2 = (yh(j) - (t_p1 - t_p2))
     +                 + (ddmh - t_p2)
              t_q2 = t_q2 + yl(j) + ddml
              yh(j) = t_q1 + t_q2
              yl(j) = t_q2 - (yh(j) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>          y(j) = y(j) + ddm
                  L = 1 - J
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                  DO 90 I = J + 1,MIN(N,J+K)
!                     Y(I) = Y(I) + TEMP1*A(L+I,J)
!<<<              ddm = temp1 * a(l+i,j)
                  ddmh = temp1h * ah(l+i,j)
                  t_u1 = 134217729.0 * temp1h
                  t_p1 = t_u1
     +                     - (t_u1 - temp1h)
                  t_p2 = temp1h - t_p1
                  t_u1 = 134217729.0 * ah(l+i,j)
                  t_r1 = t_u1
     +                     - (t_u1 - ah(l+i,j))
                  t_r2 = ah(l+i,j) - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (temp1h*al(l+i,j)
     +                  + temp1l*ah(l+i,j))
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = temp1 * a(l+i,j)
!<<<              y(i) = y(i) + ddm
                  t_p1 = yh(i) + ddmh
                  t_p2 = t_p1 - yh(i)
                  t_q1 = t_p1
                  t_q2 = (yh(i) - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + yl(i) + ddml
                  yh(i) = t_q1 + t_q2
                  yl(i) = t_q2 - (yh(i) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(i) = y(i) + ddm
!                     TEMP2 = TEMP2 + A(L+I,J)*X(I)
!<<<              ddm = a(l+i,j) * x(i)
                  ddmh = ah(l+i,j) * xh(i)
                  t_u1 = 134217729.0 * ah(l+i,j)
                  t_p1 = t_u1
     +                     - (t_u1 - ah(l+i,j))
                  t_p2 = ah(l+i,j) - t_p1
                  t_u1 = 134217729.0 * xh(i)
                  t_r1 = t_u1
     +                     - (t_u1 - xh(i))
                  t_r2 = xh(i) - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (ah(l+i,j)*xl(i)
     +                  + al(l+i,j)*xh(i))
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = a(l+i,j) * x(i)
!$omp critical
!<<<              temp2 = temp2 + ddm
                  t_p1 = temp2h + ddmh
                  t_p2 = t_p1 - temp2h
                  t_q1 = t_p1
                  t_q2 = (temp2h - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + temp2l + ddml
                  temp2h = t_q1 + t_q2
                  temp2l = t_q2 - (temp2h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              temp2 = temp2 + ddm
!$omp end critical
   90             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(J) = Y(J) + ALPHA*TEMP2
!<<<          ddm = alpha * temp2
              ddmh = alphah * temp2h
              t_u1 = 134217729.0 * alphah
              t_p1 = t_u1
     +                 - (t_u1 - alphah)
              t_p2 = alphah - t_p1
              t_u1 = 134217729.0 * temp2h
              t_r1 = t_u1
     +                 - (t_u1 - temp2h)
              t_r2 = temp2h - t_r1
              ddml = ((t_p1*t_r1
     +                - ddmh)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              ddml = ddml
     +              + (alphah*temp2l
     +              + alphal*temp2h)
              t_s1 = ddmh + ddml
              ddml = ddml - (t_s1 - ddmh)
              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          ddm = alpha * temp2
!<<<          y(j) = y(j) + ddm
              t_p1 = yh(j) + ddmh
              t_p2 = t_p1 - yh(j)
              t_q1 = t_p1
              t_q2 = (yh(j) - (t_p1 - t_p2))
     +                 + (ddmh - t_p2)
              t_q2 = t_q2 + yl(j) + ddml
              yh(j) = t_q1 + t_q2
              yl(j) = t_q2 - (yh(j) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>          y(j) = y(j) + ddm
  100         CONTINUE
          ELSE
              JX = KX
              JY = KY
              DO 120 J = 1,N
!                 TEMP1 = ALPHA*X(JX)
!<<<          temp1 = alpha * x(jx)
              temp1h = alphah * xh(jx)
              t_u1 = 134217729.0 * alphah
              t_p1 = t_u1
     +                 - (t_u1 - alphah)
              t_p2 = alphah - t_p1
              t_u1 = 134217729.0 * xh(jx)
              t_r1 = t_u1
     +                 - (t_u1 - xh(jx))
              t_r2 = xh(jx) - t_r1
              temp1l = ((t_p1*t_r1
     +                - temp1h)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              temp1l = temp1l
     +              + (alphah*xl(jx)
     +              + alphal*xh(jx))
              t_s1 = temp1h + temp1l
              temp1l = temp1l - (t_s1 - temp1h)
              temp1h = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          temp1 = alpha * x(jx)
!                 TEMP2 = ZERO
              temp2h = zeroh
              temp2l = zerol
!                 Y(JY) = Y(JY) + TEMP1*A(1,J)
!<<<          ddm = temp1 * a(1,j)
              ddmh = temp1h * ah(1,j)
              t_u1 = 134217729.0 * temp1h
              t_p1 = t_u1
     +                 - (t_u1 - temp1h)
              t_p2 = temp1h - t_p1
              t_u1 = 134217729.0 * ah(1,j)
              t_r1 = t_u1
     +                 - (t_u1 - ah(1,j))
              t_r2 = ah(1,j) - t_r1
              ddml = ((t_p1*t_r1
     +                - ddmh)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              ddml = ddml
     +              + (temp1h*al(1,j)
     +              + temp1l*ah(1,j))
              t_s1 = ddmh + ddml
              ddml = ddml - (t_s1 - ddmh)
              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          ddm = temp1 * a(1,j)
!<<<          y(jy) = y(jy) + ddm
              t_p1 = yh(jy) + ddmh
              t_p2 = t_p1 - yh(jy)
              t_q1 = t_p1
              t_q2 = (yh(jy) - (t_p1 - t_p2))
     +                 + (ddmh - t_p2)
              t_q2 = t_q2 + yl(jy) + ddml
              yh(jy) = t_q1 + t_q2
              yl(jy) = t_q2 - (yh(jy) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>          y(jy) = y(jy) + ddm
                  L = 1 - J
                  IX = JX
                  IY = JY
!$omp parallel
!$omp do
!$omp+ private(I,ix,iy)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                  DO 110 I = J + 1,MIN(N,J+K)
!!                    IX = IX + INCX
!!                    IY = IY + INCY
                      ix = jx + (i - j)*incx
                      iy = jy + (i - j)*incy
!                     Y(IY) = Y(IY) + TEMP1*A(L+I,J)
!<<<              ddm = temp1 * a(l+i,j)
                  ddmh = temp1h * ah(l+i,j)
                  t_u1 = 134217729.0 * temp1h
                  t_p1 = t_u1
     +                     - (t_u1 - temp1h)
                  t_p2 = temp1h - t_p1
                  t_u1 = 134217729.0 * ah(l+i,j)
                  t_r1 = t_u1
     +                     - (t_u1 - ah(l+i,j))
                  t_r2 = ah(l+i,j) - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (temp1h*al(l+i,j)
     +                  + temp1l*ah(l+i,j))
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = temp1 * a(l+i,j)
!<<<              y(iy) = y(iy) + ddm
                  t_p1 = yh(iy) + ddmh
                  t_p2 = t_p1 - yh(iy)
                  t_q1 = t_p1
                  t_q2 = (yh(iy) - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + yl(iy) + ddml
                  yh(iy) = t_q1 + t_q2
                  yl(iy) = t_q2 - (yh(iy) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              y(iy) = y(iy) + ddm
!                     TEMP2 = TEMP2 + A(L+I,J)*X(IX)
!<<<              ddm = a(l+i,j) * x(ix)
                  ddmh = ah(l+i,j) * xh(ix)
                  t_u1 = 134217729.0 * ah(l+i,j)
                  t_p1 = t_u1
     +                     - (t_u1 - ah(l+i,j))
                  t_p2 = ah(l+i,j) - t_p1
                  t_u1 = 134217729.0 * xh(ix)
                  t_r1 = t_u1
     +                     - (t_u1 - xh(ix))
                  t_r2 = xh(ix) - t_r1
                  ddml = ((t_p1*t_r1
     +                    - ddmh)
     +                    + t_p1 * t_r2
     +                    + t_p2 * t_r1)
     +                    + t_p2 * t_r2
                  ddml = ddml
     +                  + (ah(l+i,j)*xl(ix)
     +                  + al(l+i,j)*xh(ix))
                  t_s1 = ddmh + ddml
                  ddml = ddml - (t_s1 - ddmh)
                  ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>              ddm = a(l+i,j) * x(ix)
!$omp critical
!<<<              temp2 = temp2 + ddm
                  t_p1 = temp2h + ddmh
                  t_p2 = t_p1 - temp2h
                  t_q1 = t_p1
                  t_q2 = (temp2h - (t_p1 - t_p2))
     +                     + (ddmh - t_p2)
                  t_q2 = t_q2 + temp2l + ddml
                  temp2h = t_q1 + t_q2
                  temp2l = t_q2 - (temp2h - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>              temp2 = temp2 + ddm
!$omp end critical
  110             CONTINUE
!$omp end do
!$omp end parallel
!                 Y(JY) = Y(JY) + ALPHA*TEMP2
!<<<          ddm = alpha * temp2
              ddmh = alphah * temp2h
              t_u1 = 134217729.0 * alphah
              t_p1 = t_u1
     +                 - (t_u1 - alphah)
              t_p2 = alphah - t_p1
              t_u1 = 134217729.0 * temp2h
              t_r1 = t_u1
     +                 - (t_u1 - temp2h)
              t_r2 = temp2h - t_r1
              ddml = ((t_p1*t_r1
     +                - ddmh)
     +                + t_p1 * t_r2
     +                + t_p2 * t_r1)
     +                + t_p2 * t_r2
              ddml = ddml
     +              + (alphah*temp2l
     +              + alphal*temp2h)
              t_s1 = ddmh + ddml
              ddml = ddml - (t_s1 - ddmh)
              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>          ddm = alpha * temp2
!<<<          y(jy) = y(jy) + ddm
              t_p1 = yh(jy) + ddmh
              t_p2 = t_p1 - yh(jy)
              t_q1 = t_p1
              t_q2 = (yh(jy) - (t_p1 - t_p2))
     +                 + (ddmh - t_p2)
              t_q2 = t_q2 + yl(jy) + ddml
              yh(jy) = t_q1 + t_q2
              yl(jy) = t_q2 - (yh(jy) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>          y(jy) = y(jy) + ddm
                  JX = JX + INCX
                  JY = JY + INCY
  120         CONTINUE
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = ddsbmv"
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DSBMV .
*
      END
