      SUBROUTINE DDSCAL(N,DAH,DAL,DXH,DXL,INCX)
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
*     .. Scalar Arguments ..
      DOUBLE PRECISION DAH,DAL
      INTEGER INCX,N
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION DXH(*),DXL(*)
      double precision s1,s2,q1,q2,r1,r2,p1,p2,t1,t2,u1,u2
*     ..
*
*  Purpose
*  =======
*
*     DDSCAL scales a vector by a constant.
*     uses unrolled loops for increment equal to one.
*
*  Further Details
*  ===============
*
*     jack dongarra, linpack, 3/11/78.
*     modified 3/93 to return if incx .le. 0.
*     modified 12/3/93, array(1) declarations changed to array(*)
*     quad precision by Osamu Hino 01/04/2010
*
*  =====================================================================
*
*     .. Local Scalars ..
      INTEGER I,M,MP1,NINCX
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MOD
*     ..
      IF (N.LE.0 .OR. INCX.LE.0) RETURN
      IF (INCX.EQ.1) GO TO 20
*
*        code for increment not equal to 1
*
      NINCX = N*INCX
!$omp parallel
!$omp do
!$omp& private(I,q1,q2,u1,p1,p2,r1,r2)
      DO 10 I = 1,NINCX,INCX
!         DX(I) = DA*DX(I)
          q1 = DXH(I)*DAH
          u1 = 134217729.0*DXH(I)
          p1 = u1 - (u1-DXH(I))
          p2 = DXH(I) - p1
          u1 = 134217729.0*DAH
          r1 = u1 - (u1-DAH)
          r2 = DAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + DAL * DXH(I)
          q2 = q2 + DAH * DXL(I)
          DXH(I) = q1 + q2
          DXL(I) = q2 - (DXH(I) - q1)
   10 CONTINUE
!$omp end parallel
      RETURN
*
*        code for increment equal to 1
*
*
*        clean-up loop
*
   20 M = MOD(N,5)
      IF (M.EQ.0) GO TO 40
      DO 30 I = 1,M
!         DX(I) = DA*DX(I)
          q1 = DXH(I)*DAH
          u1 = 134217729.0*DXH(I)
          p1 = u1 - (u1-DXH(I))
          p2 = DXH(I) - p1
          u1 = 134217729.0*DAH
          r1 = u1 - (u1-DAH)
          r2 = DAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + DAL * DXH(I)
          q2 = q2 + DAH * DXL(I)
          DXH(I) = q1 + q2
          DXL(I) = q2 - (DXH(I) - q1)
   30 CONTINUE
      IF (N.LT.5) RETURN
   40 MP1 = M + 1
!$omp parallel
!$omp do
!$omp& private(I,q1,q2,u1,p1,p2,r1,r2)
      DO 50 I = MP1,N,5
!         DX(I) = DA*DX(I)
          q1 = DXH(I)*DAH
          u1 = 134217729.0*DXH(I)
          p1 = u1 - (u1-DXH(I))
          p2 = DXH(I) - p1
          u1 = 134217729.0*DAH
          r1 = u1 - (u1-DAH)
          r2 = DAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + DAL * DXH(I)
          q2 = q2 + DAH * DXL(I)
          DXH(I) = q1 + q2
          DXL(I) = q2 - (DXH(I) - q1)
!         DX(I+1) = DA*DX(I+1)
          q1 = DXH(I+1)*DAH
          u1 = 134217729.0*DXH(I+1)
          p1 = u1 - (u1-DXH(I+1))
          p2 = DXH(I+1) - p1
          u1 = 134217729.0*DAH
          r1 = u1 - (u1-DAH)
          r2 = DAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + DAL * DXH(I+1)
          q2 = q2 + DAH * DXL(I+1)
          DXH(I+1) = q1 + q2
          DXL(I+1) = q2 - (DXH(I+1) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!         DX(I+2) = DA*DX(I+2)
          q1 = DXH(I+2)*DAH
          u1 = 134217729.0*DXH(I+2)
          p1 = u1 - (u1-DXH(I+2))
          p2 = DXH(I+2) - p1
          u1 = 134217729.0*DAH
          r1 = u1 - (u1-DAH)
          r2 = DAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + DAL * DXH(I+2)
          q2 = q2 + DAH * DXL(I+2)
          DXH(I+2) = q1 + q2
          DXL(I+2) = q2 - (DXH(I+2) - q1)
!         DX(I+3) = DA*DX(I+3)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
          q1 = DXH(I+3)*DAH
          u1 = 134217729.0*DXH(I+3)
          p1 = u1 - (u1-DXH(I+3))
          p2 = DXH(I+3) - p1
          u1 = 134217729.0*DAH
          r1 = u1 - (u1-DAH)
          r2 = DAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + DAL * DXH(I+3)
          q2 = q2 + DAH * DXL(I+3)
          DXH(I+3) = q1 + q2
          DXL(I+3) = q2 - (DXH(I+3) - q1)
!         DX(I+4) = DA*DX(I+4)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
          q1 = DXH(I+4)*DAH
          u1 = 134217729.0*DXH(I+4)
          p1 = u1 - (u1-DXH(I+4))
          p2 = DXH(I+4) - p1
          u1 = 134217729.0*DAH
          r1 = u1 - (u1-DAH)
          r2 = DAH - r1
          q2 = ((p1*r1-q1) + p1*r2 + p2*r1) + p2*r2
          q2 = q2 + DAL * DXH(I+4)
          q2 = q2 + DAH * DXL(I+4)
          DXH(I+4) = q1 + q2
          DXL(I+4) = q2 - (DXH(I+4) - q1)
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
   50 CONTINUE
!$omp end parallel
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = DDSCAL"
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
      END
