!     SUBROUTINE DSYRK(UPLO,TRANS,N,K,ALPHA,A,LDA,BETA,C,LDC)
      subroutine ddsyrk(uplo, trans, n, k, alphah, alphal,
     +                  ah, al, lda, betah, betal, ch, cl, ldc)
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE PRECISION ALPHA,BETA
      double precision alphah, alphal
      double precision betah, betal
      INTEGER K,LDA,LDC,N
      CHARACTER TRANS,UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE PRECISION A(LDA,*),C(LDC,*)
      double precision ah(lda,*), al(lda,*)
      double precision ch(ldc,*), cl(ldc,*)
*     ..
*
*  Purpose
*  =======
*
*  DSYRK  performs one of the symmetric rank k operations
*
*     C := alpha*A*A' + beta*C,
*
*  or
*
*     C := alpha*A'*A + beta*C,
*
*  where  alpha and beta  are scalars, C is an  n by n  symmetric matrix
*  and  A  is an  n by k  matrix in the first case and a  k by n  matrix
*  in the second case.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On  entry,   UPLO  specifies  whether  the  upper  or  lower
*           triangular  part  of the  array  C  is to be  referenced  as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the  upper triangular part of  C
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the  lower triangular part of  C
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  TRANS  - CHARACTER*1.
*           On entry,  TRANS  specifies the operation to be performed as
*           follows:
*
*              TRANS = 'N' or 'n'   C := alpha*A*A' + beta*C.
*
*              TRANS = 'T' or 't'   C := alpha*A'*A + beta*C.
*
*              TRANS = 'C' or 'c'   C := alpha*A'*A + beta*C.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry,  N specifies the order of the matrix C.  N must be
*           at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry with  TRANS = 'N' or 'n',  K  specifies  the number
*           of  columns   of  the   matrix   A,   and  on   entry   with
*           TRANS = 'T' or 't' or 'C' or 'c',  K  specifies  the  number
*           of rows of the matrix  A.  K must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - DOUBLE PRECISION.
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  A      - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
*           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
*           Before entry with  TRANS = 'N' or 'n',  the  leading  n by k
*           part of the array  A  must contain the matrix  A,  otherwise
*           the leading  k by n  part of the array  A  must contain  the
*           matrix A.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n'
*           then  LDA must be at least  max( 1, n ), otherwise  LDA must
*           be at least  max( 1, k ).
*           Unchanged on exit.
*
*  BETA   - DOUBLE PRECISION.
*           On entry, BETA specifies the scalar beta.
*           Unchanged on exit.
*
*  C      - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
*           Before entry  with  UPLO = 'U' or 'u',  the leading  n by n
*           upper triangular part of the array C must contain the upper
*           triangular part  of the  symmetric matrix  and the strictly
*           lower triangular part of C is not referenced.  On exit, the
*           upper triangular part of the array  C is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry  with  UPLO = 'L' or 'l',  the leading  n by n
*           lower triangular part of the array C must contain the lower
*           triangular part  of the  symmetric matrix  and the strictly
*           upper triangular part of C is not referenced.  On exit, the
*           lower triangular part of the array  C is overwritten by the
*           lower triangular part of the updated matrix.
*
*  LDC    - INTEGER.
*           On entry, LDC specifies the first dimension of C as declared
*           in  the  calling  (sub)  program.   LDC  must  be  at  least
*           max( 1, n ).
*           Unchanged on exit.
*
*
*  Level 3 Blas routine.
*
*  -- Written on 8-February-1989.
*     Jack Dongarra, Argonne National Laboratory.
*     Iain Duff, AERE Harwell.
*     Jeremy Du Croz, Numerical Algorithms Group Ltd.
*     Sven Hammarling, Numerical Algorithms Group Ltd.
*
*
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      double precision temph, templ
      INTEGER I,INFO,J,L,NROWA
      LOGICAL UPPER
*     ..
*     .. Parameters ..
!     DOUBLE PRECISION ONE,ZERO
!     PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
      double precision oneh, onel
      double precision zeroh, zerol
      double precision ddmh, ddml
      double precision ddah, ddal
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "ddsyrk"
#endif
*     ..
      zeroh = 0.0d+0
      zerol = 0.0d+0
      oneh = 1.0d+0
      onel = 0.0d+0
*
*     Test the input parameters.
*
      IF (LSAME(TRANS,'N')) THEN
          NROWA = N
      ELSE
          NROWA = K
      END IF
      UPPER = LSAME(UPLO,'U')
*
      INFO = 0
      IF ((.NOT.UPPER) .AND. (.NOT.LSAME(UPLO,'L'))) THEN
          INFO = 1
      ELSE IF ((.NOT.LSAME(TRANS,'N')) .AND.
     +         (.NOT.LSAME(TRANS,'T')) .AND.
     +         (.NOT.LSAME(TRANS,'C'))) THEN
          INFO = 2
      ELSE IF (N.LT.0) THEN
          INFO = 3
      ELSE IF (K.LT.0) THEN
          INFO = 4
      ELSE IF (LDA.LT.MAX(1,NROWA)) THEN
          INFO = 7
      ELSE IF (LDC.LT.MAX(1,N)) THEN
          INFO = 10
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DSYRK ',INFO)
!         RETURN
          goto 9999
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. (((ALPHA.EQ.ZERO).OR.
!    +    (K.EQ.0)).AND. (BETA.EQ.ONE))) RETURN
      if((n.eq.0) .or. (((
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +  ).or.(k.eq.0)).and.(
!       (beta .eq. one)
     + ((betah .eq. oneh) .and.
     +  (betal .eq. onel))
     +))) goto 9999
*
*     And when  alpha.eq.zero.
*
!     IF (ALPHA.EQ.ZERO) THEN
      if(
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +) then
          IF (UPPER) THEN
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I,J)
                  DO 20 J = 1,N
                      DO 10 I = 1,J
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   10                 CONTINUE
   20             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I,J)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                  DO 40 J = 1,N
                      DO 30 I = 1,J
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
                          t_ch = betah * ch(i,j)
                          t_u1 = 134217729.0 * betah
                          t_p1 = t_u1
     +                             - (t_u1 - betah)
                          t_p2 = betah - t_p1
                          t_u1 = 134217729.0 * ch(i,j)
                          t_r1 = t_u1
     +                             - (t_u1 - ch(i,j))
                          t_r2 = ch(i,j) - t_r1
                          t_cl = ((t_p1*t_r1
     +                             - t_ch)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          t_cl = t_cl
     +                             + (betah*cl(i,j)
     +                             +  betal*ch(i,j))
                          t_s1 = t_ch + t_cl
                          cl(i,j) = t_cl - (t_s1 - t_ch)
                          ch(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
   30                 CONTINUE
   40             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          ELSE
!             IF (BETA.EQ.ZERO) THEN
              if(
!               (beta .eq. zero)
     +         ((betah .eq. zeroh) .and.
     +          (betal .eq. zerol))
     +        ) then
!$omp parallel
!$omp do
!$omp+ private(I,J)
                  DO 60 J = 1,N
                      DO 50 I = J,N
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   50                 CONTINUE
   60             CONTINUE
!$omp end do
!$omp end parallel
              ELSE
!$omp parallel
!$omp do
!$omp+ private(I,J)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                  DO 80 J = 1,N
                      DO 70 I = J,N
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
                          t_ch = betah * ch(i,j)
                          t_u1 = 134217729.0 * betah
                          t_p1 = t_u1
     +                             - (t_u1 - betah)
                          t_p2 = betah - t_p1
                          t_u1 = 134217729.0 * ch(i,j)
                          t_r1 = t_u1
     +                             - (t_u1 - ch(i,j))
                          t_r2 = ch(i,j) - t_r1
                          t_cl = ((t_p1*t_r1
     +                             - t_ch)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          t_cl = t_cl
     +                             + (betah*cl(i,j)
     +                             +  betal*ch(i,j))
                          t_s1 = t_ch + t_cl
                          cl(i,j) = t_cl - (t_s1 - t_ch)
                          ch(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
   70                 CONTINUE
   80             CONTINUE
!$omp end do
!$omp end parallel
              END IF
          END IF
          RETURN
      END IF
*
*     Start the operations.
*
      IF (LSAME(TRANS,'N')) THEN
*
*        Form  C := alpha*A*A' + beta*C.
*
          IF (UPPER) THEN
              DO 130 J = 1,N
!                 IF (BETA.EQ.ZERO) THEN
                  if(
!                   (beta .eq. zero)
     +             ((betah .eq. zeroh) .and.
     +              (betal .eq. zerol))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 90 I = 1,J
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   90                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  else if(
!                   (beta .ne. one)
     +             ((betah .ne. oneh) .or.
     +              (betal .ne. onel))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                      DO 100 I = 1,J
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
                          t_ch = betah * ch(i,j)
                          t_u1 = 134217729.0 * betah
                          t_p1 = t_u1
     +                             - (t_u1 - betah)
                          t_p2 = betah - t_p1
                          t_u1 = 134217729.0 * ch(i,j)
                          t_r1 = t_u1
     +                             - (t_u1 - ch(i,j))
                          t_r2 = ch(i,j) - t_r1
                          t_cl = ((t_p1*t_r1
     +                             - t_ch)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          t_cl = t_cl
     +                             + (betah*cl(i,j)
     +                             +  betal*ch(i,j))
                          t_s1 = t_ch + t_cl
                          cl(i,j) = t_cl - (t_s1 - t_ch)
                          ch(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
  100                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
                  DO 120 L = 1,K
!                     IF (A(J,L).NE.ZERO) THEN
                      if(
!                       (a(j,l) .ne. zero)
     +                 ((ah(j,l) .ne. zeroh) .or.
     +                  (al(j,l) .ne. zerol))
     +                ) then
!                         TEMP = ALPHA*A(J,L)
!<<<                      temp = alpha * a(j,l)
                          temph = alphah * ah(j,l)
                          t_u1 = 134217729.0 * alphah
                          t_p1 = t_u1
     +                             - (t_u1 - alphah)
                          t_p2 = alphah - t_p1
                          t_u1 = 134217729.0 * ah(j,l)
                          t_r1 = t_u1
     +                             - (t_u1 - ah(j,l))
                          t_r2 = ah(j,l) - t_r1
                          templ = ((t_p1*t_r1
     +                            - temph)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          templ = templ
     +                          + (alphah*al(j,l)
     +                          + alphal*ah(j,l))
                          t_s1 = temph + templ
                          templ = templ - (t_s1 - temph)
                          temph = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      temp = alpha * a(j,l)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                          DO 110 I = 1,J
!                             C(I,J) = C(I,J) + TEMP*A(I,L)
!<<<                        ddm = temp * a(i,l)
                            ddmh = temph * ah(i,l)
                            t_u1 = 134217729.0 * temph
                            t_p1 = t_u1
     +                               - (t_u1 - temph)
                            t_p2 = temph - t_p1
                            t_u1 = 134217729.0 * ah(i,l)
                            t_r1 = t_u1
     +                               - (t_u1 - ah(i,l))
                            t_r2 = ah(i,l) - t_r1
                            ddml = ((t_p1*t_r1
     +                              - ddmh)
     +                              + t_p1 * t_r2
     +                              + t_p2 * t_r1)
     +                              + t_p2 * t_r2
                            ddml = ddml
     +                            + (temph*al(i,l)
     +                            + templ*ah(i,l))
                            t_s1 = ddmh + ddml
                            ddml = ddml - (t_s1 - ddmh)
                            ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                        ddm = temp * a(i,l)
!<<<                        c(i,j) = c(i,j) + ddm
                            t_p1 = ch(i,j) + ddmh
                            t_p2 = t_p1 - ch(i,j)
                            t_q1 = t_p1
                            t_q2 = (ch(i,j) - (t_p1 - t_p2))
     +                               + (ddmh - t_p2)
                            t_q2 = t_q2 + cl(i,j) + ddml
                            ch(i,j) = t_q1 + t_q2
                            cl(i,j) = t_q2 - (ch(i,j) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                        c(i,j) = c(i,j) + ddm
  110                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  120             CONTINUE
  130         CONTINUE
          ELSE
              DO 180 J = 1,N
!                 IF (BETA.EQ.ZERO) THEN
                  if(
!                   (beta .eq. zero)
     +             ((betah .eq. zeroh) .and.
     +              (betal .eq. zerol))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 140 I = J,N
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
  140                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  else if(
!                   (beta .ne. one)
     +             ((betah .ne. oneh) .or.
     +              (betal .ne. onel))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                      DO 150 I = J,N
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
                          t_ch = betah * ch(i,j)
                          t_u1 = 134217729.0 * betah
                          t_p1 = t_u1
     +                             - (t_u1 - betah)
                          t_p2 = betah - t_p1
                          t_u1 = 134217729.0 * ch(i,j)
                          t_r1 = t_u1
     +                             - (t_u1 - ch(i,j))
                          t_r2 = ch(i,j) - t_r1
                          t_cl = ((t_p1*t_r1
     +                             - t_ch)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          t_cl = t_cl
     +                             + (betah*cl(i,j)
     +                             +  betal*ch(i,j))
                          t_s1 = t_ch + t_cl
                          cl(i,j) = t_cl - (t_s1 - t_ch)
                          ch(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
  150                 CONTINUE
!$omp end do
!$omp end parallel
                  END IF
                  DO 170 L = 1,K
!                     IF (A(J,L).NE.ZERO) THEN
                      if(
!                       (a(j,l) .ne. zero)
     +                 ((ah(j,l) .ne. zeroh) .or.
     +                  (al(j,l) .ne. zerol))
     +                ) then
!                         TEMP = ALPHA*A(J,L)
!<<<                      temp = alpha * a(j,l)
                          temph = alphah * ah(j,l)
                          t_u1 = 134217729.0 * alphah
                          t_p1 = t_u1
     +                             - (t_u1 - alphah)
                          t_p2 = alphah - t_p1
                          t_u1 = 134217729.0 * ah(j,l)
                          t_r1 = t_u1
     +                             - (t_u1 - ah(j,l))
                          t_r2 = ah(j,l) - t_r1
                          templ = ((t_p1*t_r1
     +                            - temph)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          templ = templ
     +                          + (alphah*al(j,l)
     +                          + alphal*ah(j,l))
                          t_s1 = temph + templ
                          templ = templ - (t_s1 - temph)
                          temph = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      temp = alpha * a(j,l)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                          DO 160 I = J,N
!                             C(I,J) = C(I,J) + TEMP*A(I,L)
!<<<                        ddm = temp * a(i,l)
                            ddmh = temph * ah(i,l)
                            t_u1 = 134217729.0 * temph
                            t_p1 = t_u1
     +                               - (t_u1 - temph)
                            t_p2 = temph - t_p1
                            t_u1 = 134217729.0 * ah(i,l)
                            t_r1 = t_u1
     +                               - (t_u1 - ah(i,l))
                            t_r2 = ah(i,l) - t_r1
                            ddml = ((t_p1*t_r1
     +                              - ddmh)
     +                              + t_p1 * t_r2
     +                              + t_p2 * t_r1)
     +                              + t_p2 * t_r2
                            ddml = ddml
     +                            + (temph*al(i,l)
     +                            + templ*ah(i,l))
                            t_s1 = ddmh + ddml
                            ddml = ddml - (t_s1 - ddmh)
                            ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                        ddm = temp * a(i,l)
!<<<                        c(i,j) = c(i,j) + ddm
                            t_p1 = ch(i,j) + ddmh
                            t_p2 = t_p1 - ch(i,j)
                            t_q1 = t_p1
                            t_q2 = (ch(i,j) - (t_p1 - t_p2))
     +                               + (ddmh - t_p2)
                            t_q2 = t_q2 + cl(i,j) + ddml
                            ch(i,j) = t_q1 + t_q2
                            cl(i,j) = t_q2 - (ch(i,j) - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                        c(i,j) = c(i,j) + ddm
  160                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  170             CONTINUE
  180         CONTINUE
          END IF
      ELSE
*
*        Form  C := alpha*A'*A + beta*C.
*
          IF (UPPER) THEN
              DO 210 J = 1,N
                  DO 200 I = 1,J
!                     TEMP = ZERO
                      temph = zeroh
                      templ = zerol
!$omp parallel
!$omp do
!$omp+ private(L)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                      DO 190 L = 1,K
!                         TEMP = TEMP + A(L,I)*A(L,J)
!<<<                      ddm = a(l,i) * a(l,j)
                          ddmh = ah(l,i) * ah(l,j)
                          t_u1 = 134217729.0 * ah(l,i)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(l,i))
                          t_p2 = ah(l,i) - t_p1
                          t_u1 = 134217729.0 * ah(l,j)
                          t_r1 = t_u1
     +                             - (t_u1 - ah(l,j))
                          t_r2 = ah(l,j) - t_r1
                          ddml = ((t_p1*t_r1
     +                            - ddmh)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddml = ddml
     +                          + (ah(l,i)*al(l,j)
     +                          + al(l,i)*ah(l,j))
                          t_s1 = ddmh + ddml
                          ddml = ddml - (t_s1 - ddmh)
                          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = a(l,i) * a(l,j)
!$omp critical
!<<<                      temp = temp + ddm
                          t_p1 = temph + ddmh
                          t_p2 = t_p1 - temph
                          t_q1 = t_p1
                          t_q2 = (temph - (t_p1 - t_p2))
     +                             + (ddmh - t_p2)
                          t_q2 = t_q2 + templ + ddml
                          temph = t_q1 + t_q2
                          templ = t_q2 - (temph - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp = temp + ddm
!$omp end critical
  190                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      if(
!                       (beta .eq. zero)
     +                 ((betah .eq. zeroh) .and.
     +                  (betal .eq. zerol))
     +                ) then
!                         C(I,J) = ALPHA*TEMP
!<<<                      c(i,j) = alpha * temp
                          ch(i,j) = alphah * temph
                          t_u1 = 134217729.0 * alphah
                          t_p1 = t_u1
     +                             - (t_u1 - alphah)
                          t_p2 = alphah - t_p1
                          t_u1 = 134217729.0 * temph
                          t_r1 = t_u1
     +                             - (t_u1 - temph)
                          t_r2 = temph - t_r1
                          cl(i,j) = ((t_p1*t_r1
     +                            - ch(i,j))
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          cl(i,j) = cl(i,j)
     +                          + (alphah*templ
     +                          + alphal*temph)
                          t_s1 = ch(i,j) + cl(i,j)
                          cl(i,j) = cl(i,j) - (t_s1 - ch(i,j))
                          ch(i,j) = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      c(i,j) = alpha * temp
                      ELSE
!                         C(I,J) = ALPHA*TEMP + BETA*C(I,J)
!<<<                      dda = beta * c(i,j)
                          ddah = betah * ch(i,j)
                          t_u1 = 134217729.0 * betah
                          t_p1 = t_u1
     +                             - (t_u1 - betah)
                          t_p2 = betah - t_p1
                          t_u1 = 134217729.0 * ch(i,j)
                          t_r1 = t_u1
     +                             - (t_u1 - ch(i,j))
                          t_r2 = ch(i,j) - t_r1
                          ddal = ((t_p1*t_r1
     +                            - ddah)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddal = ddal
     +                          + (betah*cl(i,j)
     +                          + betal*ch(i,j))
                          t_s1 = ddah + ddal
                          ddal = ddal - (t_s1 - ddah)
                          ddah = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      dda = beta * c(i,j)
!<<<                      ddm = alpha * temp
                          ddmh = alphah * temph
                          t_u1 = 134217729.0 * alphah
                          t_p1 = t_u1
     +                             - (t_u1 - alphah)
                          t_p2 = alphah - t_p1
                          t_u1 = 134217729.0 * temph
                          t_r1 = t_u1
     +                             - (t_u1 - temph)
                          t_r2 = temph - t_r1
                          ddml = ((t_p1*t_r1
     +                            - ddmh)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddml = ddml
     +                          + (alphah*templ
     +                          + alphal*temph)
                          t_s1 = ddmh + ddml
                          ddml = ddml - (t_s1 - ddmh)
                          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = alpha * temp
!<<<                      c(i,j) = ddm + dda
                          t_p1 = ddmh + ddah
                          t_p2 = t_p1 - ddmh
                          ch(i,j) = t_p1
                          cl(i,j) = (ddmh - (t_p1 - t_p2))
     +                          + (ddah - t_p2)
                          cl(i,j) = cl(i,j) + ddml + ddal
                          t_s1 = ch(i,j) + cl(i,j)
                          cl(i,j) = cl(i,j) - (t_s1 - ch(i,j))
                          ch(i,j) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddm + dda
                      END IF
  200             CONTINUE
  210         CONTINUE
          ELSE
              DO 240 J = 1,N
                  DO 230 I = J,N
!                     TEMP = ZERO
                      temph = zeroh
                      templ = zerol
!$omp parallel
!$omp do
!$omp+ private(L)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                      DO 220 L = 1,K
!                         TEMP = TEMP + A(L,I)*A(L,J)
!<<<                      ddm = a(l,i) * a(l,j)
                          ddmh = ah(l,i) * ah(l,j)
                          t_u1 = 134217729.0 * ah(l,i)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(l,i))
                          t_p2 = ah(l,i) - t_p1
                          t_u1 = 134217729.0 * ah(l,j)
                          t_r1 = t_u1
     +                             - (t_u1 - ah(l,j))
                          t_r2 = ah(l,j) - t_r1
                          ddml = ((t_p1*t_r1
     +                            - ddmh)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddml = ddml
     +                          + (ah(l,i)*al(l,j)
     +                          + al(l,i)*ah(l,j))
                          t_s1 = ddmh + ddml
                          ddml = ddml - (t_s1 - ddmh)
                          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = a(l,i) * a(l,j)
!$omp critical
!<<<                      temp = temp + ddm
                          t_p1 = temph + ddmh
                          t_p2 = t_p1 - temph
                          t_q1 = t_p1
                          t_q2 = (temph - (t_p1 - t_p2))
     +                             + (ddmh - t_p2)
                          t_q2 = t_q2 + templ + ddml
                          temph = t_q1 + t_q2
                          templ = t_q2 - (temph - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp = temp + ddm
!$omp end critical
  220                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (BETA.EQ.ZERO) THEN
                      if(
!                       (BETA .eq. ZERO)
     +                 ((BETAh .eq. ZEROh) .and.
     +                  (BETAl .eq. ZEROl))
     +                ) then
!                         C(I,J) = ALPHA*TEMP
!<<<                  c(i,j) = alpha * temp
                      ch(i,j) = alphah * temph
                      t_u1 = 134217729.0 * alphah
                      t_p1 = t_u1
     +                         - (t_u1 - alphah)
                      t_p2 = alphah - t_p1
                      t_u1 = 134217729.0 * temph
                      t_r1 = t_u1
     +                         - (t_u1 - temph)
                      t_r2 = temph - t_r1
                      cl(i,j) = ((t_p1*t_r1
     +                        - ch(i,j))
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      cl(i,j) = cl(i,j)
     +                      + (alphah*templ
     +                      + alphal*temph)
                      t_s1 = ch(i,j) + cl(i,j)
                      cl(i,j) = cl(i,j) - (t_s1 - ch(i,j))
                      ch(i,j) = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  c(i,j) = alpha * temp
                      ELSE
!                         C(I,J) = ALPHA*TEMP + BETA*C(I,J)
!<<<                  dda = beta * c(i,j)
                      ddah = betah * ch(i,j)
                      t_u1 = 134217729.0 * betah
                      t_p1 = t_u1
     +                         - (t_u1 - betah)
                      t_p2 = betah - t_p1
                      t_u1 = 134217729.0 * ch(i,j)
                      t_r1 = t_u1
     +                         - (t_u1 - ch(i,j))
                      t_r2 = ch(i,j) - t_r1
                      ddal = ((t_p1*t_r1
     +                        - ddah)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      ddal = ddal
     +                      + (betah*cl(i,j)
     +                      + betal*ch(i,j))
                      t_s1 = ddah + ddal
                      ddal = ddal - (t_s1 - ddah)
                      ddah = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  dda = beta * c(i,j)
!<<<                  ddm = alpha * temp
                      ddmh = alphah * temph
                      t_u1 = 134217729.0 * alphah
                      t_p1 = t_u1
     +                         - (t_u1 - alphah)
                      t_p2 = alphah - t_p1
                      t_u1 = 134217729.0 * temph
                      t_r1 = t_u1
     +                         - (t_u1 - temph)
                      t_r2 = temph - t_r1
                      ddml = ((t_p1*t_r1
     +                        - ddmh)
     +                        + t_p1 * t_r2
     +                        + t_p2 * t_r1)
     +                        + t_p2 * t_r2
                      ddml = ddml
     +                      + (alphah*templ
     +                      + alphal*temph)
                      t_s1 = ddmh + ddml
                      ddml = ddml - (t_s1 - ddmh)
                      ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = alpha * temp
!<<<                  c(i,j) = dda + ddm
                      t_p1 = ddah + ddmh
                      t_p2 = t_p1 - ddah
                      ch(i,j) = t_p1
                      cl(i,j) = (ddah - (t_p1 - t_p2))
     +                      + (ddmh - t_p2)
                      cl(i,j) = cl(i,j) + ddal + ddml
                      t_s1 = ch(i,j) + cl(i,j)
                      cl(i,j) = cl(i,j) - (t_s1 - ch(i,j))
                      ch(i,j) = t_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                  c(i,j) = dda + ddm
                      END IF
  230             CONTINUE
  240         CONTINUE
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = trans"
!     WRITE(6,1005) SUBNAME,"DIM M     = ",M
      WRITE(6,1005) SUBNAME,"DIM N     = ",N
      WRITE(6,1005) SUBNAME,"DIM K     = ",K
#include "fout3.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DSYRK .
*
      END
