!     SUBROUTINE DTBSV(UPLO,TRANS,DIAG,N,K,A,LDA,X,INCX)
      subroutine ddtbsv(uplo,trans,diag,n,k,
     +                 ah, al, lda, xh, xl, incx)
      implicit none
*     .. Scalar Arguments ..
      INTEGER INCX,K,LDA,N
      CHARACTER DIAG,TRANS,UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE PRECISION A(LDA,*),X(*)
      double precision ah(lda,*), al(lda,*)
      double precision xh(*), xl(*)
*     ..
*
*  Purpose
*  =======
*
*  DTBSV  solves one of the systems of equations
*
*     A*x = b,   or   A'*x = b,
*
*  where b and x are n element vectors and A is an n by n unit, or
*  non-unit, upper or lower triangular band matrix, with ( k + 1 )
*  diagonals.
*
*  No test for singularity or near-singularity is included in this
*  routine. Such tests must be performed before calling this routine.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the matrix is an upper or
*           lower triangular matrix as follows:
*
*              UPLO = 'U' or 'u'   A is an upper triangular matrix.
*
*              UPLO = 'L' or 'l'   A is a lower triangular matrix.
*
*           Unchanged on exit.
*
*  TRANS  - CHARACTER*1.
*           On entry, TRANS specifies the equations to be solved as
*           follows:
*
*              TRANS = 'N' or 'n'   A*x = b.
*
*              TRANS = 'T' or 't'   A'*x = b.
*
*              TRANS = 'C' or 'c'   A'*x = b.
*
*           Unchanged on exit.
*
*  DIAG   - CHARACTER*1.
*           On entry, DIAG specifies whether or not A is unit
*           triangular as follows:
*
*              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
*
*              DIAG = 'N' or 'n'   A is not assumed to be unit
*                                  triangular.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry with UPLO = 'U' or 'u', K specifies the number of
*           super-diagonals of the matrix A.
*           On entry with UPLO = 'L' or 'l', K specifies the number of
*           sub-diagonals of the matrix A.
*           K must satisfy  0 .le. K.
*           Unchanged on exit.
*
*  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
*           Before entry with UPLO = 'U' or 'u', the leading ( k + 1 )
*           by n part of the array A must contain the upper triangular
*           band part of the matrix of coefficients, supplied column by
*           column, with the leading diagonal of the matrix in row
*           ( k + 1 ) of the array, the first super-diagonal starting at
*           position 2 in row k, and so on. The top left k by k triangle
*           of the array A is not referenced.
*           The following program segment will transfer an upper
*           triangular band matrix from conventional full matrix storage
*           to band storage:
*
*                 DO 20, J = 1, N
*                    M = K + 1 - J
*                    DO 10, I = MAX( 1, J - K ), J
*                       A( M + I, J ) = matrix( I, J )
*              10    CONTINUE
*              20 CONTINUE
*
*           Before entry with UPLO = 'L' or 'l', the leading ( k + 1 )
*           by n part of the array A must contain the lower triangular
*           band part of the matrix of coefficients, supplied column by
*           column, with the leading diagonal of the matrix in row 1 of
*           the array, the first sub-diagonal starting at position 1 in
*           row 2, and so on. The bottom right k by k triangle of the
*           array A is not referenced.
*           The following program segment will transfer a lower
*           triangular band matrix from conventional full matrix storage
*           to band storage:
*
*                 DO 20, J = 1, N
*                    M = 1 - J
*                    DO 10, I = J, MIN( N, J + K )
*                       A( M + I, J ) = matrix( I, J )
*              10    CONTINUE
*              20 CONTINUE
*
*           Note that when DIAG = 'U' or 'u' the elements of the array A
*           corresponding to the diagonal elements of the matrix are not
*           referenced, but are assumed to be unity.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           ( k + 1 ).
*           Unchanged on exit.
*
*  X      - DOUBLE PRECISION array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array X must contain the n
*           element right-hand side vector b. On exit, X is overwritten
*           with the solution vector x.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*
*     .. Parameters ..
!     DOUBLE PRECISION ZERO
!     PARAMETER (ZERO=0.0D+0)
      double precision zeroh, zerol
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      double precision temph, templ
      double precision ddmh, ddml
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
      INTEGER I,INFO,IX,J,JX,KPLUS1,KX,L
      LOGICAL NOUNIT
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX,MIN
*     ..
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "ddtbsv"
#endif
*
*     Test the input parameters.
*
      zeroh = 0.0d+0
      zerol = 0.0d+0
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (.NOT.LSAME(TRANS,'N') .AND. .NOT.LSAME(TRANS,'T') .AND.
     +         .NOT.LSAME(TRANS,'C')) THEN
          INFO = 2
      ELSE IF (.NOT.LSAME(DIAG,'U') .AND. .NOT.LSAME(DIAG,'N')) THEN
          INFO = 3
      ELSE IF (N.LT.0) THEN
          INFO = 4
      ELSE IF (K.LT.0) THEN
          INFO = 5
      ELSE IF (LDA.LT. (K+1)) THEN
          INFO = 7
      ELSE IF (INCX.EQ.0) THEN
          INFO = 9
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DTBSV ',INFO)
          goto 9999
!         RETURN
      END IF
*
*     Quick return if possible.
*
!     IF (N.EQ.0) RETURN
      IF (N.EQ.0) goto 9999
*
      NOUNIT = LSAME(DIAG,'N')
*
*     Set up the start point in X if the increment is not unity. This
*     will be  ( N - 1 )*INCX  too small for descending loops.
*
      IF (INCX.LE.0) THEN
          KX = 1 - (N-1)*INCX
      ELSE IF (INCX.NE.1) THEN
          KX = 1
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed by sequentially with one pass through A.
*
      IF (LSAME(TRANS,'N')) THEN
*
*        Form  x := inv( A )*x.
*
          IF (LSAME(UPLO,'U')) THEN
              KPLUS1 = K + 1
              IF (INCX.EQ.1) THEN
                  DO 20 J = N,1,-1
!                     IF (X(J).NE.ZERO) THEN
                      if(
!                       (x(j) .ne. zero)
     +                 ((xh(j) .ne. zeroh) .or.
     +                  (xl(j) .ne. zerol))
!!!#ddb                   ne0 x(j) zero
     +                ) then
                          L = KPLUS1 - J
!                         IF (NOUNIT) X(J) = X(J)/A(KPLUS1,J)
                          if (nounit) then
!<<<                        x(j) = x(j) / a(kplus1,j)
                            t_q1 = xh(j) / ah(kplus1,j)
                            t_ch = ah(kplus1,j) * t_q1
                            t_u1 = 134217729.0 * ah(kplus1,j)
                            t_p1 = t_u1
     +                               - (t_u1 - ah(kplus1,j))
                            t_p2 = ah(kplus1,j) - t_p1
                            t_u1 = 134217729.0 * t_q1
                            t_r1 = t_u1
     +                               - (t_u1 - t_q1)
                            t_r2 = t_q1 - t_r1
                            t_cl = ((t_p1*t_r1
     +                               - t_ch)
     +                               + t_p1*t_r2
     +                               + t_p2*t_r1)
     +                               + t_p2*t_r2
                            t_cl = t_cl
     +                               + (al(kplus1,j) * t_q1)
                            t_s1 = t_ch + t_cl
                            t_cl = t_cl
     +                               - (t_s1 - t_ch)
                            t_ch = t_s1
                            t_s1 = xh(j) - t_ch
                            t_r1 = t_s1 - xh(j)
                            t_s2 = (xh(j)
     +                               - (t_s1 - t_r1))
     +                               - (t_ch + t_r1)
                            t_s2 = t_s2 - t_cl
                            t_s2 = t_s2 + xl(j)
                            t_q2 = (t_s1 + t_s2)
     +                               / ah(kplus1,j)
                            xh(j) = t_q1 + t_q2
                            xl(j) = t_q2
     +                            - (xh(j) - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                        x(j) = x(j) / a(kplus1,j)
                          endif
!                         TEMP = X(J)
                          temph = xh(j)
                          templ = xl(j)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                          DO 10 I = J - 1,MAX(1,J-K),-1
!                           X(I) = X(I) - TEMP*A(L+I,J)
!<<<                        ddm = temp * a(l+i,j)
                            ddmh = temph * ah(l+i,j)
                            t_u1 = 134217729.0 * temph
                            t_p1 = t_u1
     +                               - (t_u1 - temph)
                            t_p2 = temph - t_p1
                            t_u1 = 134217729.0 * ah(l+i,j)
                            t_r1 = t_u1
     +                               - (t_u1 - ah(l+i,j))
                            t_r2 = ah(l+i,j) - t_r1
                            ddml = ((t_p1*t_r1
     +                              - ddmh)
     +                              + t_p1 * t_r2
     +                              + t_p2 * t_r1)
     +                              + t_p2 * t_r2
                            ddml = ddml
     +                            + (temph*al(l+i,j)
     +                            + templ*ah(l+i,j))
                            t_s1 = ddmh + ddml
                            ddml = ddml - (t_s1 - ddmh)
                            ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                        ddm = temp * a(l+i,j)
!<<<                        x(i) = x(i) - ddm
                            t_p1 = xh(i) - ddmh
                            t_p2 = t_p1 - xh(i)
                            t_ch = t_p1
                            t_cl = (xh(i) - (t_p1 - t_p2))
     +                               - (ddmh + t_p2)
                            t_cl = t_cl + xl(i) - ddml
                            t_s1 = t_ch + t_cl
                            xl(i) = t_cl - (t_s1 - t_ch)
                            xh(i) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                        x(i) = x(i) - ddm
   10                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
   20             CONTINUE
              ELSE
                  KX = KX + (N-1)*INCX
                  JX = KX
                  DO 40 J = N,1,-1
                      KX = KX - INCX
!                     IF (X(JX).NE.ZERO) THEN
                      if(
!                       (x(jx) .ne. zero)
     +                 ((xh(jx) .ne. zeroh) .or.
     +                  (xl(jx) .ne. zerol))
!!!#ddb                   ne0 x(jx) zero
     +                ) then
                          IX = KX
                          L = KPLUS1 - J
!                         IF (NOUNIT) X(JX) = X(JX)/A(KPLUS1,J)
                          if (nounit) then
!<<<                        x(jx) = x(jx) / a(kplus1,j)
                            t_q1 = xh(jx) / ah(kplus1,j)
                            t_ch = ah(kplus1,j) * t_q1
                            t_u1 = 134217729.0 * ah(kplus1,j)
                            t_p1 = t_u1
     +                               - (t_u1 - ah(kplus1,j))
                            t_p2 = ah(kplus1,j) - t_p1
                            t_u1 = 134217729.0 * t_q1
                            t_r1 = t_u1
     +                               - (t_u1 - t_q1)
                            t_r2 = t_q1 - t_r1
                            t_cl = ((t_p1*t_r1
     +                               - t_ch)
     +                               + t_p1*t_r2
     +                               + t_p2*t_r1)
     +                               + t_p2*t_r2
                            t_cl = t_cl
     +                               + (al(kplus1,j) * t_q1)
                            t_s1 = t_ch + t_cl
                            t_cl = t_cl
     +                               - (t_s1 - t_ch)
                            t_ch = t_s1
                            t_s1 = xh(jx) - t_ch
                            t_r1 = t_s1 - xh(jx)
                            t_s2 = (xh(jx)
     +                               - (t_s1 - t_r1))
     +                               - (t_ch + t_r1)
                            t_s2 = t_s2 - t_cl
                            t_s2 = t_s2 + xl(jx)
                            t_q2 = (t_s1 + t_s2)
     +                               / ah(kplus1,j)
                            xh(jx) = t_q1 + t_q2
                            xl(jx) = t_q2
     +                            - (xh(jx) - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                        x(jx) = x(jx) / a(kplus1,j)
                          endif
!                         TEMP = X(JX)
                          temph = xh(jx)
                          templ = xl(jx)
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                          DO 30 I = J - 1,MAX(1,J-K),-1
                            ix = kx + (i - j + 1)*incx
!                           X(IX) = X(IX) - TEMP*A(L+I,J)
!<<<                        ddm = temp * a(l+i,j)
                            ddmh = temph * ah(l+i,j)
                            t_u1 = 134217729.0 * temph
                            t_p1 = t_u1
     +                               - (t_u1 - temph)
                            t_p2 = temph - t_p1
                            t_u1 = 134217729.0 * ah(l+i,j)
                            t_r1 = t_u1
     +                               - (t_u1 - ah(l+i,j))
                            t_r2 = ah(l+i,j) - t_r1
                            ddml = ((t_p1*t_r1
     +                              - ddmh)
     +                              + t_p1 * t_r2
     +                              + t_p2 * t_r1)
     +                              + t_p2 * t_r2
                            ddml = ddml
     +                            + (temph*al(l+i,j)
     +                            + templ*ah(l+i,j))
                            t_s1 = ddmh + ddml
                            ddml = ddml - (t_s1 - ddmh)
                            ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                        ddm = temp * a(l+i,j)
!<<<                        x(ix) = x(ix) - ddm
                            t_p1 = xh(ix) - ddmh
                            t_p2 = t_p1 - xh(ix)
                            t_ch = t_p1
                            t_cl = (xh(ix) - (t_p1 - t_p2))
     +                               - (ddmh + t_p2)
                            t_cl = t_cl + xl(ix) - ddml
                            t_s1 = t_ch + t_cl
                            xl(ix) = t_cl - (t_s1 - t_ch)
                            xh(ix) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                        x(ix) = x(ix) - ddm
!!                          IX = IX - INCX
   30                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
                      JX = JX - INCX
   40             CONTINUE
              END IF
          ELSE
              IF (INCX.EQ.1) THEN
                  DO 60 J = 1,N
!                     IF (X(J).NE.ZERO) THEN
                      if(
!                       (x(j) .ne. zero)
     +                 ((xh(j) .ne. zeroh) .or.
     +                  (xl(j) .ne. zerol))
!!!#ddb                   ne0 x(j) zero
     +                ) then
                          L = 1 - J
!                         IF (NOUNIT) X(J) = X(J)/A(1,J)
                          if (nounit) then
!<<<                        x(j) = x(j) / a(1,j)
                            t_q1 = xh(j) / ah(1,j)
                            t_ch = ah(1,j) * t_q1
                            t_u1 = 134217729.0 * ah(1,j)
                            t_p1 = t_u1
     +                               - (t_u1 - ah(1,j))
                            t_p2 = ah(1,j) - t_p1
                            t_u1 = 134217729.0 * t_q1
                            t_r1 = t_u1
     +                               - (t_u1 - t_q1)
                            t_r2 = t_q1 - t_r1
                            t_cl = ((t_p1*t_r1
     +                               - t_ch)
     +                               + t_p1*t_r2
     +                               + t_p2*t_r1)
     +                               + t_p2*t_r2
                            t_cl = t_cl
     +                               + (al(1,j) * t_q1)
                            t_s1 = t_ch + t_cl
                            t_cl = t_cl
     +                               - (t_s1 - t_ch)
                            t_ch = t_s1
                            t_s1 = xh(j) - t_ch
                            t_r1 = t_s1 - xh(j)
                            t_s2 = (xh(j)
     +                               - (t_s1 - t_r1))
     +                               - (t_ch + t_r1)
                            t_s2 = t_s2 - t_cl
                            t_s2 = t_s2 + xl(j)
                            t_q2 = (t_s1 + t_s2)
     +                               / ah(1,j)
                            xh(j) = t_q1 + t_q2
                            xl(j) = t_q2
     +                            - (xh(j) - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                        x(j) = x(j) / a(1,j)
                          endif
!                         TEMP = X(J)
                          temph = xh(j)
                          templ = xl(j)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                          DO 50 I = J + 1,MIN(N,J+K)
!                           X(I) = X(I) - TEMP*A(L+I,J)
!<<<                        ddm = temp * a(l+i,j)
                            ddmh = temph * ah(l+i,j)
                            t_u1 = 134217729.0 * temph
                            t_p1 = t_u1
     +                               - (t_u1 - temph)
                            t_p2 = temph - t_p1
                            t_u1 = 134217729.0 * ah(l+i,j)
                            t_r1 = t_u1
     +                               - (t_u1 - ah(l+i,j))
                            t_r2 = ah(l+i,j) - t_r1
                            ddml = ((t_p1*t_r1
     +                              - ddmh)
     +                              + t_p1 * t_r2
     +                              + t_p2 * t_r1)
     +                              + t_p2 * t_r2
                            ddml = ddml
     +                            + (temph*al(l+i,j)
     +                            + templ*ah(l+i,j))
                            t_s1 = ddmh + ddml
                            ddml = ddml - (t_s1 - ddmh)
                            ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                        ddm = temp * a(l+i,j)
!<<<                        x(i) = x(i) - ddm
                            t_p1 = xh(i) - ddmh
                            t_p2 = t_p1 - xh(i)
                            t_ch = t_p1
                            t_cl = (xh(i) - (t_p1 - t_p2))
     +                               - (ddmh + t_p2)
                            t_cl = t_cl + xl(i) - ddml
                            t_s1 = t_ch + t_cl
                            xl(i) = t_cl - (t_s1 - t_ch)
                            xh(i) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                        x(i) = x(i) - ddm
   50                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
   60             CONTINUE
              ELSE
                  JX = KX
                  DO 80 J = 1,N
                      KX = KX + INCX
!                     IF (X(JX).NE.ZERO) THEN
                      if(
!                       (x(jx) .ne. zero)
     +                 ((xh(jx) .ne. zeroh) .or.
     +                  (xl(jx) .ne. zerol))
!!!#ddb                   ne0 x(jx) zero
     +                ) then
                          IX = KX
                          L = 1 - J
!                         IF (NOUNIT) X(JX) = X(JX)/A(1,J)
                          if (nounit) then
!<<<                        x(jx) = x(jx) / a(1,j)
                            t_q1 = xh(jx) / ah(1,j)
                            t_ch = ah(1,j) * t_q1
                            t_u1 = 134217729.0 * ah(1,j)
                            t_p1 = t_u1
     +                               - (t_u1 - ah(1,j))
                            t_p2 = ah(1,j) - t_p1
                            t_u1 = 134217729.0 * t_q1
                            t_r1 = t_u1
     +                               - (t_u1 - t_q1)
                            t_r2 = t_q1 - t_r1
                            t_cl = ((t_p1*t_r1
     +                               - t_ch)
     +                               + t_p1*t_r2
     +                               + t_p2*t_r1)
     +                               + t_p2*t_r2
                            t_cl = t_cl
     +                               + (al(1,j) * t_q1)
                            t_s1 = t_ch + t_cl
                            t_cl = t_cl
     +                               - (t_s1 - t_ch)
                            t_ch = t_s1
                            t_s1 = xh(jx) - t_ch
                            t_r1 = t_s1 - xh(jx)
                            t_s2 = (xh(jx)
     +                               - (t_s1 - t_r1))
     +                               - (t_ch + t_r1)
                            t_s2 = t_s2 - t_cl
                            t_s2 = t_s2 + xl(jx)
                            t_q2 = (t_s1 + t_s2)
     +                               / ah(1,j)
                            xh(jx) = t_q1 + t_q2
                            xl(jx) = t_q2
     +                            - (xh(jx) - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                        x(jx) = x(jx) / a(1,j)
                          endif
!                         TEMP = X(JX)
                          temph = xh(jx)
                          templ = xl(jx)
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                          DO 70 I = J + 1,MIN(N,J+K)
                            ix = kx + (i - j - 1)*incx
!                           X(IX) = X(IX) - TEMP*A(L+I,J)
!<<<                        ddm = temp * a(l+i,j)
                            ddmh = temph * ah(l+i,j)
                            t_u1 = 134217729.0 * temph
                            t_p1 = t_u1
     +                               - (t_u1 - temph)
                            t_p2 = temph - t_p1
                            t_u1 = 134217729.0 * ah(l+i,j)
                            t_r1 = t_u1
     +                               - (t_u1 - ah(l+i,j))
                            t_r2 = ah(l+i,j) - t_r1
                            ddml = ((t_p1*t_r1
     +                              - ddmh)
     +                              + t_p1 * t_r2
     +                              + t_p2 * t_r1)
     +                              + t_p2 * t_r2
                            ddml = ddml
     +                            + (temph*al(l+i,j)
     +                            + templ*ah(l+i,j))
                            t_s1 = ddmh + ddml
                            ddml = ddml - (t_s1 - ddmh)
                            ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                        ddm = temp * a(l+i,j)
!<<<                        x(ix) = x(ix) - ddm
                            t_p1 = xh(ix) - ddmh
                            t_p2 = t_p1 - xh(ix)
                            t_ch = t_p1
                            t_cl = (xh(ix) - (t_p1 - t_p2))
     +                               - (ddmh + t_p2)
                            t_cl = t_cl + xl(ix) - ddml
                            t_s1 = t_ch + t_cl
                            xl(ix) = t_cl - (t_s1 - t_ch)
                            xh(ix) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                        x(ix) = x(ix) - ddm
!!                          IX = IX + INCX
   70                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
                      JX = JX + INCX
   80             CONTINUE
              END IF
          END IF
      ELSE
*
*        Form  x := inv( A')*x.
*
          IF (LSAME(UPLO,'U')) THEN
              KPLUS1 = K + 1
              IF (INCX.EQ.1) THEN
                  DO 100 J = 1,N
!                     TEMP = X(J)
                      temph = xh(j)
                      templ = xl(j)
                      L = KPLUS1 - J
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                      DO 90 I = MAX(1,J-K),J - 1
!                         TEMP = TEMP - A(L+I,J)*X(I)
!<<<                      ddm = a(l+i,j) * x(i)
                          ddmh = ah(l+i,j) * xh(i)
                          t_u1 = 134217729.0 * ah(l+i,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(l+i,j))
                          t_p2 = ah(l+i,j) - t_p1
                          t_u1 = 134217729.0 * xh(i)
                          t_r1 = t_u1
     +                             - (t_u1 - xh(i))
                          t_r2 = xh(i) - t_r1
                          ddml = ((t_p1*t_r1
     +                            - ddmh)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddml = ddml
     +                          + (ah(l+i,j)*xl(i)
     +                          + al(l+i,j)*xh(i))
                          t_s1 = ddmh + ddml
                          ddml = ddml - (t_s1 - ddmh)
                          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = a(l+i,j) * x(i)
!$omp critical
!<<<                      temp = temp - ddm
                          t_p1 = temph - ddmh
                          t_p2 = t_p1 - temph
                          t_ch = t_p1
                          t_cl = (temph - (t_p1 - t_p2))
     +                             - (ddmh + t_p2)
                          t_cl = t_cl + templ - ddml
                          t_s1 = t_ch + t_cl
                          templ = t_cl - (t_s1 - t_ch)
                          temph = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                      temp = temp - ddm
!$omp end critical
   90                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (NOUNIT) TEMP = TEMP/A(KPLUS1,J)
                      if (nounit) then
!<<<                      temp = temp / a(kplus1,j)
                          t_q1 = temph / ah(kplus1,j)
                          t_ch = ah(kplus1,j) * t_q1
                          t_u1 = 134217729.0 * ah(kplus1,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(kplus1,j))
                          t_p2 = ah(kplus1,j) - t_p1
                          t_u1 = 134217729.0 * t_q1
                          t_r1 = t_u1
     +                             - (t_u1 - t_q1)
                          t_r2 = t_q1 - t_r1
                          t_cl = ((t_p1*t_r1
     +                             - t_ch)
     +                             + t_p1*t_r2
     +                             + t_p2*t_r1)
     +                             + t_p2*t_r2
                          t_cl = t_cl
     +                             + (al(kplus1,j) * t_q1)
                          t_s1 = t_ch + t_cl
                          t_cl = t_cl
     +                             - (t_s1 - t_ch)
                          t_ch = t_s1
                          t_s1 = temph - t_ch
                          t_r1 = t_s1 - temph
                          t_s2 = (temph
     +                             - (t_s1 - t_r1))
     +                             - (t_ch + t_r1)
                          t_s2 = t_s2 - t_cl
                          t_s2 = t_s2 + templ
                          t_q2 = (t_s1 + t_s2)
     +                             / ah(kplus1,j)
                          temph = t_q1 + t_q2
                          templ = t_q2
     +                          - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                      temp = temp / a(kplus1,j)
                      endif
!                     X(J) = TEMP
                      xh(j) = temph
                      xl(j) = templ
  100             CONTINUE
              ELSE
                  JX = KX
                  DO 120 J = 1,N
!                     TEMP = X(JX)
                      temph = xh(jx)
                      templ = xl(jx)
                      IX = KX
                      L = KPLUS1 - J
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                      DO 110 I = MAX(1,J-K),J - 1
                          ix = kx + (i - MAX(1,J-K))*incx
!                         TEMP = TEMP - A(L+I,J)*X(IX)
!<<<                      ddm = a(l+i,j) * x(ix)
                          ddmh = ah(l+i,j) * xh(ix)
                          t_u1 = 134217729.0 * ah(l+i,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(l+i,j))
                          t_p2 = ah(l+i,j) - t_p1
                          t_u1 = 134217729.0 * xh(ix)
                          t_r1 = t_u1
     +                             - (t_u1 - xh(ix))
                          t_r2 = xh(ix) - t_r1
                          ddml = ((t_p1*t_r1
     +                            - ddmh)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddml = ddml
     +                          + (ah(l+i,j)*xl(ix)
     +                          + al(l+i,j)*xh(ix))
                          t_s1 = ddmh + ddml
                          ddml = ddml - (t_s1 - ddmh)
                          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = a(l+i,j) * x(ix)
!$omp critical
!<<<                      temp = temp - ddm
                          t_p1 = temph - ddmh
                          t_p2 = t_p1 - temph
                          t_ch = t_p1
                          t_cl = (temph - (t_p1 - t_p2))
     +                             - (ddmh + t_p2)
                          t_cl = t_cl + templ - ddml
                          t_s1 = t_ch + t_cl
                          templ = t_cl - (t_s1 - t_ch)
                          temph = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                      temp = temp - ddm
!$omp end critical
!!                        IX = IX + INCX
  110                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (NOUNIT) TEMP = TEMP/A(KPLUS1,J)
                      if (nounit) then
!<<<                      temp = temp / a(kplus1,j)
                          t_q1 = temph / ah(kplus1,j)
                          t_ch = ah(kplus1,j) * t_q1
                          t_u1 = 134217729.0 * ah(kplus1,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(kplus1,j))
                          t_p2 = ah(kplus1,j) - t_p1
                          t_u1 = 134217729.0 * t_q1
                          t_r1 = t_u1
     +                             - (t_u1 - t_q1)
                          t_r2 = t_q1 - t_r1
                          t_cl = ((t_p1*t_r1
     +                             - t_ch)
     +                             + t_p1*t_r2
     +                             + t_p2*t_r1)
     +                             + t_p2*t_r2
                          t_cl = t_cl
     +                             + (al(kplus1,j) * t_q1)
                          t_s1 = t_ch + t_cl
                          t_cl = t_cl
     +                             - (t_s1 - t_ch)
                          t_ch = t_s1
                          t_s1 = temph - t_ch
                          t_r1 = t_s1 - temph
                          t_s2 = (temph
     +                             - (t_s1 - t_r1))
     +                             - (t_ch + t_r1)
                          t_s2 = t_s2 - t_cl
                          t_s2 = t_s2 + templ
                          t_q2 = (t_s1 + t_s2)
     +                             / ah(kplus1,j)
                          temph = t_q1 + t_q2
                          templ = t_q2
     +                          - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                      temp = temp / a(kplus1,j)
                      endif
!                     X(JX) = TEMP
                      xh(jx) = temph
                      xl(jx) = templ
                      JX = JX + INCX
                      IF (J.GT.K) KX = KX + INCX
  120             CONTINUE
              END IF
          ELSE
              IF (INCX.EQ.1) THEN
                  DO 140 J = N,1,-1
!                     TEMP = X(J)
                      temph = xh(j)
                      templ = xl(j)
                      L = 1 - J
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                      DO 130 I = MIN(N,J+K),J + 1,-1
!                         TEMP = TEMP - A(L+I,J)*X(I)
!<<<                      ddm = a(l+i,j) * x(i)
                          ddmh = ah(l+i,j) * xh(i)
                          t_u1 = 134217729.0 * ah(l+i,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(l+i,j))
                          t_p2 = ah(l+i,j) - t_p1
                          t_u1 = 134217729.0 * xh(i)
                          t_r1 = t_u1
     +                             - (t_u1 - xh(i))
                          t_r2 = xh(i) - t_r1
                          ddml = ((t_p1*t_r1
     +                            - ddmh)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddml = ddml
     +                          + (ah(l+i,j)*xl(i)
     +                          + al(l+i,j)*xh(i))
                          t_s1 = ddmh + ddml
                          ddml = ddml - (t_s1 - ddmh)
                          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = a(l+i,j) * x(i)
!$omp critical
!<<<                      temp = temp - ddm
                          t_p1 = temph - ddmh
                          t_p2 = t_p1 - temph
                          t_ch = t_p1
                          t_cl = (temph - (t_p1 - t_p2))
     +                             - (ddmh + t_p2)
                          t_cl = t_cl + templ - ddml
                          t_s1 = t_ch + t_cl
                          templ = t_cl - (t_s1 - t_ch)
                          temph = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                      temp = temp - ddm
!$omp end critical
  130                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (NOUNIT) TEMP = TEMP/A(1,J)
                      if (nounit) then
!<<<                      temp = temp / a(1,j)
                          t_q1 = temph / ah(1,j)
                          t_ch = ah(1,j) * t_q1
                          t_u1 = 134217729.0 * ah(1,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(1,j))
                          t_p2 = ah(1,j) - t_p1
                          t_u1 = 134217729.0 * t_q1
                          t_r1 = t_u1
     +                             - (t_u1 - t_q1)
                          t_r2 = t_q1 - t_r1
                          t_cl = ((t_p1*t_r1
     +                             - t_ch)
     +                             + t_p1*t_r2
     +                             + t_p2*t_r1)
     +                             + t_p2*t_r2
                          t_cl = t_cl
     +                             + (al(1,j) * t_q1)
                          t_s1 = t_ch + t_cl
                          t_cl = t_cl
     +                             - (t_s1 - t_ch)
                          t_ch = t_s1
                          t_s1 = temph - t_ch
                          t_r1 = t_s1 - temph
                          t_s2 = (temph
     +                             - (t_s1 - t_r1))
     +                             - (t_ch + t_r1)
                          t_s2 = t_s2 - t_cl
                          t_s2 = t_s2 + templ
                          t_q2 = (t_s1 + t_s2)
     +                             / ah(1,j)
                          temph = t_q1 + t_q2
                          templ = t_q2
     +                          - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                      temp = temp / a(1,j)
                      endif
!                     X(J) = TEMP
                      xh(j) = temph
                      xl(j) = templ
  140             CONTINUE
              ELSE
                  KX = KX + (N-1)*INCX
                  JX = KX
                  DO 160 J = N,1,-1
!                     TEMP = X(JX)
                      temph = xh(jx)
                      templ = xl(jx)
                      IX = KX
                      L = 1 - J
!$omp parallel
!$omp do
!$omp+ private(I,ix)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                      DO 150 I = MIN(N,J+K),J + 1,-1
                          ix = kx + (i - MIN(N,J+K))*incx
!                         TEMP = TEMP - A(L+I,J)*X(IX)
!<<<                      ddm = a(l+i,j) * x(ix)
                          ddmh = ah(l+i,j) * xh(ix)
                          t_u1 = 134217729.0 * ah(l+i,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(l+i,j))
                          t_p2 = ah(l+i,j) - t_p1
                          t_u1 = 134217729.0 * xh(ix)
                          t_r1 = t_u1
     +                             - (t_u1 - xh(ix))
                          t_r2 = xh(ix) - t_r1
                          ddml = ((t_p1*t_r1
     +                            - ddmh)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          ddml = ddml
     +                          + (ah(l+i,j)*xl(ix)
     +                          + al(l+i,j)*xh(ix))
                          t_s1 = ddmh + ddml
                          ddml = ddml - (t_s1 - ddmh)
                          ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddm = a(l+i,j) * x(ix)
!$omp critical
!<<<                      temp = temp - ddm
                          t_p1 = temph - ddmh
                          t_p2 = t_p1 - temph
                          t_ch = t_p1
                          t_cl = (temph - (t_p1 - t_p2))
     +                             - (ddmh + t_p2)
                          t_cl = t_cl + templ - ddml
                          t_s1 = t_ch + t_cl
                          templ = t_cl - (t_s1 - t_ch)
                          temph = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                      temp = temp - ddm
!$omp end critical
!!                        IX = IX - INCX
  150                 CONTINUE
!$omp end do
!$omp end parallel
!                     IF (NOUNIT) TEMP = TEMP/A(1,J)
                      if (nounit) then
!<<<                      temp = temp / a(1,j)
                          t_q1 = temph / ah(1,j)
                          t_ch = ah(1,j) * t_q1
                          t_u1 = 134217729.0 * ah(1,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(1,j))
                          t_p2 = ah(1,j) - t_p1
                          t_u1 = 134217729.0 * t_q1
                          t_r1 = t_u1
     +                             - (t_u1 - t_q1)
                          t_r2 = t_q1 - t_r1
                          t_cl = ((t_p1*t_r1
     +                             - t_ch)
     +                             + t_p1*t_r2
     +                             + t_p2*t_r1)
     +                             + t_p2*t_r2
                          t_cl = t_cl
     +                             + (al(1,j) * t_q1)
                          t_s1 = t_ch + t_cl
                          t_cl = t_cl
     +                             - (t_s1 - t_ch)
                          t_ch = t_s1
                          t_s1 = temph - t_ch
                          t_r1 = t_s1 - temph
                          t_s2 = (temph
     +                             - (t_s1 - t_r1))
     +                             - (t_ch + t_r1)
                          t_s2 = t_s2 - t_cl
                          t_s2 = t_s2 + templ
                          t_q2 = (t_s1 + t_s2)
     +                             / ah(1,j)
                          temph = t_q1 + t_q2
                          templ = t_q2
     +                          - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                      temp = temp / a(1,j)
                      endif
!                     X(JX) = TEMP
                      xh(jx) = temph
                      xl(jx) = templ
                      JX = JX - INCX
                      IF ((N-J).GE.K) KX = KX - INCX
  160             CONTINUE
              END IF
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = ddtbsv"
      WRITE(6,'(2A)') "trans    = ",trans
#include "fout.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DTBSV .
*
      END
