!     SUBROUTINE DTRSM(SIDE,UPLO,TRANSA,DIAG,M,N,ALPHA,A,LDA,B,LDB)
      subroutine ddtrsm(side, uplo, transa, diag, m, n,
     +                 alphah, alphal, ah, al, lda, bh, bl, ldb)
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE PRECISION ALPHA
      double precision ALPHAh, ALPHAl
      INTEGER LDA,LDB,M,N
      CHARACTER DIAG,SIDE,TRANSA,UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE PRECISION A(LDA,*),B(LDB,*)
      double precision ah(lda,*), al(lda,*)
      double precision bh(ldb,*), bl(ldb,*)
*     ..
*
*  Purpose
*  =======
*
*  DTRSM  solves one of the matrix equations
*
*     op( A )*X = alpha*B,   or   X*op( A ) = alpha*B,
*
*  where alpha is a scalar, X and B are m by n matrices, A is a unit, or
*  non-unit,  upper or lower triangular matrix  and  op( A )  is one  of
*
*     op( A ) = A   or   op( A ) = A'.
*
*  The matrix X is overwritten on B.
*
*  Arguments
*  ==========
*
*  SIDE   - CHARACTER*1.
*           On entry, SIDE specifies whether op( A ) appears on the left
*           or right of X as follows:
*
*              SIDE = 'L' or 'l'   op( A )*X = alpha*B.
*
*              SIDE = 'R' or 'r'   X*op( A ) = alpha*B.
*
*           Unchanged on exit.
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the matrix A is an upper or
*           lower triangular matrix as follows:
*
*              UPLO = 'U' or 'u'   A is an upper triangular matrix.
*
*              UPLO = 'L' or 'l'   A is a lower triangular matrix.
*
*           Unchanged on exit.
*
*  TRANSA - CHARACTER*1.
*           On entry, TRANSA specifies the form of op( A ) to be used in
*           the matrix multiplication as follows:
*
*              TRANSA = 'N' or 'n'   op( A ) = A.
*
*              TRANSA = 'T' or 't'   op( A ) = A'.
*
*              TRANSA = 'C' or 'c'   op( A ) = A'.
*
*           Unchanged on exit.
*
*  DIAG   - CHARACTER*1.
*           On entry, DIAG specifies whether or not A is unit triangular
*           as follows:
*
*              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
*
*              DIAG = 'N' or 'n'   A is not assumed to be unit
*                                  triangular.
*
*           Unchanged on exit.
*
*  M      - INTEGER.
*           On entry, M specifies the number of rows of B. M must be at
*           least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of B.  N must be
*           at least zero.
*           Unchanged on exit.
*
*  ALPHA  - DOUBLE PRECISION.
*           On entry,  ALPHA specifies the scalar  alpha. When  alpha is
*           zero then  A is not referenced and  B need not be set before
*           entry.
*           Unchanged on exit.
*
*  A      - DOUBLE PRECISION array of DIMENSION ( LDA, k ), where k is m
*           when  SIDE = 'L' or 'l'  and is  n  when  SIDE = 'R' or 'r'.
*           Before entry  with  UPLO = 'U' or 'u',  the  leading  k by k
*           upper triangular part of the array  A must contain the upper
*           triangular matrix  and the strictly lower triangular part of
*           A is not referenced.
*           Before entry  with  UPLO = 'L' or 'l',  the  leading  k by k
*           lower triangular part of the array  A must contain the lower
*           triangular matrix  and the strictly upper triangular part of
*           A is not referenced.
*           Note that when  DIAG = 'U' or 'u',  the diagonal elements of
*           A  are not referenced either,  but are assumed to be  unity.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
*           LDA  must be at least  max( 1, m ),  when  SIDE = 'R' or 'r'
*           then LDA must be at least max( 1, n ).
*           Unchanged on exit.
*
*  B      - DOUBLE PRECISION array of DIMENSION ( LDB, n ).
*           Before entry,  the leading  m by n part of the array  B must
*           contain  the  right-hand  side  matrix  B,  and  on exit  is
*           overwritten by the solution matrix  X.
*
*  LDB    - INTEGER.
*           On entry, LDB specifies the first dimension of B as declared
*           in  the  calling  (sub)  program.   LDB  must  be  at  least
*           max( 1, m ).
*           Unchanged on exit.
*
*
*  Level 3 Blas routine.
*
*
*  -- Written on 8-February-1989.
*     Jack Dongarra, Argonne National Laboratory.
*     Iain Duff, AERE Harwell.
*     Jeremy Du Croz, Numerical Algorithms Group Ltd.
*     Sven Hammarling, Numerical Algorithms Group Ltd.
*
*
!  double double modefication (f.y.)
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC MAX
*     ..
*     .. Local Scalars ..
!     DOUBLE PRECISION TEMP
      double precision temph, templ
      INTEGER I,INFO,J,K,NROWA
      LOGICAL LSIDE,NOUNIT,UPPER
*     ..
*     .. Parameters ..
!     DOUBLE PRECISION ONE,ZERO
      double precision oneh, onel
      double precision zeroh, zerol
!     PARAMETER (ONE=1.0D+0,ZERO=0.0D+0)
      double precision ddmh, ddml
      double precision ddah, ddal
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "ddtrsm"
#endif
*     ..
*
*     Test the input parameters.
*
      zeroh = 0.0d+0
      zerol = 0.0d+0
      oneh = 1.0d+0
      onel = 0.0d+0
      LSIDE = LSAME(SIDE,'L')
      IF (LSIDE) THEN
          NROWA = M
      ELSE
          NROWA = N
      END IF
      NOUNIT = LSAME(DIAG,'N')
      UPPER = LSAME(UPLO,'U')
*
      INFO = 0
      IF ((.NOT.LSIDE) .AND. (.NOT.LSAME(SIDE,'R'))) THEN
          INFO = 1
      ELSE IF ((.NOT.UPPER) .AND. (.NOT.LSAME(UPLO,'L'))) THEN
          INFO = 2
      ELSE IF ((.NOT.LSAME(TRANSA,'N')) .AND.
     +         (.NOT.LSAME(TRANSA,'T')) .AND.
     +         (.NOT.LSAME(TRANSA,'C'))) THEN
          INFO = 3
      ELSE IF ((.NOT.LSAME(DIAG,'U')) .AND. (.NOT.LSAME(DIAG,'N'))) THEN
          INFO = 4
      ELSE IF (M.LT.0) THEN
          INFO = 5
      ELSE IF (N.LT.0) THEN
          INFO = 6
      ELSE IF (LDA.LT.MAX(1,NROWA)) THEN
          INFO = 9
      ELSE IF (LDB.LT.MAX(1,M)) THEN
          INFO = 11
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('DTRSM ',INFO)
          goto 9999
!         RETURN
      END IF
*
*     Quick return if possible.
*
!     IF (M.EQ.0 .OR. N.EQ.0) RETURN
      IF (M.EQ.0 .OR. N.EQ.0) goto 9999
*
*     And when  alpha.eq.zero.
*
!     IF (ALPHA.EQ.ZERO) THEN
      if(
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +) then
          DO 20 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
              DO 10 I = 1,M
!                 B(I,J) = ZERO
                  bh(i,j) = zeroh
                  bl(i,j) = zerol
   10         CONTINUE
!$omp end do
!$omp end parallel
   20     CONTINUE
          goto 9999
!         RETURN
      END IF
*
*     Start the operations.
*
      IF (LSIDE) THEN
          IF (LSAME(TRANSA,'N')) THEN
*
*           Form  B := alpha*inv( A )*B.
*
              IF (UPPER) THEN
                  DO 60 J = 1,N
!                     IF (ALPHA.NE.ONE) THEN
                      if(
!                       (ALPHA .ne. ONE)
     +                 ((ALPHAh .ne. ONEh) .or.
     +                  (ALPHAl .ne. ONEl))
     +                ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 30 I = 1,M
!                             B(I,J) = ALPHA*B(I,J)
!<<<                          b(i,j) = alpha * b(i,j)
                              t_ch = alphah * bh(i,j)
                              t_u1 = 134217729.0 * alphah
                              t_p1 = t_u1
     +                                 - (t_u1 - alphah)
                              t_p2 = alphah - t_p1
                              t_u1 = 134217729.0 * bh(i,j)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,j))
                              t_r2 = bh(i,j) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (alphah*bl(i,j)
     +                                 +  alphal*bh(i,j))
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,j) = alpha * b(i,j)
   30                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
                      DO 50 K = M,1,-1
!                         IF (B(K,J).NE.ZERO) THEN
                          if(
!                           (b(k,j) .ne. zero)
     +                     ((bh(k,j) .ne. zeroh) .or.
     +                      (bl(k,j) .ne. zerol))
     +                    ) then
!                             IF (NOUNIT) B(K,J) = B(K,J)/A(K,K)
                              if(nounit) then
!<<<                          b(k,j) = b(k,j) / a(k,k)
                              t_q1 = bh(k,j) / ah(k,k)
                              t_ch = ah(k,k) * t_q1
                              t_u1 = 134217729.0 * ah(k,k)
                              t_p1 = t_u1
     +                                 - (t_u1 - ah(k,k))
                              t_p2 = ah(k,k) - t_p1
                              t_u1 = 134217729.0 * t_q1
                              t_r1 = t_u1
     +                                 - (t_u1 - t_q1)
                              t_r2 = t_q1 - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                 + t_p1*t_r2
     +                                 + t_p2*t_r1)
     +                                 + t_p2*t_r2
                              t_cl = t_cl
     +                                 + (al(k,k) * t_q1)
                              t_s1 = t_ch + t_cl
                              t_cl = t_cl
     +                                 - (t_s1 - t_ch)
                              t_ch = t_s1
                              t_s1 = bh(k,j) - t_ch
                              t_r1 = t_s1 - bh(k,j)
                              t_s2 = (bh(k,j)
     +                                 - (t_s1 - t_r1))
     +                                 - (t_ch + t_r1)
                              t_s2 = t_s2 - t_cl
                              t_s2 = t_s2 + bl(k,j)
                              t_q2 = (t_s1 + t_s2)
     +                                 / ah(k,k)
                              bh(k,j) = t_q1 + t_q2
                              bl(k,j) = t_q2
     +                              - (bh(k,j) - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                          b(k,j) = b(k,j) / a(k,k)
                              endif
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                              DO 40 I = 1,K - 1
!                                 B(I,J) = B(I,J) - B(K,J)*A(I,K)
!<<<                          ddm = b(k,j) * a(i,k)
                              ddmh = bh(k,j) * ah(i,k)
                              t_u1 = 134217729.0 * bh(k,j)
                              t_p1 = t_u1
     +                                 - (t_u1 - bh(k,j))
                              t_p2 = bh(k,j) - t_p1
                              t_u1 = 134217729.0 * ah(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - ah(i,k))
                              t_r2 = ah(i,k) - t_r1
                              ddml = ((t_p1*t_r1
     +                                - ddmh)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              ddml = ddml
     +                              + (bh(k,j)*al(i,k)
     +                              + bl(k,j)*ah(i,k))
                              t_s1 = ddmh + ddml
                              ddml = ddml - (t_s1 - ddmh)
                              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = b(k,j) * a(i,k)
!<<<                          b(i,j) = b(i,j) - ddm
                              t_p1 = bh(i,j) - ddmh
                              t_p2 = t_p1 - bh(i,j)
                              t_ch = t_p1
                              t_cl = (bh(i,j) - (t_p1 - t_p2))
     +                                 - (ddmh + t_p2)
                              t_cl = t_cl + bl(i,j) - ddml
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                          b(i,j) = b(i,j) - ddm
   40                         CONTINUE
!$omp end do
!$omp end parallel
                          END IF
   50                 CONTINUE
   60             CONTINUE
              ELSE
                  DO 100 J = 1,N
!                     IF (ALPHA.NE.ONE) THEN
                      if(
!                       (alpha .ne. one)
     +                 ((alphah .ne. oneh) .or.
     +                  (alphal .ne. onel))
     +                ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 70 I = 1,M
!                             B(I,J) = ALPHA*B(I,J)
!<<<                          b(i,j) = alpha * b(i,j)
                              t_ch = alphah * bh(i,j)
                              t_u1 = 134217729.0 * alphah
                              t_p1 = t_u1
     +                                 - (t_u1 - alphah)
                              t_p2 = alphah - t_p1
                              t_u1 = 134217729.0 * bh(i,j)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,j))
                              t_r2 = bh(i,j) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (alphah*bl(i,j)
     +                                 +  alphal*bh(i,j))
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,j) = alpha * b(i,j)
   70                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
                      DO 90 K = 1,M
!                         IF (B(K,J).NE.ZERO) THEN
                          if(
!                           (b(k,j) .ne. zero)
     +                     ((bh(k,j) .ne. zeroh) .or.
     +                      (bl(k,j) .ne. zerol))
     +                    ) then
!                             IF (NOUNIT) B(K,J) = B(K,J)/A(K,K)
                              if(nounit) then
!<<<                          b(k,j) = b(k,j) / a(k,k)
                              t_q1 = bh(k,j) / ah(k,k)
                              t_ch = ah(k,k) * t_q1
                              t_u1 = 134217729.0 * ah(k,k)
                              t_p1 = t_u1
     +                                 - (t_u1 - ah(k,k))
                              t_p2 = ah(k,k) - t_p1
                              t_u1 = 134217729.0 * t_q1
                              t_r1 = t_u1
     +                                 - (t_u1 - t_q1)
                              t_r2 = t_q1 - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                 + t_p1*t_r2
     +                                 + t_p2*t_r1)
     +                                 + t_p2*t_r2
                              t_cl = t_cl
     +                                 + (al(k,k) * t_q1)
                              t_s1 = t_ch + t_cl
                              t_cl = t_cl
     +                                 - (t_s1 - t_ch)
                              t_ch = t_s1
                              t_s1 = bh(k,j) - t_ch
                              t_r1 = t_s1 - bh(k,j)
                              t_s2 = (bh(k,j)
     +                                 - (t_s1 - t_r1))
     +                                 - (t_ch + t_r1)
                              t_s2 = t_s2 - t_cl
                              t_s2 = t_s2 + bl(k,j)
                              t_q2 = (t_s1 + t_s2)
     +                                 / ah(k,k)
                              bh(k,j) = t_q1 + t_q2
                              bl(k,j) = t_q2
     +                              - (bh(k,j) - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                          b(k,j) = b(k,j) / a(k,k)
                              endif
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                              DO 80 I = K + 1,M
!                                 B(I,J) = B(I,J) - B(K,J)*A(I,K)
!<<<                          ddm = b(k,j) * a(i,k)
                              ddmh = bh(k,j) * ah(i,k)
                              t_u1 = 134217729.0 * bh(k,j)
                              t_p1 = t_u1
     +                                 - (t_u1 - bh(k,j))
                              t_p2 = bh(k,j) - t_p1
                              t_u1 = 134217729.0 * ah(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - ah(i,k))
                              t_r2 = ah(i,k) - t_r1
                              ddml = ((t_p1*t_r1
     +                                - ddmh)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              ddml = ddml
     +                              + (bh(k,j)*al(i,k)
     +                              + bl(k,j)*ah(i,k))
                              t_s1 = ddmh + ddml
                              ddml = ddml - (t_s1 - ddmh)
                              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = b(k,j) * a(i,k)
!<<<                          b(i,j) = b(i,j) - ddm
                              t_p1 = bh(i,j) - ddmh
                              t_p2 = t_p1 - bh(i,j)
                              t_ch = t_p1
                              t_cl = (bh(i,j) - (t_p1 - t_p2))
     +                                 - (ddmh + t_p2)
                              t_cl = t_cl + bl(i,j) - ddml
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                          b(i,j) = b(i,j) - ddm
   80                         CONTINUE
!$omp end do
!$omp end parallel
                          END IF
   90                 CONTINUE
  100             CONTINUE
              END IF
          ELSE
*
*           Form  B := alpha*inv( A' )*B.
*
              IF (UPPER) THEN
                  DO 130 J = 1,N
                      DO 120 I = 1,M
!                         TEMP = ALPHA*B(I,J)
!<<<                      temp = alpha * b(i,j)
                          temph = alphah * bh(i,j)
                          t_u1 = 134217729.0 * alphah
                          t_p1 = t_u1
     +                             - (t_u1 - alphah)
                          t_p2 = alphah - t_p1
                          t_u1 = 134217729.0 * bh(i,j)
                          t_r1 = t_u1
     +                             - (t_u1 - bh(i,j))
                          t_r2 = bh(i,j) - t_r1
                          templ = ((t_p1*t_r1
     +                            - temph)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          templ = templ
     +                          + (alphah*bl(i,j)
     +                          + alphal*bh(i,j))
                          t_s1 = temph + templ
                          templ = templ - (t_s1 - temph)
                          temph = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      temp = alpha * b(i,j)
!$omp parallel
!$omp do
!$omp+ private(K)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                          DO 110 K = 1,I - 1
!                             TEMP = TEMP - A(K,I)*B(K,J)
!<<<                          ddm = a(k,i) * b(k,j)
                              ddmh = ah(k,i) * bh(k,j)
                              t_u1 = 134217729.0 * ah(k,i)
                              t_p1 = t_u1
     +                                 - (t_u1 - ah(k,i))
                              t_p2 = ah(k,i) - t_p1
                              t_u1 = 134217729.0 * bh(k,j)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(k,j))
                              t_r2 = bh(k,j) - t_r1
                              ddml = ((t_p1*t_r1
     +                                - ddmh)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              ddml = ddml
     +                              + (ah(k,i)*bl(k,j)
     +                              + al(k,i)*bh(k,j))
                              t_s1 = ddmh + ddml
                              ddml = ddml - (t_s1 - ddmh)
                              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = a(k,i) * b(k,j)
!$omp critical
!<<<                          temp = temp - ddm
                              t_p1 = temph - ddmh
                              t_p2 = t_p1 - temph
                              t_ch = t_p1
                              t_cl = (temph - (t_p1 - t_p2))
     +                                 - (ddmh + t_p2)
                              t_cl = t_cl + templ - ddml
                              t_s1 = t_ch + t_cl
                              templ = t_cl - (t_s1 - t_ch)
                              temph = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                          temp = temp - ddm
!$omp end critical
  110                     CONTINUE
!$omp end do
!$omp end parallel
!                         IF (NOUNIT) TEMP = TEMP/A(I,I)
                          if(nounit) then
!<<<                          temp = temp / a(i,i)
                              t_q1 = temph / ah(i,i)
                              t_ch = ah(i,i) * t_q1
                              t_u1 = 134217729.0 * ah(i,i)
                              t_p1 = t_u1
     +                                 - (t_u1 - ah(i,i))
                              t_p2 = ah(i,i) - t_p1
                              t_u1 = 134217729.0 * t_q1
                              t_r1 = t_u1
     +                                 - (t_u1 - t_q1)
                              t_r2 = t_q1 - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                 + t_p1*t_r2
     +                                 + t_p2*t_r1)
     +                                 + t_p2*t_r2
                              t_cl = t_cl
     +                                 + (al(i,i) * t_q1)
                              t_s1 = t_ch + t_cl
                              t_cl = t_cl
     +                                 - (t_s1 - t_ch)
                              t_ch = t_s1
                              t_s1 = temph - t_ch
                              t_r1 = t_s1 - temph
                              t_s2 = (temph
     +                                 - (t_s1 - t_r1))
     +                                 - (t_ch + t_r1)
                              t_s2 = t_s2 - t_cl
                              t_s2 = t_s2 + templ
                              t_q2 = (t_s1 + t_s2)
     +                                 / ah(i,i)
                              temph = t_q1 + t_q2
                              templ = t_q2
     +                              - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                          temp = temp / a(i,i)
                          endif
!                         B(I,J) = TEMP
                          bh(i,j) = temph
                          bl(i,j) = templ
  120                 CONTINUE
  130             CONTINUE
              ELSE
                  DO 160 J = 1,N
                      DO 150 I = M,1,-1
!                         TEMP = ALPHA*B(I,J)
!<<<                      temp = alpha * b(i,j)
                          temph = alphah * bh(i,j)
                          t_u1 = 134217729.0 * alphah
                          t_p1 = t_u1
     +                             - (t_u1 - alphah)
                          t_p2 = alphah - t_p1
                          t_u1 = 134217729.0 * bh(i,j)
                          t_r1 = t_u1
     +                             - (t_u1 - bh(i,j))
                          t_r2 = bh(i,j) - t_r1
                          templ = ((t_p1*t_r1
     +                            - temph)
     +                            + t_p1 * t_r2
     +                            + t_p2 * t_r1)
     +                            + t_p2 * t_r2
                          templ = templ
     +                          + (alphah*bl(i,j)
     +                          + alphal*bh(i,j))
                          t_s1 = temph + templ
                          templ = templ - (t_s1 - temph)
                          temph = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      temp = alpha * b(i,j)
!$omp parallel
!$omp do
!$omp+ private(K)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                          DO 140 K = I + 1,M
!                             TEMP = TEMP - A(K,I)*B(K,J)
!<<<                          ddm = a(k,i) * b(k,j)
                              ddmh = ah(k,i) * bh(k,j)
                              t_u1 = 134217729.0 * ah(k,i)
                              t_p1 = t_u1
     +                                 - (t_u1 - ah(k,i))
                              t_p2 = ah(k,i) - t_p1
                              t_u1 = 134217729.0 * bh(k,j)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(k,j))
                              t_r2 = bh(k,j) - t_r1
                              ddml = ((t_p1*t_r1
     +                                - ddmh)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              ddml = ddml
     +                              + (ah(k,i)*bl(k,j)
     +                              + al(k,i)*bh(k,j))
                              t_s1 = ddmh + ddml
                              ddml = ddml - (t_s1 - ddmh)
                              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = a(k,i) * b(k,j)
!$omp critical
!<<<                          temp = temp - ddm
                              t_p1 = temph - ddmh
                              t_p2 = t_p1 - temph
                              t_ch = t_p1
                              t_cl = (temph - (t_p1 - t_p2))
     +                                 - (ddmh + t_p2)
                              t_cl = t_cl + templ - ddml
                              t_s1 = t_ch + t_cl
                              templ = t_cl - (t_s1 - t_ch)
                              temph = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                          temp = temp - ddm
!$omp end critical
  140                     CONTINUE
!$omp end do
!$omp end parallel
!                         IF (NOUNIT) TEMP = TEMP/A(I,I)
                          if(nounit) then
!<<<                          temp = temp / a(i,i)
                              t_q1 = temph / ah(i,i)
                              t_ch = ah(i,i) * t_q1
                              t_u1 = 134217729.0 * ah(i,i)
                              t_p1 = t_u1
     +                                 - (t_u1 - ah(i,i))
                              t_p2 = ah(i,i) - t_p1
                              t_u1 = 134217729.0 * t_q1
                              t_r1 = t_u1
     +                                 - (t_u1 - t_q1)
                              t_r2 = t_q1 - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                 + t_p1*t_r2
     +                                 + t_p2*t_r1)
     +                                 + t_p2*t_r2
                              t_cl = t_cl
     +                                 + (al(i,i) * t_q1)
                              t_s1 = t_ch + t_cl
                              t_cl = t_cl
     +                                 - (t_s1 - t_ch)
                              t_ch = t_s1
                              t_s1 = temph - t_ch
                              t_r1 = t_s1 - temph
                              t_s2 = (temph
     +                                 - (t_s1 - t_r1))
     +                                 - (t_ch + t_r1)
                              t_s2 = t_s2 - t_cl
                              t_s2 = t_s2 + templ
                              t_q2 = (t_s1 + t_s2)
     +                                 / ah(i,i)
                              temph = t_q1 + t_q2
                              templ = t_q2
     +                              - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                          temp = temp / a(i,i)
                          endif
!                         B(I,J) = TEMP
                          bh(i,j) = temph
                          bl(i,j) = templ
  150                 CONTINUE
  160             CONTINUE
              END IF
          END IF
      ELSE
          IF (LSAME(TRANSA,'N')) THEN
*
*           Form  B := alpha*B*inv( A ).
*
              IF (UPPER) THEN
                  DO 210 J = 1,N
!                     IF (ALPHA.NE.ONE) THEN
                      if(
!                       (alpha .ne. one)
     +                 ((alphah .ne. oneh) .or.
     +                  (alphal .ne. onel))
     +                ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 170 I = 1,M
!                             B(I,J) = ALPHA*B(I,J)
!<<<                          b(i,j) = alpha * b(i,j)
                              t_ch = alphah * bh(i,j)
                              t_u1 = 134217729.0 * alphah
                              t_p1 = t_u1
     +                                 - (t_u1 - alphah)
                              t_p2 = alphah - t_p1
                              t_u1 = 134217729.0 * bh(i,j)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,j))
                              t_r2 = bh(i,j) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (alphah*bl(i,j)
     +                                 +  alphal*bh(i,j))
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,j) = alpha * b(i,j)
  170                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
                      DO 190 K = 1,J - 1
!                         IF (A(K,J).NE.ZERO) THEN
                          if(
!                           (a(k,j) .ne. zero)
     +                     ((ah(k,j) .ne. zeroh) .or.
     +                      (al(k,j) .ne. zerol))
     +                    ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                              DO 180 I = 1,M
!                                 B(I,J) = B(I,J) - A(K,J)*B(I,K)
!<<<                          ddm = a(k,j) * b(i,k)
                              ddmh = ah(k,j) * bh(i,k)
                              t_u1 = 134217729.0 * ah(k,j)
                              t_p1 = t_u1
     +                                 - (t_u1 - ah(k,j))
                              t_p2 = ah(k,j) - t_p1
                              t_u1 = 134217729.0 * bh(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,k))
                              t_r2 = bh(i,k) - t_r1
                              ddml = ((t_p1*t_r1
     +                                - ddmh)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              ddml = ddml
     +                              + (ah(k,j)*bl(i,k)
     +                              + al(k,j)*bh(i,k))
                              t_s1 = ddmh + ddml
                              ddml = ddml - (t_s1 - ddmh)
                              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = a(k,j) * b(i,k)
!<<<                          b(i,j) = b(i,j) - ddm
                              t_p1 = bh(i,j) - ddmh
                              t_p2 = t_p1 - bh(i,j)
                              t_ch = t_p1
                              t_cl = (bh(i,j) - (t_p1 - t_p2))
     +                                 - (ddmh + t_p2)
                              t_cl = t_cl + bl(i,j) - ddml
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                          b(i,j) = b(i,j) - ddm
  180                         CONTINUE
!$omp end do
!$omp end parallel
                          END IF
  190                 CONTINUE
                      IF (NOUNIT) THEN
!                         TEMP = ONE/A(J,J)
!<<<                      temp = one / a(j,j)
                          t_q1 = oneh / ah(j,j)
                          temph = ah(j,j) * t_q1
                          t_u1 = 134217729.0 * ah(j,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(j,j))
                          t_p2 = ah(j,j) - t_p1
                          t_u1 = 134217729.0 * t_q1
                          t_r1 = t_u1
     +                             - (t_u1 - t_q1)
                          t_r2 = t_q1 - t_r1
                          templ = ((t_p1*t_r1
     +                           - temph)
     +                           + t_p1*t_r2
     +                           + t_p2*t_r1)
     +                           + t_p2*t_r2
                          templ = templ + (al(j,j)*t_q1)
                          t_s1 = temph + templ
                          templ = templ - (t_s1 - temph)
                          temph = t_s1
                          t_s1 = oneh - temph
                          t_r1 = t_s1 - oneh
                          t_s2 = (oneh 
     +                             - (t_s1 - t_r1))
     +                             - (temph + t_r1)
                          t_s2 = t_s2 - templ
                          t_s2 = t_s2 + onel
                          t_q2 = (t_s1 + t_s2)
     +                             / ah(j,j)
                          temph = t_q1 + t_q2
                          templ = t_q2
     +                          - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                      temp = one / a(j,j)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 200 I = 1,M
!                             B(I,J) = TEMP*B(I,J)
!<<<                          b(i,j) = temp * b(i,j)
                              t_ch = temph * bh(i,j)
                              t_u1 = 134217729.0 * temph
                              t_p1 = t_u1
     +                                 - (t_u1 - temph)
                              t_p2 = temph - t_p1
                              t_u1 = 134217729.0 * bh(i,j)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,j))
                              t_r2 = bh(i,j) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (temph*bl(i,j)
     +                                 +  templ*bh(i,j))
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,j) = temp * b(i,j)
  200                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  210             CONTINUE
              ELSE
                  DO 260 J = N,1,-1
!                     IF (ALPHA.NE.ONE) THEN
                      if(
!                       (ALPHA .ne. ONE)
     +                 ((ALPHAh .ne. ONEh) .or.
     +                  (ALPHAl .ne. ONEl))
     +                ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 220 I = 1,M
!                             B(I,J) = ALPHA*B(I,J)
!<<<                          b(i,j) = alpha * b(i,j)
                              t_ch = alphah * bh(i,j)
                              t_u1 = 134217729.0 * alphah
                              t_p1 = t_u1
     +                                 - (t_u1 - alphah)
                              t_p2 = alphah - t_p1
                              t_u1 = 134217729.0 * bh(i,j)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,j))
                              t_r2 = bh(i,j) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (alphah*bl(i,j)
     +                                 +  alphal*bh(i,j))
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,j) = alpha * b(i,j)
  220                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
                      DO 240 K = J + 1,N
!                         IF (A(K,J).NE.ZERO) THEN
                          if(
!                           (a(k,j) .ne. zero)
     +                     ((ah(k,j) .ne. zeroh) .or.
     +                      (al(k,j) .ne. zerol))
     +                    ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                              DO 230 I = 1,M
!                                 B(I,J) = B(I,J) - A(K,J)*B(I,K)
!<<<                          ddm = a(k,j) * b(i,k)
                              ddmh = ah(k,j) * bh(i,k)
                              t_u1 = 134217729.0 * ah(k,j)
                              t_p1 = t_u1
     +                                 - (t_u1 - ah(k,j))
                              t_p2 = ah(k,j) - t_p1
                              t_u1 = 134217729.0 * bh(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,k))
                              t_r2 = bh(i,k) - t_r1
                              ddml = ((t_p1*t_r1
     +                                - ddmh)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              ddml = ddml
     +                              + (ah(k,j)*bl(i,k)
     +                              + al(k,j)*bh(i,k))
                              t_s1 = ddmh + ddml
                              ddml = ddml - (t_s1 - ddmh)
                              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = a(k,j) * b(i,k)
!<<<                          b(i,j) = b(i,j) - ddm
                              t_p1 = bh(i,j) - ddmh
                              t_p2 = t_p1 - bh(i,j)
                              t_ch = t_p1
                              t_cl = (bh(i,j) - (t_p1 - t_p2))
     +                                 - (ddmh + t_p2)
                              t_cl = t_cl + bl(i,j) - ddml
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                          b(i,j) = b(i,j) - ddm
  230                         CONTINUE
!$omp end do
!$omp end parallel
                          END IF
  240                 CONTINUE
                      IF (NOUNIT) THEN
!                         TEMP = ONE/A(J,J)
!<<<                      temp = one / a(j,j)
                          t_q1 = oneh / ah(j,j)
                          temph = ah(j,j) * t_q1
                          t_u1 = 134217729.0 * ah(j,j)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(j,j))
                          t_p2 = ah(j,j) - t_p1
                          t_u1 = 134217729.0 * t_q1
                          t_r1 = t_u1
     +                             - (t_u1 - t_q1)
                          t_r2 = t_q1 - t_r1
                          templ = ((t_p1*t_r1
     +                           - temph)
     +                           + t_p1*t_r2
     +                           + t_p2*t_r1)
     +                           + t_p2*t_r2
                          templ = templ + (al(j,j)*t_q1)
                          t_s1 = temph + templ
                          templ = templ - (t_s1 - temph)
                          temph = t_s1
                          t_s1 = oneh - temph
                          t_r1 = t_s1 - oneh
                          t_s2 = (oneh 
     +                             - (t_s1 - t_r1))
     +                             - (temph + t_r1)
                          t_s2 = t_s2 - templ
                          t_s2 = t_s2 + onel
                          t_q2 = (t_s1 + t_s2)
     +                             / ah(j,j)
                          temph = t_q1 + t_q2
                          templ = t_q2
     +                          - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                      temp = one / a(j,j)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 250 I = 1,M
!                             B(I,J) = TEMP*B(I,J)
!<<<                          b(i,j) = temp * b(i,j)
                              t_ch = temph * bh(i,j)
                              t_u1 = 134217729.0 * temph
                              t_p1 = t_u1
     +                                 - (t_u1 - temph)
                              t_p2 = temph - t_p1
                              t_u1 = 134217729.0 * bh(i,j)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,j))
                              t_r2 = bh(i,j) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (temph*bl(i,j)
     +                                 +  templ*bh(i,j))
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,j) = temp * b(i,j)
  250                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  260             CONTINUE
              END IF
          ELSE
*
*           Form  B := alpha*B*inv( A' ).
*
              IF (UPPER) THEN
                  DO 310 K = N,1,-1
                      IF (NOUNIT) THEN
!                         TEMP = ONE/A(K,K)
!<<<                      temp = one / a(k,k)
                          t_q1 = oneh / ah(k,k)
                          temph = ah(k,k) * t_q1
                          t_u1 = 134217729.0 * ah(k,k)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(k,k))
                          t_p2 = ah(k,k) - t_p1
                          t_u1 = 134217729.0 * t_q1
                          t_r1 = t_u1
     +                             - (t_u1 - t_q1)
                          t_r2 = t_q1 - t_r1
                          templ = ((t_p1*t_r1
     +                           - temph)
     +                           + t_p1*t_r2
     +                           + t_p2*t_r1)
     +                           + t_p2*t_r2
                          templ = templ + (al(k,k)*t_q1)
                          t_s1 = temph + templ
                          templ = templ - (t_s1 - temph)
                          temph = t_s1
                          t_s1 = oneh - temph
                          t_r1 = t_s1 - oneh
                          t_s2 = (oneh 
     +                             - (t_s1 - t_r1))
     +                             - (temph + t_r1)
                          t_s2 = t_s2 - templ
                          t_s2 = t_s2 + onel
                          t_q2 = (t_s1 + t_s2)
     +                             / ah(k,k)
                          temph = t_q1 + t_q2
                          templ = t_q2
     +                          - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                      temp = one / a(k,k)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 270 I = 1,M
!                             B(I,K) = TEMP*B(I,K)
!<<<                          b(i,k) = temp * b(i,k)
                              t_ch = temph * bh(i,k)
                              t_u1 = 134217729.0 * temph
                              t_p1 = t_u1
     +                                 - (t_u1 - temph)
                              t_p2 = temph - t_p1
                              t_u1 = 134217729.0 * bh(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,k))
                              t_r2 = bh(i,k) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (temph*bl(i,k)
     +                                 +  templ*bh(i,k))
                              t_s1 = t_ch + t_cl
                              bl(i,k) = t_cl - (t_s1 - t_ch)
                              bh(i,k) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,k) = temp * b(i,k)
  270                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
                      DO 290 J = 1,K - 1
!                         IF (A(J,K).NE.ZERO) THEN
                          if(
!                           (a(j,k) .ne. zero)
     +                     ((ah(j,k) .ne. zeroh) .or.
     +                      (al(j,k) .ne. zerol))
     +                    ) then
!                             TEMP = A(J,K)
                              temph = ah(j,k)
                              templ = al(j,k)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                              DO 280 I = 1,M
!                                 B(I,J) = B(I,J) - TEMP*B(I,K)
!<<<                          ddm = temp * b(i,k)
                              ddmh = temph * bh(i,k)
                              t_u1 = 134217729.0 * temph
                              t_p1 = t_u1
     +                                 - (t_u1 - temph)
                              t_p2 = temph - t_p1
                              t_u1 = 134217729.0 * bh(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,k))
                              t_r2 = bh(i,k) - t_r1
                              ddml = ((t_p1*t_r1
     +                                - ddmh)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              ddml = ddml
     +                              + (temph*bl(i,k)
     +                              + templ*bh(i,k))
                              t_s1 = ddmh + ddml
                              ddml = ddml - (t_s1 - ddmh)
                              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = temp * b(i,k)
!<<<                          b(i,j) = b(i,j) - ddm
                              t_p1 = bh(i,j) - ddmh
                              t_p2 = t_p1 - bh(i,j)
                              t_ch = t_p1
                              t_cl = (bh(i,j) - (t_p1 - t_p2))
     +                                 - (ddmh + t_p2)
                              t_cl = t_cl + bl(i,j) - ddml
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                          b(i,j) = b(i,j) - ddm
  280                         CONTINUE
!$omp end do
!$omp end parallel
                          END IF
  290                 CONTINUE
!                     IF (ALPHA.NE.ONE) THEN
                      if(
!                       (alpha .ne. one)
     +                 ((alphah .ne. oneh) .or.
     +                  (alphal .ne. onel))
     +                ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 300 I = 1,M
!                             B(I,K) = ALPHA*B(I,K)
!<<<                          b(i,k) = alpha * b(i,k)
                              t_ch = alphah * bh(i,k)
                              t_u1 = 134217729.0 * alphah
                              t_p1 = t_u1
     +                                 - (t_u1 - alphah)
                              t_p2 = alphah - t_p1
                              t_u1 = 134217729.0 * bh(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,k))
                              t_r2 = bh(i,k) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (alphah*bl(i,k)
     +                                 +  alphal*bh(i,k))
                              t_s1 = t_ch + t_cl
                              bl(i,k) = t_cl - (t_s1 - t_ch)
                              bh(i,k) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,k) = alpha * b(i,k)
  300                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  310             CONTINUE
              ELSE
                  DO 360 K = 1,N
                      IF (NOUNIT) THEN
!                         TEMP = ONE/A(K,K)
!<<<                      temp = one / a(k,k)
                          t_q1 = oneh / ah(k,k)
                          temph = ah(k,k) * t_q1
                          t_u1 = 134217729.0 * ah(k,k)
                          t_p1 = t_u1
     +                             - (t_u1 - ah(k,k))
                          t_p2 = ah(k,k) - t_p1
                          t_u1 = 134217729.0 * t_q1
                          t_r1 = t_u1
     +                             - (t_u1 - t_q1)
                          t_r2 = t_q1 - t_r1
                          templ = ((t_p1*t_r1
     +                           - temph)
     +                           + t_p1*t_r2
     +                           + t_p2*t_r1)
     +                           + t_p2*t_r2
                          templ = templ + (al(k,k)*t_q1)
                          t_s1 = temph + templ
                          templ = templ - (t_s1 - temph)
                          temph = t_s1
                          t_s1 = oneh - temph
                          t_r1 = t_s1 - oneh
                          t_s2 = (oneh 
     +                             - (t_s1 - t_r1))
     +                             - (temph + t_r1)
                          t_s2 = t_s2 - templ
                          t_s2 = t_s2 + onel
                          t_q2 = (t_s1 + t_s2)
     +                             / ah(k,k)
                          temph = t_q1 + t_q2
                          templ = t_q2
     +                          - (temph - t_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>                      temp = one / a(k,k)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 320 I = 1,M
!                             B(I,K) = TEMP*B(I,K)
!<<<                          b(i,k) = temp * b(i,k)
                              t_ch = temph * bh(i,k)
                              t_u1 = 134217729.0 * temph
                              t_p1 = t_u1
     +                                 - (t_u1 - temph)
                              t_p2 = temph - t_p1
                              t_u1 = 134217729.0 * bh(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,k))
                              t_r2 = bh(i,k) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (temph*bl(i,k)
     +                                 +  templ*bh(i,k))
                              t_s1 = t_ch + t_cl
                              bl(i,k) = t_cl - (t_s1 - t_ch)
                              bh(i,k) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,k) = temp * b(i,k)
  320                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
                      DO 340 J = K + 1,N
!                         IF (A(J,K).NE.ZERO) THEN
                          if(
!                           (a(j,k) .ne. zero)
     +                     ((ah(j,k) .ne. zeroh) .or.
     +                      (al(j,k) .ne. zerol))
     +                    ) then
!                             TEMP = A(J,K)
                              temph = ah(j,k)
                              templ = al(j,k)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
!$omp+ private(ddmh, ddml)
                              DO 330 I = 1,M
!                                 B(I,J) = B(I,J) - TEMP*B(I,K)
!<<<                          ddm = temp * b(i,k)
                              ddmh = temph * bh(i,k)
                              t_u1 = 134217729.0 * temph
                              t_p1 = t_u1
     +                                 - (t_u1 - temph)
                              t_p2 = temph - t_p1
                              t_u1 = 134217729.0 * bh(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,k))
                              t_r2 = bh(i,k) - t_r1
                              ddml = ((t_p1*t_r1
     +                                - ddmh)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              ddml = ddml
     +                              + (temph*bl(i,k)
     +                              + templ*bh(i,k))
                              t_s1 = ddmh + ddml
                              ddml = ddml - (t_s1 - ddmh)
                              ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = temp * b(i,k)
!<<<                          b(i,j) = b(i,j) - ddm
                              t_p1 = bh(i,j) - ddmh
                              t_p2 = t_p1 - bh(i,j)
                              t_ch = t_p1
                              t_cl = (bh(i,j) - (t_p1 - t_p2))
     +                                 - (ddmh + t_p2)
                              t_cl = t_cl + bl(i,j) - ddml
                              t_s1 = t_ch + t_cl
                              bl(i,j) = t_cl - (t_s1 - t_ch)
                              bh(i,j) = t_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>                          b(i,j) = b(i,j) - ddm
  330                         CONTINUE
!$omp end do
!$omp end parallel
                          END IF
  340                 CONTINUE
!                     IF (ALPHA.NE.ONE) THEN
                      if(
!                       (alpha .ne. one)
     +                 ((alphah .ne. oneh) .or.
     +                  (alphal .ne. onel))
     +                ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(t_ch, t_cl)
!$omp+ private(t_p1, t_p2)
!$omp+ private(t_q1, t_q2)
!$omp+ private(t_r1, t_r2)
!$omp+ private(t_s1, t_s2)
!$omp+ private(t_t1, t_t2)
!$omp+ private(t_u1, t_u2)
!$omp+ private(t_x1, t_x2)
!$omp+ private(t_y1, t_y2)
!$omp+ private(t_z1, t_z2)
!$omp+ private(t_x, t_ax)
!$omp+ private(t_t1h, t_t1l)
!$omp+ private(t_t2h, t_t2l)
                          DO 350 I = 1,M
!                             B(I,K) = ALPHA*B(I,K)
!<<<                          b(i,k) = alpha * b(i,k)
                              t_ch = alphah * bh(i,k)
                              t_u1 = 134217729.0 * alphah
                              t_p1 = t_u1
     +                                 - (t_u1 - alphah)
                              t_p2 = alphah - t_p1
                              t_u1 = 134217729.0 * bh(i,k)
                              t_r1 = t_u1
     +                                 - (t_u1 - bh(i,k))
                              t_r2 = bh(i,k) - t_r1
                              t_cl = ((t_p1*t_r1
     +                                 - t_ch)
     +                                + t_p1 * t_r2
     +                                + t_p2 * t_r1)
     +                                + t_p2 * t_r2
                              t_cl = t_cl
     +                                 + (alphah*bl(i,k)
     +                                 +  alphal*bh(i,k))
                              t_s1 = t_ch + t_cl
                              bl(i,k) = t_cl - (t_s1 - t_ch)
                              bh(i,k) = t_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                          b(i,k) = alpha * b(i,k)
  350                     CONTINUE
!$omp end do
!$omp end parallel
                      END IF
  360             CONTINUE
              END IF
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = ddtrsm"
!     WRITE(6,1005) SUBNAME,"DIM M     = ",M
      WRITE(6,1005) SUBNAME,"DIM N     = ",N
      WRITE(6,1005) SUBNAME,"DIM K     = ",K
      WRITE(6,'(2A)') "transa    = ",transa
#include "fout3.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of DTRSM .
*
      END
