!     SUBROUTINE ZHER2(UPLO,N,ALPHA,X,INCX,Y,INCY,A,LDA)
      subroutine zzher2(uplo, n, alphah, alphal, xh, xl, incx,
     +                   yh, yl, incy, ah, al, lda)
      use bailey
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE COMPLEX ALPHA
      double complex alphah, alphal
      INTEGER INCX,INCY,LDA,N
      CHARACTER UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE COMPLEX A(LDA,*),X(*),Y(*)
      double complex ah(lda,*), al(lda,*)
      double complex xh(*), xl(*)
      double complex yh(*), yl(*)
*     ..
*
*  Purpose
*  =======
*
*  ZHER2  performs the hermitian rank 2 operation
*
*     A := alpha*x*conjg( y' ) + conjg( alpha )*y*conjg( x' ) + A,
*
*  where alpha is a scalar, x and y are n element vectors and A is an n
*  by n hermitian matrix.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the upper or lower
*           triangular part of the array A is to be referenced as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the upper triangular part of A
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the lower triangular part of A
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the order of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - COMPLEX*16      .
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  X      - COMPLEX*16       array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCX ) ).
*           Before entry, the incremented array X must contain the n
*           element vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  Y      - COMPLEX*16       array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array Y must contain the n
*           element vector y.
*           Unchanged on exit.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, n ).
*           Before entry with  UPLO = 'U' or 'u', the leading n by n
*           upper triangular part of the array A must contain the upper
*           triangular part of the hermitian matrix and the strictly
*           lower triangular part of A is not referenced. On exit, the
*           upper triangular part of the array A is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry with UPLO = 'L' or 'l', the leading n by n
*           lower triangular part of the array A must contain the lower
*           triangular part of the hermitian matrix and the strictly
*           upper triangular part of A is not referenced. On exit, the
*           lower triangular part of the array A is overwritten by the
*           lower triangular part of the updated matrix.
*           Note that the imaginary parts of the diagonal elements need
*           not be set, they are assumed to be zero, and on exit they
*           are set to zero.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, n ).
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*
!     double double modefication 2011/3 (f.y.)
*     .. Parameters ..
!     DOUBLE COMPLEX ZERO
!     PARAMETER (ZERO= (0.0D+0,0.0D+0))
      double complex ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
!     DOUBLE COMPLEX TEMP1,TEMP2
      double complex temp1h, temp1l
      double complex temp2h, temp2l
      INTEGER I,INFO,IX,IY,J,JX,JY,KX,KY
*     ..
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC DBLE,DCONJG,MAX
      double complex ddmxh, ddmxl
      double complex ddaxh, ddaxl
      double precision ddmh, ddml
      double precision ddah, ddal
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
!     tmp vars
      double complex x_ch, x_cl
      double complex x_p1, x_p2
      double complex x_q1, x_q2
      double complex x_r1, x_r2
      double complex x_s1, x_s2
      double complex x_t1, x_t2
      double complex x_u1, x_u2
      double complex x_x1, x_x2
      double complex x_y1, x_y2
      double complex x_z1, x_z2
      double complex x_x, x_ax
      double complex x_t1h, x_t1l
      double complex x_t2h, x_t2l
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "zzher2"
#endif
*     ..
*
*     Test the input parameters.
*
      zeroh = 0.0d+0
      zerol = 0.0d+0
      INFO = 0
      IF (.NOT.LSAME(UPLO,'U') .AND. .NOT.LSAME(UPLO,'L')) THEN
          INFO = 1
      ELSE IF (N.LT.0) THEN
          INFO = 2
      ELSE IF (INCX.EQ.0) THEN
          INFO = 5
      ELSE IF (INCY.EQ.0) THEN
          INFO = 7
      ELSE IF (LDA.LT.MAX(1,N)) THEN
          INFO = 9
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('ZHER2 ',INFO)
          RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. (ALPHA.EQ.ZERO)) RETURN
      if((n.eq.0) .or. (
!       (ALPHA .eq. ZERO)
     + ((ALPHAh .eq. ZEROh) .and.
     +  (ALPHAl .eq. ZEROl))
     +)) goto 9999
*
*     Set up the start points in X and Y if the increments are not both
*     unity.
*
      IF ((INCX.NE.1) .OR. (INCY.NE.1)) THEN
          IF (INCX.GT.0) THEN
              KX = 1
          ELSE
              KX = 1 - (N-1)*INCX
          END IF
          IF (INCY.GT.0) THEN
              KY = 1
          ELSE
              KY = 1 - (N-1)*INCY
          END IF
          JX = KX
          JY = KY
      END IF
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through the triangular part
*     of A.
*
      IF (LSAME(UPLO,'U')) THEN
*
*        Form  A  when A is stored in the upper triangle.
*
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
              DO 20 J = 1,N
!                 IF ((X(J).NE.ZERO) .OR. (Y(J).NE.ZERO)) THEN
                  if((
!                   (x(j) .ne. zero)
     +             ((xh(j) .ne. zeroh) .or.
     +              (xl(j) .ne. zerol))
     +            ).or.(
!                   (y(j) .ne. zero)
     +             ((yh(j) .ne. zeroh) .or.
     +              (yl(j) .ne. zerol))
     +            )) then
!                     TEMP1 = ALPHA*DCONJG(Y(J))
                      ddmxh = dconjg(yh(j))
                      ddmxl = dconjg(yl(j))
!<<<                  temp1 = alpha * ddmx
c                     temp1h = alphah * ddmxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddmxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddmxh)
c                     x_r2 = ddmxh - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*ddmxl
c    +                      + alphal*ddmxh)
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * ddmx
                      call bmul(alphah, alphal, ddmxh, ddmxl,
     +                    temp1h, temp1l)
!                     TEMP2 = DCONJG(ALPHA*X(J))
!<<<                  ddmx = alpha * x(j)
c                     ddmxh = alphah * xh(j)
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * xh(j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - xh(j))
c                     x_r2 = xh(j) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (alphah*xl(j)
c    +                      + alphal*xh(j))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = alpha * x(j)
                      call bmul(alphah, alphal, xh(j), xl(j),
     +                    ddmxh, ddmxl)
                      temp2h = dconjg(ddmxh)
                      temp2l = dconjg(ddmxl)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 10 I = 1,J - 1
!                         A(I,J) = A(I,J) + X(I)*TEMP1 + Y(I)*TEMP2
!<<<                      ddax = y(i) * temp2
c                         ddaxh = yh(i) * temp2h
c                         x_u1 = 134217729.0 * yh(i)
c                         x_p1 = x_u1
c    +                             - (x_u1 - yh(i))
c                         x_p2 = yh(i) - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (yh(i)*temp2l
c    +                          + yl(i)*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = y(i) * temp2
                          call bmul(yh(i), yl(i), temp2h, temp2l,
     +                        ddaxh, ddaxl)
!<<<                      ddmx = x(i) * temp1
c                         ddmxh = xh(i) * temp1h
c                         x_u1 = 134217729.0 * xh(i)
c                         x_p1 = x_u1
c    +                             - (x_u1 - xh(i))
c                         x_p2 = xh(i) - x_p1
c                         x_u1 = 134217729.0 * temp1h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_r2 = temp1h - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (xh(i)*temp1l
c    +                          + xl(i)*temp1h)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = x(i) * temp1
                          call bmul(xh(i), xl(i), temp1h, temp1l,
     +                        ddmxh, ddmxl)
!<<<                      ddmx = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         x_q1 = x_p1
c                         x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                             + (ddaxh - x_p2)
c                         x_q2 = x_q2 + ddmxl + ddaxl
c                         ddmxh = x_q1 + x_q2
c                         ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddmx = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
!<<<                      a(i,j) = a(i,j) + ddmx
c                         x_p1 = ah(i,j) + ddmxh
c                         x_p2 = x_p1 - ah(i,j)
c                         x_q1 = x_p1
c                         x_q2 = (ah(i,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + al(i,j) + ddmxl
c                         ah(i,j) = x_q1 + x_q2
c                         al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      a(i,j) = a(i,j) + ddmx
                          call bsum(ah(i,j), al(i,j), ddmxh, ddmxl,
     +                        ah(i,j), al(i,j))
   10                 CONTINUE
!$omp end do
!$omp end parallel
!                     A(J,J) = DBLE(A(J,J)) +
!    +                         DBLE(X(J)*TEMP1+Y(J)*TEMP2)
!<<<                  ddax = y(j) * temp2
c                     ddaxh = yh(j) * temp2h
c                     x_u1 = 134217729.0 * yh(j)
c                     x_p1 = x_u1
c    +                         - (x_u1 - yh(j))
c                     x_p2 = yh(j) - x_p1
c                     x_u1 = 134217729.0 * temp2h
c                     x_r1 = x_u1
c    +                         - (x_u1 - temp2h)
c                     x_r2 = temp2h - x_r1
c                     ddaxl = ((x_p1*x_r1
c    +                        - ddaxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddaxl = ddaxl
c    +                      + (yh(j)*temp2l
c    +                      + yl(j)*temp2h)
c                     x_s1 = ddaxh + ddaxl
c                     ddaxl = ddaxl - (x_s1 - ddaxh)
c                     ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddax = y(j) * temp2
                      call bmul(yh(j), yl(j), temp2h, temp2l,
     +                    ddaxh, ddaxl)
!<<<                  ddmx = x(j) * temp1
c                     ddmxh = xh(j) * temp1h
c                     x_u1 = 134217729.0 * xh(j)
c                     x_p1 = x_u1
c    +                         - (x_u1 - xh(j))
c                     x_p2 = xh(j) - x_p1
c                     x_u1 = 134217729.0 * temp1h
c                     x_r1 = x_u1
c    +                         - (x_u1 - temp1h)
c                     x_r2 = temp1h - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (xh(j)*temp1l
c    +                      + xl(j)*temp1h)
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = x(j) * temp1
                      call bmul(xh(j), xl(j), temp1h, temp1l,
     +                    ddmxh, ddmxl)
!<<<                  ddmx = ddmx + ddax
c                     x_p1 = ddmxh + ddaxh
c                     x_p2 = x_p1 - ddmxh
c                     x_q1 = x_p1
c                     x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                         + (ddaxh - x_p2)
c                     x_q2 = x_q2 + ddmxl + ddaxl
c                     ddmxh = x_q1 + x_q2
c                     ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  ddmx = ddmx + ddax
                      call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                    ddmxh, ddmxl)
                      ddmh = dble(ddmxh)
                      ddml = dble(ddmxl)
                      ddah = dble(ah(j,j))
                      ddal = dble(al(j,j))
!<<<                  dda = dda + ddm
c                     t_p1 = ddah + ddmh
c                     t_p2 = t_p1 - ddah
c                     t_q1 = t_p1
c                     t_q2 = (ddah - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddal + ddml
c                     ddah = t_q1 + t_q2
c                     ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  dda = dda + ddm
                      call bsum(ddah, ddal, ddmh, ddml,
     +                    ddah, ddal)
                      ah(j,j) = ddah
                      al(j,j) = ddal
!!!#ddb               add a(j,j) dda ddm
                  ELSE
!                     A(J,J) = DBLE(A(J,J))
                      ah(j,j) = dble(ah(j,j))
                      al(j,j) = dble(al(j,j))
                  END IF
   20         CONTINUE
          ELSE
              DO 40 J = 1,N
!                 IF ((X(JX).NE.ZERO) .OR. (Y(JY).NE.ZERO)) THEN
                  if((
!                   (x(jx) .ne. zero)
     +             ((xh(jx) .ne. zeroh) .or.
     +              (xl(jx) .ne. zerol))
     +            ).or.(
!                   (y(jy) .ne. zero)
     +             ((yh(jy) .ne. zeroh) .or.
     +              (yl(jy) .ne. zerol))
     +            )) then
!                     TEMP1 = ALPHA*DCONJG(Y(JY))
                      ddaxh = dconjg(yh(jy))
                      ddaxl = dconjg(yl(jy))
!<<<                  temp1 = alpha * ddax
c                     temp1h = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temp1h, temp1l)
!                     TEMP2 = DCONJG(ALPHA*X(JX))
!<<<                  ddmx = alpha * x(jx)
c                     ddmxh = alphah * xh(jx)
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * xh(jx)
c                     x_r1 = x_u1
c    +                         - (x_u1 - xh(jx))
c                     x_r2 = xh(jx) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (alphah*xl(jx)
c    +                      + alphal*xh(jx))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = alpha * x(jx)
                      call bmul(alphah, alphal, xh(jx), xl(jx),
     +                    ddmxh, ddmxl)
                      temp2h = dconjg(ddmxh)
                      temp2l = dconjg(ddmxl)
                      IX = KX
                      IY = KY
!$omp parallel
!$omp do
!$omp+ private(I,ix,iy)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 30 I = 1,J - 1
                          ix = kx + (i - 1)*incx
                          iy = ky + (i - 1)*incy
!                         A(I,J) = A(I,J) + X(IX)*TEMP1 + Y(IY)*TEMP2
!<<<                      ddax = y(iy) * temp2
c                         ddaxh = yh(iy) * temp2h
c                         x_u1 = 134217729.0 * yh(iy)
c                         x_p1 = x_u1
c    +                             - (x_u1 - yh(iy))
c                         x_p2 = yh(iy) - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (yh(iy)*temp2l
c    +                          + yl(iy)*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = y(iy) * temp2
                          call bmul(yh(iy), yl(iy), temp2h, temp2l,
     +                        ddaxh, ddaxl)
!<<<                      ddmx = x(ix) * temp1
c                         ddmxh = xh(ix) * temp1h
c                         x_u1 = 134217729.0 * xh(ix)
c                         x_p1 = x_u1
c    +                             - (x_u1 - xh(ix))
c                         x_p2 = xh(ix) - x_p1
c                         x_u1 = 134217729.0 * temp1h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_r2 = temp1h - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (xh(ix)*temp1l
c    +                          + xl(ix)*temp1h)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = x(ix) * temp1
                          call bmul(xh(ix), xl(ix), temp1h, temp1l,
     +                        ddmxh, ddmxl)
!<<<                      ddmx = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         x_q1 = x_p1
c                         x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                             + (ddaxh - x_p2)
c                         x_q2 = x_q2 + ddmxl + ddaxl
c                         ddmxh = x_q1 + x_q2
c                         ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddmx = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
!<<<                      a(i,j) = a(i,j) + ddmx
c                         x_p1 = ah(i,j) + ddmxh
c                         x_p2 = x_p1 - ah(i,j)
c                         x_q1 = x_p1
c                         x_q2 = (ah(i,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + al(i,j) + ddmxl
c                         ah(i,j) = x_q1 + x_q2
c                         al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      a(i,j) = a(i,j) + ddmx
                          call bsum(ah(i,j), al(i,j), ddmxh, ddmxl,
     +                        ah(i,j), al(i,j))
!!                        IX = IX + INCX
!!                        IY = IY + INCY
   30                 CONTINUE
!$omp end do
!$omp end parallel
!                     A(J,J) = DBLE(A(J,J)) +
!    +                         DBLE(X(JX)*TEMP1+Y(JY)*TEMP2)
!<<<                  ddax = y(jy) * temp2
c                     ddaxh = yh(jy) * temp2h
c                     x_u1 = 134217729.0 * yh(jy)
c                     x_p1 = x_u1
c    +                         - (x_u1 - yh(jy))
c                     x_p2 = yh(jy) - x_p1
c                     x_u1 = 134217729.0 * temp2h
c                     x_r1 = x_u1
c    +                         - (x_u1 - temp2h)
c                     x_r2 = temp2h - x_r1
c                     ddaxl = ((x_p1*x_r1
c    +                        - ddaxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddaxl = ddaxl
c    +                      + (yh(jy)*temp2l
c    +                      + yl(jy)*temp2h)
c                     x_s1 = ddaxh + ddaxl
c                     ddaxl = ddaxl - (x_s1 - ddaxh)
c                     ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddax = y(jy) * temp2
                      call bmul(yh(jy), yl(jy), temp2h, temp2l,
     +                    ddaxh, ddaxl)
!<<<                  ddmx = x(jx) * temp1
c                     ddmxh = xh(jx) * temp1h
c                     x_u1 = 134217729.0 * xh(jx)
c                     x_p1 = x_u1
c    +                         - (x_u1 - xh(jx))
c                     x_p2 = xh(jx) - x_p1
c                     x_u1 = 134217729.0 * temp1h
c                     x_r1 = x_u1
c    +                         - (x_u1 - temp1h)
c                     x_r2 = temp1h - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (xh(jx)*temp1l
c    +                      + xl(jx)*temp1h)
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = x(jx) * temp1
                      call bmul(xh(jx), xl(jx), temp1h, temp1l,
     +                    ddmxh, ddmxl)
!<<<                  ddmx = ddmx + ddax
c                     x_p1 = ddmxh + ddaxh
c                     x_p2 = x_p1 - ddmxh
c                     x_q1 = x_p1
c                     x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                         + (ddaxh - x_p2)
c                     x_q2 = x_q2 + ddmxl + ddaxl
c                     ddmxh = x_q1 + x_q2
c                     ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  ddmx = ddmx + ddax
                      call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                    ddmxh, ddmxl)
                      ddmh = dble(ddmxh)
                      ddml = dble(ddmxl)
                      ddah = dble(ah(j,j))
                      ddal = dble(al(j,j))
!<<<                  dda = dda + ddm
c                     t_p1 = ddah + ddmh
c                     t_p2 = t_p1 - ddah
c                     t_q1 = t_p1
c                     t_q2 = (ddah - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddal + ddml
c                     ddah = t_q1 + t_q2
c                     ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  dda = dda + ddm
                      call bsum(ddah, ddal, ddmh, ddml,
     +                    ddah, ddal)
                      ah(j,j) = ddah
                      al(j,j) = ddal
!!!#ddb               add a(j,j) dda ddm
                  ELSE
!                     A(J,J) = DBLE(A(J,J))
                      ah(j,j) = dble(ah(j,j))
                      al(j,j) = dble(al(j,j))
                  END IF
                  JX = JX + INCX
                  JY = JY + INCY
   40         CONTINUE
          END IF
      ELSE
*
*        Form  A  when A is stored in the lower triangle.
*
          IF ((INCX.EQ.1) .AND. (INCY.EQ.1)) THEN
              DO 60 J = 1,N
!                 IF ((X(J).NE.ZERO) .OR. (Y(J).NE.ZERO)) THEN
                  if((
!                   (x(j) .ne. zero)
     +             ((xh(j) .ne. zeroh) .or.
     +              (xl(j) .ne. zerol))
     +            ).or.(
!                   (y(j) .ne. zero)
     +             ((yh(j) .ne. zeroh) .or.
     +              (yl(j) .ne. zerol))
     +            )) then
!                     TEMP1 = ALPHA*DCONJG(Y(J))
                      ddaxh = dconjg(yh(j))
                      ddaxl = dconjg(yl(j))
!<<<                  temp1 = alpha * ddax
c                     temp1h = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temp1h, temp1l)
!                     TEMP2 = DCONJG(ALPHA*X(J))
!<<<                  ddmx = alpha * x(j)
c                     ddmxh = alphah * xh(j)
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * xh(j)
c                     x_r1 = x_u1
c    +                         - (x_u1 - xh(j))
c                     x_r2 = xh(j) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (alphah*xl(j)
c    +                      + alphal*xh(j))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = alpha * x(j)
                      call bmul(alphah, alphal, xh(j), xl(j),
     +                    ddmxh, ddmxl)
                      temp2h = dconjg(ddmxh)
                      temp2l = dconjg(ddmxl)
!                     A(J,J) = DBLE(A(J,J)) +
!    +                         DBLE(X(J)*TEMP1+Y(J)*TEMP2)
!<<<                  ddax = y(j) * temp2
c                     ddaxh = yh(j) * temp2h
c                     x_u1 = 134217729.0 * yh(j)
c                     x_p1 = x_u1
c    +                         - (x_u1 - yh(j))
c                     x_p2 = yh(j) - x_p1
c                     x_u1 = 134217729.0 * temp2h
c                     x_r1 = x_u1
c    +                         - (x_u1 - temp2h)
c                     x_r2 = temp2h - x_r1
c                     ddaxl = ((x_p1*x_r1
c    +                        - ddaxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddaxl = ddaxl
c    +                      + (yh(j)*temp2l
c    +                      + yl(j)*temp2h)
c                     x_s1 = ddaxh + ddaxl
c                     ddaxl = ddaxl - (x_s1 - ddaxh)
c                     ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddax = y(j) * temp2
                      call bmul(yh(j), yl(j), temp2h, temp2l,
     +                    ddaxh, ddaxl)
!<<<                  ddmx = x(j) * temp1
c                     ddmxh = xh(j) * temp1h
c                     x_u1 = 134217729.0 * xh(j)
c                     x_p1 = x_u1
c    +                         - (x_u1 - xh(j))
c                     x_p2 = xh(j) - x_p1
c                     x_u1 = 134217729.0 * temp1h
c                     x_r1 = x_u1
c    +                         - (x_u1 - temp1h)
c                     x_r2 = temp1h - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (xh(j)*temp1l
c    +                      + xl(j)*temp1h)
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = x(j) * temp1
                      call bmul(xh(j), xl(j), temp1h, temp1l,
     +                    ddmxh, ddmxl)
!<<<                  ddmx = ddmx + ddax
c                     x_p1 = ddmxh + ddaxh
c                     x_p2 = x_p1 - ddmxh
c                     x_q1 = x_p1
c                     x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                         + (ddaxh - x_p2)
c                     x_q2 = x_q2 + ddmxl + ddaxl
c                     ddmxh = x_q1 + x_q2
c                     ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  ddmx = ddmx + ddax
                      call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                    ddmxh, ddmxl)
                      ddmh = dble(ddmxh)
                      ddml = dble(ddmxl)
                      ddah = dble(ah(j,j))
                      ddal = dble(al(j,j))
!<<<                  dda = dda + ddm
c                     t_p1 = ddah + ddmh
c                     t_p2 = t_p1 - ddah
c                     t_q1 = t_p1
c                     t_q2 = (ddah - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddal + ddml
c                     ddah = t_q1 + t_q2
c                     ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  dda = dda + ddm
                      call bsum(ddah, ddal, ddmh, ddml,
     +                    ddah, ddal)
                      ah(j,j) = ddah
                      al(j,j) = ddal
!!!#ddb               add a(j,j) dda ddm
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 50 I = J + 1,N
!                         A(I,J) = A(I,J) + X(I)*TEMP1 + Y(I)*TEMP2
!<<<                      ddax = y(i) * temp2
c                         ddaxh = yh(i) * temp2h
c                         x_u1 = 134217729.0 * yh(i)
c                         x_p1 = x_u1
c    +                             - (x_u1 - yh(i))
c                         x_p2 = yh(i) - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (yh(i)*temp2l
c    +                          + yl(i)*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = y(i) * temp2
                          call bmul(yh(i), yl(i), temp2h, temp2l,
     +                        ddaxh, ddaxl)
!<<<                      ddmx = x(i) * temp1
c                         ddmxh = xh(i) * temp1h
c                         x_u1 = 134217729.0 * xh(i)
c                         x_p1 = x_u1
c    +                             - (x_u1 - xh(i))
c                         x_p2 = xh(i) - x_p1
c                         x_u1 = 134217729.0 * temp1h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_r2 = temp1h - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (xh(i)*temp1l
c    +                          + xl(i)*temp1h)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = x(i) * temp1
                          call bmul(xh(i), xl(i), temp1h, temp1l,
     +                        ddmxh, ddmxl)
!<<<                      ddmx = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         x_q1 = x_p1
c                         x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                             + (ddaxh - x_p2)
c                         x_q2 = x_q2 + ddmxl + ddaxl
c                         ddmxh = x_q1 + x_q2
c                         ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddmx = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
!<<<                      a(i,j) = a(i,j) + ddmx
c                         x_p1 = ah(i,j) + ddmxh
c                         x_p2 = x_p1 - ah(i,j)
c                         x_q1 = x_p1
c                         x_q2 = (ah(i,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + al(i,j) + ddmxl
c                         ah(i,j) = x_q1 + x_q2
c                         al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      a(i,j) = a(i,j) + ddmx
                          call bsum(ah(i,j), al(i,j), ddmxh, ddmxl,
     +                        ah(i,j), al(i,j))
   50                 CONTINUE
!$omp end do
!$omp end parallel
                  ELSE
!                     A(J,J) = DBLE(A(J,J))
                      ah(j,j) = dble(ah(j,j))
                      al(j,j) = dble(al(j,j))
                  END IF
   60         CONTINUE
          ELSE
              DO 80 J = 1,N
!                 IF ((X(JX).NE.ZERO) .OR. (Y(JY).NE.ZERO)) THEN
                  if((
!                   (x(jx) .ne. zero)
     +             ((xh(jx) .ne. zeroh) .or.
     +              (xl(jx) .ne. zerol))
     +            ).or.(
!                   (y(jy) .ne. zero)
     +             ((yh(jy) .ne. zeroh) .or.
     +              (yl(jy) .ne. zerol))
     +            )) then
!                     TEMP1 = ALPHA*DCONJG(Y(JY))
                      ddaxh = dconjg(yh(jy))
                      ddaxl = dconjg(yl(jy))
!<<<                  temp1 = alpha * ddax
c                     temp1h = alphah * ddaxh
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * ddaxh
c                     x_r1 = x_u1
c    +                         - (x_u1 - ddaxh)
c                     x_r2 = ddaxh - x_r1
c                     temp1l = ((x_p1*x_r1
c    +                        - temp1h)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     temp1l = temp1l
c    +                      + (alphah*ddaxl
c    +                      + alphal*ddaxh)
c                     x_s1 = temp1h + temp1l
c                     temp1l = temp1l - (x_s1 - temp1h)
c                     temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  temp1 = alpha * ddax
                      call bmul(alphah, alphal, ddaxh, ddaxl,
     +                    temp1h, temp1l)
!                     TEMP2 = DCONJG(ALPHA*X(JX))
!<<<                  ddmx = alpha * x(jx)
c                     ddmxh = alphah * xh(jx)
c                     x_u1 = 134217729.0 * alphah
c                     x_p1 = x_u1
c    +                         - (x_u1 - alphah)
c                     x_p2 = alphah - x_p1
c                     x_u1 = 134217729.0 * xh(jx)
c                     x_r1 = x_u1
c    +                         - (x_u1 - xh(jx))
c                     x_r2 = xh(jx) - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (alphah*xl(jx)
c    +                      + alphal*xh(jx))
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = alpha * x(jx)
                      call bmul(alphah, alphal, xh(jx), xl(jx),
     +                    ddmxh, ddmxl)
                      temp2h = dconjg(ddmxh)
                      temp2l = dconjg(ddmxl)
!                     A(J,J) = DBLE(A(J,J)) +
!    +                         DBLE(X(JX)*TEMP1+Y(JY)*TEMP2)
!<<<                  ddax = y(jy) * temp2
c                     ddaxh = yh(jy) * temp2h
c                     x_u1 = 134217729.0 * yh(jy)
c                     x_p1 = x_u1
c    +                         - (x_u1 - yh(jy))
c                     x_p2 = yh(jy) - x_p1
c                     x_u1 = 134217729.0 * temp2h
c                     x_r1 = x_u1
c    +                         - (x_u1 - temp2h)
c                     x_r2 = temp2h - x_r1
c                     ddaxl = ((x_p1*x_r1
c    +                        - ddaxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddaxl = ddaxl
c    +                      + (yh(jy)*temp2l
c    +                      + yl(jy)*temp2h)
c                     x_s1 = ddaxh + ddaxl
c                     ddaxl = ddaxl - (x_s1 - ddaxh)
c                     ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddax = y(jy) * temp2
                      call bmul(yh(jy), yl(jy), temp2h, temp2l,
     +                    ddaxh, ddaxl)
!<<<                  ddmx = x(jx) * temp1
c                     ddmxh = xh(jx) * temp1h
c                     x_u1 = 134217729.0 * xh(jx)
c                     x_p1 = x_u1
c    +                         - (x_u1 - xh(jx))
c                     x_p2 = xh(jx) - x_p1
c                     x_u1 = 134217729.0 * temp1h
c                     x_r1 = x_u1
c    +                         - (x_u1 - temp1h)
c                     x_r2 = temp1h - x_r1
c                     ddmxl = ((x_p1*x_r1
c    +                        - ddmxh)
c    +                        + x_p1 * x_r2
c    +                        + x_p2 * x_r1)
c    +                        + x_p2 * x_r2
c                     ddmxl = ddmxl
c    +                      + (xh(jx)*temp1l
c    +                      + xl(jx)*temp1h)
c                     x_s1 = ddmxh + ddmxl
c                     ddmxl = ddmxl - (x_s1 - ddmxh)
c                     ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddmx = x(jx) * temp1
                      call bmul(xh(jx), xl(jx), temp1h, temp1l,
     +                    ddmxh, ddmxl)
!<<<                  ddmx = ddmx + ddax
c                     x_p1 = ddmxh + ddaxh
c                     x_p2 = x_p1 - ddmxh
c                     x_q1 = x_p1
c                     x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                         + (ddaxh - x_p2)
c                     x_q2 = x_q2 + ddmxl + ddaxl
c                     ddmxh = x_q1 + x_q2
c                     ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  ddmx = ddmx + ddax
                      call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                    ddmxh, ddmxl)
                      ddmh = dble(ddmxh)
                      ddml = dble(ddmxl)
                      ddah = dble(ah(j,j))
                      ddal = dble(al(j,j))
!<<<                  dda = dda + ddm
c                     t_p1 = ddah + ddmh
c                     t_p2 = t_p1 - ddah
c                     t_q1 = t_p1
c                     t_q2 = (ddah - (t_p1 - t_p2))
c    +                         + (ddmh - t_p2)
c                     t_q2 = t_q2 + ddal + ddml
c                     ddah = t_q1 + t_q2
c                     ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                  dda = dda + ddm
                      call bsum(ddah, ddal, ddmh, ddml,
     +                    ddah, ddal)
                      ah(j,j) = ddah
                      al(j,j) = ddal
!!!#ddb               add a(j,j) dda ddm
                      IX = JX
                      IY = JY
!$omp parallel
!$omp do
!$omp+ private(I,ix,iy)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 70 I = J + 1,N
!!                        IX = IX + INCX
!!                        IY = IY + INCY
                          ix = jx + (i - j)*incx
                          iy = jy + (i - j)*incy
!                         A(I,J) = A(I,J) + X(IX)*TEMP1 + Y(IY)*TEMP2
!<<<                      ddax = y(iy) * temp2
c                         ddaxh = yh(iy) * temp2h
c                         x_u1 = 134217729.0 * yh(iy)
c                         x_p1 = x_u1
c    +                             - (x_u1 - yh(iy))
c                         x_p2 = yh(iy) - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (yh(iy)*temp2l
c    +                          + yl(iy)*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = y(iy) * temp2
                          call bmul(yh(jy), yl(jy), temp2h, temp2l,
     +                        ddaxh, ddaxl)
!<<<                      ddmx = x(ix) * temp1
c                         ddmxh = xh(ix) * temp1h
c                         x_u1 = 134217729.0 * xh(ix)
c                         x_p1 = x_u1
c    +                             - (x_u1 - xh(ix))
c                         x_p2 = xh(ix) - x_p1
c                         x_u1 = 134217729.0 * temp1h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_r2 = temp1h - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (xh(ix)*temp1l
c    +                          + xl(ix)*temp1h)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = x(ix) * temp1
                          call bmul(xh(ix), xl(ix), temp1h, temp1l,
     +                        ddmxh, ddmxl)
!<<<                      ddmx = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         x_q1 = x_p1
c                         x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                             + (ddaxh - x_p2)
c                         x_q2 = x_q2 + ddmxl + ddaxl
c                         ddmxh = x_q1 + x_q2
c                         ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddmx = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
!<<<                      a(i,j) = a(i,j) + ddmx
c                         x_p1 = ah(i,j) + ddmxh
c                         x_p2 = x_p1 - ah(i,j)
c                         x_q1 = x_p1
c                         x_q2 = (ah(i,j) - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + al(i,j) + ddmxl
c                         ah(i,j) = x_q1 + x_q2
c                         al(i,j) = x_q2 - (ah(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      a(i,j) = a(i,j) + ddmx
                          call bsum(ah(i,j), al(i,j), ddmxh, ddmxl,
     +                        ah(i,j), al(i,j))
   70                 CONTINUE
!$omp end do
!$omp end parallel
                  ELSE
!                     A(J,J) = DBLE(A(J,J))
                      ah(j,j) = dble(ah(j,j))
                      al(j,j) = dble(al(j,j))
                  END IF
                  JX = JX + INCX
                  JY = JY + INCY
   80         CONTINUE
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = zzher2"
#include "fout2.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of ZHER2 .
*
      END
