!     SUBROUTINE ZHER2K(UPLO,TRANS,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
      subroutine zzher2k(uplo, trans, n, k,
     +                    alphah, alphal, ah, al, lda, bh, bl, ldb,
     +                    betah, betal, ch, cl, ldc)
      use bailey
      implicit none
*     .. Scalar Arguments ..
!     DOUBLE COMPLEX ALPHA
      double complex alphah, alphal
!     DOUBLE PRECISION BETA
      double precision betah, betal
      INTEGER K,LDA,LDB,LDC,N
      CHARACTER TRANS,UPLO
*     ..
*     .. Array Arguments ..
!     DOUBLE COMPLEX A(LDA,*),B(LDB,*),C(LDC,*)
      double complex ah(lda,*), al(lda,*)
      double complex bh(ldb,*), bl(ldb,*)
      double complex ch(ldc,*), cl(ldc,*)
*     ..
*
*  Purpose
*  =======
*
*  ZHER2K  performs one of the hermitian rank 2k operations
*
*     C := alpha*A*conjg( B' ) + conjg( alpha )*B*conjg( A' ) + beta*C,
*
*  or
*
*     C := alpha*conjg( A' )*B + conjg( alpha )*conjg( B' )*A + beta*C,
*
*  where  alpha and beta  are scalars with  beta  real,  C is an  n by n
*  hermitian matrix and  A and B  are  n by k matrices in the first case
*  and  k by n  matrices in the second case.
*
*  Arguments
*  ==========
*
*  UPLO   - CHARACTER*1.
*           On  entry,   UPLO  specifies  whether  the  upper  or  lower
*           triangular  part  of the  array  C  is to be  referenced  as
*           follows:
*
*              UPLO = 'U' or 'u'   Only the  upper triangular part of  C
*                                  is to be referenced.
*
*              UPLO = 'L' or 'l'   Only the  lower triangular part of  C
*                                  is to be referenced.
*
*           Unchanged on exit.
*
*  TRANS  - CHARACTER*1.
*           On entry,  TRANS  specifies the operation to be performed as
*           follows:
*
*              TRANS = 'N' or 'n'    C := alpha*A*conjg( B' )          +
*                                         conjg( alpha )*B*conjg( A' ) +
*                                         beta*C.
*
*              TRANS = 'C' or 'c'    C := alpha*conjg( A' )*B          +
*                                         conjg( alpha )*conjg( B' )*A +
*                                         beta*C.
*
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry,  N specifies the order of the matrix C.  N must be
*           at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry with  TRANS = 'N' or 'n',  K  specifies  the number
*           of  columns  of the  matrices  A and B,  and on  entry  with
*           TRANS = 'C' or 'c',  K  specifies  the number of rows of the
*           matrices  A and B.  K must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - COMPLEX*16         .
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, ka ), where ka is
*           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
*           Before entry with  TRANS = 'N' or 'n',  the  leading  n by k
*           part of the array  A  must contain the matrix  A,  otherwise
*           the leading  k by n  part of the array  A  must contain  the
*           matrix A.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n'
*           then  LDA must be at least  max( 1, n ), otherwise  LDA must
*           be at least  max( 1, k ).
*           Unchanged on exit.
*
*  B      - COMPLEX*16       array of DIMENSION ( LDB, kb ), where kb is
*           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
*           Before entry with  TRANS = 'N' or 'n',  the  leading  n by k
*           part of the array  B  must contain the matrix  B,  otherwise
*           the leading  k by n  part of the array  B  must contain  the
*           matrix B.
*           Unchanged on exit.
*
*  LDB    - INTEGER.
*           On entry, LDB specifies the first dimension of B as declared
*           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n'
*           then  LDB must be at least  max( 1, n ), otherwise  LDB must
*           be at least  max( 1, k ).
*           Unchanged on exit.
*
*  BETA   - DOUBLE PRECISION            .
*           On entry, BETA specifies the scalar beta.
*           Unchanged on exit.
*
*  C      - COMPLEX*16          array of DIMENSION ( LDC, n ).
*           Before entry  with  UPLO = 'U' or 'u',  the leading  n by n
*           upper triangular part of the array C must contain the upper
*           triangular part  of the  hermitian matrix  and the strictly
*           lower triangular part of C is not referenced.  On exit, the
*           upper triangular part of the array  C is overwritten by the
*           upper triangular part of the updated matrix.
*           Before entry  with  UPLO = 'L' or 'l',  the leading  n by n
*           lower triangular part of the array C must contain the lower
*           triangular part  of the  hermitian matrix  and the strictly
*           upper triangular part of C is not referenced.  On exit, the
*           lower triangular part of the array  C is overwritten by the
*           lower triangular part of the updated matrix.
*           Note that the imaginary parts of the diagonal elements need
*           not be set,  they are assumed to be zero,  and on exit they
*           are set to zero.
*
*  LDC    - INTEGER.
*           On entry, LDC specifies the first dimension of C as declared
*           in  the  calling  (sub)  program.   LDC  must  be  at  least
*           max( 1, n ).
*           Unchanged on exit.
*
*
*  Level 3 Blas routine.
*
*  -- Written on 8-February-1989.
*     Jack Dongarra, Argonne National Laboratory.
*     Iain Duff, AERE Harwell.
*     Jeremy Du Croz, Numerical Algorithms Group Ltd.
*     Sven Hammarling, Numerical Algorithms Group Ltd.
*
*  -- Modified 8-Nov-93 to set C(J,J) to DBLE( C(J,J) ) when BETA = 1.
*     Ed Anderson, Cray Research Inc.
*
*
*     .. External Functions ..
      LOGICAL LSAME
      EXTERNAL LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC DBLE,DCONJG,MAX
*     ..
*     .. Local Scalars ..
!     DOUBLE COMPLEX TEMP1,TEMP2
      double complex temp1h, temp1l
      double complex temp2h, temp2l
      INTEGER I,INFO,J,L,NROWA
      LOGICAL UPPER
*     ..
*     .. Parameters ..
!     DOUBLE PRECISION ONE
!     PARAMETER (ONE=1.0D+0)
!     DOUBLE COMPLEX ZERO
!     PARAMETER (ZERO= (0.0D+0,0.0D+0))
      double precision oneh, onel
      double complex zeroh, zerol
      double complex ddmxh, ddmxl
      double complex ddaxh, ddaxl
      double complex ddbxh, ddbxl
      double complex ddxxh, ddxxl
      double precision ddmh, ddml
      double precision ddah, ddal
      double precision ddxh, ddxl
      double precision ddzeroh, ddzerol
!     tmp vars
      double precision t_ch, t_cl
      double precision t_p1, t_p2
      double precision t_q1, t_q2
      double precision t_r1, t_r2
      double precision t_s1, t_s2
      double precision t_t1, t_t2
      double precision t_u1, t_u2
      double precision t_x1, t_x2
      double precision t_y1, t_y2
      double precision t_z1, t_z2
      double precision t_x, t_ax
      double precision t_t1h, t_t1l
      double precision t_t2h, t_t2l
!     tmp vars
      double complex x_ch, x_cl
      double complex x_p1, x_p2
      double complex x_q1, x_q2
      double complex x_r1, x_r2
      double complex x_s1, x_s2
      double complex x_t1, x_t2
      double complex x_u1, x_u2
      double complex x_x1, x_x2
      double complex x_y1, x_y2
      double complex x_z1, x_z2
      double complex x_x, x_ax
      double complex x_t1h, x_t1l
      double complex x_t2h, x_t2l
!!!!! x_ will be default
#ifdef FPCOUNTER
#include "fpcounter.h"
#endif
#ifdef FPCOUNTER
#include "finit.h"
      SUBNAME = "zzher2k"
#endif
      oneh = 1.0d+0
      onel = 0.0d+0
      zeroh = 0.0d+0
      zerol = 0.0d+0
      ddzeroh = 0.0d+0
      ddzerol = 0.0d+0
*     ..
*
*     Test the input parameters.
*
      IF (LSAME(TRANS,'N')) THEN
          NROWA = N
      ELSE
          NROWA = K
      END IF
      UPPER = LSAME(UPLO,'U')
*
      INFO = 0
      IF ((.NOT.UPPER) .AND. (.NOT.LSAME(UPLO,'L'))) THEN
          INFO = 1
      ELSE IF ((.NOT.LSAME(TRANS,'N')) .AND.
     +         (.NOT.LSAME(TRANS,'C'))) THEN
          INFO = 2
      ELSE IF (N.LT.0) THEN
          INFO = 3
      ELSE IF (K.LT.0) THEN
          INFO = 4
      ELSE IF (LDA.LT.MAX(1,NROWA)) THEN
          INFO = 7
      ELSE IF (LDB.LT.MAX(1,NROWA)) THEN
          INFO = 9
      ELSE IF (LDC.LT.MAX(1,N)) THEN
          INFO = 12
      END IF
      IF (INFO.NE.0) THEN
          CALL XERBLA('ZHER2K',INFO)
          goto 9999
!         RETURN
      END IF
*
*     Quick return if possible.
*
!     IF ((N.EQ.0) .OR. (((ALPHA.EQ.ZERO).OR.
!    +    (K.EQ.0)).AND. (BETA.EQ.ONE))) RETURN
      if((N.EQ.0) .OR. ((
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +  .or. (K.EQ.0)) .and.
!       (beta .eq. one)
     + ((betah .eq. oneh) .and.
     +  (betal .eq. onel))
     +)) goto 9999

*
*     And when  alpha.eq.zero.
*
!     IF (ALPHA.EQ.ZERO) THEN
      if(
!       (alpha .eq. zero)
     + ((alphah .eq. zeroh) .and.
     +  (alphal .eq. zerol))
     +) then
          IF (UPPER) THEN
!             IF (BETA.EQ.DBLE(ZERO)) THEN
              if(
!               (beta .eq. ddzero)
     +         ((betah .eq. ddzeroh) .and.
     +          (betal .eq. ddzerol))
     +        ) then
                  DO 20 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 10 I = 1,J
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   10                 CONTINUE
!$omp end do
!$omp end parallel
   20             CONTINUE
              ELSE
                  DO 40 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                      DO 30 I = 1,J - 1
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
c                         x_ch = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         x_cl = ((x_p1*x_r1
c    +                             - x_ch)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         x_cl = x_cl
c    +                             + (betah*cl(i,j)
c    +                             +  betal*ch(i,j))
c                         x_s1 = x_ch + x_cl
c                         cl(i,j) = x_cl - (x_s1 - x_ch)
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ch(i,j), cl(i,j))
   30                 CONTINUE
!$omp end do
!$omp end parallel
!                     C(J,J) = BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!!!#ddb                 mul c(j,j) beta dda
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
                      ch(j,j) = ddmh
                      cl(j,j) = ddml
   40             CONTINUE
              END IF
          ELSE
!             IF (BETA.EQ.DBLE(ZERO)) THEN
              if(
!               (beta .eq. ddzero)
     +         ((betah .eq. ddzeroh) .and.
     +          (betal .eq. ddzerol))
     +        ) then
                  DO 60 J = 1,N
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 50 I = J,N
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   50                 CONTINUE
!$omp end do
!$omp end parallel
   60             CONTINUE
              ELSE
                  DO 80 J = 1,N
!                     C(J,J) = BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!!!#ddb                 mul c(j,j) beta  dda
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
                      ch(j,j) = ddmh
                      cl(j,j) = ddml
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                      DO 70 I = J + 1,N
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
c                         x_ch = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         x_cl = ((x_p1*x_r1
c    +                             - x_ch)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         x_cl = x_cl
c    +                             + (betah*cl(i,j)
c    +                             +  betal*ch(i,j))
c                         x_s1 = x_ch + x_cl
c                         cl(i,j) = x_cl - (x_s1 - x_ch)
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ch(i,j), cl(i,j))
   70                 CONTINUE
!$omp end do
!$omp end parallel
   80             CONTINUE
              END IF
          END IF
          RETURN
      END IF
*
*     Start the operations.
*
      IF (LSAME(TRANS,'N')) THEN
*
*        Form  C := alpha*A*conjg( B' ) + conjg( alpha )*B*conjg( A' ) +
*                   C.
*
          IF (UPPER) THEN
              DO 130 J = 1,N
!                 IF (BETA.EQ.DBLE(ZERO)) THEN
                  if(
!                   (beta .eq. ddzero)
     +             ((betah .eq. ddzeroh) .and.
     +              (betal .eq. ddzerol))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 90 I = 1,J
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
   90                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  else if(
!                   (beta .ne. one)
     +             ((betah .ne. oneh) .or.
     +              (betal .ne. onel))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                      DO 100 I = 1,J - 1
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
c                         x_ch = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         x_cl = ((x_p1*x_r1
c    +                             - x_ch)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         x_cl = x_cl
c    +                             + (betah*cl(i,j)
c    +                             +  betal*ch(i,j))
c                         x_s1 = x_ch + x_cl
c                         cl(i,j) = x_cl - (x_s1 - x_ch)
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ch(i,j), cl(i,j))
  100                 CONTINUE
!$omp end do
!$omp end parallel
!                     C(J,J) = BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!!!#ddb                 mul c(j,j) beta dda
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
                      ch(j,j) = ddmh
                      cl(j,j) = ddml
                  ELSE
!                     C(J,J) = DBLE(C(J,J))
                      ch(j,j) = dble(ch(j,j))
                      cl(j,j) = dble(cl(j,j))
                  END IF
                  DO 120 L = 1,K
!                     IF ((A(J,L).NE.ZERO) .OR. (B(J,L).NE.ZERO)) THEN
                      if(
!                       (a(j,l) .ne. zero)
     +                 ((ah(j,l) .ne. zeroh) .or.
     +                  (al(j,l) .ne. zerol))
     +                  .or.
!                       (b(j,l) .ne. zero)
     +                 ((bh(j,l) .ne. zeroh) .or.
     +                  (bl(j,l) .ne. zerol))
     +                ) then
!                         TEMP1 = ALPHA*DCONJG(B(J,L))
                          ddaxh = dconjg(bh(j,l))
                          ddaxl = dconjg(bl(j,l))
!<<<                      temp1 = alpha * ddax
c                         temp1h = alphah * ddaxh
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * ddaxh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_r2 = ddaxh - x_r1
c                         temp1l = ((x_p1*x_r1
c    +                            - temp1h)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         temp1l = temp1l
c    +                          + (alphah*ddaxl
c    +                          + alphal*ddaxh)
c                         x_s1 = temp1h + temp1l
c                         temp1l = temp1l - (x_s1 - temp1h)
c                         temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      temp1 = alpha * ddax
                          call bmul(alphah, alphal, ddaxh, ddaxl,
     +                        temp1h, temp1l)
!                         TEMP2 = DCONJG(ALPHA*A(J,L))
!<<<                      ddmx = alpha * a(j,l)
c                         ddmxh = alphah * ah(j,l)
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * ah(j,l)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(j,l))
c                         x_r2 = ah(j,l) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (alphah*al(j,l)
c    +                          + alphal*ah(j,l))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = alpha * a(j,l)
                          call bmul(alphah, alphal, ah(j,l), al(j,l),
     +                        ddmxh, ddmxl)
                          temp2h = dconjg(ddmxh)
                          temp2l = dconjg(ddmxl)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                          DO 110 I = 1,J - 1
!                             C(I,J) = C(I,J) + A(I,L)*TEMP1 +
!    +                                 B(I,L)*TEMP2
!<<<                          ddax = b(i,l) * temp2
c                             ddaxh = bh(i,l) * temp2h
c                             x_u1 = 134217729.0 * bh(i,l)
c                             x_p1 = x_u1
c    +                                 - (x_u1 - bh(i,l))
c                             x_p2 = bh(i,l) - x_p1
c                             x_u1 = 134217729.0 * temp2h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp2h)
c                             x_r2 = temp2h - x_r1
c                             ddaxl = ((x_p1*x_r1
c    +                                - ddaxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddaxl = ddaxl
c    +                              + (bh(i,l)*temp2l
c    +                              + bl(i,l)*temp2h)
c                             x_s1 = ddaxh + ddaxl
c                             ddaxl = ddaxl - (x_s1 - ddaxh)
c                             ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddax = b(i,l) * temp2
                              call bmul(bh(i,l), bl(i,l),
     +                            temp2h, temp2l, ddaxh, ddaxl)

                               
!<<<                          ddmx = a(i,l) * temp1
c                             ddmxh = ah(i,l) * temp1h
c                             x_u1 = 134217729.0 * ah(i,l)
c                             x_p1 = x_u1
c    +                                 - (x_u1 - ah(i,l))
c                             x_p2 = ah(i,l) - x_p1
c                             x_u1 = 134217729.0 * temp1h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp1h)
c                             x_r2 = temp1h - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (ah(i,l)*temp1l
c    +                              + al(i,l)*temp1h)
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = a(i,l) * temp1
                              call bmul(ah(i,l), al(i,l),
     +                            temp1h, temp1l, ddmxh, ddmxl)
!<<<                          ddmx = ddmx + ddax
c                             x_p1 = ddmxh + ddaxh
c                             x_p2 = x_p1 - ddmxh
c                             x_q1 = x_p1
c                             x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                                 + (ddaxh - x_p2)
c                             x_q2 = x_q2 + ddmxl + ddaxl
c                             ddmxh = x_q1 + x_q2
c                             ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddmx = ddmx + ddax
                              call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                            ddmxh, ddmxl)
!<<<                          c(i,j) = c(i,j) + ddmx
c                             x_p1 = ch(i,j) + ddmxh
c                             x_p2 = x_p1 - ch(i,j)
c                             x_q1 = x_p1
c                             x_q2 = (ch(i,j) - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + cl(i,j) + ddmxl
c                             ch(i,j) = x_q1 + x_q2
c                             cl(i,j) = x_q2 - (ch(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          c(i,j) = c(i,j) + ddmx
                              call bsum(ch(i,j), cl(i,j),
     +                            ddmxh, ddmxl, ch(i,j), cl(i,j))
  110                     CONTINUE
!$omp end do
!$omp end parallel
!                         C(J,J) = DBLE(C(J,J)) +
!    +                             DBLE(A(J,L)*TEMP1+B(J,L)*TEMP2)
!<<<                      ddax = b(j,l) * temp2
c                         ddaxh = bh(j,l) * temp2h
c                         x_u1 = 134217729.0 * bh(j,l)
c                         x_p1 = x_u1
c    +                             - (x_u1 - bh(j,l))
c                         x_p2 = bh(j,l) - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (bh(j,l)*temp2l
c    +                          + bl(j,l)*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = b(j,l) * temp2
                          call bmul(bh(j,l), bl(j,l), temp2h, temp2l,
     +                        ddaxh, ddaxl)
!<<<                      ddmx = a(j,l) * temp1
c                         ddmxh = ah(j,l) * temp1h
c                         x_u1 = 134217729.0 * ah(j,l)
c                         x_p1 = x_u1
c    +                             - (x_u1 - ah(j,l))
c                         x_p2 = ah(j,l) - x_p1
c                         x_u1 = 134217729.0 * temp1h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_r2 = temp1h - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ah(j,l)*temp1l
c    +                          + al(j,l)*temp1h)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = a(j,l) * temp1
                          call bmul(ah(j,l), al(j,l), temp1h, temp1l,
     +                        ddmxh, ddmxl)
!<<<                      ddmx = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         x_q1 = x_p1
c                         x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                             + (ddaxh - x_p2)
c                         x_q2 = x_q2 + ddmxl + ddaxl
c                         ddmxh = x_q1 + x_q2
c                         ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddmx = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
                          ddmh = dble(ddmxh)
                          ddml = dble(ddmxl)
                          ddah = dble(ch(j,j))
                          ddal = dble(cl(j,j))
!!!#ddb                     add c(j,j) dda ddm
!<<<                      dda = dda + ddm
c                         t_p1 = ddah + ddmh
c                         t_p2 = t_p1 - ddah
c                         t_q1 = t_p1
c                         t_q2 = (ddah - (t_p1 - t_p2))
c    +                             + (ddmh - t_p2)
c                         t_q2 = t_q2 + ddal + ddml
c                         ddah = t_q1 + t_q2
c                         ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      dda = dda + ddm
                          call bsum(ddah, ddal, ddmh, ddml,
     +                        ddah, ddal)
                          ch(j,j) = ddah
                          cl(j,j) = ddal
                      END IF
  120             CONTINUE
  130         CONTINUE
          ELSE
              DO 180 J = 1,N
!                 IF (BETA.EQ.DBLE(ZERO)) THEN
                  if(
!                   (BETA .eq. ddzero)
     +             ((BETAh .eq. ddzeroh) .and.
     +              (BETAl .eq. ddzerol))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
                      DO 140 I = J,N
!                         C(I,J) = ZERO
                          ch(i,j) = zeroh
                          cl(i,j) = zerol
  140                 CONTINUE
!$omp end do
!$omp end parallel
!                 ELSE IF (BETA.NE.ONE) THEN
                  else if(
!                   (beta .ne. one)
     +             ((betah .ne. oneh) .or.
     +              (betal .ne. onel))
     +            ) then
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
                      DO 150 I = J + 1,N
!                         C(I,J) = BETA*C(I,J)
!<<<                      c(i,j) = beta * c(i,j)
c                         x_ch = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         x_cl = ((x_p1*x_r1
c    +                             - x_ch)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         x_cl = x_cl
c    +                             + (betah*cl(i,j)
c    +                             +  betal*ch(i,j))
c                         x_s1 = x_ch + x_cl
c                         cl(i,j) = x_cl - (x_s1 - x_ch)
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>                      c(i,j) = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ch(i,j), cl(i,j))
  150                 CONTINUE
!$omp end do
!$omp end parallel
!                     C(J,J) = BETA*DBLE(C(J,J))
                      ddah = dble(ch(j,j))
                      ddal = dble(cl(j,j))
!!!#ddb                 mul c(j,j) beta dda
!<<<                  ddm = beta * dda
c                     ddmh = betah * ddah
c                     t_u1 = 134217729.0 * betah
c                     t_p1 = t_u1
c    +                         - (t_u1 - betah)
c                     t_p2 = betah - t_p1
c                     t_u1 = 134217729.0 * ddah
c                     t_r1 = t_u1
c    +                         - (t_u1 - ddah)
c                     t_r2 = ddah - t_r1
c                     ddml = ((t_p1*t_r1
c    +                        - ddmh)
c    +                        + t_p1 * t_r2
c    +                        + t_p2 * t_r1)
c    +                        + t_p2 * t_r2
c                     ddml = ddml
c    +                      + (betah*ddal
c    +                      + betal*ddah)
c                     t_s1 = ddmh + ddml
c                     ddml = ddml - (t_s1 - ddmh)
c                     ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                  ddm = beta * dda
                      call bmul(betah, betal, ddah, ddal,
     +                    ddmh, ddml)
                      ch(j,j) = ddmh
                      cl(j,j) = ddml
                  ELSE
!                     C(J,J) = DBLE(C(J,J))
                      ch(j,j) = dble(ch(j,j))
                      cl(j,j) = dble(cl(j,j))
                  END IF
                  DO 170 L = 1,K
!                     IF ((A(J,L).NE.ZERO) .OR. (B(J,L).NE.ZERO)) THEN
                      if(
!                       (a(j,l) .ne. zero)
     +                 ((ah(j,l) .ne. zeroh) .or.
     +                  (al(j,l) .ne. zerol))
     +                  .or.
!                       (b(j,l) .ne. zero)
     +                 ((bh(j,l) .ne. zeroh) .or.
     +                  (bl(j,l) .ne. zerol))
     +                ) then
!                         TEMP1 = ALPHA*DCONJG(B(J,L))
                          ddaxh = dconjg(bh(j,l))
                          ddaxl = dconjg(bl(j,l))
!<<<                      temp1 = alpha * ddax
c                         temp1h = alphah * ddaxh
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * ddaxh
c                         x_r1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_r2 = ddaxh - x_r1
c                         temp1l = ((x_p1*x_r1
c    +                            - temp1h)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         temp1l = temp1l
c    +                          + (alphah*ddaxl
c    +                          + alphal*ddaxh)
c                         x_s1 = temp1h + temp1l
c                         temp1l = temp1l - (x_s1 - temp1h)
c                         temp1h = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      temp1 = alpha * ddax
                          call bmul(alphah, alphal, ddaxh, ddaxl,
     +                        temp1h, temp1l)
!                         TEMP2 = DCONJG(ALPHA*A(J,L))
!<<<                      ddmx = alpha * a(j,l)
c                         ddmxh = alphah * ah(j,l)
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * ah(j,l)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(j,l))
c                         x_r2 = ah(j,l) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (alphah*al(j,l)
c    +                          + alphal*ah(j,l))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = alpha * a(j,l)
                          call bmul(alphah, alphal, ah(j,l), al(j,l),
     +                        ddmxh, ddmxl)
                          temp2h = dconjg(ddmxh)
                          temp2l = dconjg(ddmxl)
!$omp parallel
!$omp do
!$omp+ private(I)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                          DO 160 I = J + 1,N
!                             C(I,J) = C(I,J) + A(I,L)*TEMP1 +
!    +                                 B(I,L)*TEMP2
!<<<                          ddax = b(i,l) * temp2
c                             ddaxh = bh(i,l) * temp2h
c                             x_u1 = 134217729.0 * bh(i,l)
c                             x_p1 = x_u1
c    +                                 - (x_u1 - bh(i,l))
c                             x_p2 = bh(i,l) - x_p1
c                             x_u1 = 134217729.0 * temp2h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp2h)
c                             x_r2 = temp2h - x_r1
c                             ddaxl = ((x_p1*x_r1
c    +                                - ddaxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddaxl = ddaxl
c    +                              + (bh(i,l)*temp2l
c    +                              + bl(i,l)*temp2h)
c                             x_s1 = ddaxh + ddaxl
c                             ddaxl = ddaxl - (x_s1 - ddaxh)
c                             ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddax = b(i,l) * temp2
                              call bmul(bh(i,l), bl(i,l),
     +                            temp2h, temp2l, ddaxh, ddaxl)
!<<<                          ddmx = a(i,l) * temp1
c                             ddmxh = ah(i,l) * temp1h
c                             x_u1 = 134217729.0 * ah(i,l)
c                             x_p1 = x_u1
c    +                                 - (x_u1 - ah(i,l))
c                             x_p2 = ah(i,l) - x_p1
c                             x_u1 = 134217729.0 * temp1h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp1h)
c                             x_r2 = temp1h - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (ah(i,l)*temp1l
c    +                              + al(i,l)*temp1h)
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = a(i,l) * temp1
                              call bmul(ah(i,l), al(i,l),
     +                            temp1h, temp1l, ddmxh, ddmxl)
!<<<                          ddmx = ddmx + ddax
c                             x_p1 = ddmxh + ddaxh
c                             x_p2 = x_p1 - ddmxh
c                             x_q1 = x_p1
c                             x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                                 + (ddaxh - x_p2)
c                             x_q2 = x_q2 + ddmxl + ddaxl
c                             ddmxh = x_q1 + x_q2
c                             ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddmx = ddmx + ddax
                              call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                            ddmxh, ddmxl)
!<<<                          c(i,j) = c(i,j) + ddmx
c                             x_p1 = ch(i,j) + ddmxh
c                             x_p2 = x_p1 - ch(i,j)
c                             x_q1 = x_p1
c                             x_q2 = (ch(i,j) - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + cl(i,j) + ddmxl
c                             ch(i,j) = x_q1 + x_q2
c                             cl(i,j) = x_q2 - (ch(i,j) - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          c(i,j) = c(i,j) + ddmx
                              call bsum(ch(i,j), cl(i,j),
     +                            ddmxh, ddmxl, ch(i,j), cl(i,j))
  160                     CONTINUE
!$omp end do
!$omp end parallel
!                         C(J,J) = DBLE(C(J,J)) +
!    +                             DBLE(A(J,L)*TEMP1+B(J,L)*TEMP2)
!<<<                      ddax = b(j,l) * temp2
c                         ddaxh = bh(j,l) * temp2h
c                         x_u1 = 134217729.0 * bh(j,l)
c                         x_p1 = x_u1
c    +                             - (x_u1 - bh(j,l))
c                         x_p2 = bh(j,l) - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddaxl = ((x_p1*x_r1
c    +                            - ddaxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddaxl = ddaxl
c    +                          + (bh(j,l)*temp2l
c    +                          + bl(j,l)*temp2h)
c                         x_s1 = ddaxh + ddaxl
c                         ddaxl = ddaxl - (x_s1 - ddaxh)
c                         ddaxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddax = b(j,l) * temp2
                          call bmul(bh(j,l), bl(j,l), temp2h, temp2l,
     +                        ddaxh, ddaxl)
!<<<                      ddmx = a(j,l) * temp1
c                         ddmxh = ah(j,l) * temp1h
c                         x_u1 = 134217729.0 * ah(j,l)
c                         x_p1 = x_u1
c    +                             - (x_u1 - ah(j,l))
c                         x_p2 = ah(j,l) - x_p1
c                         x_u1 = 134217729.0 * temp1h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_r2 = temp1h - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ah(j,l)*temp1l
c    +                          + al(j,l)*temp1h)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = a(j,l) * temp1
                          call bmul(ah(j,l), al(j,l), temp1h, temp1l,
     +                        ddmxh, ddmxl)
!<<<                      ddmx = ddmx + ddax
c                         x_p1 = ddmxh + ddaxh
c                         x_p2 = x_p1 - ddmxh
c                         x_q1 = x_p1
c                         x_q2 = (ddmxh - (x_p1 - x_p2))
c    +                             + (ddaxh - x_p2)
c                         x_q2 = x_q2 + ddmxl + ddaxl
c                         ddmxh = x_q1 + x_q2
c                         ddmxl = x_q2 - (ddmxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddmx = ddmx + ddax
                          call bsum(ddmxh, ddmxl, ddaxh, ddaxl,
     +                        ddmxh, ddmxl)
                          ddmh = dble(ddmxh)
                          ddml = dble(ddmxl)
                          ddah = dble(ch(j,j))
                          ddal = dble(cl(j,j))
!!!#ddb                     add c(j,j) dda ddm
!<<<                      dda = dda + ddm
c                         t_p1 = ddah + ddmh
c                         t_p2 = t_p1 - ddah
c                         t_q1 = t_p1
c                         t_q2 = (ddah - (t_p1 - t_p2))
c    +                             + (ddmh - t_p2)
c                         t_q2 = t_q2 + ddal + ddml
c                         ddah = t_q1 + t_q2
c                         ddal = t_q2 - (ddah - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      dda = dda + ddm
                          call bsum(ddah, ddal, ddmh, ddml,
     +                        ddah, ddal)
                          ch(j,j) = ddah
                          cl(j,j) = ddal
                      END IF
  170             CONTINUE
  180         CONTINUE
          END IF
      ELSE
*
*        Form  C := alpha*conjg( A' )*B + conjg( alpha )*conjg( B' )*A +
*                   C.
*
          IF (UPPER) THEN
              DO 210 J = 1,N
                  DO 200 I = 1,J
!                     TEMP1 = ZERO
                      temp1h = zeroh
                      temp1l = zerol
!                     TEMP2 = ZERO
                      temp2h = zeroh
                      temp2l = zerol
!$omp parallel
!$omp do
!$omp+ private(L)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 190 L = 1,K
!                         TEMP1 = TEMP1 + DCONJG(A(L,I))*B(L,J)
                          ddaxh = dconjg(ah(l,i))
                          ddaxl = dconjg(al(l,i))
!<<<                      ddmx = ddax * b(l,j)
c                         ddmxh = ddaxh * bh(l,j)
c                         x_u1 = 134217729.0 * ddaxh
c                         x_p1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_p2 = ddaxh - x_p1
c                         x_u1 = 134217729.0 * bh(l,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - bh(l,j))
c                         x_r2 = bh(l,j) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ddaxh*bl(l,j)
c    +                          + ddaxl*bh(l,j))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = ddax * b(l,j)
                          call bmul(ddaxh, ddaxl, bh(l,j), bl(l,j),
     +                        ddmxh, ddmxl)
!$omp critical
!<<<                      temp1 = temp1 + ddmx
c                         x_p1 = temp1h + ddmxh
c                         x_p2 = x_p1 - temp1h
c                         x_q1 = x_p1
c                         x_q2 = (temp1h - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + temp1l + ddmxl
c                         temp1h = x_q1 + x_q2
c                         temp1l = x_q2 - (temp1h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp1 = temp1 + ddmx
                          call bsum(temp1h, temp1l, ddmxh, ddmxl,
     +                        temp1h, temp1l)
!$omp end critical
!                         TEMP2 = TEMP2 + DCONJG(B(L,I))*A(L,J)
                          ddaxh = dconjg(bh(l,i))
                          ddaxl = dconjg(bl(l,i))
!<<<                      ddmx = ddax * a(l,j)
c                         ddmxh = ddaxh * ah(l,j)
c                         x_u1 = 134217729.0 * ddaxh
c                         x_p1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_p2 = ddaxh - x_p1
c                         x_u1 = 134217729.0 * ah(l,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(l,j))
c                         x_r2 = ah(l,j) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ddaxh*al(l,j)
c    +                          + ddaxl*ah(l,j))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = ddax * a(l,j)
                          call bmul(ddaxh, ddaxl, ah(l,j), al(l,j),
     +                        ddmxh, ddmxl)
!$omp critical
!<<<                      temp2 = temp2 + ddmx
c                         x_p1 = temp2h + ddmxh
c                         x_p2 = x_p1 - temp2h
c                         x_q1 = x_p1
c                         x_q2 = (temp2h - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + temp2l + ddmxl
c                         temp2h = x_q1 + x_q2
c                         temp2l = x_q2 - (temp2h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp2 = temp2 + ddmx
                          call bsum(temp2h, temp2l, ddmxh, ddmxl,
     +                        temp2h, temp2l)
!$omp end critical
  190                 CONTINUE
!$omp end do
!$omp end parallel
                      IF (I.EQ.J) THEN
!                         IF (BETA.EQ.DBLE(ZERO)) THEN
                          if(
!                           (beta .eq. ddzero)
     +                     ((betah .eq. ddzeroh) .and.
     +                      (betal .eq. ddzerol))
     +                    ) then
!                             C(J,J) = DBLE(ALPHA*TEMP1+
!    +                                 DCONJG(ALPHA)*TEMP2)
                              ddaxh = dconjg(alphah)
                              ddaxl = dconjg(alphal)
!<<<                          ddmx = ddax * temp2
c                             ddmxh = ddaxh * temp2h
c                             x_u1 = 134217729.0 * ddaxh
c                             x_p1 = x_u1
c    +                                 - (x_u1 - ddaxh)
c                             x_p2 = ddaxh - x_p1
c                             x_u1 = 134217729.0 * temp2h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp2h)
c                             x_r2 = temp2h - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (ddaxh*temp2l
c    +                              + ddaxl*temp2h)
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = ddax * temp2
                              call bmul(ddaxh, ddaxl, temp2h, temp2l,
     +                            ddmxh, ddmxl)
!<<<                          ddxx = alpha * temp1
c                             ddxxh = alphah * temp1h
c                             x_u1 = 134217729.0 * alphah
c                             x_p1 = x_u1
c    +                                 - (x_u1 - alphah)
c                             x_p2 = alphah - x_p1
c                             x_u1 = 134217729.0 * temp1h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp1h)
c                             x_r2 = temp1h - x_r1
c                             ddxxl = ((x_p1*x_r1
c    +                                - ddxxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddxxl = ddxxl
c    +                              + (alphah*temp1l
c    +                              + alphal*temp1h)
c                             x_s1 = ddxxh + ddxxl
c                             ddxxl = ddxxl - (x_s1 - ddxxh)
c                             ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddxx = alpha * temp1
                              call bmul(alphah, alphal, temp1h, temp1l,
     +                            ddxxh, ddxxl)
!<<<                          ddxx = ddxx + ddmx
c                             x_p1 = ddxxh + ddmxh
c                             x_p2 = x_p1 - ddxxh
c                             x_q1 = x_p1
c                             x_q2 = (ddxxh - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + ddxxl + ddmxl
c                             ddxxh = x_q1 + x_q2
c                             ddxxl = x_q2 - (ddxxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddxx = ddxx + ddmx
                              call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                            ddxxh, ddxxl)
                              ch(j,j) = dble(ddxxh)
                              cl(j,j) = dble(ddxxl)
                          ELSE
!                             C(J,J) = BETA*DBLE(C(J,J)) +
!    +                                 DBLE(ALPHA*TEMP1+
!    +                                 DCONJG(ALPHA)*TEMP2)
                                       ddaxh = dconjg(alphah)
                                       ddaxl = dconjg(alphal)
!<<<                          ddmx = ddax * temp2
c                             ddmxh = ddaxh * temp2h
c                             x_u1 = 134217729.0 * ddaxh
c                             x_p1 = x_u1
c    +                                 - (x_u1 - ddaxh)
c                             x_p2 = ddaxh - x_p1
c                             x_u1 = 134217729.0 * temp2h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp2h)
c                             x_r2 = temp2h - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (ddaxh*temp2l
c    +                              + ddaxl*temp2h)
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = ddax * temp2
                              call bmul(ddaxh, ddaxl, temp2h, temp2l,
     +                            ddmxh, ddmxl)
!<<<                          ddxx = alpha * temp1
c                             ddxxh = alphah * temp1h
c                             x_u1 = 134217729.0 * alphah
c                             x_p1 = x_u1
c    +                                 - (x_u1 - alphah)
c                             x_p2 = alphah - x_p1
c                             x_u1 = 134217729.0 * temp1h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp1h)
c                             x_r2 = temp1h - x_r1
c                             ddxxl = ((x_p1*x_r1
c    +                                - ddxxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddxxl = ddxxl
c    +                              + (alphah*temp1l
c    +                              + alphal*temp1h)
c                             x_s1 = ddxxh + ddxxl
c                             ddxxl = ddxxl - (x_s1 - ddxxh)
c                             ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddxx = alpha * temp1
                              call bmul(alphah, alphal, temp1h, temp1l,
     +                            ddxxh, ddxxl)
!<<<                          ddxx = ddxx + ddmx
c                             x_p1 = ddxxh + ddmxh
c                             x_p2 = x_p1 - ddxxh
c                             x_q1 = x_p1
c                             x_q2 = (ddxxh - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + ddxxl + ddmxl
c                             ddxxh = x_q1 + x_q2
c                             ddxxl = x_q2 - (ddxxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddxx = ddxx + ddmx
                              call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                            ddxxh, ddxxl)
                                       ddah = dble(ddxxh)
                                       ddal = dble(ddxxl)
                                       ddmh = dble(ch(j,j))
                                       ddml = dble(cl(j,j))
!<<<                          ddx = beta * ddm
c                             ddxh = betah * ddmh
c                             t_u1 = 134217729.0 * betah
c                             t_p1 = t_u1
c    +                                 - (t_u1 - betah)
c                             t_p2 = betah - t_p1
c                             t_u1 = 134217729.0 * ddmh
c                             t_r1 = t_u1
c    +                                 - (t_u1 - ddmh)
c                             t_r2 = ddmh - t_r1
c                             ddxl = ((t_p1*t_r1
c    +                                - ddxh)
c    +                                + t_p1 * t_r2
c    +                                + t_p2 * t_r1)
c    +                                + t_p2 * t_r2
c                             ddxl = ddxl
c    +                              + (betah*ddml
c    +                              + betal*ddmh)
c                             t_s1 = ddxh + ddxl
c                             ddxl = ddxl - (t_s1 - ddxh)
c                             ddxh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddx = beta * ddm
                              call bmul(betah, betal, ddmh, ddml,
     +                            ddxh, ddxl)
!!!#ddb                                  add c(j,j) ddx dda
!<<<                          ddx = ddx + dda
c                             t_p1 = ddxh + ddah
c                             t_p2 = t_p1 - ddxh
c                             t_q1 = t_p1
c                             t_q2 = (ddxh - (t_p1 - t_p2))
c    +                                 + (ddah - t_p2)
c                             t_q2 = t_q2 + ddxl + ddal
c                             ddxh = t_q1 + t_q2
c                             ddxl = t_q2 - (ddxh - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddx = ddx + dda
                              ch(j,j) = ddxh
                              cl(j,j) = ddxl
!!!!!!                      dda == DBLE(ALPHA*TEMP1+DCONJG(ALPHA)*TEMP2)
!!!!!!                      ddx == BETA*DBLE(C(J,J))
                          END IF
                      ELSE
!                         IF (BETA.EQ.DBLE(ZERO)) THEN
                          if(
!                           (BETA .eq. ddzero)
     +                     ((BETAh .eq. ddzeroh) .and.
     +                      (BETAl .eq. ddzerol))
     +                    ) then
!                             C(I,J) = ALPHA*TEMP1 + DCONJG(ALPHA)*TEMP2
                              ddaxh = dconjg(alphah)
                              ddaxl = dconjg(alphal)
!<<<                          ddmx = ddax * TEMP2
c                             ddmxh = ddaxh * TEMP2h
c                             x_u1 = 134217729.0 * ddaxh
c                             x_p1 = x_u1
c    +                                 - (x_u1 - ddaxh)
c                             x_p2 = ddaxh - x_p1
c                             x_u1 = 134217729.0 * TEMP2h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - TEMP2h)
c                             x_r2 = TEMP2h - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (ddaxh*TEMP2l
c    +                              + ddaxl*TEMP2h)
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = ddax * TEMP2
                              call bmul(ddaxh, ddaxl, temp2h, temp2l,
     +                            ddmxh, ddmxl)
!<<<                          ddxx = alpha * temp1
c                             ddxxh = alphah * temp1h
c                             x_u1 = 134217729.0 * alphah
c                             x_p1 = x_u1
c    +                                 - (x_u1 - alphah)
c                             x_p2 = alphah - x_p1
c                             x_u1 = 134217729.0 * temp1h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp1h)
c                             x_r2 = temp1h - x_r1
c                             ddxxl = ((x_p1*x_r1
c    +                                - ddxxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddxxl = ddxxl
c    +                              + (alphah*temp1l
c    +                              + alphal*temp1h)
c                             x_s1 = ddxxh + ddxxl
c                             ddxxl = ddxxl - (x_s1 - ddxxh)
c                             ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddxx = alpha * temp1
                              call bmul(alphah, alphal, temp1h, temp1l,
     +                            ddxxh, ddxxl)
!<<<                          c(i,j) = ddxx + ddmx
c                             x_p1 = ddxxh + ddmxh
c                             x_p2 = x_p1 - ddxxh
c                             ch(i,j) = x_p1
c                             cl(i,j) = (ddxxh - (x_p1 - x_p2))
c    +                              + (ddmxh - x_p2)
c                             cl(i,j) = cl(i,j) + ddxxl + ddmxl
c                             x_s1 = ch(i,j) + cl(i,j)
c                             cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                             ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                          c(i,j) = ddxx + ddmx
                              call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                            ch(i,j), cl(i,j))
                          ELSE
!                             C(I,J) = BETA*C(I,J) + ALPHA*TEMP1 +
!    +                                 DCONJG(ALPHA)*TEMP2
                              ddaxh = dconjg(alphah)
                              ddaxl = dconjg(alphal)
!<<<                          ddmx = ddax * temp2
c                             ddmxh = ddaxh * temp2h
c                             x_u1 = 134217729.0 * ddaxh
c                             x_p1 = x_u1
c    +                                 - (x_u1 - ddaxh)
c                             x_p2 = ddaxh - x_p1
c                             x_u1 = 134217729.0 * temp2h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp2h)
c                             x_r2 = temp2h - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (ddaxh*temp2l
c    +                              + ddaxl*temp2h)
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = ddax * temp2
                              call bmul(ddaxh, ddaxl, temp2h, temp2l,
     +                            ddmxh, ddmxl)
!<<<                          ddxx = alpha * temp1
c                             ddxxh = alphah * temp1h
c                             x_u1 = 134217729.0 * alphah
c                             x_p1 = x_u1
c    +                                 - (x_u1 - alphah)
c                             x_p2 = alphah - x_p1
c                             x_u1 = 134217729.0 * temp1h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp1h)
c                             x_r2 = temp1h - x_r1
c                             ddxxl = ((x_p1*x_r1
c    +                                - ddxxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddxxl = ddxxl
c    +                              + (alphah*temp1l
c    +                              + alphal*temp1h)
c                             x_s1 = ddxxh + ddxxl
c                             ddxxl = ddxxl - (x_s1 - ddxxh)
c                             ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddxx = alpha * temp1
                              call bmul(alphah, alphal, temp1h, temp1l,
     +                            ddxxh, ddxxl)
!<<<                          ddbx = beta * c(i,j)
c                             ddbxh = betah * ch(i,j)
c                             x_u1 = 134217729.0 * betah
c                             x_p1 = x_u1
c    +                                 - (x_u1 - betah)
c                             x_p2 = betah - x_p1
c                             x_u1 = 134217729.0 * ch(i,j)
c                             x_r1 = x_u1
c    +                                 - (x_u1 - ch(i,j))
c                             x_r2 = ch(i,j) - x_r1
c                             ddbxl = ((x_p1*x_r1
c    +                                - ddbxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddbxl = ddbxl
c    +                              + (betah*cl(i,j)
c    +                              + betal*ch(i,j))
c                             x_s1 = ddbxh + ddbxl
c                             ddbxl = ddbxl - (x_s1 - ddbxh)
c                             ddbxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddbx = beta * c(i,j)
                              call bmul(betah, betal, ch(i,j), cl(i,j),
     +                           ddbxh, ddbxl)
!<<<                          ddxx = ddxx + ddmx
c                             x_p1 = ddxxh + ddmxh
c                             x_p2 = x_p1 - ddxxh
c                             x_q1 = x_p1
c                             x_q2 = (ddxxh - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + ddxxl + ddmxl
c                             ddxxh = x_q1 + x_q2
c                             ddxxl = x_q2 - (ddxxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddxx = ddxx + ddmx
                              call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                            ddxxh, ddxxl)
!<<<                          c(i,j) = ddbx + ddxx
c                             x_p1 = ddbxh + ddxxh
c                             x_p2 = x_p1 - ddbxh
c                             ch(i,j) = x_p1
c                             cl(i,j) = (ddbxh - (x_p1 - x_p2))
c    +                              + (ddxxh - x_p2)
c                             cl(i,j) = cl(i,j) + ddbxl + ddxxl
c                             x_s1 = ch(i,j) + cl(i,j)
c                             cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                             ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                          c(i,j) = ddbx + ddxx
                              call bsum(ddbxh, ddbxl, ddxxh, ddxxl,
     +                            ch(i,j), cl(i,j))
                          END IF
                      END IF
  200             CONTINUE
  210         CONTINUE
          ELSE
              DO 240 J = 1,N
                  DO 230 I = J,N
!                     TEMP1 = ZERO
                      temp1h = zeroh
                      temp1l = zerol
!                     TEMP2 = ZERO
                      temp2h = zeroh
                      temp2l = zerol
!$omp parallel
!$omp do
!$omp+ private(L)
!     tmp vars -> private
!$omp+ private(x_ch, x_cl)
!$omp+ private(x_p1, x_p2)
!$omp+ private(x_q1, x_q2)
!$omp+ private(x_r1, x_r2)
!$omp+ private(x_s1, x_s2)
!$omp+ private(x_t1, x_t2)
!$omp+ private(x_u1, x_u2)
!$omp+ private(x_x1, x_x2)
!$omp+ private(x_y1, x_y2)
!$omp+ private(x_z1, x_z2)
!$omp+ private(x_x, x_ax)
!$omp+ private(x_t1h, x_t1l)
!$omp+ private(x_t2h, x_t2l)
!$omp+ private(ddmxh, ddmxl)
!$omp+ private(ddaxh, ddaxl)
                      DO 220 L = 1,K
!                         TEMP1 = TEMP1 + DCONJG(A(L,I))*B(L,J)
                          ddaxh = dconjg(ah(l,i))
                          ddaxl = dconjg(al(l,i))
!<<<                      ddmx = ddax * b(l,j)
c                         ddmxh = ddaxh * bh(l,j)
c                         x_u1 = 134217729.0 * ddaxh
c                         x_p1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_p2 = ddaxh - x_p1
c                         x_u1 = 134217729.0 * bh(l,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - bh(l,j))
c                         x_r2 = bh(l,j) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ddaxh*bl(l,j)
c    +                          + ddaxl*bh(l,j))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = ddax * b(l,j)
                          call bmul(ddaxh, ddaxl, bh(l,j), bl(l,j),
     +                        ddmxh, ddmxl)
!$omp critical
!<<<                      temp1 = temp1 + ddmx
c                         x_p1 = temp1h + ddmxh
c                         x_p2 = x_p1 - temp1h
c                         x_q1 = x_p1
c                         x_q2 = (temp1h - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + temp1l + ddmxl
c                         temp1h = x_q1 + x_q2
c                         temp1l = x_q2 - (temp1h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp1 = temp1 + ddmx
                          call bsum(temp1h, temp1l, ddmxh, ddmxl,
     +                        temp1h, temp1l)
!$omp end critical
!                         TEMP2 = TEMP2 + DCONJG(B(L,I))*A(L,J)
                          ddaxh = dconjg(bh(l,i))
                          ddaxl = dconjg(bl(l,i))
!<<<                      ddmx = ddax * a(l,j)
c                         ddmxh = ddaxh * ah(l,j)
c                         x_u1 = 134217729.0 * ddaxh
c                         x_p1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_p2 = ddaxh - x_p1
c                         x_u1 = 134217729.0 * ah(l,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ah(l,j))
c                         x_r2 = ah(l,j) - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ddaxh*al(l,j)
c    +                          + ddaxl*ah(l,j))
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = ddax * a(l,j)
                          call bmul(ddaxh, ddaxl, ah(l,j), al(l,j),
     +                        ddmxh, ddmxl)
!$omp critical
!<<<                      temp2 = temp2 + ddmx
c                         x_p1 = temp2h + ddmxh
c                         x_p2 = x_p1 - temp2h
c                         x_q1 = x_p1
c                         x_q2 = (temp2h - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + temp2l + ddmxl
c                         temp2h = x_q1 + x_q2
c                         temp2l = x_q2 - (temp2h - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      temp2 = temp2 + ddmx
                          call bsum(temp2h, temp2l, ddmxh, ddmxl,
     +                        temp2h, temp2l)
!$omp end critical
  220                 CONTINUE
!$omp end do
!$omp end parallel
                      IF (I.EQ.J) THEN
!                         IF (BETA.EQ.DBLE(ZERO)) THEN
                          if(
!                           (BETA .eq. ddzero)
     +                     ((BETAh .eq. ddzeroh) .and.
     +                      (BETAl .eq. ddzerol))
     +                    ) then
!                             C(J,J) = DBLE(ALPHA*TEMP1+
!    +                                 DCONJG(ALPHA)*TEMP2)
                              ddaxh = dconjg(alphah)
                              ddaxl = dconjg(alphal)
!<<<                          ddmx = ddax * temp2
c                             ddmxh = ddaxh * temp2h
c                             x_u1 = 134217729.0 * ddaxh
c                             x_p1 = x_u1
c    +                                 - (x_u1 - ddaxh)
c                             x_p2 = ddaxh - x_p1
c                             x_u1 = 134217729.0 * temp2h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp2h)
c                             x_r2 = temp2h - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (ddaxh*temp2l
c    +                              + ddaxl*temp2h)
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = ddax * temp2
                              call bmul(ddaxh, ddaxl, temp2h, temp2l,
     +                            ddmxh, ddmxl)
!<<<                          ddxx = alpha * temp1
c                             ddxxh = alphah * temp1h
c                             x_u1 = 134217729.0 * alphah
c                             x_p1 = x_u1
c    +                                 - (x_u1 - alphah)
c                             x_p2 = alphah - x_p1
c                             x_u1 = 134217729.0 * temp1h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp1h)
c                             x_r2 = temp1h - x_r1
c                             ddxxl = ((x_p1*x_r1
c    +                                - ddxxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddxxl = ddxxl
c    +                              + (alphah*temp1l
c    +                              + alphal*temp1h)
c                             x_s1 = ddxxh + ddxxl
c                             ddxxl = ddxxl - (x_s1 - ddxxh)
c                             ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddxx = alpha * temp1
                              call bmul(alphah, alphal, temp1h, temp1l,
     +                            ddxxh, ddxxl)
!<<<                          ddxx = ddxx + ddmx
c                             x_p1 = ddxxh + ddmxh
c                             x_p2 = x_p1 - ddxxh
c                             x_q1 = x_p1
c                             x_q2 = (ddxxh - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + ddxxl + ddmxl
c                             ddxxh = x_q1 + x_q2
c                             ddxxl = x_q2 - (ddxxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddxx = ddxx + ddmx
                              call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                            ddxxh, ddxxl)
                              ch(j,j) = dble(ddxxh)
                              cl(j,j)  = dble(ddxxl)
                          ELSE
!                             C(J,J) = BETA*DBLE(C(J,J)) +
!    +                                 DBLE(ALPHA*TEMP1+
!    +                                 DCONJG(ALPHA)*TEMP2)
                              ddaxh = dconjg(alphah)
                              ddaxl = dconjg(alphal)
!<<<                          ddmx = ddax * temp2
c                             ddmxh = ddaxh * temp2h
c                             x_u1 = 134217729.0 * ddaxh
c                             x_p1 = x_u1
c    +                                 - (x_u1 - ddaxh)
c                             x_p2 = ddaxh - x_p1
c                             x_u1 = 134217729.0 * temp2h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp2h)
c                             x_r2 = temp2h - x_r1
c                             ddmxl = ((x_p1*x_r1
c    +                                - ddmxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddmxl = ddmxl
c    +                              + (ddaxh*temp2l
c    +                              + ddaxl*temp2h)
c                             x_s1 = ddmxh + ddmxl
c                             ddmxl = ddmxl - (x_s1 - ddmxh)
c                             ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddmx = ddax * temp2
                              call bmul(ddaxh, ddaxl, temp2h, temp2l,
     +                            ddmxh, ddmxl)
!<<<                          ddxx = alpha * temp1
c                             ddxxh = alphah * temp1h
c                             x_u1 = 134217729.0 * alphah
c                             x_p1 = x_u1
c    +                                 - (x_u1 - alphah)
c                             x_p2 = alphah - x_p1
c                             x_u1 = 134217729.0 * temp1h
c                             x_r1 = x_u1
c    +                                 - (x_u1 - temp1h)
c                             x_r2 = temp1h - x_r1
c                             ddxxl = ((x_p1*x_r1
c    +                                - ddxxh)
c    +                                + x_p1 * x_r2
c    +                                + x_p2 * x_r1)
c    +                                + x_p2 * x_r2
c                             ddxxl = ddxxl
c    +                              + (alphah*temp1l
c    +                              + alphal*temp1h)
c                             x_s1 = ddxxh + ddxxl
c                             ddxxl = ddxxl - (x_s1 - ddxxh)
c                             ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddxx = alpha * temp1
                              call bmul(alphah, alphal, temp1h, temp1l,
     +                            ddxxh, ddxxl)
!<<<                          ddxx = ddxx + ddmx
c                             x_p1 = ddxxh + ddmxh
c                             x_p2 = x_p1 - ddxxh
c                             x_q1 = x_p1
c                             x_q2 = (ddxxh - (x_p1 - x_p2))
c    +                                 + (ddmxh - x_p2)
c                             x_q2 = x_q2 + ddxxl + ddmxl
c                             ddxxh = x_q1 + x_q2
c                             ddxxl = x_q2 - (ddxxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddxx = ddxx + ddmx
                              call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                            ddxxh, ddxxl)
                              ddxh = dble(ddxxh)
                              ddxl = dble(ddxxl)
                              ddah = dble(ch(j,j))
                              ddal = dble(cl(j,j))
!<<<                          ddm = beta * dda
c                             ddmh = betah * ddah
c                             t_u1 = 134217729.0 * betah
c                             t_p1 = t_u1
c    +                                 - (t_u1 - betah)
c                             t_p2 = betah - t_p1
c                             t_u1 = 134217729.0 * ddah
c                             t_r1 = t_u1
c    +                                 - (t_u1 - ddah)
c                             t_r2 = ddah - t_r1
c                             ddml = ((t_p1*t_r1
c    +                                - ddmh)
c    +                                + t_p1 * t_r2
c    +                                + t_p2 * t_r1)
c    +                                + t_p2 * t_r2
c                             ddml = ddml
c    +                              + (betah*ddal
c    +                              + betal*ddah)
c                             t_s1 = ddmh + ddml
c                             ddml = ddml - (t_s1 - ddmh)
c                             ddmh = t_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                          ddm = beta * dda
                              call bmul(betah, betal, ddah, ddal,
     +                            ddmh, ddml)
!!!#ddb                         add C(J,J) ddm ddx
!<<<                          ddm = ddm + ddx
c                             t_p1 = ddmh + ddxh
c                             t_p2 = t_p1 - ddmh
c                             t_q1 = t_p1
c                             t_q2 = (ddmh - (t_p1 - t_p2))
c    +                                 + (ddxh - t_p2)
c                             t_q2 = t_q2 + ddml + ddxl
c                             ddmh = t_q1 + t_q2
c                             ddml = t_q2 - (ddmh - t_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                          ddm = ddm + ddx
                              call bsum(ddmh, ddml, ddxh, ddxl,
     +                            ddmh, ddml)
                              ch(j,j) = ddmh
                              cl(j,j) = ddml
!!!!!!                     ddxx == ALPHA*TEMP1+DCONJG(ALPHA)*TEMP2
!!!!!!                     ddx == DBLE(ALPHA*TEMP1+DCONJG(ALPHA)*TEMP2)
!!!!!!                     ddm == BETA*DBLE(C(J,J))
                          END IF
                      ELSE
!                         IF (BETA.EQ.DBLE(ZERO)) THEN
                          if(
!                           (beta .eq. ddzero)
     +                     ((betah .eq. ddzeroh) .and.
     +                      (betal .eq. ddzerol))
     +                    ) then
!                             C(I,J) = ALPHA*TEMP1 + DCONJG(ALPHA)*TEMP2
                              ddaxh = dconjg(alphah)
                              ddaxl = dconjg(alphal)
!<<<                      ddmx = ddax * temp2
c                         ddmxh = ddaxh * temp2h
c                         x_u1 = 134217729.0 * ddaxh
c                         x_p1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_p2 = ddaxh - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ddaxh*temp2l
c    +                          + ddaxl*temp2h)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = ddax * temp2
                          call bmul(ddaxh, ddaxl, temp2h, temp2l,
     +                        ddmxh, ddmxl)
!<<<                      ddxx = alpha * temp1
c                         ddxxh = alphah * temp1h
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temp1h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_r2 = temp1h - x_r1
c                         ddxxl = ((x_p1*x_r1
c    +                            - ddxxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddxxl = ddxxl
c    +                          + (alphah*temp1l
c    +                          + alphal*temp1h)
c                         x_s1 = ddxxh + ddxxl
c                         ddxxl = ddxxl - (x_s1 - ddxxh)
c                         ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddxx = alpha * temp1
                          call bmul(alphah, alphal, temp1h, temp1l,
     +                        ddxxh, ddxxl)
!<<<                      c(i,j) = ddxx + ddmx
c                         x_p1 = ddxxh + ddmxh
c                         x_p2 = x_p1 - ddxxh
c                         ch(i,j) = x_p1
c                         cl(i,j) = (ddxxh - (x_p1 - x_p2))
c    +                          + (ddmxh - x_p2)
c                         cl(i,j) = cl(i,j) + ddxxl + ddmxl
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddxx + ddmx
                          call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                        ch(i,j), cl(i,j))
                          ELSE
!                             C(I,J) = BETA*C(I,J) + ALPHA*TEMP1 +
!    +                                 DCONJG(ALPHA)*TEMP2
                              ddaxh = dconjg(alphah)
                              ddaxl = dconjg(alphal)
!<<<                      ddmx = ddax * temp2
c                         ddmxh = ddaxh * temp2h
c                         x_u1 = 134217729.0 * ddaxh
c                         x_p1 = x_u1
c    +                             - (x_u1 - ddaxh)
c                         x_p2 = ddaxh - x_p1
c                         x_u1 = 134217729.0 * temp2h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp2h)
c                         x_r2 = temp2h - x_r1
c                         ddmxl = ((x_p1*x_r1
c    +                            - ddmxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddmxl = ddmxl
c    +                          + (ddaxh*temp2l
c    +                          + ddaxl*temp2h)
c                         x_s1 = ddmxh + ddmxl
c                         ddmxl = ddmxl - (x_s1 - ddmxh)
c                         ddmxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddmx = ddax * temp2
                          call bmul(ddaxh, ddaxl, temp2h, temp2l,
     +                        ddmxh, ddmxl)
!<<<                      ddxx = alpha * temp1
c                         ddxxh = alphah * temp1h
c                         x_u1 = 134217729.0 * alphah
c                         x_p1 = x_u1
c    +                             - (x_u1 - alphah)
c                         x_p2 = alphah - x_p1
c                         x_u1 = 134217729.0 * temp1h
c                         x_r1 = x_u1
c    +                             - (x_u1 - temp1h)
c                         x_r2 = temp1h - x_r1
c                         ddxxl = ((x_p1*x_r1
c    +                            - ddxxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddxxl = ddxxl
c    +                          + (alphah*temp1l
c    +                          + alphal*temp1h)
c                         x_s1 = ddxxh + ddxxl
c                         ddxxl = ddxxl - (x_s1 - ddxxh)
c                         ddxxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddxx = alpha * temp1
                          call bmul(alphah, alphal, temp1h, temp1l,
     +                        ddxxh, ddxxl)
!<<<                      ddxx = ddxx + ddmx
c                         x_p1 = ddxxh + ddmxh
c                         x_p2 = x_p1 - ddxxh
c                         x_q1 = x_p1
c                         x_q2 = (ddxxh - (x_p1 - x_p2))
c    +                             + (ddmxh - x_p2)
c                         x_q2 = x_q2 + ddxxl + ddmxl
c                         ddxxh = x_q1 + x_q2
c                         ddxxl = x_q2 - (ddxxh - x_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>                      ddxx = ddxx + ddmx
                          call bsum(ddxxh, ddxxl, ddmxh, ddmxl,
     +                        ddxxh, ddxxl)
!<<<                      ddbx = beta * c(i,j)
c                         ddbxh = betah * ch(i,j)
c                         x_u1 = 134217729.0 * betah
c                         x_p1 = x_u1
c    +                             - (x_u1 - betah)
c                         x_p2 = betah - x_p1
c                         x_u1 = 134217729.0 * ch(i,j)
c                         x_r1 = x_u1
c    +                             - (x_u1 - ch(i,j))
c                         x_r2 = ch(i,j) - x_r1
c                         ddbxl = ((x_p1*x_r1
c    +                            - ddbxh)
c    +                            + x_p1 * x_r2
c    +                            + x_p2 * x_r1)
c    +                            + x_p2 * x_r2
c                         ddbxl = ddbxl
c    +                          + (betah*cl(i,j)
c    +                          + betal*ch(i,j))
c                         x_s1 = ddbxh + ddbxl
c                         ddbxl = ddbxl - (x_s1 - ddbxh)
c                         ddbxh = x_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>                      ddbx = beta * c(i,j)
                          call bmul(betah, betal, ch(i,j), cl(i,j),
     +                        ddbxh, ddbxl)
!<<<                      c(i,j) = ddbx + ddxx
c                         x_p1 = ddbxh + ddxxh
c                         x_p2 = x_p1 - ddbxh
c                         ch(i,j) = x_p1
c                         cl(i,j) = (ddbxh - (x_p1 - x_p2))
c    +                          + (ddxxh - x_p2)
c                         cl(i,j) = cl(i,j) + ddbxl + ddxxl
c                         x_s1 = ch(i,j) + cl(i,j)
c                         cl(i,j) = cl(i,j) - (x_s1 - ch(i,j))
c                         ch(i,j) = x_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>                      c(i,j) = ddbx + ddxx
                          call bsum(ddbxh, ddbxl, ddxxh, ddxxl,
     +                        ch(i,j), cl(i,j))
                          END IF
                      END IF
  230             CONTINUE
  240         CONTINUE
          END IF
      END IF
*
 9999 continue
#ifdef FPCOUNTER
      WRITE(6,1000) "MODULE    = trans"
!     WRITE(6,1005) SUBNAME,"DIM M     = ",M
      WRITE(6,1005) SUBNAME,"DIM N     = ",N
      WRITE(6,1005) SUBNAME,"DIM K     = ",K
#include "fout3.h"
#include "fformat.h"
#endif
      RETURN
*
*     End of ZHER2K.
*
      END
