#!/usr/bin/env perl

use strict;
use warnings;

if (@ARGV == 2) {
	convert(@ARGV);
}
elsif (@ARGV && @ARGV % 2 == 0) {
	my $max = @ARGV / 2;
	my @doubles = @ARGV[0 .. ($max - 1)];
	my @quads = @ARGV[$max .. $#ARGV];
	for (my $i = 0; $i < $max; $i++) {
		convert($doubles[$i], $quads[$i]);
	}
}
else {
	die "Invalid argument(s).\n";
}


sub convert {
	my($double_file, $quad_file) = @_;

	my($double_routine) = ($double_file =~ /(\w+)\.f$/i);
	my($quad_routine) = ($quad_file =~ /(\w+)\.f$/i);

	print "[$double_file -> $quad_file]\n";

	my @patched;
	open my $BEFORE, '<:utf8', $double_file or die $!;
	while (<$BEFORE>) {
		my $original_line = $_;

		# Datatypes
		s/DOUBLE PRECISION/REAL(KIND(0Q0))/ig;
		s/DOUBLE COMPLEX/COMPLEX(KIND(0Q0))/ig;

		# Functions
		# grep "INTRINSIC" *.f is useful
		s/\b$double_routine\b/$quad_routine/ig;
		s/DCABS1/QCABS1/ig;
		s/C?DABS/ABS/ig;
		s/DCMPLX/CMPLX/ig;
		s/DCONJG/CONJG/ig;
		s/DBLE/REAL/ig;
		s/DIMAG/IMAG/ig;
		s/DSIGN/SIGN/ig;
		s/DSQRT/SQRT/ig;

		# Constants
		s/(?<=[.\d])d(?=[-+]?\d)/q/g;
		s/(?<=[.\d])D(?=[-+]?\d)/Q/g;

		push @patched, $_;
		next if $_ eq $original_line;
		printf "%3d| %s", $., $_;
	}
	close $BEFORE;

	open my $AFTER, '>:utf8', $quad_file or die $!;
	foreach (@patched) {
		print $AFTER $_;
	}
	close $AFTER;
}
