    MODULE BLAS_UTILITY

#ifdef QUAD_PRECISION
    USE QPCONVERT
#endif
    IMPLICIT NONE

    INTERFACE RANDOM_COMPLEX
    MODULE PROCEDURE RANDOM_ZCOMPLEX_SCALAR, RANDOM_ZCOMPLEX_VECTOR, RANDOM_ZCOMPLEX_MATRIX
#ifdef QUAD_PRECISION
    MODULE PROCEDURE RANDOM_WCOMPLEX_SCALAR, RANDOM_WCOMPLEX_VECTOR, RANDOM_WCOMPLEX_MATRIX
#endif
    END INTERFACE

    INTERFACE GEMATRIX
    MODULE PROCEDURE GEMATRIX_R, GEMATRIX_C
    END INTERFACE

    INTERFACE GBMATRIX
    MODULE PROCEDURE GBMATRIX_R
    END INTERFACE

    INTERFACE HEMATRIX
    MODULE PROCEDURE HEMATRIX_C
    END INTERFACE

    INTERFACE HBMATRIX
    MODULE PROCEDURE HBMATRIX_C
    END INTERFACE

    INTERFACE SYMATRIX
    MODULE PROCEDURE SYMATRIX_R
    END INTERFACE

    INTERFACE SBMATRIX
    MODULE PROCEDURE SBMATRIX_R
    END INTERFACE

    INTERFACE TRMATRIX
    MODULE PROCEDURE TRMATRIX_R
    END INTERFACE

    INTERFACE TBMATRIX
    MODULE PROCEDURE TBMATRIX_R
    END INTERFACE

    INTERFACE PRINT_VECTOR
    MODULE PROCEDURE PRINT_DVECTOR, PRINT_ZVECTOR
#ifdef QUAD_PRECISION
    MODULE PROCEDURE PRINT_QVECTOR, PRINT_WVECTOR
    MODULE PROCEDURE PRINT_DDVECTOR, PRINT_ZZVECTOR
#endif
    END INTERFACE

    INTERFACE PRINT_MATRIX
    MODULE PROCEDURE PRINT_DMATRIX, PRINT_ZMATRIX 
#ifdef QUAD_PRECISION
    MODULE PROCEDURE PRINT_QMATRIX, PRINT_WMATRIX
    MODULE PROCEDURE PRINT_DDMATRIX, PRINT_ZZMATRIX
#endif
    END INTERFACE

    INTERFACE SIZE_CHECK
    MODULE PROCEDURE SIZE_CHECK_VECTOR_R, SIZE_CHECK_VECTOR_C
    MODULE PROCEDURE SIZE_CHECK_MATRIX_R, SIZE_CHECK_MATRIX_C
    END INTERFACE

    CONTAINS


#ifdef QUAD_PRECISION
    SUBROUTINE RANDOM_WCOMPLEX_SCALAR(Z)

    COMPLEX(16), INTENT(OUT) :: Z

    COMPLEX(16) :: ZA(1)

    CALL RANDOM_COMPLEX(ZA)
    Z = ZA(1)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE RANDOM_WCOMPLEX_VECTOR(Z)

    COMPLEX(16), INTENT(OUT) :: Z(:)

    REAL(16), ALLOCATABLE :: RE(:), IM(:)

    ALLOCATE(RE(SIZE(Z,1)))
    ALLOCATE(IM(SIZE(Z,1)))
    CALL RANDOM_NUMBER(RE)
    CALL RANDOM_NUMBER(IM)
    Z = CMPLX(RE, IM, 16)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE RANDOM_WCOMPLEX_MATRIX(Z)

    COMPLEX(16), INTENT(OUT) :: Z(:,:)

    INTEGER :: I

    DO I = 1, SIZE(Z, 1)
        CALL RANDOM_COMPLEX(Z(I,:))
    END DO

    END SUBROUTINE
#endif


    SUBROUTINE RANDOM_ZCOMPLEX_SCALAR(Z)

    DOUBLE COMPLEX, INTENT(OUT) :: Z

    DOUBLE COMPLEX :: ZA(1)

    CALL RANDOM_COMPLEX(ZA)
    Z = ZA(1)

    END SUBROUTINE


    SUBROUTINE RANDOM_ZCOMPLEX_VECTOR(Z)

    DOUBLE COMPLEX, INTENT(OUT) :: Z(:)

    DOUBLE PRECISION, ALLOCATABLE :: RE(:), IM(:)

    ALLOCATE(RE(SIZE(Z,1)))
    ALLOCATE(IM(SIZE(Z,1)))
    CALL RANDOM_NUMBER(RE)
    CALL RANDOM_NUMBER(IM)
    Z = CMPLX(RE, IM)

    END SUBROUTINE


    SUBROUTINE RANDOM_ZCOMPLEX_MATRIX(Z)

    DOUBLE COMPLEX, INTENT(OUT) :: Z(:,:)

    INTEGER :: I

    DO I = 1, SIZE(Z, 1)
        CALL RANDOM_COMPLEX(Z(I,:))
    END DO

    END SUBROUTINE


    SUBROUTINE GEMATRIX_R(M, N, AH, AL)

    INTEGER, INTENT(IN) :: M, N
    DOUBLE PRECISION, INTENT(OUT) :: AH(:,:), AL(:,:)

#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:)
#endif

    CALL SIZE_CHECK(AH, AL, M, N)

#ifdef QUAD_PRECISION
    ALLOCATE(QA(M,N))
    CALL RANDOM_NUMBER(QA)
    CALL Q2DD(QA, AH, AL)
#else
    CALL RANDOM_NUMBER(AH)
    AL = 0.0D0
#endif

    END SUBROUTINE


    SUBROUTINE GEMATRIX_C(M, N, AH, AL)

    INTEGER, INTENT(IN) :: M, N
    DOUBLE COMPLEX, INTENT(OUT) :: AH(:,:), AL(:,:)

#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:)
#endif

    CALL SIZE_CHECK(AH, AL, M, N)

#ifdef QUAD_PRECISION
    ALLOCATE(QA(M,N))
    CALL RANDOM_COMPLEX(QA)
    CALL W2ZZ(QA, AH, AL)
#else
    CALL RANDOM_COMPLEX(AH)
    AL = 0.0D0
#endif

    END SUBROUTINE


    SUBROUTINE GBMATRIX_R(M, N, KL, KU, BH, BL)

    INTEGER, INTENT(IN) :: M, N, KL, KU
    DOUBLE PRECISION, INTENT(OUT) :: BH(:,:), BL(:,:)

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:)
    INTEGER :: I, J

    CALL SIZE_CHECK(BH, BL, KL+KU+1, N)

    ALLOCATE(AH(M,N))
    ALLOCATE(AL(M,N))
    CALL GEMATRIX(M, N, AH, AL)
    BH(:,:) = 0.0D0
    DO J = 1, N
        DO I = MAX(1, J - KU), MIN(M, J + KL)
            BH(KU + 1 + I - J, J) = AH(I, J)
            BL(KU + 1 + I - J, J) = AL(I, J)
        END DO
    END DO

    END SUBROUTINE


    SUBROUTINE HEMATRIX_C(N, AH, AL)

    INTEGER, INTENT(IN) :: N
    DOUBLE COMPLEX, INTENT(OUT) :: AH(:,:), AL(:,:)

    INTEGER :: I, J

    CALL GEMATRIX(N, N, AH, AL)
    DO I=1, N
        AH(I, I) = REAL(AH(I, I))
        AL(I, I) = REAL(AL(I, I))
    END DO
    DO I=1, N-1
        DO J=I+1, N
            AH(J,I) = CONJG(AH(I,J))
            AL(J,I) = CONJG(AL(I,J))
        END DO
    END DO

    END SUBROUTINE


    SUBROUTINE HBMATRIX_C(N, K, BH, BL)

    INTEGER, INTENT(IN) :: N, K
    DOUBLE COMPLEX, INTENT(OUT) :: BH(:,:), BL(:,:)

    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:)
    INTEGER :: I, J

    CALL SIZE_CHECK(BH, BL, K+1, N)

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    CALL HEMATRIX(N, AH, AL)

    BH = 0.0D0
    BL = 0.0D0
    DO J = 1, N
        DO I = MAX(1, J - K), J
            BH(K + 1 - J + I, J) = AH(I, J)
            BL(K + 1 - J + I, J) = AL(I, J)
        END DO
    END DO

    END SUBROUTINE


    SUBROUTINE SYMATRIX_R(N, AH, AL)

    INTEGER, INTENT(IN) :: N
    DOUBLE PRECISION, INTENT(OUT) :: AH(:,:), AL(:,:)

    INTEGER :: I, J

    CALL GEMATRIX(N, N, AH, AL)
    DO I=1, N-1
        DO J=I+1, N
            AH(J,I) = AH(I,J)
            AL(J,I) = AL(I,J)
        END DO
    END DO

    END SUBROUTINE


    SUBROUTINE SBMATRIX_R(N, K, BH, BL)

    INTEGER, INTENT(IN) :: N, K
    DOUBLE PRECISION, INTENT(OUT) :: BH(:,:), BL(:,:)

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:)
    INTEGER :: I, J

    CALL SIZE_CHECK(BH, BL, K+1, N)

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    CALL GEMATRIX(N, N, AH, AL)

    BH = 0.0D0
    BL = 0.0D0
    DO J = 1, N
        DO I = MAX(1, J - K), J
            BH(K + 1 - J + I, J) = AH(I, J)
            BL(K + 1 - J + I, J) = AL(I, J)
        END DO
    END DO

    END SUBROUTINE


    SUBROUTINE TRMATRIX_R(UPLO, DIAG, N, AH, AL)

    CHARACTER, INTENT(IN) :: UPLO, DIAG
    INTEGER, INTENT(IN) :: N
    DOUBLE PRECISION, INTENT(OUT) :: AH(:,:), AL(:,:)

    INTEGER :: I, J

    CALL GEMATRIX(N, N, AH, AL)
    IF (DIAG .EQ. 'U' .OR. DIAG .EQ. 'u') THEN
        DO I=1, N
            AH(I, I) = 1.0D0
            AL(I, I) = 0.0D0
        END DO
    END IF

    IF (UPLO .EQ. 'U' .OR. UPLO .EQ. 'u') THEN
        DO I=2, N
            DO J=1, I-1
                AH(I, J) = 0.0D0
                AL(I, J) = 0.0D0
            END DO
        END DO
    ELSE IF (UPLO .EQ. 'L' .OR. UPLO .EQ. 'l') THEN
        DO I=1, N-1
            DO J=I+1, N
                AH(I, J) = 0.0D0
                AL(I, J) = 0.0D0
            END DO
        END DO
    END IF

    END SUBROUTINE


    SUBROUTINE TBMATRIX_R(UPLO, DIAG, N, K, BH, BL)

    CHARACTER, INTENT(IN) :: UPLO, DIAG
    INTEGER, INTENT(IN) :: N, K
    DOUBLE PRECISION, INTENT(OUT) :: BH(:,:), BL(:,:)

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:)
    INTEGER :: I, J

    CALL SIZE_CHECK(BH, BL, K+1, N)

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    CALL GEMATRIX(N, N, AH, AL)
    IF (DIAG .EQ. 'U' .OR. DIAG .EQ. 'u') THEN
        DO I = 1, N
            AH(I,I) = 1.0D0
            AL(I,I) = 0.0D0
        END DO
    END IF

    BH = 0.0D0
    BL = 0.0D0
    IF (UPLO .EQ. 'U' .OR. UPLO .EQ. 'u') THEN
        DO J = 1, N
            DO I = MAX(1, J - K), J
                BH(K + 1 - J + I, J) = AH(I, J)
                BL(K + 1 - J + I, J) = AL(I, J)
            END DO
        END DO
    ELSE IF (UPLO .EQ. 'L' .OR. UPLO .EQ. 'l') THEN
        DO J = 1, N
            DO I = J, MIN(N, J + K)
                BH(1 - J + I, J) = AH(I, J)
                BL(1 - J + I, J) = AL(I, J)
            END DO
        END DO
    END IF

    END SUBROUTINE


    SUBROUTINE PRINT_DVECTOR(A)

    DOUBLE PRECISION, INTENT(IN) :: A(:)

    INTEGER I

    DO I=1, SIZE(A, 1)
        PRINT "(i5, e40.32)", I, A(I)
    END DO

    END SUBROUTINE

    SUBROUTINE PRINT_DMATRIX(A)

    DOUBLE PRECISION, INTENT(IN) :: A(:,:)

    INTEGER M, N, I, J

    M = SIZE(A, 1)
    N = SIZE(A, 2)
    DO I=1, M
        DO J=1, N
            PRINT "(i5, i5, e40.32)", I, J, A(I,J)
        END DO
    END DO

    END SUBROUTINE

#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_QVECTOR(A)

    REAL(16), INTENT(IN) :: A(:)

    INTEGER I

    DO I=1, SIZE(A, 1)
        PRINT "(i5, e40.32)", I, A(I)
    END DO

    END SUBROUTINE


    SUBROUTINE PRINT_QMATRIX(A)

    REAL(16), INTENT(IN) :: A(:,:)

    INTEGER M, N, I, J

    M = SIZE(A, 1)
    N = SIZE(A, 2)
    DO I=1, M
        DO J=1, N
            PRINT "(i5, i5, e40.32)", I, J, A(I,J)
        END DO
    END DO

    END SUBROUTINE

    SUBROUTINE PRINT_DDVECTOR(AH,AL)

    DOUBLE PRECISION, INTENT(IN) :: AH(:), AL(:)
    REAL(16) :: Q

    INTEGER I

    DO I=1, SIZE(AH, 1)
       CALL DD2Q( AH(I), AL(I), Q )
       PRINT "(i5, e40.32)", I, Q
    END DO

    END SUBROUTINE

    SUBROUTINE PRINT_DDMATRIX(AH,AL)

    DOUBLE PRECISION, INTENT(IN) :: AH(:,:), AL(:,:)
    REAL(16) Q

    INTEGER M, N, I, J

    M = SIZE(AH, 1)
    N = SIZE(AH, 2)
    DO I=1, M
        DO J=1, N
           CALL DD2Q( AH(I,J), AL(I,J), Q )
           PRINT "(i5, i5, e40.32)", I, J, Q
        END DO
    END DO

    END SUBROUTINE
#endif

    SUBROUTINE PRINT_ZVECTOR(A)

    DOUBLE COMPLEX, INTENT(IN) :: A(:)

    INTEGER I

    DO I=1, SIZE(A, 1)
        PRINT "(i5, e40.32, e40.32)", I, REAL(A(I)), IMAG(A(I))
    END DO

    END SUBROUTINE

    SUBROUTINE PRINT_ZMATRIX(A)

    DOUBLE COMPLEX, INTENT(IN) :: A(:,:)

    INTEGER M, N, I, J

    M = SIZE(A, 1)
    N = SIZE(A, 2)
    DO I=1, M
        DO J=1, N
            PRINT "(i5, i5, e40.32, e40.32)", I, J, A(I,J)
        END DO
    END DO

    END SUBROUTINE

#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_ZZVECTOR(AH,AL)

    DOUBLE COMPLEX, INTENT(IN) :: AH(:), AL(:)
    COMPLEX(16) :: W

    INTEGER I

    DO I=1, SIZE(AH, 1)
       CALL ZZ2W( AH(I), AL(I), W )
       PRINT "(i5, e40.32, e40.32)", I, REAL(W), IMAG(W)
    END DO

    END SUBROUTINE

    SUBROUTINE PRINT_ZZMATRIX(AH,AL)

    DOUBLE COMPLEX, INTENT(IN) :: AH(:,:), AL(:,:)
    COMPLEX(16) :: W

    INTEGER :: I, J

    DO I=1, SIZE(AH, 1)
        DO J=1, SIZE(AH, 2)
           CALL ZZ2W( AH(I,J), AL(I,J), W )
           PRINT "(i5, i5, e40.32, e40.32)", I, J, REAL(W), IMAG(W)
        END DO
    END DO

    END SUBROUTINE

    SUBROUTINE PRINT_WVECTOR(A)

    COMPLEX(16), INTENT(IN) :: A(:)

    INTEGER :: I

    DO I=1, SIZE(A, 1)
        PRINT "(i5, e40.32, e40.32)", I, REAL(A(I)), IMAG(A(I))
    END DO

    END SUBROUTINE


    SUBROUTINE PRINT_WMATRIX(A)

    COMPLEX(16), INTENT(IN) :: A(:,:)

    INTEGER :: I, J

    DO I=1, SIZE(A, 1)
        DO J=1, SIZE(A, 2)
            PRINT "(i5, i5, e40.32, e40.32)", I, J, A(I,J)
        END DO
    END DO

    END SUBROUTINE
#endif


    INTEGER FUNCTION VECTOR_SIZE(N, INC)

    INTEGER, INTENT(IN) :: N, INC

    VECTOR_SIZE = (N - 1) * ABS(INC) + 1
    RETURN

    END FUNCTION


    SUBROUTINE TRANS_VECTOR_SIZE(TRANS, M, N, INCX, INCY, NX, NY)

    CHARACTER, INTENT(IN) :: TRANS
    INTEGER, INTENT(IN) :: M, N, INCX, INCY
    INTEGER, INTENT(OUT) :: NX, NY

    IF (TRANS .EQ. 'N' .OR. TRANS .EQ. 'n') THEN
        NX = VECTOR_SIZE(N, INCX)
        NY = VECTOR_SIZE(M, INCY)
    ELSE
        NX = VECTOR_SIZE(M, INCX)
        NY = VECTOR_SIZE(N, INCY)
    END IF
    RETURN

    END SUBROUTINE


    SUBROUTINE SIZE_CHECK_VECTOR_R(DDH, DDL, N) 

    DOUBLE PRECISION, INTENT(IN) :: DDH(:), DDL(:)
    INTEGER, INTENT(IN) :: N

    IF (SIZE(DDH,1) .NE. N .OR. SIZE(DDL,1) .NE. N) THEN
        PRINT *, "invalid size"
    END IF

    END SUBROUTINE


    SUBROUTINE SIZE_CHECK_VECTOR_C(DDH, DDL, N) 

    DOUBLE COMPLEX, INTENT(IN) :: DDH(:), DDL(:)
    INTEGER, INTENT(IN) :: N

    IF (SIZE(DDH,1) .NE. N .OR. SIZE(DDL,1) .NE. N) THEN
        PRINT *, "invalid size"
    END IF

    END SUBROUTINE


    SUBROUTINE SIZE_CHECK_MATRIX_R(DDH, DDL, M, N) 

    DOUBLE PRECISION, INTENT(IN) :: DDH(:,:), DDL(:,:)
    INTEGER, INTENT(IN) :: M, N

    IF (SIZE(DDH,1) .NE. M .OR. SIZE(DDH,2) .NE. N) THEN
        PRINT *, "invalid size"
    END IF
    IF (SIZE(DDL,1) .NE. M .OR. SIZE(DDL,2) .NE. N) THEN
        PRINT *, "invalid size"
    END IF

    END SUBROUTINE


    SUBROUTINE SIZE_CHECK_MATRIX_C(DDH, DDL, M, N) 

    DOUBLE COMPLEX, INTENT(IN) :: DDH(:,:), DDL(:,:)
    INTEGER, INTENT(IN) :: M, N

    IF (SIZE(DDH,1) .NE. M .OR. SIZE(DDH,2) .NE. N) THEN
        PRINT *, "invalid size"
    END IF
    IF (SIZE(DDL,1) .NE. M .OR. SIZE(DDL,2) .NE. N) THEN
        PRINT *, "invalid size"
    END IF

    END SUBROUTINE


    END MODULE
