    MODULE LEVEL1_TESTS

    USE BLAS_UTILITY
#ifdef QUAD_PRECISION
    USE QPCONVERT
#endif
    USE TEST_UTILITY
    IMPLICIT NONE

    INTERFACE

    INTEGER FUNCTION IDDAMAX(N, XH, XL, INCX)
    DOUBLE PRECISION XH(*), XL(*)
    INTEGER N, INCX
    END FUNCTION

    DOUBLE PRECISION FUNCTION DDOT(N, X, INCX, Y, INCY)
    DOUBLE PRECISION X(*), Y(*)
    INTEGER N, INCX, INCY
    END FUNCTION

    DOUBLE PRECISION FUNCTION DNRM2(N, X, INCX)
    DOUBLE PRECISION X(*)
    INTEGER N, INCX
    END FUNCTION

    DOUBLE PRECISION FUNCTION DASUM(N, X, INCX)
    DOUBLE PRECISION X(*)
    INTEGER N, INCX
    END FUNCTION

    INTEGER FUNCTION IDAMAX(N, X, INCX)
    DOUBLE PRECISION X(*)
    INTEGER N, INCX
    END FUNCTION

    DOUBLE COMPLEX FUNCTION ZDOTC(N, X, INCX, Y, INCY)
    DOUBLE COMPLEX X(*), Y(*)
    INTEGER N, INCX, INCY
    END FUNCTION

    DOUBLE COMPLEX FUNCTION ZDOTU(N, X, INCX, Y, INCY)
    DOUBLE COMPLEX X(*), Y(*)
    INTEGER N, INCX, INCY
    END FUNCTION

#ifdef QUAD_PRECISION
    REAL(16) FUNCTION QDOT(N, X, INCX, Y, INCY)
    REAL(16) X(*), Y(*)
    INTEGER N, INCX, INCY
    END FUNCTION

    REAL(16) FUNCTION QNRM2(N, X, INCX)
    REAL(16) X(*)
    INTEGER N, INCX
    END FUNCTION

    REAL(16) FUNCTION QASUM(N, X, INCX)
    REAL(16) X(*)
    INTEGER N, INCX
    END FUNCTION

    INTEGER FUNCTION IQAMAX(N, X, INCX)
    REAL(16) X(*)
    INTEGER N, INCX
    END FUNCTION

    COMPLEX(16) FUNCTION WDOTC(N, X, INCX, Y, INCY)
    COMPLEX(16) X(*), Y(*)
    INTEGER N, INCX, INCY
    END FUNCTION

    COMPLEX(16) FUNCTION WDOTU(N, X, INCX, Y, INCY)
    COMPLEX(16) X(*), Y(*)
    INTEGER N, INCX, INCY
    END FUNCTION
#endif

    END INTERFACE


    CONTAINS


    SUBROUTINE DDSWAP_TEST(N, INCX, INCY)
 
    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "SWAP"

    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DX(:), DY(:)
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QX(:), QY(:)
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(DX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL, DX)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL, DY)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
#endif

    CALL CLOCK_START()
    CALL DDSWAP(N, XH, XL, INCX, YH, YL, INCY)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSWAP(N, DX, INCX, DY, INCY)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSWAP(N, QX, INCX, QY, INCY)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX, QX)
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX)
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDSCAL_TEST(N, INCX)

    INTEGER, INTENT(IN) :: N, INCX
    CHARACTER(*), PARAMETER :: FNAME = "SCAL"

    DOUBLE PRECISION :: AH, AL
    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DX(:)
#ifdef QUAD_PRECISION
    REAL(16) :: QA
    REAL(16), ALLOCATABLE :: QX(:)
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(DX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL, DX)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
#endif

    CALL CLOCK_START()
    CALL DDSCAL(N, AH, AL, XH, XL, INCX)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSCAL(N, AH, DX, INCX)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSCAL(N, QA, QX, INCX)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX, QX)
#else
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX)
#endif

    END SUBROUTINE


    SUBROUTINE DDCOPY_TEST(N, INCX, INCY)

    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "COPY"

    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DY(:)
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QX(:), QY(:)
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "", N, INCY, YH, YL, DY)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
#endif

    CALL CLOCK_START()
    CALL DDCOPY(N, XH, XL, INCX, YH, YL, INCY)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DCOPY(N, XH, INCX, DY, INCY)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QCOPY(N, QX, INCX, QY, INCY)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDAXPY_TEST(N, INCX, INCY)

    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "AXPY"

    DOUBLE PRECISION :: AH, AL
    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DY(:)
#ifdef QUAD_PRECISION
    REAL(16) :: QA
    REAL(16), ALLOCATABLE :: QX(:), QY(:)
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL, DY)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
#endif

    CALL CLOCK_START()
    CALL DDAXPY(N, AH, AL, XH, XL, INCX, YH, YL, INCY)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DAXPY(N, AH, XH, INCX, DY, INCY)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QAXPY(N, QA, QX, INCX, QY, INCY)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDDOT_TEST(N, INCX, INCY)

    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "DOT"

    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION :: DDRH, DDRL, DR
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QX(:), QY(:)
    REAL(16) :: QR
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
#endif

    CALL CLOCK_START()
    CALL DDDOT(N, XH, XL, INCX, YH, YL, INCY, DDRH, DDRL)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    DR = DDOT(N, XH, INCX, YH, INCY)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    QR = QDOT(N, QX, INCX, QY, INCY)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "ret", DDRH, DDRL, DR, QR)
#else
    CALL TEST_VERIFY(FNAME, "ret", DDRH, DDRL, DR)
#endif

    END SUBROUTINE


    SUBROUTINE DDNRM2_TEST(N, INCX)

    INTEGER, INTENT(IN) :: N, INCX
    CHARACTER(*), PARAMETER :: FNAME = "NRM2"

    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:)
    DOUBLE PRECISION :: DDRH, DDRL, DR
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QX(:)
    REAL(16) :: QR
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(XH, XL, QX)
#endif

    CALL CLOCK_START()
    CALL DDNRM2(N, XH, XL, INCX, DDRH, DDRL)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    DR = DNRM2(N, XH, INCX)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    QR = QNRM2(N, QX, INCX)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "ret", DDRH, DDRL, DR, QR)
#else
    CALL TEST_VERIFY(FNAME, "ret", DDRH, DDRL, DR)
#endif

    END SUBROUTINE


    SUBROUTINE DDASUM_TEST(N, INCX)

    INTEGER, INTENT(IN) :: N, INCX
    CHARACTER(*), PARAMETER :: FNAME = "ASUM"

    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:)
    DOUBLE PRECISION :: DDRH, DDRL, DR
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QX(:)
    REAL(16) :: QR
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(XH, XL, QX)
#endif

    CALL CLOCK_START()
    CALL DDASUM(N, XH, XL, INCX, DDRH, DDRL)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    DR = DASUM(N, XH, INCX)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    QR = QASUM(N, QX, INCX)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "ret", DDRH, DDRL, DR, QR)
#else
    CALL TEST_VERIFY(FNAME, "ret", DDRH, DDRL, DR)
#endif

    END SUBROUTINE


    SUBROUTINE IDDAMAX_TEST(N, INCX)

    INTEGER, INTENT(IN) :: N, INCX
    CHARACTER(*), PARAMETER :: FNAME = "AMAX"

    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:)
    INTEGER :: IDD, ID
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QX(:)
    INTEGER :: IQ
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(XH, XL, QX)
#endif

    CALL CLOCK_START()
    IDD = IDDAMAX(N, XH, XL, INCX)
    CALL CLOCK_STOP("IDD"//FNAME)

    CALL CLOCK_START()
    ID = IDAMAX(N, XH, INCX)
    CALL CLOCK_STOP("ID"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    IQ = IQAMAX(N, QX, INCX)
    CALL CLOCK_STOP("IQ"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "ret", IDD, ID, IQ)
#else
    CALL TEST_VERIFY(FNAME, "ret", IDD, ID)
#endif

    END SUBROUTINE


    SUBROUTINE DDROT_TEST(N, INCX, INCY)

    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "ROT"

#ifndef QUAD_PRECISION
    DOUBLE PRECISION :: DA, DB
#endif
    DOUBLE PRECISION :: CH, CL, SH, SL
    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DX(:), DY(:)
#ifdef QUAD_PRECISION
    REAL(16) :: QA, QB, QC, QS
    REAL(16), ALLOCATABLE :: QX(:), QY(:)
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(DX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL, DX)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL, DY)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
#endif

#ifdef QUAD_PRECISION
    CALL RANDOM_NUMBER(QA)
    CALL RANDOM_NUMBER(QB)
    CALL QROTG(QA, QB, QC, QS)
    CALL TEST_PRINT(FNAME, "C", "INPUT", QC)
    CALL TEST_PRINT(FNAME, "S", "INPUT", QS)
    CALL Q2DD(QC, CH, CL)
    CALL Q2DD(QS, SH, SL)
#else
    CALL RANDOM_NUMBER(DA)
    CALL RANDOM_NUMBER(DB)
    CALL DROTG(DA, DB, CH, SH)
    CALL TEST_PRINT(FNAME, "C", "INPUT", CH)
    CALL TEST_PRINT(FNAME, "S", "INPUT", SH)
    CL = 0.0D0
    SL = 0.0D0
#endif

    CALL CLOCK_START()
    CALL DDROT(N, XH, XL, INCX, YH, YL, INCY, CH, CL, SH, SL)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DROT(N, DX, INCX, DY, INCY, CH, SH)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QROT(N, QX, INCX, QY, INCY, QC, QS)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX, QX)
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX)
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDROTG_TEST()

    CHARACTER(*), PARAMETER :: FNAME = "ROTG"

    DOUBLE PRECISION :: AH, AL, BH, BL, CH, CL, SH, SL
    DOUBLE PRECISION :: DA, DB, DC, DS
#ifdef QUAD_PRECISION
    REAL(16) :: QA, QB, QC, QS
#endif

    CALL TEST_PREPARE(FNAME, "A", AH, AL, DA)
    CALL TEST_PREPARE(FNAME, "B", BH, BL, DB)
#ifdef QUAD_PRECISION
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(BH, BL, QB)
#endif

    CALL CLOCK_START()
    CALL DDROTG(AH, AL, BH, BL, CH, CL, SH, SL)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DROTG(DA, DB, DC, DS)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QROTG(QA, QB, QC, QS)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA, QA)
    CALL TEST_VERIFY(FNAME, "B", BH, BL, DB, QB)
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC, QC)
    CALL TEST_VERIFY(FNAME, "S", SH, SL, DS, QS)
#else
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA)
    CALL TEST_VERIFY(FNAME, "B", BH, BL, DB)
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC)
    CALL TEST_VERIFY(FNAME, "S", SH, SL, DS)
#endif

    END SUBROUTINE


    SUBROUTINE DDROTM_TEST(N, INCX, INCY)

    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "ROTM"

#ifdef QUAD_PRECISION
    REAL(16) :: QD1, QD2, QX1, QY1
    REAL(16) :: QPARAM(5)
#else
    DOUBLE PRECISION :: DD1, DD2, DX1, DY1
#endif
    DOUBLE PRECISION :: PARAMH(5), PARAML(5)
    DOUBLE PRECISION, ALLOCATABLE :: XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DX(:), DY(:)
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QX(:), QY(:)
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(DX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL, DX)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL, DY)
#ifdef QUAD_PRECISION
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
#endif

#ifdef QUAD_PRECISION
    CALL RANDOM_NUMBER(QD1)
    CALL RANDOM_NUMBER(QD2)
    CALL RANDOM_NUMBER(QX1)
    CALL RANDOM_NUMBER(QY1)
    CALL QROTMG(QD1, QD2, QX1, QY1, QPARAM)
    CALL Q2DD(QPARAM, PARAMH, PARAML)
    CALL TEST_PRINT(FNAME, "PARAM", "INPUT", QPARAM)
#else
    CALL RANDOM_NUMBER(DD1)
    CALL RANDOM_NUMBER(DD2)
    CALL RANDOM_NUMBER(DX1)
    CALL RANDOM_NUMBER(DY1)
    CALL DROTMG(DD1, DD2, DX1, DY1, PARAMH)
    CALL TEST_PRINT(FNAME, "PARAM", "INPUT", PARAMH)
    PARAML = 0.0D0
#endif

    CALL CLOCK_START()
    CALL DDROTM(N, XH, XL, INCX, YH, YL, INCY, PARAMH, PARAML)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DROTM(N, DX, INCX, DY, INCY, PARAMH)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QROTM(N, QX, INCX, QY, INCY, QPARAM)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX, QX)
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX)
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDROTMG_TEST()

    CHARACTER(*), PARAMETER :: FNAME = "ROTMG"

    DOUBLE PRECISION :: DDD1H, DDD1L, DDD2H, DDD2L, DDX1H, DDX1L, DDY1H, DDY1L, DDPARAMH(5), DDPARAML(5)
    DOUBLE PRECISION :: DD1, DD2, DX1, DY1, DPARAM(5)
#ifdef QUAD_PRECISION
    REAL(16) :: QD1, QD2, QX1, QY1, QPARAM(5)
#endif

    CALL TEST_PREPARE(FNAME, "D1", DDD1H, DDD1L, DD1)
    CALL TEST_PREPARE(FNAME, "D2", DDD2H, DDD2L, DD2)
    CALL TEST_PREPARE(FNAME, "X1", DDX1H, DDX1L, DX1)
    CALL TEST_PREPARE(FNAME, "Y1", DDY1H, DDY1L, DY1)
    DPARAM = 0.0D0
    DDPARAMH = 0.0D0
    DDPARAML = 0.0D0
#ifdef QUAD_PRECISION
    CALL TEST_PREPARE_Q(DDD1H, DDD1L, QD1)
    CALL TEST_PREPARE_Q(DDD2H, DDD2L, QD2)
    CALL TEST_PREPARE_Q(DDX1H, DDX1L, QX1)
    CALL TEST_PREPARE_Q(DDY1H, DDY1L, QY1)
    QPARAM = 0.0Q0
#endif

    CALL CLOCK_START()
    CALL DDROTMG(DDD1H, DDD1L, DDD2H, DDD2L, DDX1H, DDX1L, DDY1H, DDY1L, DDPARAMH, DDPARAML)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DROTMG(DD1, DD2, DX1, DY1, DPARAM)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QROTMG(QD1, QD2, QX1, QY1, QPARAM)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "D1", DDD1H, DDD1L, DD1, QD1)
    CALL TEST_VERIFY(FNAME, "D2", DDD2H, DDD2L, DD2, QD2)
    CALL TEST_VERIFY(FNAME, "X1", DDX1H, DDX1L, DX1, QX1)
    CALL TEST_VERIFY(FNAME, "PARAM", DDPARAMH, DDPARAML, DPARAM, QPARAM)
#else
    CALL TEST_VERIFY(FNAME, "D1", DDD1H, DDD1L, DD1)
    CALL TEST_VERIFY(FNAME, "D2", DDD2H, DDD2L, DD2)
    CALL TEST_VERIFY(FNAME, "X1", DDX1H, DDX1L, DX1)
    CALL TEST_VERIFY(FNAME, "PARAM", DDPARAMH, DDPARAML, DPARAM)
#endif

    END SUBROUTINE


    SUBROUTINE ZZDOTC_TEST(N, INCX, INCY)

    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "DOTC"

    DOUBLE COMPLEX, ALLOCATABLE :: XH(:), XL(:), YH(:), YL(:)
    DOUBLE COMPLEX :: ZZRH, ZZRL, ZR
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: WX(:), WY(:)
    COMPLEX(16) :: WR
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL)
#ifdef QUAD_PRECISION
    ALLOCATE(WX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(WY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(XH, XL, WX)
    CALL TEST_PREPARE_Q(YH, YL, WY)
#endif

    CALL CLOCK_START()
    CALL ZZDOTC(N, XH, XL, INCX, YH, YL, INCY, ZZRH, ZZRL)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    ZR = ZDOTC(N, XH, INCX, YH, INCY)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    WR = WDOTC(N, WX, INCX, WY, INCY)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "ret", ZZRH, ZZRL, ZR, WR)
#else
    CALL TEST_VERIFY(FNAME, "ret", ZZRH, ZZRL, ZR)
#endif

    END SUBROUTINE


    SUBROUTINE ZZDOTU_TEST(N, INCX, INCY)

    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "DOTU"

    DOUBLE COMPLEX, ALLOCATABLE :: XH(:), XL(:), YH(:), YL(:)
    DOUBLE COMPLEX :: ZZRH, ZZRL, ZR
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: WX(:), WY(:)
    COMPLEX(16) :: WR
#endif

    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL)
#ifdef QUAD_PRECISION
    ALLOCATE(WX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(WY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(XH, XL, WX)
    CALL TEST_PREPARE_Q(YH, YL, WY)
#endif

    CALL CLOCK_START()
    CALL ZZDOTU(N, XH, XL, INCX, YH, YL, INCY, ZZRH, ZZRL)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    ZR = ZDOTU(N, XH, INCX, YH, INCY)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    WR = WDOTU(N, WX, INCX, WY, INCY)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "ret", ZZRH, ZZRL, ZR, WR)
#else
    CALL TEST_VERIFY(FNAME, "ret", ZZRH, ZZRL, ZR)
#endif

    END SUBROUTINE


    END MODULE
