    PROGRAM LEVEL2

    USE LEVEL2_TESTS
    USE TEST_UTILITY
    IMPLICIT NONE

    ! increment
    INTEGER :: INCX, INCY

    ! size of matrix
    INTEGER :: M, N

    ! bandwidth (KL = K)
    INTEGER :: K, KU

    ! style of matrix
    CHARACTER :: UPLO, DIAG, TRANS

    INCX = 1
    INCY = 1
    M = 5
    N = 5
    K = 2
    KU = 1
    UPLO = 'U'
    DIAG = 'N'
    TRANS = 'N'

    CALL INIT_RANDOM_SEED()

    DO
        IF (SELECT() .EQ. 0) EXIT
    END DO
    STOP


    CONTAINS


    INTEGER FUNCTION SELECT()

    PRINT *, "QPBLAS Level 2 functions"
    !        "  ##: ####10####  ##: ####10####  ##: ####10####  ##: ####10####"
    PRINT *, "   1: ddgemv       2: ddsymv       3: ddtrmv       4: ddtrsv"
    PRINT *, "   5: ddsyr        6: ddsyr2       7: ddgbmv       8: ddger"
    PRINT *, "   9: ddsbmv      10: ddtbmv      11: ddtbsv      12: zzgerc"
    PRINT *, "  13: zzgeru      14: zzhbmv      15: zzhemv      16: zzher"
    PRINT *, "  17: zzher2"
    PRINT *, ""
    PRINT *, "  70: INCX        71: INCY        72: ALPHA       73: BETA"
    PRINT *, "  74: M           75: N           76: K[L]        77: KU"
    PRINT *, "  80: UPLO        81: DIAG        83: TRANS"
    PRINT *, "  96: TEST_ALL    97: VERBOSE     98: VERIFY      99: BENCHMARK"
    PRINT *, "   0: EXIT"

    DO
        WRITE (*,fmt='(a)', advance='no') "Please select the number of function to test: "
        READ *, SELECT
        IF (SELECT .GE. 0 .AND. SELECT .LE. 17) THEN
            EXIT
        ELSE IF (SELECT .EQ. 70) THEN
            CALL CHANGE_INTEGER("INCX", INCX)
        ELSE IF (SELECT .EQ. 71) THEN
            CALL CHANGE_INTEGER("INCY", INCY)
        ELSE IF (SELECT .EQ. 72) THEN
            CALL CHANGE_INTEGER("ALPHA", FIXED_ALPHA)
        ELSE IF (SELECT .EQ. 73) THEN
            CALL CHANGE_INTEGER("BETA", FIXED_BETA)
        ELSE IF (SELECT .EQ. 74) THEN
            CALL CHANGE_INTEGER("M", M)
        ELSE IF (SELECT .EQ. 75) THEN
            CALL CHANGE_INTEGER("N", N)
        ELSE IF (SELECT .EQ. 76) THEN
            CALL CHANGE_INTEGER("K[L]", K)
        ELSE IF (SELECT .EQ. 77) THEN
            CALL CHANGE_INTEGER("KU", KU)
        ELSE IF (SELECT .EQ. 80) THEN
            CALL CHANGE_CHARACTER("UPLO", UPLO)
        ELSE IF (SELECT .EQ. 81) THEN
            CALL CHANGE_CHARACTER("DIAG", DIAG)
        ELSE IF (SELECT .EQ. 83) THEN
            CALL CHANGE_CHARACTER("TRANS", TRANS)
        ELSE IF (SELECT .EQ. 96) THEN
            EXIT
        ELSE IF (SELECT .EQ. 97) THEN
            CALL TOGGLE_MODE_VERBOSE()
        ELSE IF (SELECT .EQ. 98) THEN
            CALL TOGGLE_MODE_VERIFY()
        ELSE IF (SELECT .EQ. 99) THEN
            CALL TOGGLE_MODE_BENCHMARK()
        END IF
    END DO

    IF (SELECT .EQ. 0) THEN
        ! noop
    ELSE IF (SELECT .EQ. 1) THEN
        CALL DDGEMV_TEST(TRANS, M, N, INCX, INCY)
    ELSE IF (SELECT .EQ. 2) THEN
        CALL DDSYMV_TEST(UPLO, N, INCX, INCY)
    ELSE IF (SELECT .EQ. 3) THEN
        CALL DDTRMV_TEST(UPLO, TRANS, DIAG, N, INCX)
    ELSE IF (SELECT .EQ. 4) THEN
        CALL DDTRSV_TEST(UPLO, TRANS, DIAG, N, INCX)
    ELSE IF (SELECT .EQ. 5) THEN
        CALL DDSYR_TEST(UPLO, N, INCX)
    ELSE IF (SELECT .EQ. 6) THEN
        CALL DDSYR2_TEST(UPLO, N, INCX, INCY)
    ELSE IF (SELECT .EQ. 7) THEN
        CALL DDGBMV_TEST(TRANS, M, N, K, KU, INCX, INCY)
    ELSE IF (SELECT .EQ. 8) THEN
        CALL DDGER_TEST(M, N, INCX, INCY)
    ELSE IF (SELECT .EQ. 9) THEN
        CALL DDSBMV_TEST(UPLO, N, K, INCX, INCY)
    ELSE IF (SELECT .EQ. 10) THEN
        CALL DDTBMV_TEST(UPLO, TRANS, DIAG, N, K, INCX)
    ELSE IF (SELECT .EQ. 11) THEN
        CALL DDTBSV_TEST(UPLO, TRANS, DIAG, N, K, INCX)
    ELSE IF (SELECT .EQ. 12) THEN
        CALL ZZGERC_TEST(M, N, INCX, INCY)
    ELSE IF (SELECT .EQ. 13) THEN
        CALL ZZGERU_TEST(M, N, INCX, INCY)
    ELSE IF (SELECT .EQ. 14) THEN
        CALL ZZHBMV_TEST(UPLO, N, K, INCX, INCY)
    ELSE IF (SELECT .EQ. 15) THEN
        CALL ZZHEMV_TEST(UPLO, N, INCX, INCY)
    ELSE IF (SELECT .EQ. 16) THEN
        CALL ZZHER_TEST(UPLO, N, INCX)
    ELSE IF (SELECT .EQ. 17) THEN
        CALL ZZHER2_TEST(UPLO, N, INCX, INCY)
    ELSE IF (SELECT .EQ. 96) THEN
        PRINT *, "-----"
        CALL DDGEMV_TEST(TRANS, M, N, INCX, INCY)
        PRINT *, "-----"
        CALL DDSYMV_TEST(UPLO, N, INCX, INCY)
        PRINT *, "-----"
        CALL DDTRMV_TEST(UPLO, TRANS, DIAG, N, INCX)
        PRINT *, "-----"
        CALL DDTRSV_TEST(UPLO, TRANS, DIAG, N, INCX)
        PRINT *, "-----"
        CALL DDSYR_TEST(UPLO, N, INCX)
        PRINT *, "-----"
        CALL DDSYR2_TEST(UPLO, N, INCX, INCY)
        PRINT *, "-----"
        CALL DDGBMV_TEST(TRANS, M, N, K, KU, INCX, INCY)
        PRINT *, "-----"
        CALL DDGER_TEST(M, N, INCX, INCY)
        PRINT *, "-----"
        CALL DDSBMV_TEST(UPLO, N, K, INCX, INCY)
        PRINT *, "-----"
        CALL DDTBMV_TEST(UPLO, TRANS, DIAG, N, K, INCX)
        PRINT *, "-----"
        CALL DDTBSV_TEST(UPLO, TRANS, DIAG, N, K, INCX)
        PRINT *, "-----"
        CALL ZZGERC_TEST(M, N, INCX, INCY)
        PRINT *, "-----"
        CALL ZZGERU_TEST(M, N, INCX, INCY)
        PRINT *, "-----"
        CALL ZZHBMV_TEST(UPLO, N, K, INCX, INCY)
        PRINT *, "-----"
        CALL ZZHEMV_TEST(UPLO, N, INCX, INCY)
        PRINT *, "-----"
        CALL ZZHER_TEST(UPLO, N, INCX)
        PRINT *, "-----"
        CALL ZZHER2_TEST(UPLO, N, INCX, INCY)
        PRINT *, "-----"
    ELSE
        PRINT *, "INTERNAL ERROR"
    END IF
    PRINT *, ""
    PRINT *, ""

    END FUNCTION


    END PROGRAM
