    MODULE LEVEL2_TESTS

    USE BLAS_UTILITY
    USE TEST_UTILITY
    IMPLICIT NONE


    CONTAINS


    SUBROUTINE DDGEMV_TEST(TRANS, M, N, INCX, INCY)

    CHARACTER, INTENT(IN) :: TRANS
    INTEGER, INTENT(IN) :: M, N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "GEMV"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DY(:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: NX, NY, LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    REAL(16) :: QALPHA, QBETA
#endif

    CALL TRANS_VECTOR_SIZE(TRANS, M, N, INCX, INCY, NX, NY)

    ALLOCATE(AH(M,N))
    ALLOCATE(AL(M,N))
    ALLOCATE(XH(NX))
    ALLOCATE(XL(NX))
    ALLOCATE(YH(NY))
    ALLOCATE(YL(NY))
    ALLOCATE(DY(NY))
    CALL GEMATRIX(M, N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", NX, 0, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", NY, 0, YH, YL, DY)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(M,N))
    ALLOCATE(QX(NX))
    ALLOCATE(QY(NY))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDGEMV(TRANS, M, N, ALPHAH, ALPHAL, AH, AL, LDA, XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DGEMV(TRANS, M, N, ALPHAH, AH, LDA, XH, INCX, BETAH, DY, INCY)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QGEMV(TRANS, M, N, QALPHA, QA, LDA, QX, INCX, QBETA, QY, INCY)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDSYMV_TEST(UPLO, N, INCX, INCY)

    CHARACTER, INTENT(IN) :: UPLO
    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "SYMV"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DY(:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    REAL(16) :: QALPHA, QBETA
#endif

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL SYMATRIX(N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL, DY)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(N,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDSYMV(UPLO, N, ALPHAH, ALPHAL, AH, AL, LDA, XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSYMV(UPLO, N, ALPHAH, AH, LDA, XH, INCX, BETAH, DY, INCY)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSYMV(UPLO, N, QALPHA, QA, LDA, QX, INCX, QBETA, QY, INCY)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDTRMV_TEST(UPLO, TRANS, DIAG, N, INCX)

    CHARACTER, INTENT(IN) :: UPLO, TRANS, DIAG
    INTEGER, INTENT(IN) :: N, INCX
    CHARACTER(*), PARAMETER :: FNAME = "TRMV"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DX(:)
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:)
#endif

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(DX(VECTOR_SIZE(N, INCX)))
    CALL TRMATRIX(UPLO, DIAG, N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL, DX)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(N,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDTRMV(UPLO, TRANS, DIAG, N, AH, AL, LDA, XH, XL, INCX)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DTRMV(UPLO, TRANS, DIAG, N, AH, LDA, DX, INCX)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QTRMV(UPLO, TRANS, DIAG, N, QA, LDA, QX, INCX)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX, QX)
#else
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX)
#endif

    END SUBROUTINE


    SUBROUTINE DDTRSV_TEST(UPLO, TRANS, DIAG, N, INCX)

    CHARACTER, INTENT(IN) :: UPLO, TRANS, DIAG
    INTEGER, INTENT(IN) :: N, INCX
    CHARACTER(*), PARAMETER :: FNAME = "TRSV"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DX(:)
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:)
#endif

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(DX(VECTOR_SIZE(N, INCX)))
    CALL TRMATRIX(UPLO, DIAG, N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL, DX)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(N,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDTRSV(UPLO, TRANS, DIAG, N, AH, AL, LDA, XH, XL, INCX)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DTRSV(UPLO, TRANS, DIAG, N, AH, LDA, DX, INCX)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QTRSV(UPLO, TRANS, DIAG, N, QA, LDA, QX, INCX)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX, QX)
#else
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX)
#endif

    END SUBROUTINE


    SUBROUTINE DDSYR_TEST(UPLO, N, INCX)

    CHARACTER, INTENT(IN) :: UPLO
    INTEGER, INTENT(IN) :: N, INCX
    CHARACTER(*), PARAMETER :: FNAME = "SYR"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DA(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:)
    REAL(16) :: QALPHA
#endif

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    ALLOCATE(DA(N,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    CALL SYMATRIX(N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL, DA)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(N,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDSYR(UPLO, N, ALPHAH, ALPHAL, XH, XL, INCX, AH, AL, LDA)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSYR(UPLO, N, ALPHAH, XH, INCX, DA, LDA)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSYR(UPLO, N, QALPHA, QX, INCX, QA, LDA)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA)
#endif

    END SUBROUTINE


    SUBROUTINE DDSYR2_TEST(UPLO, N, INCX, INCY)

    CHARACTER, INTENT(IN) :: UPLO
    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "SYR2"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DA(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    REAL(16) :: QALPHA
#endif

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    ALLOCATE(DA(N,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    CALL SYMATRIX(N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL, DA)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(N,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDSYR2(UPLO, N, ALPHAH, ALPHAL, XH, XL, INCX, YH, YL, INCY, AH, AL, LDA)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSYR2(UPLO, N, ALPHAH, XH, INCX, YH, INCY, DA, LDA)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSYR2(UPLO, N, QALPHA, QX, INCX, QY, INCY, QA, LDA)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA)
#endif

    END SUBROUTINE


    SUBROUTINE DDGBMV_TEST(TRANS, M, N, KL, KU, INCX, INCY)

    CHARACTER, INTENT(IN) :: TRANS
    INTEGER, INTENT(IN) :: M, N, KL, KU, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "GBMV"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DY(:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: NX, NY, LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    REAL(16) :: QALPHA, QBETA
#endif

    CALL TRANS_VECTOR_SIZE(TRANS, M, N, INCX, INCY, NX, NY)

    ALLOCATE(AH(KL+KU+1,N))
    ALLOCATE(AL(KL+KU+1,N))
    ALLOCATE(XH(NX))
    ALLOCATE(XL(NX))
    ALLOCATE(YH(NY))
    ALLOCATE(YL(NY))
    ALLOCATE(DY(NY))
    CALL GBMATRIX(M, N, KL, KU, AH, AL)
    CALL TEST_PREPARE(FNAME, "A (banded matrix)", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", NX, 0, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", NY, 0, YH, YL, DY)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(KL+KU+1,N))
    ALLOCATE(QX(NX))
    ALLOCATE(QY(NY))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDGBMV(TRANS, M, N, KL, KU, ALPHAH, ALPHAL, AH, AL, LDA, XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DGBMV(TRANS, M, N, KL, KU, ALPHAH, AH, LDA, XH, INCX, BETAH, DY, INCY)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QGBMV(TRANS, M, N, KL, KU, QALPHA, QA, LDA, QX, INCX, QBETA, QY, INCY)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDGER_TEST(M, N, INCX, INCY)

    INTEGER, INTENT(IN) :: M, N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "GER"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DA(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    REAL(16) :: QALPHA
#endif

    ALLOCATE(AH(M,N))
    ALLOCATE(AL(M,N))
    ALLOCATE(DA(M,N))
    ALLOCATE(XH(VECTOR_SIZE(M, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(M, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    CALL GEMATRIX(M, N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL, DA)
    CALL TEST_PREPARE(FNAME, "X", M, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(M,N))
    ALLOCATE(QX(VECTOR_SIZE(M, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDGER(M, N, ALPHAH, ALPHAL, XH, XL, INCX, YH, YL, INCY, AH, AL, LDA)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DGER(M, N, ALPHAH, XH, INCX, YH, INCY, DA, LDA)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QGER(M, N, QALPHA, QX, INCX, QY, INCY, QA, LDA)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA)
#endif

    END SUBROUTINE


    SUBROUTINE DDSBMV_TEST(UPLO, N, K, INCX, INCY)

    CHARACTER, INTENT(IN) :: UPLO
    INTEGER, INTENT(IN) :: N, K, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "SBMV"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DY(:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    REAL(16) :: QALPHA, QBETA
#endif

    ALLOCATE(AH(K+1,N))
    ALLOCATE(AL(K+1,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL SBMATRIX(N, K, AH, AL)
    CALL TEST_PREPARE(FNAME, "A (banded matrix)", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL, DY)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(K+1,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDSBMV(UPLO, N, K, ALPHAH, ALPHAL, AH, AL, LDA, XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSBMV(UPLO, N, K, ALPHAH, AH, LDA, XH, INCX, BETAH, DY, INCY)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSBMV(UPLO, N, K, QALPHA, QA, LDA, QX, INCX, QBETA, QY, INCY)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE DDTBMV_TEST(UPLO, TRANS, DIAG, N, K, INCX)

    CHARACTER, INTENT(IN) :: UPLO, TRANS, DIAG
    INTEGER, INTENT(IN) :: N, K, INCX
    CHARACTER(*), PARAMETER :: FNAME = "TBMV"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DX(:)
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:)
#endif

    ALLOCATE(AH(K+1,N))
    ALLOCATE(AL(K+1,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(DX(VECTOR_SIZE(N, INCX)))
    CALL TBMATRIX(UPLO, DIAG, N, K, AH, AL)
    CALL TEST_PREPARE(FNAME, "A (banded matrix)", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL, DX)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(K+1,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDTBMV(UPLO, TRANS, DIAG, N, K, AH, AL, LDA, XH, XL, INCX)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DTBMV(UPLO, TRANS, DIAG, N, K, AH, LDA, DX, INCX)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QTBMV(UPLO, TRANS, DIAG, N, K, QA, LDA, QX, INCX)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX, QX)
#else
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX)
#endif

    END SUBROUTINE


    SUBROUTINE DDTBSV_TEST(UPLO, TRANS, DIAG, N, K, INCX)

    CHARACTER, INTENT(IN) :: UPLO, TRANS, DIAG
    INTEGER, INTENT(IN) :: N, K, INCX
    CHARACTER(*), PARAMETER :: FNAME = "TBSV"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:)
    DOUBLE PRECISION, ALLOCATABLE :: DX(:)
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QX(:)
#endif

    ALLOCATE(AH(K+1,N))
    ALLOCATE(AL(K+1,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(DX(VECTOR_SIZE(N, INCX)))
    CALL TBMATRIX(UPLO, DIAG, N, K, AH, AL)
    CALL TEST_PREPARE(FNAME, "A (banded matrix)", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL, DX)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(K+1,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL DDTBSV(UPLO, TRANS, DIAG, N, K, AH, AL, LDA, XH, XL, INCX)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DTBSV(UPLO, TRANS, DIAG, N, K, AH, LDA, DX, INCX)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QTBSV(UPLO, TRANS, DIAG, N, K, QA, LDA, QX, INCX)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX, QX)
#else
    CALL TEST_VERIFY(FNAME, "X", XH, XL, DX)
#endif

    END SUBROUTINE


    SUBROUTINE ZZGERC_TEST(M, N, INCX, INCY)

    INTEGER, INTENT(IN) :: M, N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "GERC"

    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE COMPLEX, ALLOCATABLE :: DA(:,:)
    DOUBLE COMPLEX :: ALPHAH, ALPHAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    COMPLEX(16) :: QALPHA
#endif

    ALLOCATE(AH(M,N))
    ALLOCATE(AL(M,N))
    ALLOCATE(DA(M,N))
    ALLOCATE(XH(VECTOR_SIZE(M, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(M, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    CALL GEMATRIX(M, N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL, DA)
    CALL TEST_PREPARE(FNAME, "X", M, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(M,N))
    ALLOCATE(QX(VECTOR_SIZE(M, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL ZZGERC(M, N, ALPHAH, ALPHAL, XH, XL, INCX, YH, YL, INCY, AH, AL, LDA)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZGERC(M, N, ALPHAH, XH, INCX, YH, INCY, DA, LDA)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WGERC(M, N, QALPHA, QX, INCX, QY, INCY, QA, LDA)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA)
#endif

    END SUBROUTINE


    SUBROUTINE ZZGERU_TEST(M, N, INCX, INCY)

    INTEGER, INTENT(IN) :: M, N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "GERU"

    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE COMPLEX, ALLOCATABLE :: DA(:,:)
    DOUBLE COMPLEX :: ALPHAH, ALPHAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    COMPLEX(16) :: QALPHA
#endif

    ALLOCATE(AH(M,N))
    ALLOCATE(AL(M,N))
    ALLOCATE(DA(M,N))
    ALLOCATE(XH(VECTOR_SIZE(M, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(M, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    CALL GEMATRIX(M, N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL, DA)
    CALL TEST_PREPARE(FNAME, "X", M, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(M,N))
    ALLOCATE(QX(VECTOR_SIZE(M, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL ZZGERU(M, N, ALPHAH, ALPHAL, XH, XL, INCX, YH, YL, INCY, AH, AL, LDA)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZGERU(M, N, ALPHAH, XH, INCX, YH, INCY, DA, LDA)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WGERU(M, N, QALPHA, QX, INCX, QY, INCY, QA, LDA)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA)
#endif

    END SUBROUTINE


    SUBROUTINE ZZHBMV_TEST(UPLO, N, K, INCX, INCY)

    CHARACTER, INTENT(IN) :: UPLO
    INTEGER, INTENT(IN) :: N, K, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "HBMV"

    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE COMPLEX, ALLOCATABLE :: DY(:)
    DOUBLE COMPLEX :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    COMPLEX(16) :: QALPHA, QBETA
#endif

    ALLOCATE(AH(K+1,N))
    ALLOCATE(AL(K+1,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL HBMATRIX(N, K, AH, AL)
    CALL TEST_PREPARE(FNAME, "A (banded matrix)", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL, DY)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(K+1,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL ZZHBMV(UPLO, N, K, ALPHAH, ALPHAL, AH, AL, LDA, XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZHBMV(UPLO, N, K, ALPHAH, AH, LDA, XH, INCX, BETAH, DY, INCY)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WHBMV(UPLO, N, K, QALPHA, QA, LDA, QX, INCX, QBETA, QY, INCY)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE ZZHEMV_TEST(UPLO, N, INCX, INCY)

    CHARACTER, INTENT(IN) :: UPLO
    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "HEMV"

    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE COMPLEX, ALLOCATABLE :: DY(:)
    DOUBLE COMPLEX :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    COMPLEX(16) :: QALPHA, QBETA
#endif

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    ALLOCATE(DY(VECTOR_SIZE(N, INCY)))
    CALL HEMATRIX(N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL, DY)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(N,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL ZZHEMV(UPLO, N, ALPHAH, ALPHAL, AH, AL, LDA, XH, XL, INCX, BETAH, BETAL, YH, YL, INCY)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZHEMV(UPLO, N, ALPHAH, AH, LDA, XH, INCX, BETAH, DY, INCY)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WHEMV(UPLO, N, QALPHA, QA, LDA, QX, INCX, QBETA, QY, INCY)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY, QY)
#else
    CALL TEST_VERIFY(FNAME, "Y", YH, YL, DY)
#endif

    END SUBROUTINE


    SUBROUTINE ZZHER_TEST(UPLO, N, INCX)

    CHARACTER, INTENT(IN) :: UPLO
    INTEGER, INTENT(IN) :: N, INCX
    CHARACTER(*), PARAMETER :: FNAME = "HER"

    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:)
    DOUBLE COMPLEX, ALLOCATABLE :: DA(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QX(:)
    REAL(16) :: QALPHA
#endif

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    ALLOCATE(DA(N,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    CALL HEMATRIX(N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL, DA)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(N,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL ZZHER(UPLO, N, ALPHAH, ALPHAL, XH, XL, INCX, AH, AL, LDA)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZHER(UPLO, N, ALPHAH, XH, INCX, DA, LDA)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WHER(UPLO, N, QALPHA, QX, INCX, QA, LDA)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA)
#endif

    END SUBROUTINE


    SUBROUTINE ZZHER2_TEST(UPLO, N, INCX, INCY)

    CHARACTER, INTENT(IN) :: UPLO
    INTEGER, INTENT(IN) :: N, INCX, INCY
    CHARACTER(*), PARAMETER :: FNAME = "HER2"

    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), XH(:), XL(:), YH(:), YL(:)
    DOUBLE COMPLEX, ALLOCATABLE :: DA(:,:)
    DOUBLE COMPLEX :: ALPHAH, ALPHAL
    INTEGER :: LDA
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QX(:), QY(:)
    COMPLEX(16) :: QALPHA
#endif

    ALLOCATE(AH(N,N))
    ALLOCATE(AL(N,N))
    ALLOCATE(DA(N,N))
    ALLOCATE(XH(VECTOR_SIZE(N, INCX)))
    ALLOCATE(XL(VECTOR_SIZE(N, INCX)))
    ALLOCATE(YH(VECTOR_SIZE(N, INCY)))
    ALLOCATE(YL(VECTOR_SIZE(N, INCY)))
    CALL HEMATRIX(N, AH, AL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL, DA)
    CALL TEST_PREPARE(FNAME, "X", N, INCX, XH, XL)
    CALL TEST_PREPARE(FNAME, "Y", N, INCY, YH, YL)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(N,N))
    ALLOCATE(QX(VECTOR_SIZE(N, INCX)))
    ALLOCATE(QY(VECTOR_SIZE(N, INCY)))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(XH, XL, QX)
    CALL TEST_PREPARE_Q(YH, YL, QY)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)

    CALL CLOCK_START()
    CALL ZZHER2(UPLO, N, ALPHAH, ALPHAL, XH, XL, INCX, YH, YL, INCY, AH, AL, LDA)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZHER2(UPLO, N, ALPHAH, XH, INCX, YH, INCY, DA, LDA)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WHER2(UPLO, N, QALPHA, QX, INCX, QY, INCY, QA, LDA)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, "A", AH, AL, DA)
#endif

    END SUBROUTINE


    END MODULE
