    MODULE LEVEL3_TESTS

    USE BLAS_UTILITY
    USE TEST_UTILITY
    IMPLICIT NONE

    LOGICAL LSAME
    EXTERNAL LSAME

    CONTAINS


    SUBROUTINE DDGEMM_TEST(TRANSA, TRANSB, M, N, K)

    CHARACTER, INTENT(IN) :: TRANSA, TRANSB
    INTEGER, INTENT(IN) :: M, N, K
    CHARACTER(*), PARAMETER :: FNAME = "GEMM"
    
    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), BH(:,:), BL(:,:), CH(:,:), CL(:,:)
    DOUBLE PRECISION, ALLOCATABLE :: DC(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA, LDB, LDC
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QB(:,:), QC(:,:)
    REAL(16) :: QALPHA, QBETA
#endif

    IF (LSAME(TRANSA, 'N')) THEN
        ALLOCATE(AH(M,K))
        ALLOCATE(AL(M,K))
    ELSE
        ALLOCATE(AH(K,M))
        ALLOCATE(AL(K,M))
    END IF
    IF (LSAME(TRANSB, 'N')) THEN
        ALLOCATE(BH(K,N))
        ALLOCATE(BL(K,N))
    ELSE
        ALLOCATE(BH(N,K))
        ALLOCATE(BL(N,K))
    END IF
    ALLOCATE(CH(M,N))
    ALLOCATE(CL(M,N))
    ALLOCATE(DC(M,N))
    CALL GEMATRIX(SIZE(AH,1), SIZE(AH,2), AH, AL)
    CALL GEMATRIX(SIZE(BH,1), SIZE(BH,2), BH, BL)
    CALL GEMATRIX(M, N, CH, CL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "B", BH, BL)
    CALL TEST_PREPARE(FNAME, "C", CH, CL, DC)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QB(SIZE(BH,1), SIZE(BH,2)))
    ALLOCATE(QC(M,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(BH, BL, QB)
    CALL TEST_PREPARE_Q(CH, CL, QC)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)
    LDB = SIZE(BH, 1)
    LDC = SIZE(CH, 1)

    CALL CLOCK_START()
    CALL DDGEMM(TRANSA, TRANSB, M, N, K, ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DGEMM(TRANSA, TRANSB, M, N, K, ALPHAH, AH, LDA, BH, LDB, BETAH, DC, LDC)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QGEMM(TRANSA, TRANSB, M, N, K, QALPHA, QA, LDA, QB, LDB, QBETA, QC, LDC)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC, QC)
#else
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC)
#endif

    END SUBROUTINE


    SUBROUTINE DDSYMM_TEST(SIDE, UPLO, M, N)

    CHARACTER, INTENT(IN) :: SIDE, UPLO
    INTEGER, INTENT(IN) :: M, N
    CHARACTER(*), PARAMETER :: FNAME = "SYMM"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), BH(:,:), BL(:,:), CH(:,:), CL(:,:)
    DOUBLE PRECISION, ALLOCATABLE :: DC(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA, LDB, LDC
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QB(:,:), QC(:,:)
    REAL(16) :: QALPHA, QBETA
#endif

    IF (LSAME(SIDE, 'L')) THEN
        ALLOCATE(AH(M,M))
        ALLOCATE(AL(M,M))
    ELSE
        ALLOCATE(AH(N,N))
        ALLOCATE(AL(N,N))
    END IF
    ALLOCATE(BH(M,N))
    ALLOCATE(BL(M,N))
    ALLOCATE(CH(M,N))
    ALLOCATE(CL(M,N))
    ALLOCATE(DC(M,N))
    CALL SYMATRIX(SIZE(AH,1), AH, AL)
    CALL GEMATRIX(M, N, BH, BL)
    CALL GEMATRIX(M, N, CH, CL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "B", BH, BL)
    CALL TEST_PREPARE(FNAME, "C", CH, CL, DC)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QB(M,N))
    ALLOCATE(QC(M,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(BH, BL, QB)
    CALL TEST_PREPARE_Q(CH, CL, QC)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)
    LDB = SIZE(BH, 1)
    LDC = SIZE(CH, 1)

    CALL CLOCK_START()
    CALL DDSYMM(SIDE, UPLO, M, N, ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSYMM(SIDE, UPLO, M, N, ALPHAH, AH, LDA, BH, LDB, BETAH, DC, LDC)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSYMM(SIDE, UPLO, M, N, QALPHA, QA, LDA, QB, LDB, QBETA, QC, LDC)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC, QC)
#else
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC)
#endif

    END SUBROUTINE


    SUBROUTINE DDSYR2K_TEST(UPLO, TRANS, N, K)

    CHARACTER, INTENT(IN) :: UPLO, TRANS
    INTEGER, INTENT(IN) :: N, K
    CHARACTER(*), PARAMETER :: FNAME = "SYR2K"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), BH(:,:), BL(:,:), CH(:,:), CL(:,:)
    DOUBLE PRECISION, ALLOCATABLE :: DC(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA, LDB, LDC
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QB(:,:), QC(:,:)
    REAL(16) :: QALPHA, QBETA
#endif

    IF (LSAME(TRANS, 'N')) THEN
        ALLOCATE(AH(N,K))
        ALLOCATE(AL(N,K))
        ALLOCATE(BH(N,K))
        ALLOCATE(BL(N,K))
    ELSE
        ALLOCATE(AH(K,N))
        ALLOCATE(AL(K,N))
        ALLOCATE(BH(K,N))
        ALLOCATE(BL(K,N))
    END IF
    ALLOCATE(CH(N,N))
    ALLOCATE(CL(N,N))
    ALLOCATE(DC(N,N))
    CALL GEMATRIX(SIZE(AH,1), SIZE(AH,2), AH, AL)
    CALL GEMATRIX(SIZE(BH,1), SIZE(BH,2), BH, BL)
    CALL SYMATRIX(N, CH, CL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "B", BH, BL)
    CALL TEST_PREPARE(FNAME, "C", CH, CL, DC)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QB(SIZE(BH,1), SIZE(BH,2)))
    ALLOCATE(QC(N,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(BH, BL, QB)
    CALL TEST_PREPARE_Q(CH, CL, QC)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)
    LDB = SIZE(BH, 1)
    LDC = SIZE(CH, 1)

    CALL CLOCK_START()
    CALL DDSYR2K(UPLO, TRANS, N, K, ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSYR2K(UPLO, TRANS, N, K, ALPHAH, AH, LDA, BH, LDB, BETAH, DC, LDC)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSYR2K(UPLO, TRANS, N, K, QALPHA, QA, LDA, QB, LDB, QBETA, QC, LDC)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC, QC)
#else
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC)
#endif

    END SUBROUTINE


    SUBROUTINE DDSYRK_TEST(UPLO, TRANS, N, K)

    CHARACTER, INTENT(IN) :: UPLO, TRANS
    INTEGER, INTENT(IN) :: N, K
    CHARACTER(*), PARAMETER :: FNAME = "SYRK"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), CH(:,:), CL(:,:)
    DOUBLE PRECISION, ALLOCATABLE :: DC(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA, LDC
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QC(:,:)
    REAL(16) :: QALPHA, QBETA
#endif

    IF (LSAME(TRANS, 'N')) THEN
        ALLOCATE(AH(N,K))
        ALLOCATE(AL(N,K))
    ELSE
        ALLOCATE(AH(K,N))
        ALLOCATE(AL(K,N))
    END IF
    ALLOCATE(CH(N,N))
    ALLOCATE(CL(N,N))
    ALLOCATE(DC(N,N))
    CALL GEMATRIX(SIZE(AH,1), SIZE(AH,2), AH, AL)
    CALL SYMATRIX(N, CH, CL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "C", CH, CL, DC)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QC(N,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(CH, CL, QC)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)
    LDC = SIZE(CH, 1)

    CALL CLOCK_START()
    CALL DDSYRK(UPLO, TRANS, N, K, ALPHAH, ALPHAL, AH, AL, LDA, BETAH, BETAL, CH, CL, LDC)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DSYRK(UPLO, TRANS, N, K, ALPHAH, AH, LDA, BETAH, DC, LDC)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QSYRK(UPLO, TRANS, N, K, QALPHA, QA, LDA, QBETA, QC, LDC)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC, QC)
#else
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC)
#endif

    END SUBROUTINE


    SUBROUTINE DDTRMM_TEST(SIDE, UPLO, TRANSA, DIAG, M, N)

    CHARACTER, INTENT(IN) :: SIDE, UPLO, TRANSA, DIAG
    INTEGER, INTENT(IN) :: M, N
    CHARACTER(*), PARAMETER :: FNAME = "TRMM"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), BH(:,:), BL(:,:)
    DOUBLE PRECISION, ALLOCATABLE :: DB(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL
    INTEGER :: LDA, LDB
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QB(:,:)
    REAL(16) :: QALPHA
#endif

    IF (LSAME(SIDE, 'L')) THEN
        ALLOCATE(AH(M,M))
        ALLOCATE(AL(M,M))
    ELSE
        ALLOCATE(AH(N,N))
        ALLOCATE(AL(N,N))
    END IF
    ALLOCATE(BH(M,N))
    ALLOCATE(BL(M,N))
    ALLOCATE(DB(M,N))
    CALL SYMATRIX(SIZE(AH,1), AH, AL)
    CALL GEMATRIX(M, N, BH, BL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "B", BH, BL, DB)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QB(M,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(BH, BL, QB)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)
    LDB = SIZE(BH, 1)

    CALL CLOCK_START()
    CALL DDTRMM(SIDE, UPLO, TRANSA, DIAG, M, N, ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DTRMM(SIDE, UPLO, TRANSA, DIAG, M, N, ALPHAH, AH, LDA, DB, LDB)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QTRMM(SIDE, UPLO, TRANSA, DIAG, M, N, QALPHA, QA, LDA, QB, LDB)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "B", BH, BL, DB, QB)
#else
    CALL TEST_VERIFY(FNAME, "B", BH, BL, DB)
#endif

    END SUBROUTINE


    SUBROUTINE DDTRSM_TEST(SIDE, UPLO, TRANSA, DIAG, M, N)

    CHARACTER, INTENT(IN) :: SIDE, UPLO, TRANSA, DIAG
    INTEGER, INTENT(IN) :: M, N
    CHARACTER(*), PARAMETER :: FNAME = "TRSM"

    DOUBLE PRECISION, ALLOCATABLE :: AH(:,:), AL(:,:), BH(:,:), BL(:,:)
    DOUBLE PRECISION, ALLOCATABLE :: DB(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL
    INTEGER :: LDA, LDB
#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:), QB(:,:)
    REAL(16) :: QALPHA
#endif

    IF (LSAME(SIDE, 'L')) THEN
        ALLOCATE(AH(M,M))
        ALLOCATE(AL(M,M))
    ELSE
        ALLOCATE(AH(N,N))
        ALLOCATE(AL(N,N))
    END IF
    ALLOCATE(BH(M,N))
    ALLOCATE(BL(M,N))
    ALLOCATE(DB(M,N))
    CALL SYMATRIX(SIZE(AH,1), AH, AL)
    CALL GEMATRIX(M, N, BH, BL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "B", BH, BL, DB)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QB(M,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(BH, BL, QB)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
#endif

    LDA = SIZE(AH, 1)
    LDB = SIZE(BH, 1)

    CALL CLOCK_START()
    CALL DDTRSM(SIDE, UPLO, TRANSA, DIAG, M, N, ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB)
    CALL CLOCK_STOP("DD"//FNAME)

    CALL CLOCK_START()
    CALL DTRSM(SIDE, UPLO, TRANSA, DIAG, M, N, ALPHAH, AH, LDA, DB, LDB)
    CALL CLOCK_STOP("D"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL QTRSM(SIDE, UPLO, TRANSA, DIAG, M, N, QALPHA, QA, LDA, QB, LDB)
    CALL CLOCK_STOP("Q"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "B", BH, BL, DB, QB)
#else
    CALL TEST_VERIFY(FNAME, "B", BH, BL, DB)
#endif

    END SUBROUTINE


    SUBROUTINE ZZHEMM_TEST(SIDE, UPLO, M, N)

    CHARACTER, INTENT(IN) :: SIDE, UPLO
    INTEGER, INTENT(IN) :: M, N
    CHARACTER(*), PARAMETER :: FNAME = "HEMM"

    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), BH(:,:), BL(:,:), CH(:,:), CL(:,:)
    DOUBLE COMPLEX, ALLOCATABLE :: DC(:,:)
    DOUBLE COMPLEX :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA, LDB, LDC
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QB(:,:), QC(:,:)
    COMPLEX(16) :: QALPHA, QBETA
#endif

    IF (LSAME(SIDE, 'L')) THEN
        ALLOCATE(AH(M,M))
        ALLOCATE(AL(M,M))
    ELSE
        ALLOCATE(AH(N,N))
        ALLOCATE(AL(N,N))
    END IF
    ALLOCATE(BH(M,N))
    ALLOCATE(BL(M,N))
    ALLOCATE(CH(M,N))
    ALLOCATE(CL(M,N))
    ALLOCATE(DC(M,N))
    CALL HEMATRIX(SIZE(AH,1), AH, AL)
    CALL GEMATRIX(M, N, BH, BL)
    CALL GEMATRIX(M, N, CH, CL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "B", BH, BL)
    CALL TEST_PREPARE(FNAME, "C", CH, CL, DC)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QB(M,N))
    ALLOCATE(QC(M,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(BH, BL, QB)
    CALL TEST_PREPARE_Q(CH, CL, QC)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)
    LDB = SIZE(BH, 1)
    LDC = SIZE(CH, 1)

    CALL CLOCK_START()
    CALL ZZHEMM(SIDE, UPLO, M, N, ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZHEMM(SIDE, UPLO, M, N, ALPHAH, AH, LDA, BH,  LDB, BETAH, DC, LDC)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WHEMM(SIDE, UPLO, M, N, QALPHA, QA, LDA, QB,  LDB, QBETA, QC, LDC)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC, QC)
#else
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC)
#endif

    END SUBROUTINE


    SUBROUTINE ZZHER2K_TEST(UPLO, TRANS, N, K)

    CHARACTER, INTENT(IN) :: UPLO, TRANS
    INTEGER, INTENT(IN) :: N, K
    CHARACTER(*), PARAMETER :: FNAME = "HER2K"
    
    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), BH(:,:), BL(:,:), CH(:,:), CL(:,:)
    DOUBLE COMPLEX, ALLOCATABLE :: DC(:,:)
    DOUBLE COMPLEX :: ALPHAH, ALPHAL
    DOUBLE PRECISION :: BETAH, BETAL
    INTEGER :: LDA, LDB, LDC
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QB(:,:), QC(:,:)
    COMPLEX(16) :: QALPHA
    REAL(16) :: QBETA
#endif

    IF (LSAME(TRANS, 'N')) THEN
        ALLOCATE(AH(N,K))
        ALLOCATE(AL(N,K))
        ALLOCATE(BH(N,K))
        ALLOCATE(BL(N,K))
    ELSE
        ALLOCATE(AH(K,N))
        ALLOCATE(AL(K,N))
        ALLOCATE(BH(K,N))
        ALLOCATE(BL(K,N))
    END IF
    ALLOCATE(CH(N,N))
    ALLOCATE(CL(N,N))
    ALLOCATE(DC(N,N))
    CALL GEMATRIX(SIZE(AH,1), SIZE(AH,2), AH, AL)
    CALL GEMATRIX(SIZE(BH,1), SIZE(BH,2), BH, BL)
    CALL HEMATRIX(N, CH, CL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "B", BH, BL)
    CALL TEST_PREPARE(FNAME, "C", CH, CL, DC)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QB(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QC(N,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(BH, BL, QB)
    CALL TEST_PREPARE_Q(CH, CL, QC)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)
    LDB = SIZE(BH, 1)
    LDC = SIZE(CH, 1)

    CALL CLOCK_START()
    CALL ZZHER2K(UPLO, TRANS, N, K, ALPHAH, ALPHAL, AH, AL, LDA, BH, BL, LDB, BETAH, BETAL, CH, CL, LDC)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZHER2K(UPLO, TRANS, N, K, ALPHAH, AH, LDA, BH, LDB, BETAH, DC, LDC)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WHER2K(UPLO, TRANS, N, K, QALPHA, QA, LDA, QB, LDB, QBETA, QC, LDC)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC, QC)
#else
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC)
#endif

    END SUBROUTINE


    SUBROUTINE ZZHERK_TEST(UPLO, TRANS, N, K)

    CHARACTER, INTENT(IN) :: UPLO, TRANS
    INTEGER, INTENT(IN) :: N, K
    CHARACTER(*), PARAMETER :: FNAME = "HERK"
    
    DOUBLE COMPLEX, ALLOCATABLE :: AH(:,:), AL(:,:), CH(:,:), CL(:,:)
    DOUBLE COMPLEX, ALLOCATABLE :: DC(:,:)
    DOUBLE PRECISION :: ALPHAH, ALPHAL, BETAH, BETAL
    INTEGER :: LDA, LDC
#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:), QC(:,:)
    REAL(16) :: QALPHA, QBETA
#endif

    IF (LSAME(TRANS, 'N')) THEN
        ALLOCATE(AH(N,K))
        ALLOCATE(AL(N,K))
    ELSE
        ALLOCATE(AH(K,N))
        ALLOCATE(AL(K,N))
    END IF
    ALLOCATE(CH(N,N))
    ALLOCATE(CL(N,N))
    ALLOCATE(DC(N,N))
    CALL GEMATRIX(SIZE(AH,1), SIZE(AH,2), AH, AL)
    CALL HEMATRIX(N, CH, CL)
    CALL TEST_PREPARE(FNAME, "A", AH, AL)
    CALL TEST_PREPARE(FNAME, "C", CH, CL, DC)
    CALL TEST_PREPARE_ALPHA(FNAME, ALPHAH, ALPHAL)
    CALL TEST_PREPARE_BETA(FNAME, BETAH, BETAL)
#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    ALLOCATE(QC(N,N))
    CALL TEST_PREPARE_Q(AH, AL, QA)
    CALL TEST_PREPARE_Q(CH, CL, QC)
    CALL TEST_PREPARE_Q(ALPHAH, ALPHAL, QALPHA)
    CALL TEST_PREPARE_Q(BETAH, BETAL, QBETA)
#endif

    LDA = SIZE(AH, 1)
    LDC = SIZE(CH, 1)

    CALL CLOCK_START()
    CALL ZZHERK(UPLO, TRANS, N, K, ALPHAH, ALPHAL, AH, AL, LDA, BETAH, BETAL, CH, CL, LDC)
    CALL CLOCK_STOP("ZZ"//FNAME)

    CALL CLOCK_START()
    CALL ZHERK(UPLO, TRANS, N, K, ALPHAH, AH, LDA, BETAH, DC, LDC)
    CALL CLOCK_STOP("Z"//FNAME)

#ifdef QUAD_PRECISION
    CALL CLOCK_START()
    CALL WHERK(UPLO, TRANS, N, K, QALPHA, QA, LDA, QBETA, QC, LDC)
    CALL CLOCK_STOP("W"//FNAME)
#endif

#ifdef QUAD_PRECISION
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC, QC)
#else
    CALL TEST_VERIFY(FNAME, "C", CH, CL, DC)
#endif

    END SUBROUTINE


    END MODULE
