    MODULE QPCONVERT

    IMPLICIT NONE

    INTERFACE Q2DD
    MODULE PROCEDURE Q2DD_SCALAR, Q2DD_ARRAY, Q2DD_MATRIX
    END INTERFACE

    INTERFACE DD2Q
    MODULE PROCEDURE DD2Q_SCALAR, DD2Q_ARRAY, DD2Q_MATRIX
    END INTERFACE

    INTERFACE W2ZZ
    MODULE PROCEDURE W2ZZ_SCALAR, W2ZZ_ARRAY, W2ZZ_MATRIX
    END INTERFACE

    INTERFACE ZZ2W
    MODULE PROCEDURE ZZ2W_SCALAR, ZZ2W_ARRAY, ZZ2W_MATRIX
    END INTERFACE

    CONTAINS


    SUBROUTINE Q2DD_SCALAR(Q, DH, DL)

    REAL(16), INTENT(IN) :: Q
    DOUBLE PRECISION, INTENT(OUT) :: DH, DL

    REAL(16) :: QTEMPH, QTEMPL

    DH = Q
    QTEMPH = DH
    QTEMPL = Q - QTEMPH
    DL = QTEMPL

    END SUBROUTINE


    SUBROUTINE Q2DD_ARRAY(Q, DH, DL)

    REAL(16), INTENT(IN) :: Q(:)
    DOUBLE PRECISION, INTENT(OUT) :: DH(:), DL(:)

    INTEGER :: I

    IF (SIZE(Q, 1) .LT. 1) RETURN
    DO I = 1, SIZE(Q, 1)
        CALL Q2DD_SCALAR(Q(I), DH(I), DL(I))
    END DO

    END SUBROUTINE


    SUBROUTINE Q2DD_MATRIX(Q, DH, DL)

    REAL(16), INTENT(IN) :: Q(:,:)
    DOUBLE PRECISION, INTENT(OUT) :: DH(:,:), DL(:,:)

    INTEGER :: I

    IF (SIZE(Q, 1) .LT. 1) RETURN
    DO I = 1, SIZE(Q, 1)
        CALL Q2DD_ARRAY(Q(I,:), DH(I,:), DL(I,:))
    END DO

    END SUBROUTINE


    SUBROUTINE DD2Q_SCALAR(DH, DL, Q)

    DOUBLE PRECISION, INTENT(IN) :: DH, DL
    REAL(16), INTENT(OUT) :: Q

    REAL(16) :: QTEMPH, QTEMPL

    QTEMPH = DH
    QTEMPL = DL
    Q = QTEMPL + QTEMPH    ! DO NOT DESCRIBE: Q = DL + DH

    END SUBROUTINE


    SUBROUTINE DD2Q_ARRAY(DH, DL, Q)

    DOUBLE PRECISION, INTENT(IN) :: DH(:), DL(:)
    REAL(16), INTENT(OUT) :: Q(:)

    INTEGER :: I

    IF (SIZE(DH, 1) .LT. 1) RETURN
    DO I = 1, SIZE(DH, 1)
        CALL DD2Q_SCALAR(DH(I), DL(I), Q(I))
    END DO

    END SUBROUTINE


    SUBROUTINE DD2Q_MATRIX(DH, DL, Q)

    DOUBLE PRECISION, INTENT(IN) :: DH(:,:), DL(:,:)
    REAL(16), INTENT(OUT) :: Q(:,:)

    INTEGER :: I

    IF (SIZE(DH, 1) .LT. 1) RETURN
    DO I = 1, SIZE(DH, 1)
        CALL DD2Q_ARRAY(DH(I,:), DL(I,:), Q(I,:))
    END DO

    END SUBROUTINE


    SUBROUTINE W2ZZ_SCALAR(W, ZH, ZL)

    COMPLEX(16), INTENT(IN) :: W
    DOUBLE COMPLEX, INTENT(OUT) :: ZH, ZL

    COMPLEX(16) WTEMPH, WTEMPL

    ZH = W
    WTEMPH = ZH
    WTEMPL = W - WTEMPH
    ZL = WTEMPL

    END SUBROUTINE


    SUBROUTINE W2ZZ_ARRAY(W, ZH, ZL)

    COMPLEX(16), INTENT(IN) :: W(:)
    DOUBLE COMPLEX, INTENT(OUT) :: ZH(:), ZL(:)

    INTEGER :: I

    IF (SIZE(W, 1) .LT. 1) RETURN
    DO I = 1, SIZE(W, 1)
        CALL W2ZZ_SCALAR(W(I), ZH(I), ZL(I))
    END DO

    END SUBROUTINE


    SUBROUTINE W2ZZ_MATRIX(W, ZH, ZL)

    COMPLEX(16), INTENT(IN) :: W(:,:)
    DOUBLE COMPLEX, INTENT(OUT) :: ZH(:,:), ZL(:,:)

    INTEGER :: I

    IF (SIZE(W, 1) .LT. 1) RETURN
    DO I = 1, SIZE(W, 1)
        CALL W2ZZ_ARRAY(W(I,:), ZH(I,:), ZL(I,:))
    END DO

    END SUBROUTINE


    SUBROUTINE ZZ2W_SCALAR(ZH, ZL, W)

    DOUBLE COMPLEX, INTENT(IN) :: ZH, ZL
    COMPLEX(16), INTENT(OUT) :: W

    COMPLEX(16) :: WTEMPH, WTEMPL

    WTEMPH = ZH
    WTEMPL = ZL
    W = WTEMPL + WTEMPH    ! DO NOT DESCRIBE: W = ZL + ZH

    END SUBROUTINE


    SUBROUTINE ZZ2W_ARRAY(ZH, ZL, W)

    DOUBLE COMPLEX, INTENT(IN) :: ZH(:), ZL(:)
    COMPLEX(16), INTENT(OUT) :: W(:)

    INTEGER :: I

    IF (SIZE(ZH, 1) .LT. 1) RETURN
    DO I = 1, SIZE(ZH, 1)
        CALL ZZ2W_SCALAR(ZH(I), ZL(I), W(I))
    END DO

    END SUBROUTINE


    SUBROUTINE ZZ2W_MATRIX(ZH, ZL, W)

    DOUBLE COMPLEX, INTENT(IN) :: ZH(:,:), ZL(:,:)
    COMPLEX(16), INTENT(OUT) :: W(:,:)

    INTEGER :: I

    IF (SIZE(ZH, 1) .LT. 1) RETURN
    DO I = 1, SIZE(ZH, 1)
        CALL ZZ2W_ARRAY(ZH(I,:), ZL(I,:), W(I,:))
    END DO

    END SUBROUTINE


    END MODULE
