    MODULE TEST_UTILITY

    USE BLAS_UTILITY
#ifdef QUAD_PRECISION
    USE QPCONVERT
#endif
    IMPLICIT NONE

    INTERFACE TEST_PREPARE
    MODULE PROCEDURE TEST_PREPARE_SCALAR_R, TEST_PREPARE_SCALAR_C
    MODULE PROCEDURE TEST_PREPARE_VECTOR_R, TEST_PREPARE_VECTOR_C
    MODULE PROCEDURE TEST_PREPARE_MATRIX_R, TEST_PREPARE_MATRIX_C
    END INTERFACE

    INTERFACE TEST_PREPARE_ALPHA
    MODULE PROCEDURE TEST_PREPARE_ALPHA_R, TEST_PREPARE_ALPHA_C
    END INTERFACE

    INTERFACE TEST_PREPARE_BETA
    MODULE PROCEDURE TEST_PREPARE_BETA_R, TEST_PREPARE_BETA_C
    END INTERFACE

#ifdef QUAD_PRECISION
    INTERFACE TEST_PREPARE_Q
    MODULE PROCEDURE TEST_PREPARE_SCALAR_QR, TEST_PREPARE_SCALAR_QC
    MODULE PROCEDURE TEST_PREPARE_VECTOR_QR, TEST_PREPARE_VECTOR_QC
    MODULE PROCEDURE TEST_PREPARE_MATRIX_QR, TEST_PREPARE_MATRIX_QC
    END INTERFACE
#endif

    INTERFACE TEST_PRINT
    MODULE PROCEDURE TEST_PRINT_SCALAR_D, TEST_PRINT_SCALAR_Z
    MODULE PROCEDURE TEST_PRINT_VECTOR_D, TEST_PRINT_VECTOR_Z
    MODULE PROCEDURE TEST_PRINT_MATRIX_D, TEST_PRINT_MATRIX_Z
    MODULE PROCEDURE TEST_PRINT_MATRIX_DD, TEST_PRINT_MATRIX_ZZ
#ifdef QUAD_PRECISION
    MODULE PROCEDURE TEST_PRINT_SCALAR_Q, TEST_PRINT_SCALAR_W
    MODULE PROCEDURE TEST_PRINT_VECTOR_Q, TEST_PRINT_VECTOR_W
    MODULE PROCEDURE TEST_PRINT_MATRIX_Q, TEST_PRINT_MATRIX_W
#endif
    END INTERFACE

    INTERFACE TEST_VERIFY
    MODULE PROCEDURE TEST_VERIFY_SCALAR_I
    MODULE PROCEDURE TEST_VERIFY_SCALAR_R, TEST_VERIFY_SCALAR_C
    MODULE PROCEDURE TEST_VERIFY_VECTOR_R, TEST_VERIFY_VECTOR_C
    MODULE PROCEDURE TEST_VERIFY_MATRIX_R, TEST_VERIFY_MATRIX_C
    END INTERFACE

    INTERFACE PRINT_DISTANCE
    MODULE PROCEDURE PRINT_DISTANCE_I_I_SCALAR
    MODULE PROCEDURE PRINT_DISTANCE_D_D_SCALAR, PRINT_DISTANCE_D_DD_SCALAR
    MODULE PROCEDURE PRINT_DISTANCE_Z_Z_SCALAR, PRINT_DISTANCE_Z_ZZ_SCALAR
    MODULE PROCEDURE PRINT_DISTANCE_D_D_VECTOR, PRINT_DISTANCE_D_DD_VECTOR
    MODULE PROCEDURE PRINT_DISTANCE_Z_Z_VECTOR, PRINT_DISTANCE_Z_ZZ_VECTOR
    MODULE PROCEDURE PRINT_DISTANCE_D_D_MATRIX, PRINT_DISTANCE_D_DD_MATRIX
    MODULE PROCEDURE PRINT_DISTANCE_Z_Z_MATRIX, PRINT_DISTANCE_Z_ZZ_MATRIX
#ifdef QUAD_PRECISION
    MODULE PROCEDURE PRINT_DISTANCE_Q_D_SCALAR, PRINT_DISTANCE_Q_DD_SCALAR
    MODULE PROCEDURE PRINT_DISTANCE_W_Z_SCALAR, PRINT_DISTANCE_W_ZZ_SCALAR
    MODULE PROCEDURE PRINT_DISTANCE_Q_D_VECTOR, PRINT_DISTANCE_Q_DD_VECTOR
    MODULE PROCEDURE PRINT_DISTANCE_W_Z_VECTOR, PRINT_DISTANCE_W_ZZ_VECTOR
    MODULE PROCEDURE PRINT_DISTANCE_Q_D_MATRIX, PRINT_DISTANCE_Q_DD_MATRIX
    MODULE PROCEDURE PRINT_DISTANCE_W_Z_MATRIX, PRINT_DISTANCE_W_ZZ_MATRIX
#endif
    END INTERFACE

#ifdef ISNAN_NOT_FOUND
    INTERFACE ISNAN
    MODULE PROCEDURE ISNAN_D
#ifdef QUAD_PRECISION
    MODULE PROCEDURE ISNAN_Q
#endif
    END INTERFACE
#endif

    LOGICAL, PRIVATE :: TEST_MODE_BENCHMARK = .FALSE.
    LOGICAL, PRIVATE :: TEST_MODE_VERIFY = .FALSE.
    LOGICAL, PRIVATE :: TEST_MODE_VERBOSE = .FALSE.
    INTEGER, PRIVATE :: TIME

    INTEGER, PUBLIC :: FIXED_ALPHA = -1 ! not fixed
    INTEGER, PUBLIC :: FIXED_BETA = -1 ! not fixed


    CONTAINS


    SUBROUTINE INIT_RANDOM_SEED()

    INTEGER I, N, CLOCK
    INTEGER, ALLOCATABLE :: SEED(:)

    CALL RANDOM_SEED(SIZE=N)
    ALLOCATE(SEED(N))

    CALL SYSTEM_CLOCK(COUNT=CLOCK)

    SEED = CLOCK + 37 * (/ (i - 1, i = 1, n) /)
    CALL RANDOM_SEED(PUT = SEED)

    END SUBROUTINE


    SUBROUTINE CHANGE_INTEGER(KEY, VARIABLE)

    CHARACTER(*), INTENT(IN) :: KEY
    INTEGER, INTENT(INOUT) :: VARIABLE

    PRINT *, KEY, " =", VARIABLE
    WRITE (*,fmt='(a,a,a)', advance='no') "Please input ", KEY, ": "
    READ *, VARIABLE

    END SUBROUTINE


    SUBROUTINE CHANGE_CHARACTER(KEY, VARIABLE)

    CHARACTER(*), INTENT(IN) :: KEY
    CHARACTER(1), INTENT(INOUT) :: VARIABLE

    PRINT *, KEY, " = ", VARIABLE
    WRITE (*,fmt='(a,a,a)', advance='no') "Please input ", KEY, ": "
    READ *, VARIABLE

    END SUBROUTINE


    SUBROUTINE TOGGLE_MODE_BENCHMARK()

    TEST_MODE_BENCHMARK = .NOT. TEST_MODE_BENCHMARK

    IF (TEST_MODE_BENCHMARK) THEN
        PRINT *, "[BENCHMARK MODE] ON"
    ELSE
        PRINT *, "[BENCHMARK MODE] OFF"
    END IF

    END SUBROUTINE


    SUBROUTINE TOGGLE_MODE_VERIFY()

    TEST_MODE_VERIFY = .NOT. TEST_MODE_VERIFY

    IF (TEST_MODE_VERIFY) THEN
        PRINT *, "[VERIFY MODE] ON"
    ELSE
        PRINT *, "[VERIFY MODE] OFF"
    END IF

    END SUBROUTINE


    SUBROUTINE TOGGLE_MODE_VERBOSE()

    TEST_MODE_VERBOSE = .NOT. TEST_MODE_VERBOSE

    IF (TEST_MODE_VERBOSE) THEN
        PRINT *, "[VERBOSE MODE] ON"
    ELSE
        PRINT *, "[VERBOSE MODE] OFF"
    END IF

    END SUBROUTINE

    SUBROUTINE TEST_PREPARE_SCALAR_R(FNAME, VNAME, DDH, DDL, D)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE PRECISION, INTENT(OUT) :: DDH, DDL
    DOUBLE PRECISION, INTENT(OUT), OPTIONAL :: D

#ifdef QUAD_PRECISION
    REAL(16) :: Q
#endif

#ifdef QUAD_PRECISION
    CALL RANDOM_NUMBER(Q)
    CALL Q2DD(Q, DDH, DDL)
    CALL TEST_PRINT(FNAME, VNAME, "INPUT", Q)
#else
    CALL RANDOM_NUMBER(DDH)
    DDL = 0.0D0
    CALL TEST_PRINT(FNAME, VNAME, "INPUT", DDH)
#endif

    IF (PRESENT(D)) THEN
        D = DDH
    END IF

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_SCALAR_C(FNAME, VNAME, DDH, DDL, D)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE COMPLEX, INTENT(OUT) :: DDH, DDL
    DOUBLE COMPLEX, INTENT(OUT), OPTIONAL :: D

#ifdef QUAD_PRECISION
    COMPLEX(16) :: Q
#endif

#ifdef QUAD_PRECISION
    CALL RANDOM_COMPLEX(Q)
    CALL W2ZZ(Q, DDH, DDL)
    CALL TEST_PRINT(FNAME, VNAME, "INPUT", Q)
#else
    CALL RANDOM_COMPLEX(DDH)
    DDL = 0.0D0
    CALL TEST_PRINT(FNAME, VNAME, "INPUT", DDH)
#endif

    IF (PRESENT(D)) THEN
        D = DDH
    END IF

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_VECTOR_R(FNAME, VNAME, N, INC, DDH, DDL, D)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    INTEGER, INTENT(IN) :: N, INC
    DOUBLE PRECISION, INTENT(OUT) :: DDH(:), DDL(:)
    DOUBLE PRECISION, INTENT(OUT), OPTIONAL :: D(:)

#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: Q(:)
#endif
    INTEGER :: LENGTH

    IF (INC .NE. 0) THEN
        LENGTH = VECTOR_SIZE(N, INC)
    ELSE
        LENGTH = N
    END IF
    CALL SIZE_CHECK(DDH, DDL, LENGTH)

#ifdef QUAD_PRECISION
    ALLOCATE(Q(LENGTH))
    CALL RANDOM_NUMBER(Q)
    CALL Q2DD(Q, DDH, DDL)
    CALL TEST_PRINT(FNAME, VNAME, "INPUT", Q)
#else
    CALL RANDOM_NUMBER(DDH)
    DDL = 0.0D0
    CALL TEST_PRINT(FNAME, VNAME, "INPUT", DDH)
#endif

    IF (PRESENT(D)) THEN
        CALL SIZE_CHECK(DDH, D, LENGTH)
        D(:) = DDH(:)
    END IF

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_VECTOR_C(FNAME, VNAME, N, INC, DDH, DDL, D)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    INTEGER, INTENT(IN) :: N, INC
    DOUBLE COMPLEX, INTENT(OUT) :: DDH(:), DDL(:)
    DOUBLE COMPLEX, INTENT(OUT), OPTIONAL :: D(:)

#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: Q(:)
#endif
    INTEGER :: LENGTH

    IF (INC .NE. 0) THEN
        LENGTH = VECTOR_SIZE(N, INC)
    ELSE
        LENGTH = N
    END IF
    CALL SIZE_CHECK(DDH, DDL, LENGTH)

#ifdef QUAD_PRECISION
    ALLOCATE(Q(LENGTH))
    CALL RANDOM_COMPLEX(Q)
    CALL W2ZZ(Q, DDH, DDL)
    CALL TEST_PRINT(FNAME, VNAME, "INPUT", Q)
#else
    CALL RANDOM_COMPLEX(DDH)
    DDL = CMPLX(0.0D0, 0.0D0)
    CALL TEST_PRINT(FNAME, VNAME, "INPUT", DDH)
#endif

    IF (PRESENT(D)) THEN
        CALL SIZE_CHECK(DDH, D, LENGTH)
        D(:) = DDH(:)
    END IF

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_MATRIX_R(FNAME, VNAME, DDH, DDL, D)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE PRECISION, INTENT(IN) :: DDH(:,:), DDL(:,:)
    DOUBLE PRECISION, INTENT(OUT), OPTIONAL :: D(:,:)

    CALL TEST_PRINT(FNAME, VNAME, "INPUT", DDH, DDL)

    IF (PRESENT(D)) THEN
        CALL SIZE_CHECK(DDH, D, SIZE(DDH,1), SIZE(DDH,2))
        D(:,:) = DDH(:,:)
    END IF

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_MATRIX_C(FNAME, VNAME, DDH, DDL, D)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE COMPLEX, INTENT(IN) :: DDH(:,:), DDL(:,:)
    DOUBLE COMPLEX, INTENT(OUT), OPTIONAL :: D(:,:)

    CALL TEST_PRINT(FNAME, VNAME, "INPUT", DDH, DDL)

    IF (PRESENT(D)) THEN
        CALL SIZE_CHECK(DDH, D, SIZE(DDH,1), SIZE(DDH,2))
        D(:,:) = DDH(:,:)
    END IF

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_ALPHA_R(FNAME, DDH, DDL)

    CHARACTER(*), INTENT(IN) :: FNAME
    DOUBLE PRECISION, INTENT(OUT) :: DDH, DDL

    IF (FIXED_ALPHA .EQ. 0) THEN
        DDH = 0d0
    ELSE IF (FIXED_ALPHA .EQ. 1) THEN
        DDH = 1d0
    ELSE
        CALL TEST_PREPARE(FNAME, "ALPHA", DDH, DDL)
        RETURN
    END IF

    DDL = 0d0
    CALL TEST_PRINT(FNAME, "ALPHA", "INPUT", DDH)

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_ALPHA_C(FNAME, DDH, DDL)

    CHARACTER(*), INTENT(IN) :: FNAME
    DOUBLE COMPLEX, INTENT(OUT) :: DDH, DDL

    IF (FIXED_ALPHA .EQ. 0) THEN
        DDH = 0d0
    ELSE IF (FIXED_ALPHA .EQ. 1) THEN
        DDH = 1d0
    ELSE
        CALL TEST_PREPARE(FNAME, "ALPHA", DDH, DDL)
        RETURN
    END IF

    DDL = 0d0
    CALL TEST_PRINT(FNAME, "ALPHA", "INPUT", DDH)

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_BETA_R(FNAME, DDH, DDL)

    CHARACTER(*), INTENT(IN) :: FNAME
    DOUBLE PRECISION, INTENT(OUT) :: DDH, DDL

    IF (FIXED_BETA .EQ. 0) THEN
        DDH = 0d0
    ELSE IF (FIXED_BETA .EQ. 1) THEN
        DDH = 1d0
    ELSE
        CALL TEST_PREPARE(FNAME, "BETA", DDH, DDL)
        RETURN
    END IF

    DDL = 0d0
    CALL TEST_PRINT(FNAME, "BETA", "BETA", DDH)

    END SUBROUTINE


    SUBROUTINE TEST_PREPARE_BETA_C(FNAME, DDH, DDL)

    CHARACTER(*), INTENT(IN) :: FNAME
    DOUBLE COMPLEX, INTENT(OUT) :: DDH, DDL

    IF (FIXED_BETA .EQ. 0) THEN
        DDH = 0d0
    ELSE IF (FIXED_BETA .EQ. 1) THEN
        DDH = 1d0
    ELSE
        CALL TEST_PREPARE(FNAME, "BETA", DDH, DDL)
        RETURN
    END IF

    DDL = 0d0
    CALL TEST_PRINT(FNAME, "BETA", "BETA", DDH)

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PREPARE_SCALAR_QR(DDH, DDL, Q)

    DOUBLE PRECISION, INTENT(IN) :: DDH, DDL
    REAL(16), INTENT(OUT) :: Q

    CALL DD2Q(DDH, DDL, Q)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PREPARE_SCALAR_QC(DDH, DDL, Q)

    DOUBLE COMPLEX, INTENT(IN) :: DDH, DDL
    COMPLEX(16), INTENT(OUT) :: Q

    CALL ZZ2W(DDH, DDL, Q)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PREPARE_VECTOR_QR(DDH, DDL, Q)

    DOUBLE PRECISION, INTENT(IN) :: DDH(:), DDL(:)
    REAL(16), INTENT(OUT) :: Q(:)

    CALL SIZE_CHECK(DDH, DDL, SIZE(Q,1))
    CALL DD2Q(DDH, DDL, Q)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PREPARE_VECTOR_QC(DDH, DDL, Q)

    DOUBLE COMPLEX, INTENT(IN) :: DDH(:), DDL(:)
    COMPLEX(16), INTENT(OUT) :: Q(:)

    CALL SIZE_CHECK(DDH, DDL, SIZE(Q,1))
    CALL ZZ2W(DDH, DDL, Q)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PREPARE_MATRIX_QR(DDH, DDL, Q)

    DOUBLE PRECISION, INTENT(IN) :: DDH(:,:), DDL(:,:)
    REAL(16), INTENT(OUT) :: Q(:,:)

    CALL SIZE_CHECK(DDH, DDL, SIZE(Q,1), SIZE(Q,2))
    CALL DD2Q(DDH, DDL, Q)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PREPARE_MATRIX_QC(DDH, DDL, Q)

    DOUBLE COMPLEX, INTENT(IN) :: DDH(:,:), DDL(:,:)
    COMPLEX(16), INTENT(OUT) :: Q(:,:)

    CALL SIZE_CHECK(DDH, DDL, SIZE(Q,1), SIZE(Q,2))
    CALL ZZ2W(DDH, DDL, Q)

    END SUBROUTINE
#endif


    SUBROUTINE TEST_PRINT_SCALAR_D(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    DOUBLE PRECISION, INTENT(IN) :: VALUE

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " =", VALUE

    END SUBROUTINE


    SUBROUTINE TEST_PRINT_SCALAR_Z(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    DOUBLE COMPLEX, INTENT(IN) :: VALUE

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " =", VALUE

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PRINT_SCALAR_Q(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    REAL(16), INTENT(IN) :: VALUE

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " =", VALUE

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PRINT_SCALAR_W(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    COMPLEX(16), INTENT(IN) :: VALUE

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " =", VALUE

    END SUBROUTINE
#endif


    SUBROUTINE TEST_PRINT_VECTOR_D(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    DOUBLE PRECISION, INTENT(IN) :: VALUE(:)

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " ="
    CALL PRINT_VECTOR(VALUE)

    END SUBROUTINE


    SUBROUTINE TEST_PRINT_VECTOR_Z(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    DOUBLE COMPLEX, INTENT(IN) :: VALUE(:)

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " ="
    CALL PRINT_VECTOR(VALUE)

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PRINT_VECTOR_Q(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    REAL(16), INTENT(IN) :: VALUE(:)

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " ="
    CALL PRINT_VECTOR(VALUE)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PRINT_VECTOR_W(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    COMPLEX(16), INTENT(IN) :: VALUE(:)

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " ="
    CALL PRINT_VECTOR(VALUE)

    END SUBROUTINE
#endif


    SUBROUTINE TEST_PRINT_MATRIX_D(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    DOUBLE PRECISION, INTENT(IN) :: VALUE(:,:)

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " ="
    CALL PRINT_MATRIX(VALUE)

    END SUBROUTINE


    SUBROUTINE TEST_PRINT_MATRIX_Z(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    DOUBLE COMPLEX, INTENT(IN) :: VALUE(:,:)

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " ="
    CALL PRINT_MATRIX(VALUE)

    END SUBROUTINE


    SUBROUTINE TEST_PRINT_MATRIX_DD(FNAME, VNAME, IO, AH, AL)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    DOUBLE PRECISION, INTENT(IN) :: AH(:,:), AL(:,:)

#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: QA(:,:)
#endif

#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    CALL DD2Q(AH, AL, QA)
    CALL TEST_PRINT(FNAME, VNAME, IO, QA)
#else
    CALL TEST_PRINT(FNAME, VNAME, IO, AH)
#endif

    END SUBROUTINE


    SUBROUTINE TEST_PRINT_MATRIX_ZZ(FNAME, VNAME, IO, AH, AL)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    DOUBLE COMPLEX, INTENT(IN) :: AH(:,:), AL(:,:)

#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: QA(:,:)
#endif

#ifdef QUAD_PRECISION
    ALLOCATE(QA(SIZE(AH,1), SIZE(AH,2)))
    CALL ZZ2W(AH, AL, QA)
    CALL TEST_PRINT(FNAME, VNAME, IO, QA)
#else
    CALL TEST_PRINT(FNAME, VNAME, IO, AH)
#endif

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PRINT_MATRIX_Q(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    REAL(16), INTENT(IN) :: VALUE(:,:)

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " ="
    CALL PRINT_MATRIX(VALUE)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_PRINT_MATRIX_W(FNAME, VNAME, IO, VALUE)

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME, IO
    COMPLEX(16), INTENT(IN) :: VALUE(:,:)

    IF ((.NOT. TEST_MODE_VERBOSE) .OR. (LEN(VNAME) .EQ. 0)) RETURN

    PRINT *, "[", FNAME ,"] ", IO, " ", VNAME, " ="
    CALL PRINT_MATRIX(VALUE)

    END SUBROUTINE
#endif


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_VERIFY_SCALAR_R(FNAME, VNAME, DDH, DDL, D, Q)
#else
    SUBROUTINE TEST_VERIFY_SCALAR_R(FNAME, VNAME, DDH, DDL, D)
#endif

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE PRECISION, INTENT(IN) :: DDH, DDL
    DOUBLE PRECISION, INTENT(IN) :: D
#ifdef QUAD_PRECISION
    REAL(16), INTENT(IN) :: Q
#endif

    DOUBLE PRECISION :: DDHA(1), DDLA(1)
    DOUBLE PRECISION :: DA(1)
#ifdef QUAD_PRECISION
    REAL(16) :: QA(1)
#endif

    DDHA(1) = DDH
    DDLA(1) = DDL
    DA(1) = D
#ifdef QUAD_PRECISION
    QA(1) = Q
    CALL TEST_VERIFY(FNAME, VNAME, DDHA, DDLA, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, VNAME, DDHA, DDLA, DA)
#endif

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_VERIFY_SCALAR_C(FNAME, VNAME, DDH, DDL, D, Q)
#else
    SUBROUTINE TEST_VERIFY_SCALAR_C(FNAME, VNAME, DDH, DDL, D)
#endif

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE COMPLEX, INTENT(IN) :: DDH, DDL
    DOUBLE COMPLEX, INTENT(IN) :: D
#ifdef QUAD_PRECISION
    COMPLEX(16), INTENT(IN) :: Q
#endif

    DOUBLE COMPLEX :: DDHA(1), DDLA(1)
    DOUBLE COMPLEX :: DA(1)
#ifdef QUAD_PRECISION
    COMPLEX(16) :: QA(1)
#endif

    DDHA(1) = DDH
    DDLA(1) = DDL
    DA(1) = D
#ifdef QUAD_PRECISION
    QA(1) = Q
    CALL TEST_VERIFY(FNAME, VNAME, DDHA, DDLA, DA, QA)
#else
    CALL TEST_VERIFY(FNAME, VNAME, DDHA, DDLA, DA)
#endif

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_VERIFY_SCALAR_I(FNAME, VNAME, DD, D, Q)
#else
    SUBROUTINE TEST_VERIFY_SCALAR_I(FNAME, VNAME, DD, D)
#endif

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    INTEGER, INTENT(IN) :: DD
    INTEGER, INTENT(IN) :: D
#ifdef QUAD_PRECISION
    INTEGER, INTENT(IN) :: Q
#endif

    IF (TEST_MODE_VERBOSE) THEN
#ifdef QUAD_PRECISION
        CALL TEST_PRINT("IDD"//FNAME, VNAME, "OUTPUT", REAL(DD, 16))
        CALL TEST_PRINT("ID"//FNAME, VNAME, "OUTPUT", REAL(D, 16))
        CALL TEST_PRINT("IQ"//FNAME, VNAME, "OUTPUT", REAL(Q, 16))
#else
        CALL TEST_PRINT("IDD"//FNAME, VNAME, "OUTPUT", DBLE(DD))
        CALL TEST_PRINT("ID"//FNAME, VNAME, "OUTPUT", DBLE(D))
#endif
    END IF

    IF (TEST_MODE_VERIFY) THEN
        CALL PRINT_DISTANCE("ID"//FNAME//" IDD"//FNAME//" "//VNAME, D, DD)
#ifdef QUAD_PRECISION
        CALL PRINT_DISTANCE("IQ"//FNAME//" IDD"//FNAME//" "//VNAME, Q, DD)
        CALL PRINT_DISTANCE("IQ"//FNAME//" ID"//FNAME//" "//VNAME, Q, D)
#endif
    END IF

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_VERIFY_VECTOR_R(FNAME, VNAME, DDH, DDL, D, Q)
#else
    SUBROUTINE TEST_VERIFY_VECTOR_R(FNAME, VNAME, DDH, DDL, D)
#endif

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE PRECISION, INTENT(IN) :: DDH(:), DDL(:)
    DOUBLE PRECISION, INTENT(IN) :: D(:)
#ifdef QUAD_PRECISION
    REAL(16), INTENT(IN) :: Q(:)
#endif

#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: DDQ(:)
#endif

    IF (TEST_MODE_VERBOSE) THEN
#ifdef QUAD_PRECISION
        ALLOCATE(DDQ(SIZE(DDH,1)))
        CALL DD2Q(DDH, DDL, DDQ)
        CALL TEST_PRINT("DD"//FNAME, VNAME, "OUTPUT", DDQ)
#else
        CALL TEST_PRINT("DD"//FNAME, VNAME//"H", "OUTPUT", DDH)
        CALL TEST_PRINT("DD"//FNAME, VNAME//"L", "OUTPUT", DDL)
#endif
        CALL TEST_PRINT("D"//FNAME, VNAME, "OUTPUT", D)
#ifdef QUAD_PRECISION
        CALL TEST_PRINT("Q"//FNAME, VNAME, "OUTPUT", Q)
#endif
    END IF

    IF (TEST_MODE_VERIFY) THEN
        CALL PRINT_DISTANCE("D"//FNAME//" DD"//FNAME//" "//VNAME, D, DDH, DDL)
#ifdef QUAD_PRECISION
        CALL PRINT_DISTANCE("Q"//FNAME//" DD"//FNAME//" "//VNAME, Q, DDH, DDL)
        CALL PRINT_DISTANCE("Q"//FNAME//" D"//FNAME//" "//VNAME, Q, D)
#endif
    END IF

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_VERIFY_VECTOR_C(FNAME, VNAME, DDH, DDL, D, Q)
#else
    SUBROUTINE TEST_VERIFY_VECTOR_C(FNAME, VNAME, DDH, DDL, D)
#endif

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE COMPLEX, INTENT(IN) :: DDH(:), DDL(:)
    DOUBLE COMPLEX, INTENT(IN) :: D(:)
#ifdef QUAD_PRECISION
    COMPLEX(16), INTENT(IN) :: Q(:)
#endif

#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: DDQ(:)
#endif

    IF (TEST_MODE_VERBOSE) THEN
#ifdef QUAD_PRECISION
        ALLOCATE(DDQ(SIZE(DDH,1)))
        CALL ZZ2W(DDH, DDL, DDQ)
        CALL TEST_PRINT("ZZ"//FNAME, VNAME, "OUTPUT", DDQ)
#else
        CALL TEST_PRINT("ZZ"//FNAME, VNAME//"H", "OUTPUT", DDH)
        CALL TEST_PRINT("ZZ"//FNAME, VNAME//"L", "OUTPUT", DDL)
#endif
        CALL TEST_PRINT("Z"//FNAME, VNAME, "OUTPUT", D)
#ifdef QUAD_PRECISION
        CALL TEST_PRINT("W"//FNAME, VNAME, "OUTPUT", Q)
#endif
    END IF

    IF (TEST_MODE_VERIFY) THEN
        CALL PRINT_DISTANCE("Z"//FNAME//" ZZ"//FNAME//" "//VNAME, D, DDH, DDL)
#ifdef QUAD_PRECISION
        CALL PRINT_DISTANCE("W"//FNAME//" ZZ"//FNAME//" "//VNAME, Q, DDH, DDL)
        CALL PRINT_DISTANCE("W"//FNAME//" Z"//FNAME//" "//VNAME, Q, D)
#endif
    END IF

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_VERIFY_MATRIX_R(FNAME, VNAME, DDH, DDL, D, Q)
#else
    SUBROUTINE TEST_VERIFY_MATRIX_R(FNAME, VNAME, DDH, DDL, D)
#endif

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE PRECISION, INTENT(IN) :: DDH(:,:), DDL(:,:)
    DOUBLE PRECISION, INTENT(IN) :: D(:,:)
#ifdef QUAD_PRECISION
    REAL(16), INTENT(IN) :: Q(:,:)
#endif

#ifdef QUAD_PRECISION
    REAL(16), ALLOCATABLE :: DDQ(:,:)
#endif

    IF (TEST_MODE_VERBOSE) THEN
#ifdef QUAD_PRECISION
        ALLOCATE(DDQ(SIZE(DDH,1), SIZE(DDH,2)))
        CALL DD2Q(DDH, DDL, DDQ)
        CALL TEST_PRINT("DD"//FNAME, VNAME, "OUTPUT", DDQ)
#else
        CALL TEST_PRINT("DD"//FNAME, VNAME//"H", "OUTPUT", DDH)
        CALL TEST_PRINT("DD"//FNAME, VNAME//"L", "OUTPUT", DDL)
#endif
        CALL TEST_PRINT("D"//FNAME, VNAME, "OUTPUT", D)
#ifdef QUAD_PRECISION
        CALL TEST_PRINT("Q"//FNAME, VNAME, "OUTPUT", Q)
#endif
    END IF

    IF (TEST_MODE_VERIFY) THEN
        CALL PRINT_DISTANCE("D"//FNAME//" DD"//FNAME//" "//VNAME, D, DDH, DDL)
#ifdef QUAD_PRECISION
        CALL PRINT_DISTANCE("Q"//FNAME//" DD"//FNAME//" "//VNAME, Q, DDH, DDL)
        CALL PRINT_DISTANCE("Q"//FNAME//" D"//FNAME//" "//VNAME, Q, D)
#endif
    END IF

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE TEST_VERIFY_MATRIX_C(FNAME, VNAME, DDH, DDL, D, Q)
#else
    SUBROUTINE TEST_VERIFY_MATRIX_C(FNAME, VNAME, DDH, DDL, D)
#endif

    CHARACTER(*), INTENT(IN) :: FNAME, VNAME
    DOUBLE COMPLEX, INTENT(IN) :: DDH(:,:), DDL(:,:)
    DOUBLE COMPLEX, INTENT(IN) :: D(:,:)
#ifdef QUAD_PRECISION
    COMPLEX(16), INTENT(IN) :: Q(:,:)
#endif

#ifdef QUAD_PRECISION
    COMPLEX(16), ALLOCATABLE :: DDQ(:,:)
#endif

    IF (TEST_MODE_VERBOSE) THEN
#ifdef QUAD_PRECISION
        ALLOCATE(DDQ(SIZE(DDH,1),SIZE(DDH,2)))
        CALL ZZ2W(DDH, DDL, DDQ)
        CALL TEST_PRINT("ZZ"//FNAME, VNAME, "OUTPUT", DDQ)
#else
        CALL TEST_PRINT("ZZ"//FNAME, VNAME//"H", "OUTPUT", DDH)
        CALL TEST_PRINT("ZZ"//FNAME, VNAME//"L", "OUTPUT", DDL)
#endif
        CALL TEST_PRINT("Z"//FNAME, VNAME, "OUTPUT", D)
#ifdef QUAD_PRECISION
        CALL TEST_PRINT("W"//FNAME, VNAME, "OUTPUT", Q)
#endif
    END IF

    IF (TEST_MODE_VERIFY) THEN
        CALL PRINT_DISTANCE("Z"//FNAME//" ZZ"//FNAME//" "//VNAME, D, DDH, DDL)
#ifdef QUAD_PRECISION
        CALL PRINT_DISTANCE("W"//FNAME//" ZZ"//FNAME//" "//VNAME, Q, DDH, DDL)
        CALL PRINT_DISTANCE("W"//FNAME//" Z"//FNAME//" "//VNAME, Q, D)
#endif
    END IF

    END SUBROUTINE


    SUBROUTINE PRINT_DISTANCE_D_D_VECTOR(FUNCTION_NAME, DX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE PRECISION, INTENT(IN) :: DX(:)
    DOUBLE PRECISION, INTENT(IN) :: DY(:)

    DOUBLE PRECISION MAX_ERROR, DIFF
    INTEGER I

    MAX_ERROR = 0.0d0

    IF ( SIZE(DX) .NE. SIZE(DY) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    DO I = 1, SIZE(DX)
        IF (ISNAN(DX(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF (ISNAN(DY(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        DIFF = ABS(DX(I)-DY(I))
        IF (MAX_ERROR .LT. DIFF) THEN
            MAX_ERROR = DIFF
        END IF
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE

#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_Q_D_VECTOR(FUNCTION_NAME, QX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    REAL(16), INTENT(IN) :: QX(:)
    DOUBLE PRECISION, INTENT(IN) :: DY(:)

    REAL(16) MAX_ERROR, DIFF
    INTEGER I

    MAX_ERROR = 0.0q0

    IF ( SIZE(QX) .NE. SIZE(DY) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    DO I = 1, SIZE(QX)
        IF (ISNAN(QX(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF (ISNAN(DY(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        DIFF = ABS(QX(I)-DY(I))
        IF (MAX_ERROR .LT. DIFF) THEN
            MAX_ERROR = DIFF
        END IF
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_D_DD_VECTOR(FUNCTION_NAME, DX, DYH, DYL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE PRECISION, INTENT(IN) :: DX(:)
    DOUBLE PRECISION, INTENT(IN) :: DYH(:)
    DOUBLE PRECISION, INTENT(IN) :: DYL(:)

    DOUBLE PRECISION MAX_ERROR, DIFF
    INTEGER I

    MAX_ERROR = 0.0d0

    IF ( SIZE(DX) .NE. SIZE(DYH) .OR. SIZE(DX) .NE. SIZE(DYL) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    DO I = 1, SIZE(DX)
        IF (ISNAN(DX(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF (ISNAN(DYH(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF (ISNAN(DYL(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        DIFF = ABS(DX(I)-DYH(I)-DYL(I))
        IF (MAX_ERROR .LT. DIFF) THEN
            MAX_ERROR = DIFF
        END IF
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_Q_DD_VECTOR(FUNCTION_NAME, QX, DYH, DYL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    REAL(16), INTENT(IN) :: QX(:)
    DOUBLE PRECISION, INTENT(IN) :: DYH(:)
    DOUBLE PRECISION, INTENT(IN) :: DYL(:)

    REAL(16) MAX_ERROR, QY, DIFF
    INTEGER I

    MAX_ERROR = 0.0q0

    IF ( SIZE(QX) .NE. SIZE(DYH) .OR. SIZE(QX) .NE. SIZE(DYL) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    DO I = 1, SIZE(QX)
        IF (ISNAN(QX(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF (ISNAN(DYH(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF (ISNAN(DYL(I))) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        CALL DD2Q(DYH(I),DYL(I),QY)
        DIFF = ABS(QX(I)-QY)
        IF (MAX_ERROR .LT. DIFF) THEN
            MAX_ERROR = DIFF
        END IF
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_Z_Z_VECTOR(FUNCTION_NAME, DX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE COMPLEX, INTENT(IN) :: DX(:)
    DOUBLE COMPLEX, INTENT(IN) :: DY(:)

    DOUBLE PRECISION MAX_ERROR, DIFF
    INTEGER I

    MAX_ERROR = 0.0d0

    IF ( SIZE(DX) .NE. SIZE(DY) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    DO I = 1, SIZE(DX)
        IF ( ISNAN( REAL(DX(I)) ) .OR. ISNAN( IMAG(DX(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF ( ISNAN( REAL(DY(I)) ) .OR. ISNAN( IMAG(DY(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        DIFF = ABS(DX(I)-DY(I))
        IF (MAX_ERROR .LT. DIFF) THEN
            MAX_ERROR = DIFF
        END IF
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_W_Z_VECTOR(FUNCTION_NAME, QX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    COMPLEX(16), INTENT(IN) :: QX(:)
    DOUBLE COMPLEX, INTENT(IN) :: DY(:)

    REAL(16) MAX_ERROR, DIFF
    INTEGER I

    MAX_ERROR = 0.0q0

    IF ( SIZE(QX) .NE. SIZE(DY) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    DO I = 1, SIZE(QX) 
        IF ( ISNAN( REAL(QX(I)) ) .OR. ISNAN( IMAG(QX(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF ( ISNAN( REAL(DY(I)) ) .OR. ISNAN( IMAG(DY(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        DIFF = ABS(QX(I)-DY(I))
        IF (MAX_ERROR .LT. DIFF) THEN
            MAX_ERROR = DIFF
        END IF
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_Z_ZZ_VECTOR(FUNCTION_NAME, DX, DYH, DYL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE COMPLEX, INTENT(IN) :: DX(:)
    DOUBLE COMPLEX, INTENT(IN) :: DYH(:)
    DOUBLE COMPLEX, INTENT(IN) :: DYL(:)

    DOUBLE PRECISION MAX_ERROR, DIFF
    INTEGER I

    MAX_ERROR = 0.0d0

    IF ( SIZE(DX) .NE. SIZE(DYH) .OR. SIZE(DX) .NE. SIZE(DYL) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    DO I = 1, SIZE(DX)
        IF ( ISNAN( REAL(DX(I)) ) .OR. ISNAN( IMAG(DX(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF ( ISNAN( REAL(DYH(I)) ) .OR. ISNAN( IMAG(DYH(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF ( ISNAN( REAL(DYL(I)) ) .OR. ISNAN( IMAG(DYL(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        DIFF = ABS(DX(I)-DYH(I)-DYL(I))
        IF (MAX_ERROR .LT. DIFF) THEN
            MAX_ERROR = DIFF
        END IF
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_W_ZZ_VECTOR(FUNCTION_NAME, QX, DYH, DYL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    COMPLEX(16), INTENT(IN) :: QX(:)
    DOUBLE COMPLEX, INTENT(IN) :: DYH(:)
    DOUBLE COMPLEX, INTENT(IN) :: DYL(:)

    COMPLEX(16) QY
    REAL(16) MAX_ERROR, DIFF
    INTEGER I

    MAX_ERROR = 0.0q0

    IF ( SIZE(QX) .NE. SIZE(DYH) .OR. SIZE(QX) .NE. SIZE(DYL) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    DO I = 1, SIZE(QX)
        IF ( ISNAN( REAL(QX(I)) ) .OR. ISNAN( IMAG(QX(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF ( ISNAN( REAL(DYH(I)) ) .OR. ISNAN( IMAG(DYH(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        IF ( ISNAN( REAL(DYL(I)) ) .OR. ISNAN( IMAG(DYL(I)) ) ) THEN
            WRITE (*, fmt='(a,a,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,") NaN detected"
            RETURN
        END IF
        CALL ZZ2W(DYH(I),DYL(I),QY)
        DIFF = ABS(QX(I)-QY)
        IF (MAX_ERROR .LT. DIFF) THEN
            MAX_ERROR = DIFF
        END IF
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_I_I_SCALAR(FUNCTION_NAME, IX, IY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    INTEGER, INTENT(IN) :: IX
    INTEGER, INTENT(IN) :: IY

    INTEGER DIST
    DIST = ABS(IX - IY)

    WRITE (*, fmt='(a,a,a,i8)') " [", FUNCTION_NAME, "] absolute maximum error: ", DIST

    END SUBROUTINE

    SUBROUTINE PRINT_DISTANCE_D_D_SCALAR(FUNCTION_NAME, DX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE PRECISION, INTENT(IN) :: DX
    DOUBLE PRECISION, INTENT(IN) :: DY

    DOUBLE PRECISION DAX(1)
    DOUBLE PRECISION DAY(1)

    DAX(1) = DX
    DAY(1) = DY

    CALL PRINT_DISTANCE_D_D_VECTOR(FUNCTION_NAME, DAX, DAY)

    END SUBROUTINE

#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_Q_D_SCALAR(FUNCTION_NAME, QX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    REAL(16), INTENT(IN) :: QX
    DOUBLE PRECISION, INTENT(IN) :: DY

    REAL(16) QAX(1)
    DOUBLE PRECISION DAY(1)

    QAX(1) = QX
    DAY(1) = DY

    CALL PRINT_DISTANCE_Q_D_VECTOR(FUNCTION_NAME, QAX, DAY)

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_D_DD_SCALAR(FUNCTION_NAME, DX, DYH, DYL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE PRECISION, INTENT(IN) :: DX
    DOUBLE PRECISION, INTENT(IN) :: DYH
    DOUBLE PRECISION, INTENT(IN) :: DYL

    DOUBLE PRECISION DAX(1)
    DOUBLE PRECISION DAYH(1)
    DOUBLE PRECISION DAYL(1)

    DAX(1) = DX
    DAYH(1) = DYH
    DAYL(1) = DYL

    CALL PRINT_DISTANCE_D_DD_VECTOR(FUNCTION_NAME, DAX, DAYH, DAYL)

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_Q_DD_SCALAR(FUNCTION_NAME, QX, DYH, DYL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    REAL(16), INTENT(IN) :: QX
    DOUBLE PRECISION, INTENT(IN) :: DYH
    DOUBLE PRECISION, INTENT(IN) :: DYL

    REAL(16) QAX(1)
    DOUBLE PRECISION DAYH(1)
    DOUBLE PRECISION DAYL(1)

    QAX(1) = QX
    DAYH(1) = DYH
    DAYL(1) = DYL

    CALL PRINT_DISTANCE_Q_DD_VECTOR(FUNCTION_NAME, QAX, DAYH, DAYL)

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_Z_Z_SCALAR(FUNCTION_NAME, DX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE COMPLEX, INTENT(IN) :: DX
    DOUBLE COMPLEX, INTENT(IN) :: DY

    DOUBLE COMPLEX DAX(1)
    DOUBLE COMPLEX DAY(1)

    DAX(1) = DX
    DAY(1) = DY

    CALL PRINT_DISTANCE_Z_Z_VECTOR(FUNCTION_NAME, DAX, DAY)

    END SUBROUTINE

#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_W_Z_SCALAR(FUNCTION_NAME, QX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    COMPLEX(16), INTENT(IN) :: QX
    DOUBLE COMPLEX, INTENT(IN) :: DY

    COMPLEX(16) QAX(1)
    DOUBLE COMPLEX DAY(1)

    QAX(1) = QX
    DAY(1) = DY

    CALL PRINT_DISTANCE_W_Z_VECTOR(FUNCTION_NAME, QAX, DAY)

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_Z_ZZ_SCALAR(FUNCTION_NAME, DX, DYH, DYL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE COMPLEX, INTENT(IN) :: DX
    DOUBLE COMPLEX, INTENT(IN) :: DYH
    DOUBLE COMPLEX, INTENT(IN) :: DYL

    DOUBLE COMPLEX DAX(1)
    DOUBLE COMPLEX DAYH(1)
    DOUBLE COMPLEX DAYL(1)

    DAX(1) = DX
    DAYH(1) = DYH
    DAYL(1) = DYL

    CALL PRINT_DISTANCE_Z_ZZ_VECTOR(FUNCTION_NAME, DAX, DAYH, DAYL)

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_W_ZZ_SCALAR(FUNCTION_NAME, QX, DYH, DYL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    COMPLEX(16), INTENT(IN) :: QX
    DOUBLE COMPLEX, INTENT(IN) :: DYH
    DOUBLE COMPLEX, INTENT(IN) :: DYL

    COMPLEX(16) QAX(1)
    DOUBLE COMPLEX DAYH(1)
    DOUBLE COMPLEX DAYL(1)

    QAX(1) = QX
    DAYH(1) = DYH
    DAYL(1) = DYL

    CALL PRINT_DISTANCE_W_ZZ_VECTOR(FUNCTION_NAME, QAX, DAYH, DAYL)

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_D_D_MATRIX(FUNCTION_NAME, DX, DY)

    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE PRECISION, INTENT(IN) :: DX(:,:), DY(:,:)
    DOUBLE PRECISION MAX_ERROR, DIFF
    INTEGER I, J, M, N

    MAX_ERROR = 0.0d0

    IF ( SIZE(DX,1) .NE. SIZE(DY,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF
    IF ( SIZE(DX,2) .NE. SIZE(DY,2) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    M = SIZE(DX, 1)
    N = SIZE(DX, 2)
    DO I = 1, M
        DO J = 1, N
           IF (ISNAN(DX(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF (ISNAN(DY(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           DIFF = ABS(DX(I,J)-DY(I,J))
           IF (MAX_ERROR .LT. DIFF) THEN
              MAX_ERROR = DIFF
           END IF
        END DO
    END DO

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_Q_D_MATRIX(FUNCTION_NAME, QX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    REAL(16), INTENT(IN) :: QX(:,:)
    DOUBLE PRECISION, INTENT(IN) :: DY(:,:)

    REAL(16) MAX_ERROR, DIFF
    INTEGER I, J, M, N

    MAX_ERROR = 0.0q0

    IF ( SIZE(QX,1) .NE. SIZE(DY,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF
    IF ( SIZE(QX,1) .NE. SIZE(DY,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    M = SIZE(QX, 1)
    N = SIZE(QX, 2)
    DO I = 1, M
        DO J = 1, N
           IF (ISNAN(QX(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF (ISNAN(DY(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           DIFF = ABS(QX(I,J)-DY(I,J))
           IF (MAX_ERROR .LT. DIFF) THEN
              MAX_ERROR = DIFF
           END IF
        END DO
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_D_DD_MATRIX(FUNCTION_NAME, DX, YH, YL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE PRECISION, INTENT(IN) :: DX(:,:)
    DOUBLE PRECISION, INTENT(IN) :: YH(:,:)
    DOUBLE PRECISION, INTENT(IN) :: YL(:,:)

    DOUBLE PRECISION MAX_ERROR, DIFF
    INTEGER I, J, M, N

    MAX_ERROR = 0.0d0

    IF ( SIZE(DX,1) .NE. SIZE(YH,1) .OR. SIZE(DX,1) .NE. SIZE(YL,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF
    IF ( SIZE(DX,2) .NE. SIZE(YH,2) .OR. SIZE(DX,2) .NE. SIZE(YL,2) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    M = SIZE(DX, 1)
    N = SIZE(DX, 2)
    DO I = 1, M
        DO J = 1, N
           IF (ISNAN(DX(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF (ISNAN(YH(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF (ISNAN(YL(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           DIFF = ABS(DX(I,J)-YH(I,J)-YL(I,J))
           IF (MAX_ERROR .LT. DIFF) THEN
              MAX_ERROR = DIFF
           END IF
        END DO
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_Q_DD_MATRIX(FUNCTION_NAME, QX, YH, YL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    REAL(16), INTENT(IN) :: QX(:,:)
    DOUBLE PRECISION, INTENT(IN) :: YH(:,:)
    DOUBLE PRECISION, INTENT(IN) :: YL(:,:)

    REAL(16) MAX_ERROR, QY, DIFF
    INTEGER I, J, M, N

    MAX_ERROR = 0.0q0

    IF ( SIZE(QX,1) .NE. SIZE(YH,1) .OR. SIZE(QX,1) .NE. SIZE(YL,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF
    IF ( SIZE(QX,2) .NE. SIZE(YH,2) .OR. SIZE(QX,2) .NE. SIZE(YL,2) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    M = SIZE(QX, 1)
    N = SIZE(QX, 2)
    DO I = 1, M
        DO J = 1, N
           IF (ISNAN(QX(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF (ISNAN(YH(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF (ISNAN(YL(I,J))) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           CALL DD2Q(YH(I,J),YL(I,J),QY)
           DIFF = ABS(QX(I,J)-QY)
           IF (MAX_ERROR .LT. DIFF) THEN
              MAX_ERROR = DIFF
           END IF
        END DO
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_Z_Z_MATRIX(FUNCTION_NAME, DX, DY)

    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE COMPLEX, INTENT(IN) :: DX(:,:), DY(:,:)
    DOUBLE PRECISION MAX_ERROR, DIFF
    INTEGER I, J, M, N

    MAX_ERROR = 0.0d0

    IF ( SIZE(DX,1) .NE. SIZE(DY,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF
    IF ( SIZE(DX,2) .NE. SIZE(DY,2) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    M = SIZE(DX, 1)
    N = SIZE(DX, 2)
    DO I = 1, M
        DO J = 1, N
           IF ( ISNAN(REAL(DX(I,J))) .OR. ISNAN(IMAG(DX(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF ( ISNAN(REAL(DY(I,J))) .OR. ISNAN(IMAG(DY(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           DIFF = ABS(DX(I,J)-DY(I,J))
           IF (MAX_ERROR .LT. DIFF) THEN
              MAX_ERROR = DIFF
           END IF
        END DO
    END DO

    END SUBROUTINE


#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_W_Z_MATRIX(FUNCTION_NAME, QX, DY)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    COMPLEX(16), INTENT(IN) :: QX(:,:)
    DOUBLE COMPLEX, INTENT(IN) :: DY(:,:)

    REAL(16) MAX_ERROR, DIFF
    INTEGER I, J, M, N

    MAX_ERROR = 0.0q0

    IF ( SIZE(QX,1) .NE. SIZE(DY,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF
    IF ( SIZE(QX,1) .NE. SIZE(DY,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    M = SIZE(QX, 1)
    N = SIZE(QX, 2)
    DO I = 1, M
        DO J = 1, N
           IF ( ISNAN(REAL(QX(I,J))) .OR. ISNAN(IMAG(QX(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF ( ISNAN(REAL(DY(I,J))) .OR. ISNAN(IMAG(DY(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           DIFF = ABS(QX(I,J)-DY(I,J))
           IF (MAX_ERROR .LT. DIFF) THEN
              MAX_ERROR = DIFF
           END IF
        END DO
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE
#endif


    SUBROUTINE PRINT_DISTANCE_Z_ZZ_MATRIX(FUNCTION_NAME, DX, YH, YL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    DOUBLE COMPLEX, INTENT(IN) :: DX(:,:)
    DOUBLE COMPLEX, INTENT(IN) :: YH(:,:)
    DOUBLE COMPLEX, INTENT(IN) :: YL(:,:)

    DOUBLE PRECISION MAX_ERROR, DIFF
    INTEGER I, J, M, N

    MAX_ERROR = 0.0d0

    IF ( SIZE(DX,1) .NE. SIZE(YH,1) .OR. SIZE(DX,1) .NE. SIZE(YL,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF
    IF ( SIZE(DX,2) .NE. SIZE(YH,2) .OR. SIZE(DX,2) .NE. SIZE(YL,2) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    M = SIZE(DX, 1)
    N = SIZE(DX, 2)
    DO I = 1, M
        DO J = 1, N
           IF ( ISNAN(REAL(DX(I,J))) .OR. ISNAN(IMAG(DX(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF ( ISNAN(REAL(YH(I,J))) .OR. ISNAN(IMAG(YH(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF ( ISNAN(REAL(YL(I,J))) .OR. ISNAN(IMAG(YL(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           DIFF = ABS(DX(I,J)-YH(I,J)-YL(I,J))
           IF (MAX_ERROR .LT. DIFF) THEN
              MAX_ERROR = DIFF
           END IF
        END DO
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE

#ifdef QUAD_PRECISION
    SUBROUTINE PRINT_DISTANCE_W_ZZ_MATRIX(FUNCTION_NAME, QX, YH, YL)
    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    COMPLEX(16), INTENT(IN) :: QX(:,:)
    DOUBLE COMPLEX, INTENT(IN) :: YH(:,:)
    DOUBLE COMPLEX, INTENT(IN) :: YL(:,:)
    COMPLEX(16) QY

    REAL(16) MAX_ERROR, DIFF
    INTEGER I, J, M, N

    MAX_ERROR = 0.0q0

    IF ( SIZE(QX,1) .NE. SIZE(YH,1) .OR. SIZE(QX,1) .NE. SIZE(YL,1) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF
    IF ( SIZE(QX,2) .NE. SIZE(YH,2) .OR. SIZE(QX,2) .NE. SIZE(YL,2) ) THEN
        WRITE (*,*) "array size mismatch"
        RETURN
    END IF

    M = SIZE(QX, 1)
    N = SIZE(QX, 2)
    DO I = 1, M
        DO J = 1, N
           IF ( ISNAN(REAL(QX(I,J))) .OR. ISNAN(IMAG(QX(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF ( ISNAN(REAL(YH(I,J))) .OR. ISNAN(IMAG(YH(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           IF ( ISNAN(REAL(YL(I,J))) .OR. ISNAN(IMAG(YL(I,J))) ) THEN
              WRITE (*, fmt='(a,a,a,i8,a,i8,a)') " [", FUNCTION_NAME, "] (", I ,",",J,") NaN detected"
              RETURN
           END IF
           CALL ZZ2W(YH(I,J),YL(I,J),QY)
           DIFF = ABS(QX(I,J)-QY)
           IF (MAX_ERROR .LT. DIFF) THEN
              MAX_ERROR = DIFF
           END IF
        END DO
    END DO
    WRITE (*, fmt='(a,a,a,e15.8)') " [", FUNCTION_NAME, "] absolute maximum error: ", MAX_ERROR

    END SUBROUTINE
#endif

#ifdef ISNAN_NOT_FOUND
    LOGICAL FUNCTION ISNAN_D(X)

    DOUBLE PRECISION, INTENT(IN) :: X

    ISNAN_D = X .NE. X

    END FUNCTION


#ifdef QUAD_PRECISION
    LOGICAL FUNCTION ISNAN_Q(X)

    REAL(16), INTENT(IN) :: X

    ISNAN_Q = X .NE. X

    END FUNCTION
#endif
#endif


    SUBROUTINE CLOCK_START()

    CALL SYSTEM_CLOCK(TIME)

    END SUBROUTINE


    SUBROUTINE CLOCK_STOP(FUNCTION_NAME)

    CHARACTER(*), INTENT(IN) :: FUNCTION_NAME
    INTEGER TIME2, RATE, MAX, DIFF

    IF (.NOT. TEST_MODE_BENCHMARK) RETURN

    CALL SYSTEM_CLOCK(TIME2, RATE, MAX)
    IF (TIME2 .LT. TIME) THEN
        DIFF = MAX - TIME + TIME2
    ELSE
        DIFF = TIME2 - TIME
    END IF
    WRITE (*, fmt='(a,a,a,f7.3,a)') " [", FUNCTION_NAME, "] ", DIFF / DBLE(RATE), " sec"

    END SUBROUTINE


    END MODULE
